/***********************************************************************
Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.
*****************************#NETL_2#********************************/
#define MSGRING_DUMP_MESSAGES

#include <stdio.h>
#include <sys/mman.h>
#include <fcntl.h>

#include "inttypes.h"
#include "errno.h"
#include "malloc.h"
#include "nlm_hal_fmn.h"
#include "nlm_hal_macros.h"
#include "mm.h"

/* #define DEBUG */

#define Message(a, b...)                                printf(a, ##b)
#define shift_lower_bits(x, bitshift, numofbits)        (((unsigned long long)(x) & ((1ULL << (numofbits)) - 1)) << (bitshift))
#define xtract_bits(x, bitpos, numofbits)               ((x) >> (bitpos) & ((1ULL << (numofbits)) - 1))
#define gen_outq_conf(q)                                ((1ULL<<63) | (((((q)* 512) & 0x7fff) >>10) << 10) | ((16*((q)+1)-1) << 5) | (16 * (q)))

#define DTR_MSG_SEND_RETRIES            (16)
#define DTR_MSG_SEND_TIMEOUT            (10000)
#define DTR_MSG_RECV_TIMEOUT            (10000)

#define NUM_VC                          (4)
#define MIN_DTR_VC                      (264)
#define MAX_DTR_VC                      (267)

#define MSG_TYPE_TRANSFER               (0)
#define MSG_TYPE_CRC                    (1)
#define MSG_TYPE_CHECKSUM               (2)
#define MSG_TYPE_RAID_LIST              (6)
#define MSG_TYPE_REGULAR_LIST           (7)

#define DTR_OP_IN_PROGRESS              (0)
#define DTR_OP_DONE                     (2)
#define DTR_OP_ERR                      (1)
#define MSGSND_SUCCESS                  (0)

#define RECEIVE_BUCKET_ID               (processor_id () * 4 + 0)
#define P2D_MESSAGE_SIZE                (4)
#define P2P_MESSAGE_SIZE                (3)

#define RAID_MAX_ENTRIES                (384)
#define RAID_MESSAGE_TYPE               (6)
#define RAID_ENTRY_SIZE                 (8)
#define RAID_LIST_MSG_SIZE              (3)

uint32_t state;
unsigned long shared_virt_address[32];
uint64_t entries [128] __attribute__((__aligned__(64)));
struct dtr_shm_vars {
	spinlock_t init_lock;
	int init_done;
};

extern void nlm_hal_dtr_init (void);

/********** Fill control/pkt descriptors *****************/
static __inline__
uint64_t gen_dtr_msg_format_0 (const char *addr, uint32_t type, uint32_t size)
{
  return  1ULL << 63
          | shift_lower_bits (type, 60, 3)
          | shift_lower_bits (size, 40, 20)
          | shift_lower_bits (virt_to_phys ((volatile void *)addr), 0, 40);
}

static __inline__ 
uint64_t gen_dtr_msg_format_1 (const int freeback_msg_dest_id)
{
  return  0ULL << 63
          | shift_lower_bits (1, 56, 1) /* Inform Source */
          | shift_lower_bits (freeback_msg_dest_id, 44, 12);
}

static __inline__ 
uint64_t gen_dtr_msg_format_2 (const char *addr)
{
  return  0ULL << 63
          | shift_lower_bits (1, 40, 1) /* Perform Transfer */
          | shift_lower_bits (virt_to_phys ((volatile void *) addr), 0, 40);
}

static __inline__ 
uint64_t gen_dtr_msg_format_3 (const char *addr)
{
  return (uint64_t)addr;
}

static __inline__
uint64_t gen_dtr_raid_msg_format_0 (const uint32_t entries_count, 
                                    const uint64_t* entries_address)
{
  return 1ULL << 63
         | shift_lower_bits (RAID_MESSAGE_TYPE, 60, 3)
         | shift_lower_bits ((RAID_ENTRY_SIZE * entries_count), 40, 9)
         | shift_lower_bits (virt_to_phys ((volatile void *)entries_address), 0, 40);
}

static __inline__
uint64_t gen_dtr_raid_msg_format_1 (const uint32_t raidtype, 
                                    const uint32_t operation, 
                                    const uint32_t disks,
                                    const uint32_t freeback_msg_dest_id)
{
  return 0ULL << 63
         | shift_lower_bits (1, 56, 1) /* Inform Source */
         | shift_lower_bits (freeback_msg_dest_id, 44, 12)
         | shift_lower_bits (raidtype, 10, 2)
         | shift_lower_bits (operation, 9, 1)
         | shift_lower_bits (disks, 0, 4);
}

static __inline__
uint64_t gen_dtr_raid_msg_format_2 (const void *segment_address)
{
  return 0ULL << 63
         | shift_lower_bits (segment_address, 0, 62);
}

void gen_dtr_messages (uint64_t *dtr_msg, const char *in, const uint32_t size,
                       const uint32_t msg_type, const uint32_t freeback_msg_dest_id, char *out)
{
  /* generate dtr msg descriptor */
  if (msg_type == MSG_TYPE_TRANSFER)
  {
    dtr_msg[0] = gen_dtr_msg_format_0 (in, msg_type, size);
    dtr_msg[1] = gen_dtr_msg_format_1 (freeback_msg_dest_id);
    dtr_msg[2] = gen_dtr_msg_format_2 (out);
    dtr_msg[3] = gen_dtr_msg_format_3 (in);
  }
}
/**************************Print Functions ************************/
static void print_dtr_msg_type_0_desc (uint64_t* dtr_msg)
{
  uint64_t msg0 = *(dtr_msg + 0);
  uint64_t msg1 = *(dtr_msg + 1);
  uint64_t msg2 = *(dtr_msg + 2);
  uint64_t msg3 = *(dtr_msg + 3);

  Message ("-----------------------------------------------------------------------------\n");
  Message ("DTR_DRIVER: Message Descriptor 0:        0x%016llx\n", (unsigned long long)msg0);
  Message ("DTR_DRIVER: Message Descriptor 1:        0x%016llx\n", (unsigned long long)msg1);
  Message ("DTR_DRIVER: Message Descriptor 2:        0x%016llx\n", (unsigned long long)msg2);
  Message ("DTR_DRIVER: Message Descriptor 3:        0x%016llx\n", (unsigned long long)msg3);

  Message ("DTR_DRIVER: Source addr :                0x%llx  \n", xtract_bits (msg0, 0, 40));
  Message ("DTR_DRIVER: Length :                     0x%llx  \n", xtract_bits (msg0, 40, 20));
  Message ("DTR_DRIVER: Type:                        0x%llx  \n", xtract_bits (msg0, 60, 3));
  Message ("DTR_DRIVER: FWF:                         0x%llx  \n", xtract_bits (msg0, 63, 1));

  Message ("DTR_DRIVER: Message Source :             0x%llx  \n", xtract_bits (msg1, 44, 12));
  Message ("DTR_DRIVER: Inform Source :              0x%llx  \n", xtract_bits (msg1, 56, 1));
  Message ("DTR_DRIVER: FWF:                         0x%llx  \n", xtract_bits (msg1, 63, 1));
  
  Message ("DTR_DRIVER: Dest addr :                  0x%llx  \n", xtract_bits (msg2, 0, 40));
  Message ("DTR_DRIVER: Perform Xfer :               0x%llx  \n", xtract_bits (msg2, 40, 1));
  Message ("DTR_DRIVER: FWF:                         0x%llx  \n", xtract_bits (msg2, 63, 1));

  Message ("DTR_DRIVER: Software Scratch Pad :       0x%llx  \n", xtract_bits (msg3, 0, 62));
  Message ("DTR_DRIVER: FWF:                         0x%llx  \n", xtract_bits (msg3, 63, 1));
  Message ("-----------------------------------------------------------------------------\n");
}

static void print_dtr_msg_type_7_desc (uint64_t* dtr_msg)
{
  uint64_t msg0 = *(dtr_msg + 0);
  uint64_t msg1 = *(dtr_msg + 1);
  uint64_t msg2 = *(dtr_msg + 2);

  Message ("-----------------------------------------------------------------------------\n");
  Message ("DTR_DRIVER: Message Descriptor 0:        0x%016llx\n", (unsigned long long)msg0);
  Message ("DTR_DRIVER: Message Descriptor 1:        0x%016llx\n", (unsigned long long)msg1);
  Message ("DTR_DRIVER: Message Descriptor 2:        0x%016llx\n", (unsigned long long)msg2);

  Message ("DTR_DRIVER: Source addr :                0x%llx  \n", xtract_bits (msg0, 0, 40));
  Message ("DTR_DRIVER: Length :                     0x%llx  \n", xtract_bits (msg0, 40, 20));
  Message ("DTR_DRIVER: Type:                        0x%llx  \n", xtract_bits (msg0, 60, 3));
  Message ("DTR_DRIVER: FWF:                         0x%llx  \n", xtract_bits (msg0, 63, 1));

  Message ("DTR_DRIVER: Message Source :             0x%llx  \n", xtract_bits (msg1, 44, 12));
  Message ("DTR_DRIVER: Inform Source :              0x%llx  \n", xtract_bits (msg1, 56, 1));
  Message ("DTR_DRIVER: FWF:                         0x%llx  \n", xtract_bits (msg1, 63, 1));
  
  Message ("DTR_DRIVER: Software Scratch Pad :       0x%llx  \n", xtract_bits (msg2, 0, 62));
  Message ("DTR_DRIVER: FWF:                         0x%llx  \n", xtract_bits (msg2, 63, 1));
  Message ("-----------------------------------------------------------------------------\n");
}

static void print_dtr_msg_type_6_desc (uint64_t* dtr_msg)
{
  uint64_t msg0 = *(dtr_msg + 0);
  uint64_t msg1 = *(dtr_msg + 1);
  uint64_t msg2 = *(dtr_msg + 2);

  Message ("-----------------------------------------------------------------------------\n");
  Message ("DTR_DRIVER: Message Descriptor 0:        0x%016llx\n", (unsigned long long)msg0);
  Message ("DTR_DRIVER: Message Descriptor 1:        0x%016llx\n", (unsigned long long)msg1);
  Message ("DTR_DRIVER: Message Descriptor 2:        0x%016llx\n", (unsigned long long)msg2);

  Message ("DTR_DRIVER: Source addr :                0x%llx  \n", xtract_bits (msg0, 0, 40));
  Message ("DTR_DRIVER: Length :                     0x%llx  \n", xtract_bits (msg0, 40, 9));
  Message ("DTR_DRIVER: Type:                        0x%llx  \n", xtract_bits (msg0, 60, 3));
  Message ("DTR_DRIVER: FWF:                         0x%llx  \n", xtract_bits (msg0, 63, 1));

  Message ("DTR_DRIVER: Device Count :               0x%llx  \n", xtract_bits (msg1, 0, 5));
  Message ("DTR_DRIVER: Write Syndrome :             0x%llx  \n", xtract_bits (msg1, 9, 1));
  Message ("DTR_DRIVER: RAID Type :                  0x%llx  \n", xtract_bits (msg1, 10, 2));
  Message ("DTR_DRIVER: Message Source :             0x%llx  \n", xtract_bits (msg1, 44, 12));
  Message ("DTR_DRIVER: Inform Source :              0x%llx  \n", xtract_bits (msg1, 56, 1));
  Message ("DTR_DRIVER: FWF:                         0x%llx  \n", xtract_bits (msg1, 63, 1));
  
  Message ("DTR_DRIVER: Software Scratch Pad :       0x%llx  \n", xtract_bits (msg2, 0, 62));
  Message ("DTR_DRIVER: FWF:                         0x%llx  \n", xtract_bits (msg2, 63, 1));
  Message ("-----------------------------------------------------------------------------\n");
}

static void print_raid_message_entries (uint64_t *entries, uint32_t count)
{
  uint32_t i;
  uint64_t entry;
  for (i = 0; i < count ; i++)
  {
    entry = entries [i];
    Message ("DTR_DRIVER: EntryType:  0x%llx ", xtract_bits (entry, 63, 1));
    if (xtract_bits (entry, 63, 1) == 0) /* Segment Entry */
    {
      Message ("SOD:  0x%llx, EOD:  0x%llx Segment Length:  0x%llx Segment Address:  0x%llx \n", 
          xtract_bits (entry, 62, 1), 
          xtract_bits (entry, 61, 1), 
          xtract_bits (entry, 40, 20), 
          xtract_bits (entry, 0, 40));
    }
    else /* Coefficient Entry */
    {
      Message ("Device ID:  0x%llx  QCoefficient:  0x%llx PCoefficient:  0x%llx\n", xtract_bits (entry, 40, 2), xtract_bits (entry, 8, 8), xtract_bits (entry, 0, 8));
    }
  }
}

void print_data (char ***data, int num_segments_per_device, int num_devices, const uint32_t *segment_length)
{
  uint32_t i, j, k;

  for (i = 0; i < num_segments_per_device; i++)
  {
      for (k = 0; k < num_devices; k++)
      {
        Message ("%d,%d: ", k, i);
        for (j = 0; j < segment_length [i]; j++)
        {
          Message ("%02X", data [k][i][j] & 0xff);
        }
        Message ("\t");
      }
      Message ("\n");
  }
  Message ("%s finished\n", __FUNCTION__);
}
/******************** Process msgs coming back from dtr engine *********/

uint32_t dtr_print_data_err(uint64_t msg_err)
{
  uint32_t fwf, return_code, msg_format_error, bus_error, msg_source, channel_id, access_order, csum_present, crc_present;
  uint32_t partial_csum, partial_crc;
  uint32_t QSynCheck = 0, PSynCheck = 0;

  fwf = xtract_bits (msg_err, 63, 1);
  return_code = xtract_bits (msg_err, 62, 1);
  msg_format_error = xtract_bits (msg_err, 61, 1);
  bus_error = xtract_bits (msg_err, 60, 1);
  msg_source = xtract_bits (msg_err, 59, 1);
  channel_id = xtract_bits (msg_err, 56, 2);
  
  if (fwf && (msg_format_error || bus_error) && return_code == 0)
  {
    access_order = xtract_bits (msg_err, 50, 1);
    csum_present = xtract_bits (msg_err, 49, 1);
    crc_present  = xtract_bits (msg_err, 48, 1);
    partial_csum = xtract_bits (msg_err, 32, 16);
    partial_crc = xtract_bits (msg_err, 0, 32);

    Message ("DTR_APP_ERROR: fwf %d\n", fwf);
    Message ("DTR_APP_ERROR: %s\n", return_code?"P2D Msg":"P2P Msg");
    Message ("DTR_APP_ERROR: msg_format_error %d\n", msg_format_error);
    Message ("DTR_APP_ERROR: bus_error %d\n", bus_error);
    Message ("DTR_APP_ERROR: msg_source %d\n", msg_source);
    Message ("DTR_APP_ERROR: channel_id %d\n", channel_id);
    Message ("DTR_APP_ERROR: access_order %d\n", access_order);
    Message ("DTR_APP_ERROR: csum_present %d\n", csum_present );
    Message ("DTR_APP_ERROR: crc_present %d\n", crc_present);
    return 1;
  }

  if (return_code == 1)
  {
    QSynCheck = xtract_bits (msg_err, 44, 2);
    PSynCheck = xtract_bits (msg_err, 42, 2);

    if (fwf && (msg_format_error || bus_error || QSynCheck == 1|| PSynCheck == 1)) 
    {
      Message ("DTR_APP_ERROR: fwf %d\n", fwf);
      Message ("DTR_APP_ERROR: %s\n", return_code?"P2D Msg":"P2P Msg");
      Message ("DTR_APP_ERROR: msg_format_error %d\n", msg_format_error);
      Message ("DTR_APP_ERROR: bus_error  %d\n", bus_error);
      Message ("DTR_APP_ERROR: msg_source %d\n", msg_source);
      Message ("DTR_APP_ERROR: channel_id %d\n", channel_id);
      Message ("DTR_APP_ERROR: ListCode   %d\n", (int) xtract_bits (msg_err, 54, 2));
      Message ("DTR_APP_ERROR: PacketDest %d\n", (int) xtract_bits (msg_err, 51, 1));
      Message ("DTR_APP_ERROR: QSynCheck  %d\n", QSynCheck);
      Message ("DTR_APP_ERROR: PSynCheck  %d\n", PSynCheck);
      Message ("DTR_APP_ERROR: QWriteZero %d\n", (int) xtract_bits (msg_err, 41, 1));
      Message ("DTR_APP_ERROR: PWriteZero %d\n", (int) xtract_bits (msg_err, 40, 1));
      Message ("DTR_APP_ERROR: QErrorPosition %d\n", (int) xtract_bits (msg_err, 20, 20));
      Message ("DTR_APP_ERROR: PErrorPosition %d\n", (int) xtract_bits (msg_err, 0, 20));
      return 1;
    }
  }
  Message ("DTR_DRIVER: fwf %d\n", fwf);
  Message ("DTR_DRIVER: %s\n", return_code?"P2D Msg":"P2P Msg");
  Message ("DTR_DRIVER: msg_format_error %d\n", msg_format_error);
  Message ("DTR_DRIVER: bus_error %d\n", bus_error);
  Message ("DTR_DRIVER: msg_source %d\n", msg_source);
  Message ("DTR_DRIVER: channel_id %d\n", channel_id);

  return 0;
}

void dtr_receive_msg (void)
{
  uint32_t code = 0, size = 0, sender = 0, my_vc;
  int status = 0;
  uint64_t msg[2] = {0};

  my_vc = RECEIVE_BUCKET_ID;

  status = nlh_recv_msg2 (my_vc, &sender, &size, &code, &msg[0], &msg[1]);

  if (!status)
  {
    Message ("DTR_DRIVER: Received Proper Msg status 0x%x %d %d \n", status, sender, size);
    Message ("DTR_DRIVER: Msg0 = %llx, Msg1 = %llx\n", (unsigned long long)msg[0], (unsigned long long)msg[1]);
    if (dtr_print_data_err (msg[0]))
      state = DTR_OP_ERR;
    else
      state = DTR_OP_DONE;
  }

  return;
}

int dtr_receive (void)
{
  while (1) 
  {
    dtr_receive_msg();
    if (state != DTR_OP_IN_PROGRESS)
      break;
  }

  return 0;
}

int dtr_send_msg (uint64_t *dtr_msg, uint32_t vc_id, uint32_t num_entries)
{
  if (num_entries == P2D_MESSAGE_SIZE)
    return xlp_message_send (vc_id, 4, 0 /* msg code */ , dtr_msg);
  else
    return xlp_message_send (vc_id, 3, 0 /* msg code */ , dtr_msg);
}

int dtr_send (uint64_t *dtr_msg, uint32_t vc_id, uint32_t num_entries)
{
  int ret, try;
  int timeout=0;
  while (1) 
  {
    for (try =0; try < DTR_MSG_SEND_RETRIES; try++) 
    {
      ret = dtr_send_msg (dtr_msg, vc_id, num_entries);
      if (ret == MSGSND_SUCCESS)
        return ret;
    }

    /* flag error in case of timeout and return and cleanup the state */
    timeout++;
    if (timeout > DTR_MSG_SEND_TIMEOUT) 
    {
      /* flag error and do the cleanup */
      return -1;
    }
  }
}

int send_n_recv_dtr_messages (uint64_t *dtr_msg, uint32_t vc_id, uint32_t  num_entries)
{
  int ret;

  ret = dtr_send (dtr_msg, vc_id, num_entries);
  Message ("DTR_DRIVER: sync msg sent to VC [%d], ret val = %d\n", vc_id, ret);

  if (ret != MSGSND_SUCCESS || ret == -EINPROGRESS)
    return ret;

  Message ("DTR_DRIVER: msg sent done\n");
  Message ("DTR_DRIVER: Waiting for message to receive\n");
  ret = dtr_receive ();
  Message ("DTR_DRIVER: Message Received\n");
  return ret;
}

static inline void *nlm_dtr_shm_init(void)
{
	int fd, rv;
	const char *name = "dtr-init-shm";
	struct dtr_shm_vars *m;

	fd = shm_open(name, O_CREAT | O_RDWR, 0600);
	if(fd < 0) {
		printf("shm_open failed for %s\n", name);
		return NULL;
	}

	rv = ftruncate(fd, sizeof(struct dtr_shm_vars));
	if(rv < 0) {
		printf("ftruncate failed on fd %d for %s\n", fd, name);
		goto err_exit;
	}

	m = mmap(NULL, sizeof(struct dtr_shm_vars), PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
	if(m == MAP_FAILED) {
		printf("mmap failed on fd %d for %s\n", fd, name);
		goto err_exit;
	}
	return m;

err_exit:
	shm_unlink(name);
	return NULL;
}

#ifdef XLP_SIM
void init_dtr()
{
	struct dtr_shm_vars *m;

	if ((m = nlm_dtr_shm_init()) == NULL) {
		printf("Failed to initialze nae shared memory");
	}

	spin_lock(&m->init_lock);
	if (m->init_done) {
		spin_unlock(&m->init_lock);
		return;
	} else {
		m->init_done = 1;
	}

	nlm_hal_dtr_init();

	spin_unlock(&m->init_lock);
    return;
}
#endif

/******************** DTR API ****************************/
/**
* @file_name dtr.c
*/

/**
* @defgroup dtr Data Transfer and RAID API
* @brief Description about the dtr apis supported in NETOS
*/

/**
* @brief: nlm_dtr_transfer function is used for performing 
*         data transfer of len bytes from input address to output address. 
*
* @param[in]  in    :input address
* @param[in]  len   :length of data to be transferred.
* @param[out] out   :output address
*
* @return
*  - On sucess,  returns 0
*  - On failure, returns non-zero value.
*
* @sa nlm_dtr_multi_transfer
* @ingroup dtr
*
*/
int nlm_dtr_transfer (const char *in, 
                      uint32_t size,
                      char *out)
{
  uint64_t dtr_msg[P2D_MESSAGE_SIZE] = {0};
  /* Application is running on CPU 0. Threads/Buckets for CPU0 are 0,1,2,3 */
  uint32_t freeback_msg_dest_id = RECEIVE_BUCKET_ID, dtr_vc;

  gen_dtr_messages (dtr_msg, in, size, MSG_TYPE_TRANSFER, freeback_msg_dest_id, out);
  print_dtr_msg_type_0_desc (dtr_msg);
  
  Message ("DTR_DRIVER: Messages Created \n");

  dtr_vc = MIN_DTR_VC;
  state = DTR_OP_IN_PROGRESS;

  send_n_recv_dtr_messages (dtr_msg, dtr_vc, P2D_MESSAGE_SIZE);
  if (state == DTR_OP_ERR)
    return DTR_OP_ERR;
  return 0;
}

/**
* @brief: nlm_data_multi_transfer function is used for processing 
*         a list of data transfers from input to output addresses for len bytes. 
*
* @param[in]  in        :Array of input address of size arr_size
* @param[in]  len       :Length of data to be transferred for each index of input array addresses
* @param[out] out       :Array of output address of size arr_size
* @param[in]  arr_size  :Size of all three arrays (in, out, len)
*
* @return
*  - On sucess,  returns 0
*  - On failure, returns non-zero value.
*
* @sa nlm_dtr_transfer
* @ingroup dtr
*
*/
int nlm_dtr_multi_transfer (const char **in,
                            const uint32_t *len,
                            char **out,
                            uint32_t arr_size)
{
  uint32_t i;
  uint64_t *p2d_dtr_msg, p2p_dtr_msg[P2P_MESSAGE_SIZE] = {0};
  uint32_t freeback_msg_dest_id = RECEIVE_BUCKET_ID, dtr_vc;

  p2d_dtr_msg = malloc (arr_size * P2D_MESSAGE_SIZE * sizeof (uint64_t)); 
  memset (p2d_dtr_msg, 0, arr_size * P2D_MESSAGE_SIZE * sizeof (uint64_t));

  for (i = 0; i < arr_size; i++)
  {
    gen_dtr_messages (p2d_dtr_msg + i * P2D_MESSAGE_SIZE, in[i], len[i], MSG_TYPE_TRANSFER, freeback_msg_dest_id, out[i]);
  }

  p2p_dtr_msg[0] = gen_dtr_msg_format_0 ((char *) p2d_dtr_msg, MSG_TYPE_REGULAR_LIST, arr_size * P2D_MESSAGE_SIZE * sizeof (uint64_t));
  p2p_dtr_msg[1] = gen_dtr_msg_format_1 (freeback_msg_dest_id);
  p2p_dtr_msg[2] = gen_dtr_msg_format_2 ((char *)p2d_dtr_msg);
  
  Message ("DTR_DRIVER: Messages Created \n");
  print_dtr_msg_type_7_desc (p2p_dtr_msg);

  dtr_vc = MIN_DTR_VC;
  state = DTR_OP_IN_PROGRESS;

  send_n_recv_dtr_messages (p2p_dtr_msg, dtr_vc, P2P_MESSAGE_SIZE);
  free (p2d_dtr_msg);
  if (state == DTR_OP_ERR)
    return DTR_OP_ERR;
  return 0;
}

/**
* @brief: nlm_dtr_raid is used to perform raid 5 and raid 6 operation.
*
* @param[in] disks              : Total Number of disks for raid (including disks for P and Q).
* @param[in] segment_per_disk   : Number of segments per disk.
* @param[in] data               : Array of disks having equal number of segments where each segment is pointing to equal length data.
* @param[in] segment_length     : Array of lengths for segments.
* @param[in] operation          : "Read and Check" or "Write" operation to be performed. For "Read and Check", pass 0, for "Write", pass 1.
* @param[in] raidtype           : For Raid5, pass 1 and for Raid6, pass 2.
* @param[in] p_device_id        : Disk id to which calculated p-syndrome will be written.
* @param[in] q_device_id        : Disk id to which calculated q-syndrome will be written.
*
* @return
*  - On sucess,  returns 0
*  - On failure, returns non-zero value.
*
* @ingroup dtr
*
*/

int nlm_dtr_raid (uint32_t disks,
                  uint32_t segments_per_disk,
                  char ***data,
                  const uint32_t *segment_length,
                  uint32_t operation,
                  uint32_t raidtype,
                  uint32_t p_device_id,
                  uint32_t q_device_id)
{
  uint32_t i, j, dtr_vc, freeback_msg_dest_id = RECEIVE_BUCKET_ID, entries_count;
  uint32_t master_control_value;
  uint64_t base, segment_entry;
  uint64_t raid_list_msg [RAID_LIST_MSG_SIZE] = {0};

  Message ("DTR_DRIVER: Entered %s \n", __FUNCTION__);

  base = nlm_hal_get_dev_base (0 /* node */, 0 /* bus */, 5 /* dev */, 0 /* func */);
  master_control_value = nlm_hal_read_32bit_reg (base, 0x40);
  /* Enable master control alongwith raid */
  nlm_hal_write_32bit_reg (base, 0x40, master_control_value | (0x10));

  if (0) print_data (data, segments_per_disk, disks, segment_length);

  entries_count = disks + disks * segments_per_disk;
  if ((entries_count*RAID_ENTRY_SIZE) > RAID_MAX_ENTRIES)
  {
    Message ("DTR_DRIVER: Entries %d are more than max entries %d \n", entries_count, RAID_MAX_ENTRIES);
    return 1;
  }
#if 0  
  entries = (uint64_t *) malloc (sizeof (uint64_t) * entries_count);
  memset (entries, 0, sizeof (uint64_t) * entries_count);
#endif

  raid_list_msg [0] = gen_dtr_raid_msg_format_0 (entries_count, entries);
  raid_list_msg [1] = gen_dtr_raid_msg_format_1 (raidtype, operation, disks, freeback_msg_dest_id);
  raid_list_msg [2] = gen_dtr_raid_msg_format_2 (entries);

  print_dtr_msg_type_6_desc (raid_list_msg);

  /* Loop to fill the coefficients entry */
  for (i = 0; i < disks; i++)
  {
    entries[i] = 1ULL << 63;/* Coefficient Entry */
    /* Device ID */
    if ((i + 1) == p_device_id)
      entries [i] |= shift_lower_bits (1, 40, 2);
    if ((i + 1) == q_device_id)
      entries [i] |= shift_lower_bits (2, 40, 2);

    /* QCoefficient */
    entries [i] |= shift_lower_bits ((1 << i), 8, 8);
    /* PCoefficient */
    entries[i] |= shift_lower_bits (1, 0, 8);

    if ((i + 1) == q_device_id)
    {
      entries[i] |= shift_lower_bits (0, 0, 8);
      entries [i] |= shift_lower_bits (1, 8, 8);
    }
    if ((i + 1) == p_device_id)
    {
      entries[i] |= shift_lower_bits (1, 0, 8);
      entries [i] |= shift_lower_bits (0, 8, 8);
    }
  }
  Message ("DTR_DRIVER: %s : Coefficients Entry created\n", __FUNCTION__);

  /* Loop to fill the segments entry for each disk */
  for (i = 0; i < disks; i++)
  {
    for (j = 0; j < segments_per_disk; j++)
    {
      segment_entry = entries [disks + i * segments_per_disk + j];
      segment_entry = 0;
      segment_entry = 0ULL << 63; /* Segment Entry */
      /* SOD */
      if (j == 0)
        segment_entry |= shift_lower_bits (1, 62, 1);
      /* EOD */
      if (j+1 == segments_per_disk)
        segment_entry |= shift_lower_bits (1, 61, 1);
      /* Segment Length */
      segment_entry |= shift_lower_bits (segment_length [j], 40, 20);
      /* Segment Address */
      segment_entry |= shift_lower_bits (virt_to_phys ((volatile void *) data[i][j]), 0, 40);
      entries [disks + i * segments_per_disk + j] = segment_entry;
    }
  }

  if (0) print_raid_message_entries (entries, entries_count);
  
  dtr_vc = MIN_DTR_VC;
  state = DTR_OP_IN_PROGRESS;

  send_n_recv_dtr_messages (raid_list_msg, dtr_vc, RAID_LIST_MSG_SIZE);
  /* Disable raid in master control */
  nlm_hal_write_32bit_reg (base, 0x40, master_control_value & 0xffffffef);

  if (state == DTR_OP_ERR)
    return DTR_OP_ERR;
  return 0;
}
