/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

//#include "pci.h"
#include <stdio.h>
#include <stdlib.h>

#include "mac_defines.h"
#include "gmac.h"
#include "types.h"
#include "mips-exts.h"
#include "mipsregs.h"
#include "msgring.h"
#include "iomap.h"
#include "netdevice.h"
#include "spinlock.h"
#include "packet.h"
#include "pod.h"
#include "mdio.h"
#include "asmmacros.h"
#include "spill.h"
#include "classifier.h"
#include "iomap.h"
#include "mm.h"
#include "system.h"
#include "atomic.h"
#include "shmvars.h"

//#define DEBUG 1


static int sgmii_dc_present(int block);
static void nlm_xaui_init(device_conf * current_dev, int block );
void gmac_inject_free_desc(int num_desc, int desc_bucket);
int is_valid_gmac(int gmac);
static int nlm_set_gmac_speed( device_conf *current_dev);
static void advertise_phy_speed( unsigned long mmio,int gmac , int speed , int duplex );
static void nlm_gmac_inject_free_desc( device_conf *current_dev);
static void get_phy_info(int instance, port_info * current_phy ) __attribute__((__noinline__));


extern struct block_config  blk_conf[4];

classifier_config classifier_conf_gmac = {
    .classify_gmac.enable = 1,
    .classify_gmac.rx_enable = 1,
    .classify_gmac.ttable_mode = USE_HASH,
    .classify_gmac.prepad_enable = 0,
    .classify_gmac.byte_offset = 0,
    .classify_gmac.num_classes = 4,
    .classify_gmac.ethertype = 0xffff, 
    .classify_gmac.offset_l4_prot = 0,
    .classify_gmac.offset_0 = 0,
    .classify_gmac.len_0 = 0,
    .classify_gmac.offset_1 = 0,
    .classify_gmac.len_1 = 0,
    .classify_gmac.offset_2 = 0,
    .classify_gmac.len_2 = 0,
    .classify_gmac.offset_3 = 0,
    .classify_gmac.len_3 = 0,
    .classify_gmac.offset_4 = 0,
    .classify_gmac.len_4 = 0,
    .classify_gmac.offset_5 = 0,
    .classify_gmac.len_5 = 0,

    .classify_xgs0.enable = 0,    /* XGS0 INIT */ 
    .classify_xgs1.enable = 0,    /* XGS1 INIT */ 
};

static uint32_t gmac_offsets[] = { PHOENIX_IO_GMAC_0_OFFSET, PHOENIX_IO_GMAC_1_OFFSET,
                        PHOENIX_IO_GMAC_2_OFFSET, PHOENIX_IO_GMAC_3_OFFSET,
                        PHOENIX_IO_GMAC_4_OFFSET, PHOENIX_IO_GMAC_5_OFFSET,
                        PHOENIX_IO_GMAC_6_OFFSET, PHOENIX_IO_GMAC_7_OFFSET };


// function to disable filtering of mac addresses
// void turn_off_filtering (unsigned int *_t_mmio){
void turn_off_filtering (phoenix_reg_t *_t_mmio){
  phoenix_write_reg( _t_mmio, G_MAC_FILTER_CONFIG,   FIELD_VALUE(MAC_ADDR0_VALID,1) |
             FIELD_VALUE(MAC_ADDR1_VALID,1) |
             FIELD_VALUE(MAC_ADDR2_VALID,1) |
             FIELD_VALUE(MAC_ADDR3_VALID,1)) ;
  phoenix_write_reg( _t_mmio, G_MAC_ADDR_MASK2_0, 0x0) ;
  phoenix_write_reg( _t_mmio, G_MAC_ADDR_MASK2_1, 0x0) ;
}


static void get_phy_info(int instance, port_info * current_dev )
{
	if ( is_xls() && ( instance > 3 )){
		current_dev->pcs_addr = PHOENIX_IO_GMAC_4_OFFSET;
		current_dev->serdes_addr = PHOENIX_IO_GMAC_4_OFFSET;
	}
	else {
		current_dev->pcs_addr = PHOENIX_IO_GMAC_0_OFFSET;
		current_dev->serdes_addr = PHOENIX_IO_GMAC_0_OFFSET;
	}

	current_dev->phy_addr =  PHOENIX_IO_GMAC_0_OFFSET;
	current_dev->phy_id = instance ;
	current_dev->pcs_id = 27 + instance%4;
	current_dev->serdes_id = 26;


	if ( is_xls()) {
		if ( current_dev->rgmii == 1 )
			current_dev->phy_id = 0;
		else    /* sgmii */
			current_dev->phy_id = ( 0x10 | instance );

		if ( current_dev->rgmii == 1) {
			if ( xls_board_atx_vi()) /* rgmii -0 is linked with sgmii-4 */
				current_dev->phy_addr = PHOENIX_IO_GMAC_4_OFFSET;
		}
		if ( is_xls_board_xi_xii () ) {
			if ( sgmii_dc_present(0) && (instance < 4)){
				current_dev->phy_addr = PHOENIX_IO_GMAC_0_OFFSET;
				current_dev->phy_id = ( 0x10 | instance );
			}
			else if ( sgmii_dc_present(1) && (instance > 3))	{
				current_dev->phy_addr = PHOENIX_IO_GMAC_4_OFFSET;
				current_dev->phy_id -= 4;
			}
			else {
				current_dev->phy_addr = PHOENIX_IO_GMAC_0_OFFSET;
				current_dev->phy_id = 0;
			}
		}

	} 
}
static int  xls_serdes_autoconfig( port_info *phy)
{
	volatile unsigned int i;

	if( !is_xls())
		return ERR_NT_XLS;

	arizona_gmac_phy_write( phy->pcs_addr, phy->pcs_id,PCS_CTRL ,0x1000);
	for(i =0 ; i < 0xfffff; i++ );
	arizona_gmac_phy_write( phy->pcs_addr, phy->pcs_id,PCS_CTRL,0x0200);
	for(i =0 ; i < 0xfffff; i++ );
	for(i =0 ; i < 0xfffff; i++ );

	return 0;
	
}
void nlm_sgmii_serdes_reset(void) 
{
	int i;
	unsigned int *gpio_mmio;
	gpio_mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_GPIO_OFFSET);

	for (i=0;i<1000000;i++);

	/* use 125 Mhz ref clock instead of 156.25Mhz ref clk */
	phoenix_write_reg(gpio_mmio, 0x10, 0x7103);
	phoenix_write_reg(gpio_mmio, 0x21, 0x7103);

	for (i=0;i<1000000;i++);
}


static void nlm_serdes_regs_init( port_info * phy)
{
	volatile unsigned int *mmio_gpio;
	mmio_gpio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_GPIO_OFFSET);

//  P Reg   Val
// -------------
// 26 0     6DB0
// 26 1     0FFF
// 26 2     B6D0
// 26 3     00FF
// 26 4     0000
// 26 5     0000
// 26 6     0005
// 26 7     0001
// 26 8     0000
// 26 9     0000
// 26 10    0000
	arizona_gmac_phy_write (phy->serdes_addr, phy->serdes_id, 0, 0x6DB0);
	arizona_gmac_phy_write (phy->serdes_addr, phy->serdes_id, 1, 0xFFFF); // 0x0fff
	arizona_gmac_phy_write (phy->serdes_addr, phy->serdes_id, 2, 0xB6D0);
	arizona_gmac_phy_write (phy->serdes_addr, phy->serdes_id, 3, 0x00FF);
       
	arizona_gmac_phy_write (phy->serdes_addr, phy->serdes_id, 4, 0x0000);
	arizona_gmac_phy_write (phy->serdes_addr, phy->serdes_id, 5, 0x0000);
	arizona_gmac_phy_write (phy->serdes_addr, phy->serdes_id, 6, 0x0005);
	arizona_gmac_phy_write (phy->serdes_addr, phy->serdes_id, 7, 0x0001);
	arizona_gmac_phy_write (phy->serdes_addr, phy->serdes_id, 8, 0x0000);
	arizona_gmac_phy_write (phy->serdes_addr, phy->serdes_id, 9, 0x0000);

	// program  GPIO values for serdes init parameters
	phoenix_write_reg(mmio_gpio,0x20,0x7e6802);
	phoenix_write_reg(mmio_gpio,0x10, 0x7104);

	if (is_xls_board_xaui_rework())
		nlm_sgmii_serdes_reset();


}


int nlm_check_mac_speed( device_conf * current_dev) 
{
	int speed;
	int duplex;	
	uint32_t data;
	volatile int delay;
	port_info * phy = (port_info *)current_dev->intrl;
	int try = 0;
	struct gmac_shm_vars *m = current_dev->shm;

	if(m == NULL)
		return -1;

	arizona_gmac_phy_read(phy->phy_addr, phy->phy_id, DEV_AUX_CTL_STAT, &data);

	speed = ( data >> 3 ) & 0x3;
	duplex = (data >> 5) & 0x01;
	if ( (speed == m->gmac_speed[current_dev->id]) && ( duplex == m->gmac_duplex[current_dev->id] ))
        	return -1;

	if ( m->gmac_speed[current_dev->id] != -1 ) {
		do {

			for(delay =0 ; delay < 0xffff; delay++); 
			arizona_gmac_phy_read(phy->phy_addr, phy->phy_id, PHY_STATUS, &data);
			try++;
			if ( try == 300)
				break;
		
		} while( (data & (1 << 5)) == 0);
		arizona_gmac_phy_read(phy->phy_addr, phy->phy_id, DEV_AUX_CTL_STAT, &data);

		speed = ( data >> 3 ) & 0x3;
		duplex = (data >> 5) & 0x01;
	}
	
	m->gmac_speed[current_dev->id] = speed ;
	m->gmac_duplex[current_dev->id] = duplex;

	if ( speed == 0x0 )
		speed = 10;
	else if ( speed == 0x1)
		speed = 100;
	else if ( speed == 0x2)
		speed = 1000;

	current_dev->phy_conf[0].speed = speed ;
	current_dev->phy_conf[0].duplex = duplex;

	xls_serdes_autoconfig(phy);

	nlm_set_gmac_speed(current_dev);

	return 0;
	

}

static int nlm_set_gmac_speed( device_conf *current_dev ) {

	uint32_t config = 0;
	uint32_t core_ctl =0;

	if(!is_valid_gmac(current_dev->id))
		return ERR_INVALID_GMAC;
	config = phoenix_read_reg(current_dev->mmio, G_CONFIG_1);

	phoenix_write_reg (current_dev->mmio, G_CONFIG_1, 
			FIELD_VALUE(G_SOFT_RESET,0)                   |
			FIELD_VALUE(LOOP_BACK,0)                      |
			FIELD_VALUE(RECEIVE_FLOW_CONTROL_ENABLE,0)    |
			FIELD_VALUE(TRANSMIT_FLOW_CONTROL_ENABLE,0)   |
			FIELD_VALUE(RECEIVE_ENABLE,0)                 |
			FIELD_VALUE(TRANSMIT_ENABLE,0));
   
	while (GET_FIELD(phoenix_read_reg(current_dev->mmio, G_CONFIG_1), SYNC_RECEIVE_ENABLE)) {}

	if(current_dev->phy_conf[0].speed  == 10){
		core_ctl = phoenix_read_reg(current_dev->mmio, CORE_CONTROL);
		core_ctl = core_ctl & ~(0x3);
		core_ctl = core_ctl | FREQ_2p5MHz;
		if (is_xls())
			phoenix_write_reg(current_dev->mmio, G_INTERFACE_CONTROL, SGMII_SPEED_10);
		phoenix_write_reg(current_dev->mmio, G_CONFIG_2, 0x7117);
		phoenix_write_reg(current_dev->mmio, CORE_CONTROL, core_ctl );
		printf("configuring gmac_%d in 10Mbps mode\n", current_dev->id);
	}else if (current_dev->phy_conf[0].speed  == 100) {
		core_ctl = phoenix_read_reg(current_dev->mmio, CORE_CONTROL);
		core_ctl = core_ctl & ~(0x3);
		core_ctl = core_ctl | FREQ_25MHz;
		if (is_xls())
			phoenix_write_reg( current_dev->mmio, G_INTERFACE_CONTROL, SGMII_SPEED_100);
		phoenix_write_reg(current_dev->mmio, CORE_CONTROL, core_ctl );
		phoenix_write_reg(current_dev->mmio, G_CONFIG_2, 0x7117);
		printf("configuring gmac_%d in 100Mbps mode\n", current_dev->id);
	}else if (current_dev->phy_conf[0].speed == 1000) {
		core_ctl = phoenix_read_reg(current_dev->mmio, CORE_CONTROL);
		core_ctl = core_ctl & ~(0x3);
		core_ctl = core_ctl | FREQ_125MHz;
		if (is_xls())
			phoenix_write_reg(current_dev->mmio, G_INTERFACE_CONTROL, SGMII_SPEED_1000);
		phoenix_write_reg(current_dev->mmio, G_CONFIG_2, 0x7217);
		phoenix_write_reg(current_dev->mmio, CORE_CONTROL, core_ctl ); 
		printf("configuring gmac_%d in 1000Mbps mode\n", current_dev->id);
	}else{
		printf("\nUnknown speed/duplex mode detected.");
		
	}

	phoenix_write_reg(current_dev->mmio, G_CONFIG_1, config);
	return 0;

}

static int nlm_disable_gmac_autonegotiation_and_set(device_conf * current_dev)
{
	port_info * phy = ( port_info *)current_dev->intrl;
	struct gmac_shm_vars *m;

	if((m = current_dev->shm) == NULL)
		return ERR_NO_MEM;

        if(!is_valid_gmac(current_dev->id))
                return ERR_INVALID_GMAC;

	if(m->gmac_xaui_mode[current_dev->id])
		return ERR_XAUI_MODE;

	advertise_phy_speed( phy->phy_addr, phy->phy_id, current_dev->phy_conf[0].speed, current_dev->phy_conf[0].duplex);
	nlm_set_gmac_speed( current_dev);


	return 0;
}


static int nlm_enable_gmac_autonegotiation(device_conf * current_dev )
{

	port_info * phy = ( port_info *)current_dev->intrl;

	uint32_t data = 0;
        volatile int delay =0;
	int auto_complete = 0;
	struct gmac_shm_vars *m;

	if((m = current_dev->shm) == NULL)
		return ERR_NO_MEM;


	if(!is_valid_gmac(current_dev->id))
		return ERR_INVALID_GMAC;

	if(m->gmac_xaui_mode[current_dev->id])
		return ERR_XAUI_MODE;

	/* advertise 100 and 10 full as well as half duplexity and
	1000 full duplexity */

	arizona_gmac_phy_read(phy->phy_addr, phy->phy_id, PHY_STATUS, &data);

	if ( (data & (1 << 5)) != 0 ) {
		auto_complete = 1;
	}

	arizona_gmac_phy_read(phy->phy_addr, phy->phy_id ,AUTO_NEG_ADVER,&data);
	data = data | 0x3E0;
	arizona_gmac_phy_write ( phy->phy_addr, phy->phy_id  ,AUTO_NEG_ADVER ,data);

	arizona_gmac_phy_read( phy->phy_addr, phy->phy_id ,BASE1000_T_CTL1 ,&data);
	data = data | 0x300;
	arizona_gmac_phy_write ( phy->phy_addr, phy->phy_id ,BASE1000_T_CTL1 ,data);

	/* restart auto neg */
	arizona_gmac_phy_read( phy->phy_addr, phy->phy_id ,PHY_CONTROL,&data);
	data = data & ~(0x200);
	arizona_gmac_phy_write ( phy->phy_addr, phy->phy_id ,PHY_CONTROL ,data);
	for (delay = 0; delay < 0xff; delay++);

	data = data | 0x200;
	arizona_gmac_phy_write ( phy->phy_addr, phy->phy_id ,PHY_CONTROL ,data);
	for (delay = 0; delay < 0xff; delay++);

	if ( auto_complete == 1 ) {
		int try =0;
		do 
		{

			for(delay =0 ; delay < 0xfffff; delay++); 
			arizona_gmac_phy_read(phy->phy_addr, phy->phy_id, PHY_STATUS, &data);
			try++;

			if ( try == 300)
				break;
		} while( (data & (1 << 5)) == 0);
	}
	nlm_check_mac_speed(current_dev);

	return 0;
 
}
void nlm_gmac_soft_reset(int id)
{
	phoenix_reg_t * mmio = (phoenix_reg_t *) (phoenix_io_base + gmac_offsets[id]);
	phoenix_write_reg( mmio, G_CONFIG_1, FIELD_VALUE(RECEIVE_ENABLE,0)   |
			FIELD_VALUE(TRANSMIT_ENABLE,0) ) ;
	while ((GET_FIELD(phoenix_read_reg( mmio, RX_CONTROL), RX_HALT)) == 0);
	phoenix_write_reg (mmio, RX_CONTROL, FIELD_VALUE(SOFT_RESET,1));
	while ((GET_FIELD(phoenix_read_reg( mmio , RX_CONTROL),SOFT_RESET_DONE)) == 0) {}
	phoenix_write_reg (mmio, RX_CONTROL, FIELD_VALUE(SOFT_RESET,0));
	
}

static int gmac_block_commmon_init(device_conf * current_dev, int block)
{
	int i =0;
	int gmac_id  =0;
	port_info * phy = NULL;
	struct gmac_shm_vars *m = current_dev->shm;
	if(m == NULL)
		return -1;

	if ( !atomic_cmpset_int(&m->gmac_common_init[block], 0, 1))
		return INIT_NOT_NEEDED;


	phy = ( port_info *)current_dev->intrl;

	for(i =0; i < 4; i++) {
		gmac_id =  (block * 4 ) +  i;
		if ( is_valid_gmac(gmac_id))
			nlm_gmac_soft_reset( gmac_id);
	}

	nlm_config_spill(current_dev);
	
	if( is_xls()) {
		nlm_serdes_regs_init( phy);
	}

  // setup DMA<-> memory subsystem credit registers
	phoenix_write_reg (current_dev->mmio, DMA_CR_0, 
		FIELD_VALUE(WR0_CREDITS,7) |
		FIELD_VALUE(RD0_CREDITS,7) |
		FIELD_VALUE(WR1_CREDITS,7) |
		FIELD_VALUE(RD1_CREDITS,7) |
		FIELD_VALUE(WR2_CREDITS,7) |
		FIELD_VALUE(RD2_CREDITS,7) |
		FIELD_VALUE(WR3_CREDITS,7) |
		FIELD_VALUE(RD3_CREDITS,7)
		);
  
 	// DM credit register
	phoenix_write_reg (current_dev->mmio, DMA_CR_3, 
		FIELD_VALUE(SP_CLASS_WR_CREDITS,1)  |
		FIELD_VALUE(SP_CLASS_RD_CREDITS,1)  |
		FIELD_VALUE(REG_FR_IN_WR_CREDITS,1) |
		FIELD_VALUE(REG_FR_IN_RD_CREDITS,1) |
		FIELD_VALUE(FR_OUT_WR_CREDITS,1)    |
		FIELD_VALUE(P2P_CHANNEL_RD_CREDITS,1) |
		FIELD_VALUE(FR_OUT_RD_CREDITS,1)    
		);
	
  	phoenix_write_reg ( current_dev->mmio, PARSER_CONFIG, 0);

	/* free back bucket size configuration */
	phoenix_write_reg ( current_dev->mmio, G_RFR0_BUCKET_SIZE, blk_conf[block].bucket_size[1]);
	phoenix_write_reg ( current_dev->mmio, G_TX0_BUCKET_SIZE, blk_conf[block].bucket_size[2]);
	phoenix_write_reg ( current_dev->mmio, G_TX1_BUCKET_SIZE, blk_conf[block].bucket_size[3]);
	phoenix_write_reg ( current_dev->mmio, G_TX2_BUCKET_SIZE, blk_conf[block].bucket_size[4]);
	phoenix_write_reg ( current_dev->mmio, G_TX3_BUCKET_SIZE, blk_conf[block].bucket_size[5]);

	for(i =0; i < NLM_MAX_CREDIT_COUNTER ; i++ ) 
		phoenix_write_reg ( current_dev->mmio, CC_CPU0_0 + i , blk_conf[block].credit_counter[i]);

	nlm_setup_pde(current_dev);

	return 0;

	


}
int  nlm_gmac_init( device_conf * current_dev, int rgmii)
{
	port_info * internal_info = NULL;
	int block ;
	struct gmac_shm_vars *m;


	if ( current_dev == NULL )
		return ERR_INVALID_DEVICE_CONFIG;

	if((current_dev->shm = nlm_gmac_shm_init()) == NULL)
		return ERR_NO_MEM;
	m = current_dev->shm;

	if ( !is_valid_gmac(current_dev->id)) { 
		printf("gmac_%d interface is not supported on this board\n",current_dev->id);
		return ERR_DEVICE_NOT_SUPORTTED;
	}


	spin_lock(&m->gmac_lock[current_dev->id]) ;

	if ( check_device_config( current_dev ) ==  ERR_INVALID_DEVICE_CONFIG ) {
		spin_unlock(&m->gmac_lock[current_dev->id]) ;
		return ERR_INVALID_DEVICE_CONFIG;
	}


	if ( m->gmac_dev_init[current_dev->id] == 1 ) {
		spin_unlock(&m->gmac_lock[current_dev->id]) ;
		return INIT_NOT_NEEDED;
	}

	internal_info = ( port_info *)current_dev->intrl;
	block = internal_info->block_id; 

	m->gmac_free_desc[current_dev->id] = 0;
		
	if ( current_dev->mmio == NULL )
		current_dev->mmio = ( phoenix_reg_t *) (phoenix_io_base + gmac_offsets[current_dev->id]);

	/* LITE Board does not have rgmii */
	if ( xls_board_atx_viii() || ( is_xls() && ( current_dev->id > 0 )) )
		rgmii = 0;

	internal_info->rgmii = rgmii;

	get_phy_info(current_dev->id, internal_info);


	#ifdef DEBUG
	printf("gmac[%d] mmio : %p\n",current_dev->id,current_dev->mmio);
	printf("gmac[%d] phy_addr : %lx\n",current_dev->id,internal_info->phy_addr);
	printf("gmac[%d] phy_id : %x\n",current_dev->id,internal_info->phy_id);
	printf("gmac[%d] pcs_addr : %lx\n",current_dev->id,internal_info->pcs_addr);
	printf("gmac[%d] pcs_id : %x\n",current_dev->id,internal_info->pcs_id);
	#endif
	

	if ( is_xls_board_xi_xii() ){
		if ( xlsb0_in_xaui(block)){
			if ((current_dev->id == 0) || (current_dev->id == 4 )){
				m->gmac_xaui_mode[current_dev->id] = 1;
				nlm_xaui_init(current_dev, block);
				m->gmac_dev_init[current_dev->id] = 1;
			}
			spin_unlock(&m->gmac_lock[current_dev->id]) ;
			return 0;
		} 
		else if ( sgmii_dc_present(block)){ 
			printf("SGMII daughter card is present for  %d\n",block);
		} else {	
			printf("SGMII daughter card is not present\n");
			spin_unlock(&m->gmac_lock[current_dev->id]) ;
			return ERR_INVALID_GMAC;
		}
	}

		

	phoenix_write_reg (current_dev->mmio, TX_CONTROL,
		     FIELD_VALUE(TX_THRESHOLD,512) |
		     FIELD_VALUE(TX_ENABLE,0));


	phoenix_write_reg(current_dev->mmio, RX_CONTROL, FIELD_VALUE(RX_ENABLE,0) |
			FIELD_VALUE(ERROR_STATUS, current_dev->error_mask));


	// gmac block datapath reset and mode config
	phoenix_write_reg (current_dev->mmio, G_CONFIG_1, 
		FIELD_VALUE(G_SOFT_RESET,0)                   |
		FIELD_VALUE(LOOP_BACK,0)                      |
		FIELD_VALUE(RECEIVE_FLOW_CONTROL_ENABLE,0)    |
		FIELD_VALUE(TRANSMIT_FLOW_CONTROL_ENABLE,0)   |
		FIELD_VALUE(RECEIVE_ENABLE,0)                 |
		FIELD_VALUE(TRANSMIT_ENABLE,0));
  

	
	gmac_block_commmon_init(current_dev, block);

	arizona_gmac_phy_init( current_dev->mmio );
		
	if ( current_dev->max_tx_frame_len > current_dev->max_rx_frame_len )
		current_dev->max_rx_frame_len = current_dev->max_tx_frame_len;
		

	phoenix_write_reg ( current_dev->mmio, G_MAX_FRAME_LENGTH,
			FIELD_VALUE(MAX_FM_LEN,current_dev->max_rx_frame_len));

	// setup packet descriptor page sizes

	if ( current_dev->rx_regular_buf_size & 0x1f ) {
		/* the frame size given is not aligned to cache line( 32 bit)*/
		current_dev->rx_regular_buf_size = (current_dev->rx_regular_buf_size & ~(0x01f)) + 32; 
	}

	if ( current_dev->enable_prepad == 1 )
		current_dev->rx_regular_buf_size = current_dev->rx_regular_buf_size + 32;

	phoenix_write_reg (current_dev->mmio, DESC_PACK_CTRL,
			FIELD_VALUE(BYTE_OFFSET,current_dev->byte_offset[0])    | // Byte offset of packet data
			FIELD_VALUE(PREPAD_ENABLE,current_dev->enable_prepad)  | // Prepad enable
			FIELD_VALUE(REGULAR_SIZE,current_dev->rx_regular_buf_size)  // Size of regular buffer
			);

	  	// setup default PDE

	// gmac block datapath reset and mode config
	phoenix_write_reg (current_dev->mmio, G_CONFIG_1, 
		FIELD_VALUE(G_SOFT_RESET,0)                   |
		FIELD_VALUE(LOOP_BACK,0)                      |
		FIELD_VALUE(RECEIVE_FLOW_CONTROL_ENABLE,1)    |
		FIELD_VALUE(TRANSMIT_FLOW_CONTROL_ENABLE,1)   |
		FIELD_VALUE(RECEIVE_ENABLE,1)                 |
		FIELD_VALUE(TRANSMIT_ENABLE,1));
   
	// setup gmac split mode 
	phoenix_write_reg ( current_dev->mmio, FREEQ_CARVE,
		FIELD_VALUE(FREEQ_CARVING_MODE,0)|
		FIELD_VALUE(FREEOUTQ_CARVING_MODE,0)|
		FIELD_VALUE(MAX_UNDERRUN_RETRY_COUNT,31)
	) ;


	if ( internal_info->rgmii == 1 && is_xls())
		phoenix_write_reg(current_dev->mmio, RX_CONTROL,FIELD_VALUE(RX_RGMII,1)|
				FIELD_VALUE(ERROR_STATUS, current_dev->error_mask));

		

	// set clock mux to be in gige mode
	phoenix_write_reg(current_dev->mmio, G_IPG, 
		FIELD_VALUE(N_B2B_IPG_1,0x40) |
		FIELD_VALUE(N_B2B_IPG_2,0x60) |
		FIELD_VALUE(MIN_IFG_ENFORCE,0x50) |
		FIELD_VALUE(B2B_IPG,0x58)
		);
  
	// Set filter Off
		phoenix_write_reg (current_dev->mmio, G_MAC_FILTER_CONFIG,  FIELD_VALUE(MAC_ADDR0_VALID,1) |
			FIELD_VALUE(MAC_ADDR1_VALID,1) |
			FIELD_VALUE(MAC_ADDR2_VALID,1) |
			FIELD_VALUE(MAC_ADDR3_VALID,1));
		phoenix_write_reg (current_dev->mmio, G_MAC_ADDR_MASK2_0, 0x0);
		phoenix_write_reg (current_dev->mmio, G_MAC_ADDR_MASK2_1, 0x0);
  
	// Set L2 Alloc On
	phoenix_write_reg (current_dev->mmio, L2_ALLOC_CTRL, 
			FIELD_VALUE(RX_L2_ALLOCATE,current_dev->l2_alloc_rx) | 
			FIELD_VALUE(TX_L2_ALLOCATE,current_dev->l2_alloc_tx));	
  
  
	// setup packet transmit threshold
	phoenix_write_reg (current_dev->mmio, TX_CONTROL,
			FIELD_VALUE(TX_THRESHOLD,512));   // | 	FIELD_VALUE(TX_ENABLE,1));


	if ( current_dev->enable_stat )
		clr_enable_device_stat( current_dev);

	turn_off_filtering( current_dev->mmio ) ;

	if ( current_dev->error_id != -1 ) {
		uint32_t reg = phoenix_read_reg(current_dev->mmio, CORE_CONTROL);
		reg = reg &  ~(0x3fc);
		phoenix_write_reg (current_dev->mmio,CORE_CONTROL, reg | FIELD_VALUE(USE_ERROR_ID,1) |
						FIELD_VALUE(ERROR_ID,current_dev->error_id));
	}
	else {
		uint32_t reg = phoenix_read_reg(current_dev->mmio, CORE_CONTROL);
		reg = reg &  ~(0x3fc);
		phoenix_write_reg (current_dev->mmio,CORE_CONTROL, reg ); 
	}

	
	if ( current_dev->phy_conf[0].auto_neg) {
		nlm_enable_gmac_autonegotiation(current_dev);
	}
	else
		nlm_disable_gmac_autonegotiation_and_set ( current_dev);	

	

	SET_PORT_INIT(internal_info->state);

	m->gmac_dev_init[current_dev->id] = 1;
	spin_unlock(&m->gmac_lock[current_dev->id]) ;

	return 0;
}

int  nlm_gmac_open( device_conf * current_dev )
{

	uint32_t value;
	struct gmac_shm_vars *m;

	if ( current_dev == NULL ) {
		return ERR_INVALID_OPERATION;
	}
	if((m = current_dev->shm) == NULL)
		return ERR_NO_MEM;

	spin_lock(&m->gmac_lock[current_dev->id]) ;

	port_info * internal_info =  ( port_info *)current_dev->intrl;

	if ( internal_info  == NULL ) {
		spin_unlock(&m->gmac_lock[current_dev->id]) ;
		return ERR_INVALID_OPERATION;
	}

	if ( PORT_OPEN( internal_info->state)) {
		spin_unlock(&m->gmac_lock[current_dev->id]) ;
		return ERR_INVALID_OPERATION;
	}

	nlm_gmac_inject_free_desc( current_dev);
	
	value = phoenix_read_reg(current_dev->mmio,TX_CONTROL);
	 
	phoenix_write_reg (current_dev->mmio, TX_CONTROL,value|
		     FIELD_VALUE(TX_THRESHOLD,512) |
		     FIELD_VALUE(TX_ENABLE,1));

	value = phoenix_read_reg(current_dev->mmio,RX_CONTROL);

	phoenix_write_reg(current_dev->mmio, RX_CONTROL, (value | FIELD_VALUE(RX_ENABLE,1) ));

	SET_PORT_OPEN(internal_info->state);

	spin_unlock(&m->gmac_lock[current_dev->id]) ;
	
	return 0;
	
}


int nlm_gmac_close( device_conf * current_dev )
{

	uint32_t rx;
	struct gmac_shm_vars *m;

	if ( current_dev == NULL ) {
		return ERR_INVALID_OPERATION;
	}
	if((m = current_dev->shm) == NULL)
		return ERR_NO_MEM;


	spin_lock(&m->gmac_lock[current_dev->id]) ;
	port_info * phy =  ( port_info *)current_dev->intrl;

	if ( phy == NULL ) {
		spin_unlock(&m->gmac_lock[current_dev->id]) ;
		return ERR_INVALID_OPERATION;
	}

	if ( PORT_CLOSE( phy->state)) {
		spin_unlock(&m->gmac_lock[current_dev->id]) ;
		return ERR_INVALID_OPERATION;
        }


	phoenix_write_reg (current_dev->mmio, TX_CONTROL,
		     FIELD_VALUE(TX_THRESHOLD,512) |
		     FIELD_VALUE(TX_ENABLE,0));


	rx = phoenix_read_reg(current_dev->mmio, RX_CONTROL);
	rx = rx & ~(0x1);
	phoenix_write_reg(current_dev->mmio, RX_CONTROL,rx | FIELD_VALUE(RX_ENABLE,0) );

	SET_PORT_CLOSE(	phy->state);
	spin_unlock(&m->gmac_lock[current_dev->id]) ;

	return 0;
}

static void nlm_gmac_inject_free_desc( device_conf * current_dev)
{
	int desc_bucket;
	port_info * internal = ( port_info *)current_dev->intrl;
	int block = internal->block_id;
	int num_desc = blk_conf[block].max_frin_desc ;
	int i, sent_cnt = 0;
	struct msgrng_msg free_msg;
	void * pbuf = 0;
	void * (*alloc_rxbuf)(void *arg) = blk_conf[block].pkt_mem->alloc_rx_buf;
	void * arg =  blk_conf[block].pkt_mem->arg;
	struct gmac_shm_vars *m = current_dev->shm;

	if(m == NULL)
		return;

	if ( m->gmac_free_desc[current_dev->id] == 1 )
		return;

	if ( current_dev->id < 4 )
		desc_bucket = MSGRNG_STNID_GMAC0_FR;
	else 
		desc_bucket = MSGRNG_STNID_GMAC1_FR;

	free_msg.msg1 = free_msg.msg2 = free_msg.msg3 = 0;

	printf("Sending free desc %d to Gmac station %d\n", num_desc,desc_bucket);

	for (i=0;i<num_desc;i++) {
		pbuf = alloc_rxbuf(arg );
			if(pbuf == NULL)
				break;
			else
				sent_cnt++;
		(void)mac_make_desc_rfr(&free_msg, 0, 0, blk_conf[block].pkt_mem->virt_to_phys(pbuf));
		while(message_send(1, 2, desc_bucket, &free_msg));
	}
	 m->gmac_free_desc[current_dev->id] = 1 ;

  printf("Free packets Request=%d, Sent=%d\n", num_desc, sent_cnt);
}
int nlm_xaui_reset(phoenix_reg_t *mmio)
{
	volatile uint32_t val;
	volatile int delay;

	/* Disable Core RX */
	val = phoenix_read_reg(mmio, RX_CONTROL);
	val &= ~0x1;
	phoenix_write_reg(mmio, RX_CONTROL, val);

	/* wait for rx to halt */
	while(1) {
		val = phoenix_read_reg(mmio, RX_CONTROL);
		if(val & 0x2)
			break;
        	for(delay=0; delay<0xffff; delay++){}
	}

	/* Issue a soft reset */
	val = phoenix_read_reg(mmio, RX_CONTROL);
	val |= 0x4;
	phoenix_write_reg(mmio, RX_CONTROL, val);

	/* wait for reset to complete */
	while(1) {
		val = phoenix_read_reg(mmio, RX_CONTROL);
		if(val & 0x8)
			break;
        	for(delay=0; delay<0xffff; delay++){}
	}

	/* Clear the soft reset bit */
	val = phoenix_read_reg(mmio, RX_CONTROL);
	val &= ~0x4;
	phoenix_write_reg(mmio, RX_CONTROL, val);
	return 0;
}


static void nlm_xaui_init(device_conf * current_dev, int block )
{
	uint32_t value;
	int i;
	volatile int delay =0;
	port_info * phy = NULL;

	phoenix_write_reg (current_dev->mmio, TX_CONTROL,
		     FIELD_VALUE(TX_THRESHOLD,512) |
		     FIELD_VALUE(TX_ENABLE,0));


	phoenix_write_reg(current_dev->mmio, RX_CONTROL, FIELD_VALUE(RX_ENABLE,0) | 
				FIELD_VALUE(ERROR_STATUS, current_dev->error_mask));

	phoenix_write_reg (current_dev->mmio, TX_CONTROL,
		     FIELD_VALUE(TX_THRESHOLD,512) |
		     FIELD_VALUE(TX_ENABLE,0));


	phy = ( port_info *)current_dev->intrl;


	nlm_serdes_regs_init( phy);
	for(delay =0; delay < 0xfffff;delay++);

	nlm_config_spill(current_dev);
	nlm_xaui_reset(current_dev->mmio);

	// setup packet descriptor page sizes, make it consitent with other os

	printf("Initializing port %d in XAUI mode \n", current_dev->id);
	value = phoenix_read_reg(current_dev->mmio, XG_CONFIG_1);
	phoenix_write_reg(current_dev->mmio, XG_CONFIG_1, (value | 0x50000020));
	phoenix_write_reg(current_dev->mmio, XG_MAX_FRAME_LENGTH, 0x0A000A00);

	nlm_setup_pde(current_dev);
	
	// setup DMA<-> memory subsystem credit registers
	phoenix_write_reg(current_dev->mmio, DMA_CR_0, (7<<28)|(7<<24));
	phoenix_write_reg(current_dev->mmio, DMA_CR_3,
		(2<<21)|(2<<18)|(2<<15)|(2<<12)|(2<<9)|(2<<6)|(2<<3)|(2<<0));
	phoenix_write_reg(current_dev->mmio, 0x221, (224 << 16));

	// Set L2 Alloc On
	phoenix_write_reg (current_dev->mmio, L2_ALLOC_CTRL, FIELD_VALUE(RX_L2_ALLOCATE,0xffffffff));
	
	/* free back bucket size configuration */
	phoenix_write_reg ( current_dev->mmio, G_RFR0_BUCKET_SIZE, blk_conf[block].bucket_size[1]);
	phoenix_write_reg ( current_dev->mmio, G_TX0_BUCKET_SIZE, blk_conf[block].bucket_size[2]);
	phoenix_write_reg ( current_dev->mmio, G_TX1_BUCKET_SIZE, blk_conf[block].bucket_size[3]);
	phoenix_write_reg ( current_dev->mmio, G_TX2_BUCKET_SIZE, blk_conf[block].bucket_size[4]);
	phoenix_write_reg ( current_dev->mmio, G_TX3_BUCKET_SIZE, blk_conf[block].bucket_size[5]);

	for(i =0; i < NLM_MAX_CREDIT_COUNTER ; i++ ) 
		phoenix_write_reg ( current_dev->mmio, CC_CPU0_0 + i , blk_conf[block].credit_counter[i]);
	phoenix_write_reg ( current_dev->mmio, PARSER_CONFIG, 0);

	if ( current_dev->rx_regular_buf_size & 0x1f ) {
		/* the frame size given is not aligned to cache line( 32 bit)*/
		current_dev->rx_regular_buf_size = (current_dev->rx_regular_buf_size & ~(0x01f)) + 32; 
	}

	if ( current_dev->enable_prepad == 1 )
		current_dev->rx_regular_buf_size = current_dev->rx_regular_buf_size + 32;

	phoenix_write_reg (current_dev->mmio, DESC_PACK_CTRL,
		FIELD_VALUE(BYTE_OFFSET, current_dev->byte_offset[0])    | // Byte offset of packet data
		FIELD_VALUE(PREPAD_ENABLE,current_dev->enable_prepad)  | // Prepad enable
		FIELD_VALUE(REGULAR_SIZE,current_dev->rx_regular_buf_size)  // Size of regular buffer
	);

	if ( current_dev->enable_stat )
		clr_enable_device_stat( current_dev);

	SET_PORT_INIT( phy->state); 

}

 

// parser port based class distribution mode
static inline void parser_global_port_mode(int group) {
	volatile unsigned int *mmio;
        if (group)
            mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_GMAC_4_OFFSET);
        else
	mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_GMAC_0_OFFSET);  
	phoenix_write_reg (mmio, PARSER_CONFIG, 0x00000008);
}

static int sgmii_dc_present(int block)
{
	unsigned long cpld_base = (unsigned long)(PHOENIX_CPLD_OFFSET);
	unsigned char *mmio = (unsigned char*)cpld_base;
	unsigned char value = mmio[0x0d];
	value = value & 0x03;

	switch (block)
	{
		case 0:
			if ((value == 0x0) || (value == 0x1))
				return 1;
			break;
		case 1:
			if ((value == 0x0) || (value == 0x2))
				return 1;
			break;
		default:
			return 0;
	}
	return 0;
}

static int gmac_408_8p(){
    uint32_t gmac7_disable ;
    uint32_t *gpio_procid_mmio = (uint32_t *)(phoenix_io_base + PHOENIX_IO_GPIO_OFFSET);

    gmac7_disable = ( gpio_procid_mmio[35] >> 28 ) & 0x1 ;

    // ( gmac7_disable = 1 ) => port7 is disabled 
    // ( gmac7_disable = 0 ) => port7 is enabled 

    if ( gmac7_disable) 
        return(0);
    else
        return(1);
}

int is_valid_gmac(int gmac)
{
	
	if(gmac < 0)
		return 0;

	if (is_xls_board_xi_xii())
	{
		int block = gmac/4;
			if ( xlsb0_in_xaui(block)){
				if ((gmac == 0) || (gmac == 4 ))
					return 1;
				else
                        		return 0;
			}

		switch (gmac)
		{
			case 0: /* fall through */
			case 1:
			case 2:
			case 3: 
				return (sgmii_dc_present(0));
				break;
			case 4: /* fall through */
			case 5:
			case 6:
			case 7: 
				return (sgmii_dc_present(1));
				break;
			default: 
				return 0;
		}
	}

	if(is_xls()){
		/*
				GMAC TABLE FOR XLS 

		CHIP 		   NO. OF SGMII		  NO. OF RGMII
		404/408			6			1

		404/408			8			1
		(with 8 ports)

		204/208			4			1
		104/108			3			1

		608			8			1

		LITE-BOARD_2xx		4			0

		LITE-BOARD_4xx		6			0

		LITE-BOARD_4xx		8			0
		(with 8 ports)

		*/
		if(is_xls_408() || is_xls_404()){
			if(gmac_408_8p())
				return (gmac < 8);
			return (gmac < 6);
		}else if(is_xls_208() || is_xls_204()){
			return (gmac < 4);
		}else if(is_xls_608() || is_xls_b0() ){
			return (gmac < 8);
		}else if (is_xls_108() || is_xls_104()) {
			return (gmac < 3);
		}
	}else{
		/*
						MAC TABLE FOR XLR

		BOARD		   NO. OF GMACS		  NO. OF XGMACS		   NO. OF SPI4
		ATX-I			4			0			2
		ATX-I-B			4			0			2
		ATX-II			2			2			0
		ATX-II-B		2			2			0
		ATX-III			4			0			0
		ATX-V			3			0			0

		*/
		if(xlr_board_atx_i() || xlr_board_atx_ii_b() || xlr_board_atx_iii()){
			return (gmac < 4);
		}else if(xlr_board_atx_ii()){
			return (gmac < 2);
		}else if(xlr_board_atx_v()){
			return (gmac < 3);
		}
	}
	return 0;
}


static void advertise_phy_speed( unsigned long mmio,int gmac , int speed , int duplex )
{
        volatile int delay =0;
        uint32_t base1000_ctl =0;
        uint32_t advertise =0;
        uint32_t data =0;

        if ( speed == 10 ) {
                data = data | ( duplex ? (1 << 6 ): (1 << 5 ));
        }
        else if ( speed == 100 ){
                data = data |  ( duplex ? ( 1 << 8) : ( 1 << 7))  ;
            //    data = data |  ( 1 << 8) | ( 1 << 7) | (1 << 6)  |( 1 << 5);
        }
        else if ( speed == 1000 ){
                data = data | (1 << 9) | (1 <<8 );
        }
        else {
                printf("Forced Speed is not valid\n");
                printf("Defaulting to 1000Mbps");
                data = data | (1 << 9) | (1 <<8 );
        }


	arizona_gmac_phy_read(mmio,gmac,BASE1000_T_CTL1 ,&base1000_ctl);
	base1000_ctl = base1000_ctl & ~0x300;

	arizona_gmac_phy_read(mmio,gmac,AUTO_NEG_ADVER,&advertise);
	advertise  = advertise & ~0x1E0;


        if ( speed == 1000 ) 
		base1000_ctl = base1000_ctl | data;
	else
		advertise = advertise | data;
	
	arizona_gmac_phy_write (mmio, gmac  ,BASE1000_T_CTL1 ,base1000_ctl);
        arizona_gmac_phy_write (mmio, gmac  ,AUTO_NEG_ADVER ,advertise);

	arizona_gmac_phy_read(mmio,gmac,PHY_CONTROL,&data);
	data = data & ~(0x00000200);
	arizona_gmac_phy_write (mmio, gmac  ,PHY_CONTROL ,data);
	for (delay = 0; delay < 0xfff; delay++);

	data = data | 0x200;
	arizona_gmac_phy_write (mmio, gmac  ,PHY_CONTROL ,data);
	for (delay = 0; delay < 0x8ffff; delay++);


}

void arizona_gmac_phy_init( phoenix_reg_t *mmio )
{
	int i=0;

	// lower the mgmt clock freq to minimum possible freq
	phoenix_write_reg(mmio, G_MII_MGMT_CONFIG, MGMT_CLOCK_SELECT);

	// poll for the mgmt clk to update
	for (i = 0; i < PHY_STATUS_RETRIES; i++) 
		;
}

int arizona_gmac_phy_read (unsigned long gmac_offset, int num, int reg, uint32_t *data)
{
	phoenix_reg_t *mmio = phoenix_io_mmio(gmac_offset);
	int i=0;

	// setup the phy reg to be read
	phoenix_write_reg(mmio, G_MII_MGMT_ADDRESS, (num<<8)|reg);

	// perform a read cycle
	// the rising edge is detected.
	phoenix_write_reg(mmio, G_MII_MGMT_COMMAND, 0) ;
	phoenix_write_reg(mmio, G_MII_MGMT_COMMAND, READ_COMMAND);

	// poll for the read cycle to complete
	for (i = 0; i < PHY_STATUS_RETRIES; i++) {
		if ((phoenix_read_reg(mmio, G_MII_MGMT_INDICATORS) & 0x5) == 0)  
			break;
	}

	if (i == PHY_STATUS_RETRIES) { 
		*data = 0xffffffff; 
		printf("\n!!!PHY READ FAILED!!!\n");
		return 0; 
	}

	// return the read data
	*data = (uint32_t) phoenix_read_reg (mmio, G_MII_MGMT_STATUS);

	return 1;
}

int arizona_gmac_phy_write (unsigned long gmac_offset, int num, int reg, uint32_t data)
{
	phoenix_reg_t *mmio = phoenix_io_mmio(gmac_offset);
	int i=0;

	// setup the phy reg to be read
	phoenix_write_reg(mmio, G_MII_MGMT_ADDRESS, (num<<8)|reg);

	// write the data which starts the write cycle
	phoenix_write_reg(mmio, G_MII_MGMT_CONTROL, data);

	// poll for the write cycle to complete
	for (i = 0;i<PHY_STATUS_RETRIES;i++) {
		if ((phoenix_read_reg(mmio, G_MII_MGMT_INDICATORS) & 0x1)==0) break;
	}

	// clear the write data/cycle
	//phoenix_write_reg(mmio, G_MII_MGMT_CONTROL, 0);

	if (i == PHY_STATUS_RETRIES) {
		printf("\n!!!PHY WRITE FAILED!!!\n");
		return 0;
	}  
	
	return 1;
}

