/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#include <stdio.h>
#include <stdlib.h>

#include "types.h"
#include "msgring.h"
#include "mac.h"
#include "iomap.h"
#include "parser.h"
#include "mac_defines.h"

/* Parser Config */
prsr_cfg_entry   prsr_cfg ;
p_prsr_cfg_entry p_prsr_cfg = &prsr_cfg ;

/* Translate table */

ttable   _ttable ;
p_ttable _p_ttable = &_ttable ;

/* L2CTable */

l2_table   l2_t ;  
p_l2_table p_l2_t = &l2_t;

/* L3CTable */

l3_table   l3_t ;  
p_l3_table p_l3_t = &l3_t;

/* L4CTable */

l4_table   l4_t ;  
p_l4_table p_l4_t = &l4_t;  

/* cam table and mask */

cam_table   cam_t ; 
p_cam_table p_cam_t = &cam_t; 

uint32_t  l2_protocol_index;
uint32_t  l2_protocol;
uint32_t  l3_start_index;
uint32_t  l3_checksum_length;
uint32_t  l3_ctable_index;
uint32_t  l3_checksum_valid;
uint32_t  l4_checksum_valid;
uint32_t  l3_protocol_index;
uint32_t  l3_protocol;
uint32_t  l4_start_index;
uint32_t  l4_ctable_index;
uint32_t  extract_vector;
uint32_t  field_protocol;
uint32_t  ethernet_format_check_pkt = 0;
uint32_t  ppp_format_check_pkt = 0;

volatile uint32_t parser_error[32] ;

// -------------------------
// Internal Functions
// -------------------------

static prepad calc_extract_vector (p_packet _my_pkt, int prepad_on, int portId) {

	uint32_t start_index = 0 ;
	uint32_t stop_index  = 0 ;
	uint32_t loop_index  = 0 ;
	uint32_t i ;
	uint32_t l3_cam_index = calc_l3_ctable_index(_my_pkt, prepad_on, portId) ;
	uint32_t l4_cam_index = calc_l4_ctable_index(_my_pkt, prepad_on) ;

	uint32_t l3_0_offset = p_l3_t->entry[l3_cam_index].offset_0 ;
	uint32_t l3_1_offset = p_l3_t->entry[l3_cam_index].offset_1 ;
	uint32_t l3_2_offset = p_l3_t->entry[l3_cam_index].offset_2 ;

	uint32_t l3_0_len = p_l3_t->entry[l3_cam_index].len_0 ;
	uint32_t l3_1_len = p_l3_t->entry[l3_cam_index].len_1 ;
	uint32_t l3_2_len = p_l3_t->entry[l3_cam_index].len_2 ;

	uint32_t l4_0_offset = p_l4_t->entry[l4_cam_index].offset_3 ;
	uint32_t l4_1_offset = p_l4_t->entry[l4_cam_index].offset_4 ;

	uint32_t l4_0_len = p_l4_t->entry[l4_cam_index].len_3 ;
	uint32_t l4_1_len = p_l4_t->entry[l4_cam_index].len_4 ;
	prepad   extracted_vector;
	uint32_t PACKET_OFFSET = calc_packet_offset(_my_pkt, prepad_on);

	/* initialize the vector array */

	for (i = 0 ; i < 16 ; i++) {
		extracted_vector.entry[i] = 0 ;
	}

	start_index = calc_l3_start_index(_my_pkt, prepad_on) + l3_0_offset ;
	stop_index  = calc_l3_start_index(_my_pkt, prepad_on) + l3_0_offset + l3_0_len ;
	for (i = start_index ; i < stop_index ; i++) {
		extracted_vector.entry[loop_index] = _my_pkt->data[PACKET_OFFSET+i] ;
		loop_index ++ ;
	}

	start_index = calc_l3_start_index(_my_pkt, prepad_on) + l3_1_offset ;
	stop_index  = calc_l3_start_index(_my_pkt, prepad_on) + l3_1_offset + l3_1_len ;
	for (i = start_index ; i < stop_index ; i++) {
		extracted_vector.entry[loop_index] = _my_pkt->data[PACKET_OFFSET+i] ;
		loop_index ++ ;
	}

	start_index = calc_l3_start_index(_my_pkt, prepad_on) + l3_2_offset ;
	stop_index  = calc_l3_start_index(_my_pkt, prepad_on) + l3_2_offset + l3_2_len ;
	for (i = start_index ; i < stop_index ; i++) {
		extracted_vector.entry[loop_index] = _my_pkt->data[PACKET_OFFSET+i] ;
		loop_index ++ ;
	}

	start_index = calc_l4_start_index(_my_pkt, prepad_on) + l4_0_offset ;
	stop_index  = calc_l4_start_index(_my_pkt, prepad_on) + l4_0_offset + l4_0_len ;
	for (i = start_index ; i < stop_index ; i++) {
		extracted_vector.entry[loop_index] = _my_pkt->data[PACKET_OFFSET+i] ;
		loop_index ++ ;
	}

	start_index = calc_l4_start_index(_my_pkt, prepad_on) + l4_1_offset ;
	stop_index  = calc_l4_start_index(_my_pkt, prepad_on) + l4_1_offset + l4_1_len ;
	for (i = start_index ; i < stop_index ; i++) {
		extracted_vector.entry[loop_index] = _my_pkt->data[PACKET_OFFSET+i] ;
		loop_index ++ ;
	}

	return extracted_vector;
}

static inline uint32_t calc_vector_msb (p_packet _my_pkt, int prepad_on, int portId) {
	prepad     extracted_vector ;
	uint32_t   vector_msb = 0 ;

	extracted_vector = calc_extract_vector (_my_pkt, prepad_on, portId);
	vector_msb = ( extracted_vector.entry[0] & 0xfe ) >> 1 ;
	return (vector_msb);
}

static inline uint32_t calc_cam_hit_index (p_packet _my_pkt, int prepad_on, int portId) {
	uint32_t hit_index = 5 ;
	prepad   extracted_vector ;
	extracted_vector = calc_extract_vector (_my_pkt, prepad_on, portId);

	return ( hit_index );
}

static uint32_t calc_hash_32(p_packet _my_pkt, int prepad_on, int portId) {
	prepad   extracted_vector ;
	p_packet   _hash_vector ;
	uint32_t pktCrc ;

	_hash_vector = pkt_new();
	extracted_vector       = calc_extract_vector (_my_pkt, prepad_on, portId);
	_hash_vector->data[15] = extracted_vector.entry[15] ;
	_hash_vector->data[14] = extracted_vector.entry[14] ;
	_hash_vector->data[13] = extracted_vector.entry[13] ;
	_hash_vector->data[12] = extracted_vector.entry[12] ;
	_hash_vector->data[11] = extracted_vector.entry[11] ;
	_hash_vector->data[10] = extracted_vector.entry[10] ;
	_hash_vector->data[9]  = extracted_vector.entry[9]  ;
	_hash_vector->data[8]  = extracted_vector.entry[8]  ;
	_hash_vector->data[7]  = extracted_vector.entry[7]  ;
	_hash_vector->data[6]  = extracted_vector.entry[6]  ;
	_hash_vector->data[5]  = extracted_vector.entry[5]   ;
	_hash_vector->data[4]  = extracted_vector.entry[4]   ;
	_hash_vector->data[3]  = extracted_vector.entry[3]   ;
	_hash_vector->data[2]  = extracted_vector.entry[2]   ;
	_hash_vector->data[1]  = extracted_vector.entry[1]   ;
	_hash_vector->data[0]  = extracted_vector.entry[0]   ;
	_hash_vector->len   = 16 ;

	pktCrc = crc32_for_hash(_hash_vector->data,_hash_vector->len);
	pkt_delete(_hash_vector);
	return (pktCrc);
}

#if 0
static void calc_print_results (p_packet _my_pkt, int prepad_on, int portId) {
	uint32_t i ;
	prepad   extracted_vector;

	printf("Printing expected classifier results \n");
	printf("calc_l2_protocol_index  =   %d \n",calc_l2_protocol_index (_my_pkt, prepad_on) );
	printf("calc_l2_protocol        = 0x%x \n",calc_l2_protocol (_my_pkt, prepad_on) );
	printf("calc_l3_start_index     =   %d \n",calc_l3_start_index (_my_pkt, prepad_on) );
	printf("calc_l3_checksum_length =   %d \n",calc_l3_checksum_length (_my_pkt, prepad_on) );
	printf("calc_l3_ctable_index    =   %d \n",calc_l3_ctable_index (_my_pkt, prepad_on, portId) );
	printf("calc_l3_checksum_valid  =   %d \n",calc_l3_checksum_valid (_my_pkt, prepad_on) );
	printf("calc_l4_checksum_valid  =   %d \n",calc_l4_checksum_valid (_my_pkt, prepad_on) );
	printf("calc_l3_protocol_index  =   %d \n",calc_l3_protocol_index (_my_pkt, prepad_on) );
	printf("calc_l3_protocol        = 0x%x \n",calc_l3_protocol (_my_pkt, prepad_on) );
	printf("calc_l4_start_index     =   %d \n",calc_l4_start_index (_my_pkt, prepad_on) );
	printf("calc_l4_ctable_index    =   %d \n",calc_l4_ctable_index (_my_pkt, prepad_on) );
	printf("calc_field_protocol     = 0x%x \n",calc_field_protocol (_my_pkt, prepad_on, portId) );
	printf("calc_l3_cam_hit         = 0x%x \n",calc_l3_cam_hit (_my_pkt, prepad_on) );
	printf("calc_l4_cam_hit         = 0x%x \n",calc_l4_cam_hit (_my_pkt, prepad_on) );
	printf("calc_extract_0_valid    = 0x%x \n",calc_extract_0_valid (_my_pkt, prepad_on) );
	printf("calc_extract_1_valid    = 0x%x \n",calc_extract_1_valid (_my_pkt, prepad_on) );
	printf("calc_extract_2_valid    = 0x%x \n",calc_extract_2_valid (_my_pkt, prepad_on) );
	printf("calc_extract_3_valid    = 0x%x \n",calc_extract_3_valid (_my_pkt, prepad_on) );
	printf("calc_extract_4_valid    = 0x%x \n",calc_extract_4_valid (_my_pkt, prepad_on) );
	printf("calc_extract_5_valid    = 0x%x \n",calc_extract_5_valid (_my_pkt, prepad_on) );
	printf("calc_l2_header_done     = 0x%x \n",calc_l2_header_done (_my_pkt, prepad_on) );
	printf("calc_l3_header_done     = 0x%x \n",calc_l3_header_done (_my_pkt, prepad_on) );
	printf("calc_hash_32            = 0x%x \n",calc_hash_32 (_my_pkt, prepad_on, portId) );
	printf("calc_extract_vector     = ");
	extracted_vector = calc_extract_vector(_my_pkt, prepad_on, portId);
	for (i = 0 ; i < 16 ; i++) printf("%x",extracted_vector.entry[i]);
	printf("\n");
}
#endif

static uint32_t calc_hash_7_ttable (p_packet _my_pkt, int prepad_on, int portId) {
	prepad   extracted_vector ;
	p_packet   _hash_vector ;
	uint32_t pktCrc ;

	_hash_vector = pkt_new();
	extracted_vector       = calc_extract_vector (_my_pkt, prepad_on, portId);
	_hash_vector->data[15] = extracted_vector.entry[15];
	_hash_vector->data[14] = extracted_vector.entry[14];
	_hash_vector->data[13] = extracted_vector.entry[13];
	_hash_vector->data[12] = extracted_vector.entry[12];
	_hash_vector->data[11] = extracted_vector.entry[11];
	_hash_vector->data[10] = extracted_vector.entry[10];
	_hash_vector->data[9]  = extracted_vector.entry[9] ;
	_hash_vector->data[8]  = extracted_vector.entry[8] ;
	_hash_vector->data[7]  = extracted_vector.entry[7] ;
	_hash_vector->data[6]  = extracted_vector.entry[6] ;
	_hash_vector->data[5]  = extracted_vector.entry[5]  ;
	_hash_vector->data[4]  = extracted_vector.entry[4]  ;
	_hash_vector->data[3]  = extracted_vector.entry[3]  ;
	_hash_vector->data[2]  = extracted_vector.entry[2]  ;
	_hash_vector->data[1]  = extracted_vector.entry[1]  ;
	_hash_vector->data[0]  = extracted_vector.entry[0]  ;
	_hash_vector->len   = 16 ;

	pktCrc = crc32_for_ttable(_hash_vector->data,_hash_vector->len, 
			p_prsr_cfg->crc_hash_poly);
	pkt_delete(_hash_vector);
	return (pktCrc);
}

static void deconstruct_prepad(p_prepad my_prepad) {

	printf("Deconstructing prepad \n");
	printf("use_bucket           = %x\n",get_use_bucket(my_prepad));
	printf("use_class            = %x\n",get_class(my_prepad));
	printf("bucket               = %x\n",get_bucket(my_prepad));
	printf("hash7                = %x\n",get_hash7(my_prepad));
	printf("sequence_number      = %x\n",get_seq_no(my_prepad));
	printf("l3_cam_hit           = %x\n",get_l3_cam_hit(my_prepad));
	printf("l4_cam_hit           = %x\n",get_l4_cam_hit(my_prepad));
	printf("extra_header_valid   = %x\n",get_extra_header_valid(my_prepad));
	printf("extract_0_valid      = %x\n",get_extract_0_valid(my_prepad));
	printf("extract_1_valid      = %x\n",get_extract_1_valid(my_prepad));
	printf("extract_2_valid      = %x\n",get_extract_2_valid(my_prepad));
	printf("extract_3_valid      = %x\n",get_extract_3_valid(my_prepad));
	printf("extract_4_valid      = %x\n",get_extract_4_valid(my_prepad));
	printf("l2_header_done       = %x\n",get_l2_header_done(my_prepad));
	printf("l3_header_done       = %x\n",get_l3_header_done(my_prepad));
	printf("ip_header_csum_valid = %x\n",get_ip_header_csum_valid(my_prepad));
	printf("l4_csum_valid        = %x\n",get_l4_csum_valid(my_prepad));
	printf("crc_32               = %x\n",get_crc_32(my_prepad));
	printf("l3_cam_addr          = %x\n",get_l3_cam_addr(my_prepad));
	printf("l4_cam_addr          = %x\n",get_l4_cam_addr(my_prepad));
	printf("extract              = %x%x%x%x\n",get_extract_0(my_prepad),get_extract_1(my_prepad),
			                           get_extract_2(my_prepad),get_extract_3(my_prepad));
}

/*
 *
 * Public Functions
 *
 */ 


uint32_t calc_thread_id (p_packet _my_pkt, int prepad_on, int portId) {
	uint32_t ttable_index = 0 ;
	uint32_t bucket_id = 0;

	if (p_prsr_cfg->use_hash) {
		ttable_index = ( calc_hash_7_ttable (_my_pkt, prepad_on, portId)  & 0xfe000000 ) >> 25  ;
		bucket_id = _p_ttable->entry[ttable_index].bucket_id ; 
	}

	if ( !p_prsr_cfg->use_hash && !p_prsr_cfg->use_cam && !p_prsr_cfg->use_proto ) {
		ttable_index = calc_vector_msb (_my_pkt, prepad_on, portId); 
		bucket_id = _p_ttable->entry[ttable_index].bucket_id ; 
	}

	return bucket_id;
}


int check_extraction (p_packet recv_pkt, p_prepad _prepad, int id, int port, int pkt_count) {
	int i;
	prepad extracted_vector ;

	parser_error[id] = 0 ;
	extracted_vector = calc_extract_vector(recv_pkt, 1, port);

	/* compare the extracted vector */
	for (i = 0 ; i < 16 ; i++) {
		if ( recv_pkt->data[16+i] != extracted_vector.entry[i] ) {
			parser_error[id] ++ ;
		}
	}

	if ( get_crc_32(_prepad) != calc_hash_32 (recv_pkt,1, port) ) {
		parser_error[id] ++ ;
	}

	/* compare the l3 cam address */
	if ( get_l3_cam_addr(_prepad) != calc_l3_ctable_index (recv_pkt,1, port) ) {
		parser_error[id] ++ ;
	}

	/* compare the l4 cam address */
	if ( get_l4_cam_addr(_prepad) != calc_l4_ctable_index (recv_pkt,1) ) {
		parser_error[id] ++ ;
	}

	/* compare the tcp checksum */
	if ( get_l4_csum_valid(_prepad) != calc_l4_checksum_valid (recv_pkt,1) ) {
		// do what you want
	}

	// compare the tcp checksum
	if ( get_ip_header_csum_valid(_prepad) != calc_l3_checksum_valid (recv_pkt,1) ) {
		parser_error[id] ++ ;
	}

	return 0;
}

void gmac_5t_extract(volatile unsigned int * _mmio, int mac_num) {

	int i ;
	volatile unsigned int *t_mmio;

	/* enable prepad */
	t_mmio = _mmio;
	phoenix_write_reg( t_mmio , DESC_PACK_CTRL, (phoenix_read_reg (t_mmio, DESC_PACK_CTRL)| FIELD_VALUE(PREPAD_ENABLE,0x1 )));
	t_mmio = _mmio + 0x400;
	phoenix_write_reg( t_mmio , DESC_PACK_CTRL, (phoenix_read_reg (t_mmio, DESC_PACK_CTRL)| FIELD_VALUE(PREPAD_ENABLE,0x1 )));
        if (mac_num > 2) {
		t_mmio = _mmio + 0x800;
		phoenix_write_reg( t_mmio , DESC_PACK_CTRL, (phoenix_read_reg (t_mmio, DESC_PACK_CTRL)| FIELD_VALUE(PREPAD_ENABLE,0x1 )));
		t_mmio = _mmio + 0xc00;
		phoenix_write_reg( t_mmio , DESC_PACK_CTRL, (phoenix_read_reg (t_mmio, DESC_PACK_CTRL)| FIELD_VALUE(PREPAD_ENABLE,0x1 )));
	}

	/* setup the parser config */
	t_mmio = _mmio;

	/* setup the parser config */

	p_prsr_cfg = &prsr_cfg ;
	init_prsr_cfg(p_prsr_cfg);
	p_prsr_cfg->use_global_port = 1 ;
	update_prsr_cfg(p_prsr_cfg, t_mmio);

	/* setup the l2 table */
	p_l2_t = &l2_t ;
	init_l2_table(p_l2_t);

	/* setup the l2 tables, All ethernet ports */

	for (i = 0 ; i < 4 ; i++) {
		p_l2_t->entry[i].extraHdrProtoSize    = 0x0 ;
		p_l2_t->entry[i].extraHdrProtoOffset  = 0x0 ;
		p_l2_t->entry[i].extraHdrSize         = 0x0 ;
		p_l2_t->entry[i].proto_offset         = 0x0 ;
		p_l2_t->entry[i].l2_header_offset     = 0x0 ;
		p_l2_t->entry[i].l2_proto             = L2_ETH ;
	}
	update_l2_table(GMAC_MODE, p_l2_t, t_mmio, mac_num);

	/* setup the l3 table */
	p_l3_t = &l3_t ;
	init_l3_table(p_l3_t);
	for (i = 2 ; i < 16 ; i++) {
		p_l3_t->entry[i].l3_prot_key         = 0x0800 ;
		p_l3_t->entry[i].l2_prot             = L2_ETH ;
		p_l3_t->entry[i].ip_checksum_compute =  1 ;
		p_l3_t->entry[i].l4_prot_offset      =  9 ;
		p_l3_t->entry[i].l3_header_offset    =  0 ;
		p_l3_t->entry[i].offset_0            = 12 ;
		p_l3_t->entry[i].len_0               =  4 ; 
		p_l3_t->entry[i].offset_1            = 16 ;
		p_l3_t->entry[i].len_1               =  4 ;
		p_l3_t->entry[i].offset_2            =  9 ;
		p_l3_t->entry[i].len_2               =  1 ;
	}
	update_l3_table( p_l3_t, t_mmio ) ;

	/* setup the l4 table */
	p_l4_t = &l4_t ;
	init_l4_table(p_l4_t);
	for (i = 2 ; i < 8; i++) {
		p_l4_t->entry[i].l4_prot_key          = 6 ;
		p_l4_t->entry[i].reserved             = 0 ;
		p_l4_t->entry[i].len_4                = 2 ;
		p_l4_t->entry[i].offset_4             = 2 ;
		p_l4_t->entry[i].len_3                = 2 ;
		p_l4_t->entry[i].offset_3             = 0 ;
		p_l4_t->entry[i].reserved_2           = 0 ;
	}
	update_l4_table( p_l4_t, t_mmio ) ;

}

/* function sets up the 5 tupple extraction for the spi4 */

void spi4_5t_extract(int _spi4_id) {

	int i ;
    volatile unsigned int *t_mmio;
	
	if ( _spi4_id == 0 ) {
		t_mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_0_OFFSET);
	} else {
		t_mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_1_OFFSET);
	}

	/* enable the prepad */
	phoenix_write_reg( t_mmio , DESC_PACK_CTRL, (phoenix_read_reg (t_mmio, DESC_PACK_CTRL)| FIELD_VALUE(PREPAD_ENABLE,0x1 )));

	/* setup the parser config */
	p_prsr_cfg = &prsr_cfg;
	init_prsr_cfg(p_prsr_cfg);
	p_prsr_cfg->use_global_port = 1;
	update_prsr_cfg(p_prsr_cfg, t_mmio);

	/* setup the l2 table */
	p_l2_t = &l2_t ;
	init_l2_table(p_l2_t);
	for (i = 0; i < 16; i++) {
		p_l2_t->entry[i].extraHdrProtoSize    = 0x0 ;
		p_l2_t->entry[i].extraHdrProtoOffset  = 0x0 ;
		p_l2_t->entry[i].extraHdrSize         = 0x0 ;
		p_l2_t->entry[i].proto_offset         = 0x0 ;
		p_l2_t->entry[i].l2_header_offset     = 0x0 ;
		p_l2_t->entry[i].l2_proto             = L2_ETH ;
	}
    update_l2_table_xgs (p_l2_t, t_mmio) ;

	/* setup the l3 table */
	p_l3_t = &l3_t ;
	init_l3_table(p_l3_t);
	for (i = 15; i < 16; i++) {
		p_l3_t->entry[i].l3_prot_key         = 0x8021+i ;
		p_l3_t->entry[i].l2_prot             = L2_ETH ;
		p_l3_t->entry[i].ip_checksum_compute =  1 ;
		p_l3_t->entry[i].l4_prot_offset      =  9 ;
		p_l3_t->entry[i].l3_header_offset    =  0 ;
		p_l3_t->entry[i].offset_0            = 12 ;
		p_l3_t->entry[i].len_0               =  4 ; 
		p_l3_t->entry[i].offset_1            = 16 ;
		p_l3_t->entry[i].len_1               =  4 ;
		p_l3_t->entry[i].offset_2            =  9 ;
		p_l3_t->entry[i].len_2               =  1 ;
	}
	update_l3_table( p_l3_t, t_mmio ) ;

	/* setup the l4 table */
	p_l4_t = &l4_t ;
	init_l4_table(p_l4_t);
	for (i = 7 ; i < 8; i++) {
		p_l4_t->entry[i].l4_prot_key          = 6 + i ;
		p_l4_t->entry[i].reserved             = 0 ;
		p_l4_t->entry[i].len_4                = 2 ;
		p_l4_t->entry[i].offset_4             = 2 ;
		p_l4_t->entry[i].len_3                = 2 ;
		p_l4_t->entry[i].offset_3             = 0 ;
		p_l4_t->entry[i].reserved_2           = 0 ;
	}
	update_l4_table( p_l4_t, t_mmio ) ;
}

void update_translate_table(p_ttable my_ttable , volatile unsigned int * _mmio) {
	uint32_t temp_entry = 0 ;
	uint32_t i ;

	for (i = 0 ; i < ((my_ttable->num_entries)/2) ; i++ ) {
		temp_entry = ( my_ttable->entry[i*2].class_id          <<  7 ) |
			( my_ttable->entry[i*2].bucket_id         <<  1 ) |
			( my_ttable->entry[i*2].use_bucket_id     <<  0 ) |
			( my_ttable->entry[(i*2)+1].class_id      << 23 ) |
			( my_ttable->entry[(i*2)+1].bucket_id     << 17 ) |
			( my_ttable->entry[(i*2)+1].use_bucket_id << 16 ) ;
		printf("FROM TRANSLATE TABLE : index = %x, value = %x \n",i,temp_entry);
		_mmio[TRANSLATE_TABLE+i] = temp_entry ;
	}
}

void update_cam_results_table(p_tcam_ttable my_ttable , volatile unsigned int * _mmio) {
	uint32_t temp_entry = 0 ;
	uint32_t i ;

	for (i = 0 ; i < ((my_ttable->num_entries)/2) ; i++ ) {
		temp_entry = ( my_ttable->entry[i*2].class_id          <<  7 ) |
			( my_ttable->entry[i*2].bucket_id         <<  1 ) |
			( my_ttable->entry[i*2].use_bucket_id     <<  0 ) |
			( my_ttable->entry[(i*2)+1].class_id      << 23 ) |
			( my_ttable->entry[(i*2)+1].bucket_id     << 17 ) |
			( my_ttable->entry[(i*2)+1].use_bucket_id << 16 ) ;
		printf("FROM CAM RESULTS TABLE : index = %x, value = %x \n",i,temp_entry);
		phoenix_write_reg(_mmio,(CAM_4X_128_TABLE+i), temp_entry) ;
	}
}

void update_cam_table(p_cam_table my_table, volatile unsigned int * _mmio) {
	uint32_t i, j ;
	uint32_t temp_entry ;

	// fill in the key -- location 0
	// -------------------------------
	temp_entry = ( my_table->data_0[0]  << 24 ) | 
		     ( my_table->data_0[1]  << 16 ) | 
		     ( my_table->data_0[2]  << 8 ) | 
		     ( my_table->data_0[3]  << 0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_KEY + (0*8) + 3),temp_entry) ;

	temp_entry = ( my_table->data_0[4]  <<  24 ) | 
		     ( my_table->data_0[5]  <<  16 ) | 
		     ( my_table->data_0[6]  << 8 ) | 
		     ( my_table->data_0[7]  << 0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_KEY + (0*8) + 2),temp_entry) ;

	temp_entry = ( my_table->data_0[8]  << 24 ) | 
		     ( my_table->data_0[9]  << 16 ) | 
		     ( my_table->data_0[10] << 8 ) | 
		     ( my_table->data_0[11] << 0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_KEY + (0*8) + 1), temp_entry) ;

	temp_entry = ( my_table->data_0[12] << 24 ) | 
		     ( my_table->data_0[13] << 16 ) | 
		     ( my_table->data_0[14] << 8 ) | 
		     ( my_table->data_0[15] << 0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_KEY + (0*8) + 0), temp_entry) ;

	// fill in the mask 
	// ----------------
	temp_entry = ( my_table->mask_0[0]  << 24 ) | 
		     ( my_table->mask_0[1]  << 16 ) | 
		     ( my_table->mask_0[2]  << 8 ) | 
		     ( my_table->mask_0[3]  << 0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_MASK + (0*8) + 3 ), temp_entry) ;

	temp_entry = ( my_table->mask_0[4]  << 24 ) | 
		     ( my_table->mask_0[5]  << 16 ) | 
		     ( my_table->mask_0[6]  << 8 ) | 
		     ( my_table->mask_0[7]  << 0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_MASK + (0*8) + 2), temp_entry) ;

	temp_entry = ( my_table->mask_0[8]  << 24 )  | 
		     ( my_table->mask_0[9]  << 16 ) | 
		     ( my_table->mask_0[10] << 8 ) | 
		     ( my_table->mask_0[11] << 0 ) ;
	phoenix_write_reg(_mmio,( CAM_4X_128_MASK + (0*8) + 1), temp_entry) ;

	temp_entry = ( my_table->mask_0[12] << 24 ) | 
		     ( my_table->mask_0[13] << 16 ) | 
		     ( my_table->mask_0[14] << 8 ) | 
		     ( my_table->mask_0[15] << 0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_MASK + (0*8) + 0),temp_entry) ;


	// fill in the key -- location 1
	// -------------------------------
	temp_entry = ( my_table->data_1[0]  << 24 ) | 
		     ( my_table->data_1[1]  << 16 ) | 
		     ( my_table->data_1[2]  << 8 ) | 
		     ( my_table->data_1[3]  << 0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_KEY + (1*8) + 3 ), temp_entry) ;

	temp_entry = ( my_table->data_1[4] << 24 )  | 
		     ( my_table->data_1[5] << 16 ) | 
		     ( my_table->data_1[6] << 8 ) | 
		     ( my_table->data_1[7] << 0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_KEY + (1*8) + 2), temp_entry) ;

	temp_entry = ( my_table->data_1[8]  << 24 )  | 
		     ( my_table->data_1[9]  << 16 ) | 
		     ( my_table->data_1[10] << 8 ) | 
		     ( my_table->data_1[11] << 0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_KEY + (1*8) + 1), temp_entry) ;

	temp_entry = ( my_table->data_1[12] << 24 ) | 
		     ( my_table->data_1[13] << 16 ) | 
		     ( my_table->data_1[14] << 8 ) | 
		     ( my_table->data_1[15] << 0 ) ;
	phoenix_write_reg(_mmio,( CAM_4X_128_KEY + (1*8) + 0), temp_entry) ;

	// fill in the mask 
	// ----------------
	temp_entry = ( my_table->mask_1[0] << 24 ) | 
		     ( my_table->mask_1[1] << 16 ) | 
		     ( my_table->mask_1[2] << 8 ) | 
		     ( my_table->mask_1[3] << 0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_MASK + (1*8) + 3 ), temp_entry) ;

	temp_entry = ( my_table->mask_1[4] << 24 ) | 
		     ( my_table->mask_1[5] << 16 ) | 
		     ( my_table->mask_1[6] << 8 ) | 
		     ( my_table->mask_1[7] << 0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_MASK + (1*8) + 2), temp_entry) ;

	temp_entry = ( my_table->mask_1[8]  << 24 )  | 
		     ( my_table->mask_1[9]  << 16 ) | 
		     ( my_table->mask_1[10] << 8 ) | 
		     ( my_table->mask_1[11] << 0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_MASK + (1*8) + 1), temp_entry) ;


	temp_entry = ( my_table->mask_1[12] << 24 ) | 
		     ( my_table->mask_1[13] << 16 ) | 
		     ( my_table->mask_1[14] << 8 )  | 
		     ( my_table->mask_1[15] << 0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_MASK + (1*8) + 0),temp_entry) ;


	// fill in the key -- location 2
	// -------------------------------
	temp_entry = ( my_table->data_2[0]  << 24 ) | 
		     ( my_table->data_2[1]  << 16 ) | 
		     ( my_table->data_2[2]  <<  8 ) | 
		     ( my_table->data_2[3]  <<  0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_KEY + (2*8) + 3 ), temp_entry );

	temp_entry = ( my_table->data_2[4]  << 24 ) | 
		     ( my_table->data_2[5]  << 16 ) | 
		     ( my_table->data_2[6]  <<  8 ) | 
		     ( my_table->data_2[7]  <<  0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_KEY + (2*8) + 2),temp_entry) ;

	temp_entry = ( my_table->data_2[8]  << 24 ) | 
		     ( my_table->data_2[9]  << 16 ) | 
		     ( my_table->data_2[10] <<  8 ) | 
		     ( my_table->data_2[11] <<  0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_KEY + (2*8) + 1), temp_entry) ;

	temp_entry = ( my_table->data_2[12] << 24 ) | 
		     ( my_table->data_2[13] << 16 ) | 
		     ( my_table->data_2[14] <<  8 ) | 
		     ( my_table->data_2[15] <<  0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_KEY + (2*8) + 0),temp_entry) ;

	// fill in the mask 
	// ----------------
	temp_entry = ( my_table->mask_2[0] << 24 ) | 
		     ( my_table->mask_2[1] << 16 ) | 
		     ( my_table->mask_2[2] <<  8 ) | 
		     ( my_table->mask_2[3] <<  0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_MASK + (2*8) + 3 ), temp_entry) ;

	temp_entry = ( my_table->mask_2[4]  << 24 ) | 
		     ( my_table->mask_2[5]  << 16 ) | 
		     ( my_table->mask_2[6]  <<  8 ) | 
		     ( my_table->mask_2[7]  <<  0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_MASK + (2*8) + 2),temp_entry) ;

	temp_entry = ( my_table->mask_2[8]  << 24 ) | 
		     ( my_table->mask_2[9]  << 16 ) | 
		     ( my_table->mask_2[10] <<  8 ) | 
		     ( my_table->mask_2[11] <<  0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_MASK + (2*8) + 1),temp_entry) ;

	temp_entry = ( my_table->mask_2[12]   << 24 ) | 
		     ( my_table->mask_2[13] << 16 ) | 
		     ( my_table->mask_2[14] <<  8 ) | 
		     ( my_table->mask_2[15] <<  0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_MASK + (2*8) + 0),temp_entry) ;


	// fill in the key -- location 2
	// -------------------------------
	temp_entry = ( my_table->data_3[0]  << 24 ) | 
		     ( my_table->data_3[1]  << 16 ) | 
		     ( my_table->data_3[2]  <<  8 ) | 
		     ( my_table->data_3[3]  <<  0 ) ;
	phoenix_write_reg( _mmio,(CAM_4X_128_KEY + (3*8) + 3), temp_entry) ;

	temp_entry = ( my_table->data_3[4] << 24 ) | 
		     ( my_table->data_3[5] << 16 ) | 
		     ( my_table->data_3[6] <<  8 ) | 
		     ( my_table->data_3[7] <<  0 ) ;
	phoenix_write_reg( _mmio,(CAM_4X_128_KEY + (3*8) + 2), temp_entry) ;

	temp_entry = ( my_table->data_3[8]  << 24 ) | 
		     ( my_table->data_3[9]  << 16 ) | 
		     ( my_table->data_3[10] <<  8 ) | 
		     ( my_table->data_3[11] <<  0 ) ;
	phoenix_write_reg( _mmio,(CAM_4X_128_KEY + (3*8) + 1), temp_entry) ;

	temp_entry = ( my_table->data_3[12] << 24 ) | 
		     ( my_table->data_3[13] << 16 ) | 
		     ( my_table->data_3[14] <<  8 ) | 
		     ( my_table->data_3[15] <<  0 ) ;
	phoenix_write_reg( _mmio,(CAM_4X_128_KEY + (3*8) + 0), temp_entry) ;

	// fill in the mask 
	// ----------------
	temp_entry = ( my_table->mask_3[0] << 24 ) | 
		     ( my_table->mask_3[1] << 16 ) | 
		     ( my_table->mask_3[2] <<  8 ) | 
		     ( my_table->mask_3[3] <<  0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_MASK + (3*8) + 3 ), temp_entry) ;

	temp_entry = ( my_table->mask_3[4] << 24 ) | 
		     ( my_table->mask_3[5] << 16 ) | 
		     ( my_table->mask_3[6] <<  8 ) | 
		     ( my_table->mask_3[7] <<  0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_MASK + (3*8) + 2 ), temp_entry) ;

	temp_entry = ( my_table->mask_3[8]  << 24 ) | 
		     ( my_table->mask_3[9]  << 16 ) | 
		     ( my_table->mask_3[10] <<  8 ) | 
		     ( my_table->mask_3[11] <<  0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_MASK + (3*8) + 1 ), temp_entry) ;

	temp_entry = ( my_table->mask_3[12] << 24 )  | 
		     ( my_table->mask_3[13] << 16 ) | 
		     ( my_table->mask_3[14] <<  8 ) | 
		     ( my_table->mask_3[15] <<  0 ) ;
	phoenix_write_reg(_mmio,(CAM_4X_128_MASK + (3*8) + 0 ), temp_entry) ;

	for (i = 0 ; i < 4 ; i++) {
		for ( j = 0 ; j < 4 ; j ++ ) {
			printf("key  : address = %x, data = %x \n",CAM_4X_128_KEY  + (i*8) + j,phoenix_read_reg(_mmio,(CAM_4X_128_KEY  + (i*8) + j)));
		}
	}

}

void prepad_copy(p_prepad new_prepad, p_prepad old_prepad) {
	uint32_t i ;

	for (i = 0 ; i < 32 ; i++) {
		new_prepad->entry[i] = old_prepad->entry[i] ;
	}
}

void init_translate_table(p_ttable my_ttable) {
	uint32_t i ;

	for (i = 0 ; i < 128 ; i++) {
		my_ttable->entry[i].class_id      = 0 ;
		my_ttable->entry[i].bucket_id     = 0 ;
		my_ttable->entry[i].use_bucket_id = 0 ;
	}
}

void init_cam_table(p_cam_table my_table) {
	uint32_t i;

	for (i = 0 ; i < 16 ; i++) {
		my_table->data_0[i] = 0 ; my_table->mask_0[i] = 0 ;
		my_table->data_1[i] = 0 ; my_table->mask_1[i] = 0 ;
		my_table->data_2[i] = 0 ; my_table->mask_2[i] = 0 ;
		my_table->data_3[i] = 0 ; my_table->mask_3[i] = 0 ;
	}
}

void init_tcam_translate_table(p_tcam_ttable my_ttable) {
	uint32_t i ;

	for (i = 0 ; i < 4 ; i++) {
		my_ttable->entry[i].class_id      = 0 ;
		my_ttable->entry[i].bucket_id     = 0 ;
		my_ttable->entry[i].use_bucket_id = 0 ;
	}
	my_ttable->num_entries = 4;
}

void set_desc_vlan_tag(struct msgrng_msg *msg, uint16_t vlan_tag) {

	/*
	 * clear out the locations corresponding to the tag
	 * and then fill in the values
	 */

	uint16_t vlan_tag_4_0  = ( vlan_tag & 0x1F )    >> 0 ;
	uint16_t vlan_tag_9_5  = ( vlan_tag & 0x3E0 )   >> 5 ;
	uint16_t vlan_tag_11_0 = ( vlan_tag & 0xC00 )   >> 10 ;
	uint16_t vlan_tag_15_13 = ( vlan_tag & 0xE000 ) >> 13 ;

	msg->msg0 = msg->msg0 & (uint64_t) 0xE3CFFFFFFFFFFFE0Ull ;
	msg->msg0 = msg->msg0 | ( ( (uint64_t) vlan_tag_9_5 )   <<  0 ) | // vlan_tag[9:5]
		( ( (uint64_t) vlan_tag_11_0 )  << 52 ) | // vlan_tag[11:10]
		( ( (uint64_t) vlan_tag_15_13 ) << 58 ) ; // vlan_tag[15:13]

	msg->msg1 = msg->msg1 & (uint64_t) 0xFFFFFFFFFFFFFFE0Ull ;
	msg->msg1 = msg->msg1 | ( (uint64_t) vlan_tag_4_0 ) ; // vlan_tag[4:0]
}

void init_classifier (void) {
	volatile unsigned int *_mmio ;
	int i ;

	/* clear the L2 Type table */
	_mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_GMAC_3_OFFSET); 
	phoenix_write_reg(_mmio,L2_TYPE, 0) ;

	_mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_GMAC_2_OFFSET); 
	phoenix_write_reg(_mmio,L2_TYPE, 0) ;

	_mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_GMAC_1_OFFSET); 
	phoenix_write_reg(_mmio,L2_TYPE, 0) ;

	_mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_GMAC_0_OFFSET); 
	phoenix_write_reg(_mmio,L2_TYPE, 0) ;

	/* clear the L3 CTABLE */
	for (i = 0 ; i < 32 ; i++) 
	  	phoenix_write_reg(_mmio,(L3_CTABLE+i), 0) ;

	/* clear the L4 CTABLE */
	for (i = 0 ; i < 15 ; i++) 
		phoenix_write_reg(_mmio,(L4_CTABLE+i),0) ;

	/* clear the CAM_4X_128_TABLE */
	for (i = 0 ; i < 2 ; i++) 
		phoenix_write_reg(_mmio,(CAM_4X_128_TABLE+i),0) ;

	/* clear the CAM_4X_128_KEY */
	for (i = 0 ; i < 2 ; i++) 
		phoenix_write_reg(_mmio,(CAM_4X_128_KEY+i), 0) ;

	/* clear the Translate table */
	for (i = 0 ; i < 64 ; i++ ) 
		phoenix_write_reg(_mmio,(TRANSLATE_TABLE+i),0) ;
}

void parser_error_print (parser_error_t _my_error) {
    printf("error_portId         = %d\n",_my_error.portId);
    printf("error_pkt_count      = %d\n",_my_error.pkt_count);
    printf("error_l3_cam_addr    = %d\n",_my_error.l3_cam_addr);
    printf("error_l4_cam_addr    = %d\n",_my_error.l4_cam_addr);
    printf("error_l3_csum        = %d\n",_my_error.l3_csum);
    printf("error_l4_csum        = %d\n",_my_error.l4_csum);
    printf("error_vector_mismatch= %d\n",_my_error.vector_mismatch);
    printf("error_l3_cam_hit     = %d\n",_my_error.l3_cam_hit);
    printf("error_l4_cam_hit     = %d\n",_my_error.l4_cam_hit);
    printf("error_extract_0_valid= %d\n",_my_error.extract_0_valid);
    printf("error_extract_1_valid= %d\n",_my_error.extract_1_valid);
    printf("error_extract_2_valid= %d\n",_my_error.extract_2_valid);
    printf("error_extract_3_valid= %d\n",_my_error.extract_3_valid);
    printf("error_extract_4_valid= %d\n",_my_error.extract_4_valid);
    printf("error_extract_5_valid= %d\n",_my_error.extract_5_valid);
    printf("error_l2_header_done = %d\n",_my_error.l2_header_done);
    printf("error_l3_header_done = %d\n",_my_error.l3_header_done);
    printf("error_hash_32        = 0x%x\n",_my_error.hash_32);
    deconstruct_prepad(_my_error.error_prepad);
    printf("\n\n");
    // calc_print_results(_my_error.error_packet, _my_error.error_prepad, _my_error.portId);
    printf("\n\n");
}

uint32_t calc_hash_32_5t(uint32_t ipsa, uint32_t ipda, uint32_t l4_proto, 
						 uint32_t tcp_sp, uint32_t tcp_dp) 
{
	p_packet   _hash_vector ;
	uint32_t pktCrc ;

	_hash_vector = pkt_new();
	_hash_vector->data[15]  = ( ipsa >> 24 ) & 0xff ;
	_hash_vector->data[14]  = ( ipsa >> 16 ) & 0xff ;
	_hash_vector->data[13]  = ( ipsa >>  8 ) & 0xff ;
	_hash_vector->data[12]  = ( ipsa >>  0 ) & 0xff ;
	_hash_vector->data[11]  = ( ipda >> 24 ) & 0xff ;
	_hash_vector->data[10]  = ( ipda >> 16 ) & 0xff ;
	_hash_vector->data[9]   = ( ipda >>  8 ) & 0xff ;
	_hash_vector->data[8]   = ( ipda >>  0 ) & 0xff ;
	_hash_vector->data[7]   = ( l4_proto >> 0 ) & 0xff ;
	_hash_vector->data[6]   = ( tcp_sp >> 8 ) & 0xff ;
	_hash_vector->data[5]   = ( tcp_sp >> 0 ) & 0xff ;
	_hash_vector->data[4]   = ( tcp_dp >> 8 ) & 0xff ;
	_hash_vector->data[3]   = ( tcp_dp >> 0 ) & 0xff ;
	_hash_vector->data[2]   = 0 ; 
	_hash_vector->data[1]   = 0 ;
	_hash_vector->data[0]   = 0 ;
	_hash_vector->len   = 16 ;

	pktCrc = crc32_for_hash(_hash_vector->data,_hash_vector->len);
	pkt_delete(_hash_vector);
	return (pktCrc);
}

void update_l3_table(p_l3_table my_table , volatile unsigned int * _mmio) {

	uint32_t _entry_odd ;
	uint32_t _entry_even ;
	uint32_t i ;

	for (i = 0 ; i < 16 ; i++) {
		_entry_even = my_table->entry[i].l3_prot_key         <<  0 |
			my_table->entry[i].l2_prot             << 16 |
			my_table->entry[i].ip_checksum_compute << 18 |
			my_table->entry[i].l4_prot_offset      << 20 |
			my_table->entry[i].l3_header_offset    << 26 ;

		_entry_odd  = my_table->entry[i].offset_0 << 25 |
			my_table->entry[i].len_0    << 21 |
			my_table->entry[i].offset_1 << 14 |
			my_table->entry[i].len_1    << 10 |
			my_table->entry[i].offset_2 <<  4 |
			my_table->entry[i].len_2    <<  0 ;

		phoenix_write_reg(_mmio,(L3_CTABLE+(2*i)), _entry_even );
		phoenix_write_reg(_mmio,(L3_CTABLE+(2*i)+1), _entry_odd)  ;
	}
}

void init_l3_table(p_l3_table my_table) {
	uint32_t i ;

	for (i = 0 ; i < 16 ; i++) {
		my_table->entry[i].l3_prot_key         = 0 ;
		my_table->entry[i].l2_prot             = 0 ;
		my_table->entry[i].ip_checksum_compute = 0 ;
		my_table->entry[i].l4_prot_offset      = 0 ;
		my_table->entry[i].l3_header_offset    = 0 ;
		my_table->entry[i].offset_0            = 0 ;
		my_table->entry[i].len_0               = 0 ;
		my_table->entry[i].offset_1            = 0 ;
		my_table->entry[i].len_1               = 0 ;
		my_table->entry[i].offset_2            = 0 ;
		my_table->entry[i].len_2               = 0 ;
	}
}

void update_l4_table(p_l4_table my_table , volatile unsigned int * _mmio) {
	uint32_t _entry_odd ;
	uint32_t _entry_even ;
	uint32_t i ;

	for (i = 0 ; i < 8 ; i++) {
		my_table->entry[i].reserved   = 0 ;
		my_table->entry[i].reserved_2 = 0 ;

		_entry_odd  =
			my_table->entry[i].reserved             <<  1 |
			my_table->entry[i].len_4                <<  7 |
			my_table->entry[i].offset_4             << 11 |
			my_table->entry[i].len_3                << 17 |
			my_table->entry[i].offset_3             << 21 |
			my_table->entry[i].reserved_2           << 27 ;
		_entry_even = my_table->entry[i].l4_prot_key ;

		phoenix_write_reg(_mmio,(L4_CTABLE+(2*i)), _entry_even) ;
		phoenix_write_reg(_mmio,(L4_CTABLE+(2*i)+1), _entry_odd)  ;
		printf("L4 Ctable entry %d : %x : %x\n", i, _entry_even, _entry_odd) ;
	}
}

void init_l4_table(p_l4_table my_table) {

	uint32_t i ;

	for (i = 0 ; i < 8 ; i++) {
		my_table->entry[i].l4_prot_key          = 0 ;
		my_table->entry[i].reserved             = 0 ;
		my_table->entry[i].len_4                = 0 ;
		my_table->entry[i].offset_4             = 0 ;
		my_table->entry[i].len_3                = 0 ;
		my_table->entry[i].offset_3             = 0 ;
		my_table->entry[i].reserved_2           = 0 ;
	}
}

void update_prsr_cfg(p_prsr_cfg_entry _my_entry, volatile unsigned int * _mmio) {
	uint32_t temp_entry ;

	temp_entry = ( _my_entry->use_proto         << 0 ) |
		( _my_entry->use_hash          << 1 ) |
		( _my_entry->use_cam           << 2 ) |
		( _my_entry->use_global_port   << 3 ) |
		( _my_entry->crc_hash_poly     << 8 ) |
		( _my_entry->unclassified_mask << 15 ) ;
	printf("Inside update_prsr_cfg : temp_entry = %x \n",temp_entry);
	phoenix_write_reg(_mmio,PARSER_CONFIG, temp_entry) ;
	temp_entry = ( _my_entry->parse_depth & 0x1f ) |
		     ( _my_entry->poly_mask << 5 ) ;
	printf("Inside update_parser_depth : temp_entry = %x \n",temp_entry);
	phoenix_write_reg(_mmio,PARSE_DEPTH, temp_entry );
}

void init_prsr_cfg (p_prsr_cfg_entry _my_entry) {
	_my_entry->use_proto         = 0 ;
	_my_entry->use_hash          = 0 ;
	_my_entry->use_cam           = 0 ;
	_my_entry->use_global_port   = 0 ;
	_my_entry->crc_hash_poly     = 0 ;
	_my_entry->unclassified_mask = 0 ;
}

void update_l2_table_block(int mac_mode, p_l2_table _my_table, volatile unsigned int * _mmio, int mac_num, int block, 
							net_config *net_conf) {
	uint32_t temp_entry ;
	volatile unsigned int *_mmio_0 ;
	volatile unsigned int *_mmio_1 ;
	volatile unsigned int *_mmio_2 ;
	volatile unsigned int *_mmio_3 ;
	uint32_t i ;

	if ( mac_mode == GMAC_MODE ) {
		// addresses are unique for each port of the mac
		_mmio_0  = _mmio;
		_mmio_1  = _mmio + 0x400;
		_mmio_2  = _mmio + 0x800;
		_mmio_3  = _mmio + 0xc00;

		if(block == 0){
			/*gmac0 block*/
			for (i = 0 ; i < mac_num ; i++) {
				temp_entry = (_my_table->entry[i].l2_proto << 0) | 
						 (_my_table->entry[i].l2_header_offset << 2) |
				         (_my_table->entry[i].proto_offset << 8) | 
						 (_my_table->entry[i].extraHdrSize << 14) |
				         (_my_table->entry[i].extraHdrProtoOffset << 20) | 
						 (_my_table->entry[i].extraHdrProtoSize << 26) ;
				if (i == 0 && net_conf->gmac0) { phoenix_write_reg(_mmio_0,L2_TYPE, temp_entry) ; }
				if (i == 1 && net_conf->gmac1) { phoenix_write_reg(_mmio_1,L2_TYPE, temp_entry) ; }
				if (i == 2 && net_conf->gmac2) { phoenix_write_reg(_mmio_2,L2_TYPE, temp_entry) ; }
				if (i == 3 && net_conf->gmac3) { phoenix_write_reg(_mmio_3,L2_TYPE, temp_entry) ; }
			}
		}else {
			/*gmac1 block*/
			for (i = 0 ; i < mac_num ; i++) {
				temp_entry = (_my_table->entry[i].l2_proto << 0) | 
						 (_my_table->entry[i].l2_header_offset << 2) |
				         (_my_table->entry[i].proto_offset << 8) | 
						 (_my_table->entry[i].extraHdrSize << 14) |
				         (_my_table->entry[i].extraHdrProtoOffset << 20) | 
						 (_my_table->entry[i].extraHdrProtoSize << 26) ;
				if (i == 0 && net_conf->gmac4) { phoenix_write_reg(_mmio_0,L2_TYPE, temp_entry) ; }
				if (i == 1 && net_conf->gmac5) { phoenix_write_reg(_mmio_1,L2_TYPE, temp_entry) ; }
				if (i == 2 && net_conf->gmac6) { phoenix_write_reg(_mmio_2,L2_TYPE, temp_entry) ; }
				if (i == 3 && net_conf->gmac7) { phoenix_write_reg(_mmio_3,L2_TYPE, temp_entry) ; }
			}
		}
	}
}

void update_l2_table(int mac_mode, p_l2_table _my_table, volatile unsigned int * _mmio, int mac_num) {
	uint32_t temp_entry ;
	volatile unsigned int *_mmio_0 ;
	volatile unsigned int *_mmio_1 ;
	volatile unsigned int *_mmio_2 ;
	volatile unsigned int *_mmio_3 ;
	uint32_t i ;

	if ( mac_mode == GMAC_MODE ) {
		// addresses are unique for each port of the mac
		_mmio_0  = _mmio;
		_mmio_1  = _mmio + 0x400;
		_mmio_2  = _mmio + 0x800;
		_mmio_3  = _mmio + 0xc00;

		for (i = 0 ; i < mac_num ; i++) {
			temp_entry = (_my_table->entry[i].l2_proto << 0) | 
						 (_my_table->entry[i].l2_header_offset << 2) |
				         (_my_table->entry[i].proto_offset << 8) | 
						 (_my_table->entry[i].extraHdrSize << 14) |
				         (_my_table->entry[i].extraHdrProtoOffset << 20) | 
						 (_my_table->entry[i].extraHdrProtoSize << 26) ;
			if (i == 0 ) { phoenix_write_reg(_mmio_0,L2_TYPE, temp_entry) ; }
			if (i == 1 ) { phoenix_write_reg(_mmio_1,L2_TYPE, temp_entry) ; }
			if (i == 2 ) { phoenix_write_reg(_mmio_2,L2_TYPE, temp_entry) ; }
			if (i == 3 ) { phoenix_write_reg(_mmio_3,L2_TYPE, temp_entry) ; }
		}
	}
}	

void update_l2_table_xgs (p_l2_table _my_table, volatile unsigned int * _mmio) {
	uint32_t temp_entry ;
	uint32_t i ;

	for (i = 0 ; i < 16 ; i++) {
		temp_entry = ( _my_table->entry[i].l2_proto            << 0  ) |
			( _my_table->entry[i].l2_header_offset    << 2  ) |
			( _my_table->entry[i].proto_offset        << 8  ) |
			( _my_table->entry[i].extraHdrSize        << 14 ) |
			( _my_table->entry[i].extraHdrProtoOffset << 20 ) |
			( _my_table->entry[i].extraHdrProtoSize   << 26 ) ;
		phoenix_write_reg(_mmio,(L2_TYPE+i),temp_entry) ;
		printf ("UPDATE_L2_TABLE_XGS: entry %d contains %x\n", i, temp_entry);
	}
}	

void init_l2_table (p_l2_table _my_table) {
	int i ;

	for (i = 0 ; i < 16 ; i++) {
		_my_table->entry[i].l2_proto            = 0 ;
		_my_table->entry[i].l2_header_offset    = 0 ;
		_my_table->entry[i].proto_offset        = 0 ; 
		_my_table->entry[i].extraHdrSize        = 0 ;
		_my_table->entry[i].extraHdrProtoOffset = 0 ;
		_my_table->entry[i].extraHdrProtoSize   = 0 ;
	}
}

// ------------------------------------------------------
// public functions that need to be confined to netos lib
// ------------------------------------------------------

void parser_error_init (parser_error_t _my_error) {
    _my_error.portId          = 0;    
    _my_error.pkt_count       = 0;    
    _my_error.l3_cam_addr     = 0;    
    _my_error.l4_cam_addr     = 0;    
    _my_error.l3_csum         = 0;    
    _my_error.l4_csum         = 0;    
    _my_error.vector_mismatch = 0;
    _my_error.l3_cam_hit      = 0;    
    _my_error.l4_cam_hit      = 0;    
    _my_error.extract_0_valid = 0;
    _my_error.extract_1_valid = 0;
    _my_error.extract_2_valid = 0;
    _my_error.extract_3_valid = 0;
    _my_error.extract_4_valid = 0;
    _my_error.extract_5_valid = 0;
    _my_error.l2_header_done  = 0;
    _my_error.l3_header_done  = 0;
    _my_error.hash_32         = 0;    
}
