/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

// spi4 device driver for NETOS_LIB
// 10x1g configuration
// written by Ahmed Shahid
// NLM

#include <stdio.h>
#include <stdlib.h>

#include "mac_defines.h"
#include "spi4.h"
#include "types.h"
#include "mips-exts.h"
#include "mipsregs.h"
#include "msgring.h"
#include "iomap.h"
#include "netdevice.h"
#include "spinlock.h"
#include "packet.h"
#include "pod.h"
#include "mdio.h"
#include "asmmacros.h"
#include "spill.h"
#include "vitesse_io.h"
#include "vitesse_highlevel.h"
#include "classifier.h"
#include "mm.h"
#include "vitesse_phy_ctrl.h"
#include "string.h"
#include"system.h"
#include "atomic.h"
#include "meigsii_reg.h"
#include "shmvars.h"


static uint32_t spi4_offsets[] = {	PHOENIX_IO_SPI4_0_OFFSET,
					PHOENIX_IO_SPI4_1_OFFSET
				  };

int nlm_spi4_port_close( device_conf * current_dev, int port);
int nlm_spi4_port_open( device_conf * current_dev, int port);
int nlm_spi4_inject_free_desc( device_conf * current_dev );
extern struct block_config  blk_conf[4];
int max_frame_length = 0;
classifier_config classifier_conf_spi4 = {
	/* GMAC INIT */
	.classify_gmac.enable = 0,
	.classify_gmac.rx_enable = 0,
	.classify_gmac.ttable_mode = USE_HASH,
	.classify_gmac.prepad_enable = 0,
        .classify_gmac.byte_offset = 0,
	.classify_gmac.num_classes = 4,
	.classify_gmac.ethertype = 0xffff,
	.classify_gmac.offset_l4_prot = 0,
	.classify_gmac.offset_0 = 0,
	.classify_gmac.len_0 = 0,
	.classify_gmac.offset_1 = 0,
	.classify_gmac.len_1 = 0,
	.classify_gmac.offset_2 = 0,
	.classify_gmac.len_2 = 0,
	.classify_gmac.offset_3 = 0,
	.classify_gmac.len_3 = 0,
	.classify_gmac.offset_4 = 0,
	.classify_gmac.len_4 = 0,
	.classify_gmac.offset_5 = 0,
	.classify_gmac.len_5 = 0,

	/* XGS0 INIT */
	.classify_xgs0.enable = 1,
	.classify_xgs0.rx_enable = 1,
	.classify_xgs0.ttable_mode = USE_HASH,
	.classify_xgs0.prepad_enable = 0,
        .classify_xgs0.byte_offset = 0,
	.classify_xgs0.num_classes = 4,
	.classify_xgs0.ethertype = 0xffff,
	.classify_xgs0.offset_l4_prot = 0,
	.classify_xgs0.offset_0 = 0,
	.classify_xgs0.len_0 = 0,
	.classify_xgs0.offset_1 = 0,
	.classify_xgs0.len_1 = 0,
	.classify_xgs0.offset_2 = 0,
	.classify_xgs0.len_2 = 0,
	.classify_xgs0.offset_3 = 0,
	.classify_xgs0.len_3 = 0,
	.classify_xgs0.offset_4 = 0,
	.classify_xgs0.len_4 = 0,
	.classify_xgs0.offset_5 = 0,
	.classify_xgs0.len_5 = 0,

	/* XGS1 INIT */
	.classify_xgs1.enable = 1,
	.classify_xgs1.rx_enable = 1,
	.classify_xgs1.ttable_mode = USE_HASH,
	.classify_xgs1.prepad_enable = 0,
        .classify_xgs1.byte_offset = 0,
	.classify_xgs1.num_classes = 4,
	.classify_xgs1.ethertype = 0xffff,
	.classify_xgs1.offset_l4_prot = 0,
	.classify_xgs1.offset_0 = 0,
	.classify_xgs1.len_0 = 0,
	.classify_xgs1.offset_1 = 0,
	.classify_xgs1.len_1 = 0,
	.classify_xgs1.offset_2 = 0,
	.classify_xgs1.len_2 = 0,
	.classify_xgs1.offset_3 = 0,
	.classify_xgs1.len_3 = 0,
	.classify_xgs1.offset_4 = 0,
	.classify_xgs1.len_4 = 0,
	.classify_xgs1.offset_5 = 0,
	.classify_xgs1.len_5 = 0,
};


// spi4 card detect routine for ATX-1 vitess daughter cards
static int spi4_card_detect(int device_number){

	unsigned char value;
        unsigned long cpld_base;
        unsigned char *mmio ;

        cpld_base = (unsigned long)(PHOENIX_CPLD_OFFSET);
        mmio = (unsigned char*) cpld_base;
        value = mmio[CPLD_MISC_STATUS_REG];

	if ( device_number == 1 ) {
		value = value>>3;
		if(!(value & SPI4_MASK_BIT1))
			return 1;
	}
	else {
		value = value >>4;
		if(!(value & SPI4_MASK_BIT1))
			return 1;

	}

	return 0;
}


// spi4 rx fifo carve routine
static void spi4_rx_fifo_carve(int num_interface, int num_ports) {
	int spi_rx_fifo_base[16];
	int spi_rx_fifo_size[16];
	int glu_rx_fifo_base[16];
	int glu_rx_fifo_size[16];

	int single_fifo_size;
	int total_size;
	int current_base;
	int base;
	int size;
	int i;

	int base0;
	int size0;
	int base1;
	int size1;
	int reg_addr;
	// int value;
	
	volatile unsigned int *mmio;

	if (num_interface == 1) {
		mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_0_OFFSET);
	} else {
		mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_1_OFFSET);
	}  

	// carve spi_rx_fifo
	total_size = 512;
	single_fifo_size = total_size/num_ports;
	for (i=0;i<16;i++) {
		current_base = i * single_fifo_size;
		if (i< num_ports) {
		  base = current_base;
		  size = single_fifo_size;
		} else {
		  base = total_size;
		  size = 0;
		}    
		spi_rx_fifo_base[i] = base;
		spi_rx_fifo_size[i] = size;
	}
	
	// carve glu_rx_fifo
	total_size = 96;
	single_fifo_size = total_size/num_ports;  
	for (i=0;i<16;i++) {
		current_base = i * single_fifo_size;
		if (i< num_ports) {
		  base = current_base;
		  size = single_fifo_size;
		} else {
		  base = total_size;
		  size = 0;
		}    
		glu_rx_fifo_base[i] = base;
		glu_rx_fifo_size[i] = size;
	}  

	// program spi_rx_fifo
	for (i=0;i<16;i++) {
		phoenix_write_reg(mmio,SPI_RX_FIFO_BASE,((i << 16) | spi_rx_fifo_base[i]));
		phoenix_write_reg(mmio,SPI_RX_FIFO_SIZE,((i << 16) | spi_rx_fifo_size[i]));
	}
	
	// program glu_rx_fifo
	for (i=0;i<8;i++) {
		base0 = glu_rx_fifo_base[i*2];
		base1 = glu_rx_fifo_base[i*2+1];
		size0 = glu_rx_fifo_size[i*2];
		size1 = glu_rx_fifo_size[i*2+1];
		reg_addr = GLU_RX_FIFO_BASE +i;
		phoenix_write_reg(mmio,reg_addr,((base0 << 24) | (size0 << 16) | (base1 << 8) | (size1)));
	}
}


// spi4 tx fifo carve routine
static void spi4_tx_fifo_carve(int num_interface, int num_ports) {
	int spi_tx_fifo_base[16];
	int spi_tx_fifo_size[16];
	int glu_tx_fifo_base[16];
	int glu_tx_fifo_size[16];

	int single_fifo_size;
	int total_size;
	int current_base;
	int base;
	int size;
	int i;

	int base0;
	int size0;
	int base1;
	int size1;
	int reg_addr;
	// int value;
	
	volatile unsigned int *mmio;

	if (num_interface == 1) {
		mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_0_OFFSET);
	} else {
		mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_1_OFFSET);
	}  

	// carve spi_tx_fifo
	total_size = 128;
	single_fifo_size = total_size/num_ports;
	for (i=0;i<16;i++) {
		current_base = i * single_fifo_size;
		if (i< num_ports) {
		  base = current_base;
		  size = single_fifo_size;
		} else {
		  base = total_size;
		  size = 0;
		}    
		spi_tx_fifo_base[i] = base;
		spi_tx_fifo_size[i] = size;
	}

	// carve glu_tx_fifo
	total_size = 96;
	single_fifo_size = total_size/num_ports;  
	for (i=0;i<16;i++) {
		current_base = i * single_fifo_size;
		if (i< num_ports) {
		  base = current_base;
		  size = single_fifo_size;
		} else {
		  base = total_size;
		  size = 0;
		}    
		glu_tx_fifo_base[i] = base;
		glu_tx_fifo_size[i] = size;
	}  

	// program spi_tx_fifo
	for (i=0;i<16;i++) {
		phoenix_write_reg(mmio,SPI_TX_FIFO_BASE,((i << 16) | spi_tx_fifo_base[i]));
		phoenix_write_reg(mmio,SPI_TX_FIFO_SIZE,((i << 16) | spi_tx_fifo_size[i]));
	}

	// program glu_tx_fifo
	for (i=0;i<8;i++) {
		base0 = glu_tx_fifo_base[i*2];
		base1 = glu_tx_fifo_base[i*2+1];
		size0 = glu_tx_fifo_size[i*2];
		size1 = glu_tx_fifo_size[i*2+1];
		reg_addr =  GLU_TX_FIFO_BASE +i;
		phoenix_write_reg(mmio,reg_addr,( (base0 << 24) | (size0 << 16) | (base1 << 8) | (size1)));
	}
}


// function to program maxburst1
static void spi4_set_rx_maxburst1(int num_interface, int value) {
	int i;
	volatile unsigned int *mmio;
	
	if (num_interface == 1) {
		mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_0_OFFSET);
	} else {
		mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_1_OFFSET);
	}
	
	// program spi4 max burst1 for 16 channels
	for (i=0;i<16;i++)
		phoenix_write_reg(mmio,ING_MAXBURST1,((i << 16) | value));
}


// function to program maxburst2
static void spi4_set_rx_maxburst2(int num_interface, int value) {
	int i;
	volatile unsigned int *mmio;
	
	if (num_interface == 1) 
		mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_0_OFFSET);
	else
		mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_1_OFFSET);

	
	// program spi4 max burst1 for 16 channels
	for (i=0;i<16;i++)
		phoenix_write_reg(mmio,ING_MAXBURST2,((i << 16) | value));
}


// function to program tx maxburst1
static void spi4_set_tx_maxburst1(int num_interface, int value) {
 	int i;
	volatile unsigned int *mmio;
	
	if (num_interface == 1) 
		mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_0_OFFSET);
	else
		mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_1_OFFSET);

	// program spi4 tx maxburst1 for 16 channels
	for (i=0;i<16;i++)
		phoenix_write_reg(mmio,EGR_MAXBURST1,((i << 16) | value));
}

// function to program tx maxburst2
static void spi4_set_tx_maxburst2(int num_interface, int value) {
 	int i;
	volatile unsigned int *mmio;
	
	if (num_interface == 1) 
		mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_0_OFFSET);
	else
		mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_1_OFFSET);

	// program spi4 tx maxburst2 for 16 channels
	for (i=0;i<16;i++)
		phoenix_write_reg(mmio,EGR_MAXBURST2,((i << 16) | value));
}

static int vtss_nlm_monitor_phy_status(int port, int *speed, int *duplexity)
{
	vtss_rc ret, dup;
	vtss_phy_status_t   status;

	ret = vtss_phy_status_get(port, &status);
	if(ret != VTSS_OK){
		return 0;
	}
	if ( status.autoneg_complete == 0 )
		return 1;
	dup = ret = vtss_phy_read(port, 0x1c);
	if(ret<0) return 0;
	ret >>=3;
	ret &= 0x3;
	switch(ret){
	case 2:
		(*speed) =(int) MAC_SPEED_1000;
		break;
	case 1:
		(*speed )= (int)MAC_SPEED_100;
		 break;
	case 0:
		(*speed) = (int)MAC_SPEED_10;
		break;
	}
	dup >>= 5;
	dup &= 0x01;
	if(dup){
		(*duplexity) = (int)1;
	}
	else{
		(*duplexity) = (int)0;
	}
	return 1;
}

// configure spi4 for arizona-1 daughter card settings
static void spi4_arizona_setup(int num) {
	volatile unsigned int *mmio;
	
	if (num == 1) {
		mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_0_OFFSET);
	}
	else {
		mmio = (unsigned int *)(phoenix_io_base + PHOENIX_IO_SPI4_1_OFFSET);
	}
	
	// Setup calendering to match vitess device parameters
	phoenix_write_reg ( mmio, SPI_RX_CAL_LEN, 10);
	phoenix_write_reg ( mmio, SPI_RX_CAL_M,   1);
	phoenix_write_reg ( mmio, SPI_TX_CAL_LEN, 10);
	phoenix_write_reg ( mmio, SPI_TX_CAL_M,   1);
	
	// clean interrupts and other status bits
	phoenix_write_reg (mmio, SPI_INT , FIELD_VALUE(INVALID_CW,1) |
		                    FIELD_VALUE(DIP2_ERROR,1) |
		                    FIELD_VALUE(DIP4_ERROR,1)) ;
	
	phoenix_write_reg( mmio, SPI_TX_STATUS, FIELD_VALUE(TX_SYNC,1) |
		                    FIELD_VALUE(TX_TRAIN_DETECT,1) |
		                    FIELD_VALUE(TX_CAL_ERROR,1)) ;
	
	phoenix_write_reg( mmio, SPI_RX_STATUS, FIELD_VALUE(MISSING_EOP,1) |
		                    FIELD_VALUE(MISSING_SOP,1) |
		                    FIELD_VALUE(RX_SYNC,1) |
		                    FIELD_VALUE(RX_TRAIN_DETECT,1)) ;
	
	// Total number of Entries in TxFIFO (Interface) is 128
	// Each entry in the TxFIFO is 16 bytes.
	// The following fifo carving functions distribute
	// the 128 entries evenly to all the 10 ports.
	// This gives 12 entries to all the ports

	spi4_tx_fifo_carve(num, 10);

	// Total number of Entries in RxFIFO (Interface) is 512.
	// Each entry in the RxFIFO is 16 bytes.
	// The following fifo carving functions distribute
	// the 512 entries evenly to all the 10 ports.
	// This gives 51 entries to all the ports

	spi4_rx_fifo_carve(num, 10);

	// When number of available entries in RxFIFO is more than
	// maxburst1 (or starving threshold), the rx_status signals
	// starving to the partner device. 

	spi4_set_rx_maxburst1(num, 0x20);
 
	// When number of available entries in RxFIFO is more than
	// maxburst2 (or hungry threshold), the rx_status signals
	// hungry to the partner device. 

	spi4_set_rx_maxburst2(num, 0x20);

	// set egress maxburst1
	spi4_set_tx_maxburst1(num, 0x10);
	
	// set egress maxburst2
	spi4_set_tx_maxburst2(num, 0x10);

	// disable backend burst length restricts
	phoenix_write_reg( mmio,SPI_BE_BURST,0);

	// This is for internal flow control in transmit side
	// between glue and SPi4 controller
	phoenix_write_reg(mmio,SPI_HNGY_0, FIELD_VALUE(EG_HNGY_THRESH_0,3) |
	                  FIELD_VALUE(EG_HNGY_THRESH_1,3) |
	                  FIELD_VALUE(EG_HNGY_THRESH_2,3) |
	                  FIELD_VALUE(EG_HNGY_THRESH_3,3)) ;
		  
	phoenix_write_reg(mmio,SPI_HNGY_1, FIELD_VALUE(EG_HNGY_THRESH_4,3) |
		              FIELD_VALUE(EG_HNGY_THRESH_5,3) |
		              FIELD_VALUE(EG_HNGY_THRESH_6,3) |
		              FIELD_VALUE(EG_HNGY_THRESH_7,3)) ;
		  
	phoenix_write_reg(mmio,SPI_HNGY_2, FIELD_VALUE(EG_HNGY_THRESH_8,3) |
		              FIELD_VALUE(EG_HNGY_THRESH_9,3) |
		              FIELD_VALUE(EG_HNGY_THRESH_10,3) |
		              FIELD_VALUE(EG_HNGY_THRESH_11,3)) ;

	phoenix_write_reg(mmio,SPI_HNGY_3, FIELD_VALUE(EG_HNGY_THRESH_12,3) |
		              FIELD_VALUE(EG_HNGY_THRESH_13,3) |
		              FIELD_VALUE(EG_HNGY_THRESH_14,3) |
		              FIELD_VALUE(EG_HNGY_THRESH_15,3)) ;

	phoenix_write_reg(mmio,SPI_STRV_0, FIELD_VALUE(EG_STRV_THRESH_0,5) |
		              FIELD_VALUE(EG_STRV_THRESH_1,5) |
		              FIELD_VALUE(EG_STRV_THRESH_2,5) |
		              FIELD_VALUE(EG_STRV_THRESH_3,5)) ;

	phoenix_write_reg(mmio,SPI_STRV_1, FIELD_VALUE(EG_STRV_THRESH_4,5) |
		              FIELD_VALUE(EG_STRV_THRESH_5,5) |
		              FIELD_VALUE(EG_STRV_THRESH_6,5) |
		              FIELD_VALUE(EG_STRV_THRESH_7,5)) ;
		  
	phoenix_write_reg( mmio,SPI_STRV_2, FIELD_VALUE(EG_STRV_THRESH_8,5) |
		              FIELD_VALUE(EG_STRV_THRESH_9,5) |
		              FIELD_VALUE(EG_STRV_THRESH_10,5) |
		              FIELD_VALUE(EG_STRV_THRESH_11,5)) ;
		  
	phoenix_write_reg( mmio,SPI_STRV_3, FIELD_VALUE(EG_STRV_THRESH_12,5) |
		              FIELD_VALUE(EG_STRV_THRESH_13,5) |
		              FIELD_VALUE(EG_STRV_THRESH_14,5) |
		              FIELD_VALUE(EG_STRV_THRESH_15,5)) ;
}


static int vtss_nlm_change_port_status(int port, int id,int speed, int duplexity, int forced)
{
	int i;

	uint32_t *mmio = (uint32_t *)(phoenix_io_base + PHOENIX_IO_SPI4_0_OFFSET);

	if ( id == 1 )
		mmio = (uint32_t *)(phoenix_io_base + PHOENIX_IO_SPI4_1_OFFSET);

	phoenix_write_reg( mmio, CNTRL_REG, FIELD_VALUE(SPI4_RX_ENABLE,0) |
					FIELD_VALUE(SPI4_TX_ENABLE,0)); 
	switch(speed){
	case MAC_SPEED_1000:
		speed = VTSS_SPEED_1G;
		break;
	case MAC_SPEED_100:
		speed = VTSS_SPEED_100M;
		break;
	case MAC_SPEED_10:
		speed = VTSS_SPEED_10M;
		break;
	}

	if ( speed == VTSS_SPEED_1G) {
		printf("configuring spi4 port_%d in 1000Mbps mode\n", (port -1 ));
	}
	else if ( speed == VTSS_SPEED_100M){
		printf("configuring spi4 port_%d in 100Mbps mode\n", (port -1));
	}
	else if( speed == VTSS_SPEED_10M ){
		printf("configuring spi4 port_%d in 10Mbps mode\n", (port -1 ));
	}
	else {
		printf("Invalid Speed. spi4 port_%d is down\n", (port -1 ));
	}
	i = vtss_port_set_mode(port, speed, duplexity);
	phoenix_write_reg( mmio, CNTRL_REG, FIELD_VALUE(SPI4_RX_ENABLE,1) |
 				FIELD_VALUE(SPI4_TX_ENABLE,1));
	if(i == VTSS_OK ){
		return 1;
	} else {
		return 0;
	}
}

static void  change_vits_base(int port)
{
	if(port < XLR_TOTAL_SPI4_CHANNELS){
		megis_phoenix_init(1);
	} else {
		megis_phoenix_init(2);
	}
	return;
}

static int  check_spi4_link_status(device_conf *current_dev, int port)
{
	int speed = 0, duplexity = 0,index = port;
	int id =0;
	vtss_rc ret;
	vtss_phy_status_t   status;
	struct spi4_shm_vars *m;

	if((m = current_dev->shm) == NULL)
		return ERR_NO_MEM;

	change_vits_base(port);
	if ( port >= XLR_TOTAL_SPI4_CHANNELS ) {
		port = port - XLR_TOTAL_SPI4_CHANNELS;
		id = 1;
	}

	if ( spi4_card_detect( id + 1 ) == 0 )
		return 0;
	port++;

        ret = vtss_phy_status_get(port, &status);
        if(ret != VTSS_OK){
                return 0;
        }
	
	if (vtss_nlm_monitor_phy_status(port, &speed, &duplexity) == 0 ) {
		//printf(" Port %d is Not Ready\n ",port );
		return 0;
	}

	if ( (speed == m->spi4_speed[index])&& (duplexity == m->spi4_duplex[index]))
		return 0;

	if ( m->spi4_speed[index] != -1 ) {
		
		int try =0;
		volatile uint32_t delay;
		do
		{	
			for( delay =0 ; delay<0xfffff; delay++);
			vtss_phy_status_get(port, &status);
			try++;

			if ( try == 1000)
				break;
		}while ( status.autoneg_complete == 0); 
		vtss_nlm_monitor_phy_status(port, &speed, &duplexity);
	}

	m->spi4_speed[index]  = speed;
	m->spi4_duplex[index] = duplexity;


        ret = vtss_phy_status_get(port, &status);
	
	vtss_nlm_change_port_status(port,id, speed,duplexity,0);

	return 1;
}


static int  enable_spi4_autonegotiation(int port )
{
	vtss_phy_control_t control;
	vtss_phy_autoneg_advertisement_t advertisement;
	vtss_phy_masterslave_control_t masterslave_control;

	if ( port >= ( 2 * XLR_TOTAL_SPI4_CHANNELS)  )
		 return -1;

	change_vits_base(port );

	if ( port >=  XLR_TOTAL_SPI4_CHANNELS) 
		port = port - XLR_TOTAL_SPI4_CHANNELS;
	port++;

	memset(&advertisement,0,sizeof(vtss_phy_autoneg_advertisement_t));
	vtss_phy_autoneg_advertisement_get( port, &advertisement);
	advertisement.ability_100base_x_fdx = 1; 
	advertisement.ability_100base_x_hdx = 1;
	advertisement.ability_10base_t_fdx = 1;
	advertisement.ability_10base_t_hdx = 1;
	vtss_phy_autoneg_advertisement_set( port, &advertisement);

	memset(&masterslave_control,0,sizeof(vtss_phy_masterslave_control_t));
	vtss_phy_masterslave_control_get( port, &masterslave_control);
	masterslave_control.ability_1000base_t_fdx = 1;
	vtss_phy_masterslave_control_set( port,&masterslave_control);

	memset(&control,0,sizeof(control));
	vtss_phy_control_get( port, &control );
	control.reset = 0;
	control.loopback = 0;
	control.autoneg_enable = 1;
	control.powerdown = 0;
	control.isolate = 0;		
	control.autoneg_restart = 1;
	control.fdx = 0;
	control.collision_test = 0;
	vtss_phy_control_set(port,&control );

	return 0;
}
static int  disable_spi4_autonegotiation(int port, int speed, int duplexity )
{
	vtss_phy_control_t control;
	vtss_phy_autoneg_advertisement_t advertisement;
	vtss_phy_masterslave_control_t masterslave_control;
	int id =0;

	if ( port >= ( 2 * XLR_TOTAL_SPI4_CHANNELS)  )
		return -1;
	change_vits_base(port );
	if ( port >= XLR_TOTAL_SPI4_CHANNELS) {
		port = port - XLR_TOTAL_SPI4_CHANNELS ;
		id  = 1;
	}
	port++;

	memset(&advertisement,0,sizeof(vtss_phy_autoneg_advertisement_t));
	vtss_phy_autoneg_advertisement_get( port, &advertisement);
	advertisement.ability_100base_t4 =  0;
	advertisement.ability_100base_x_fdx = 0; 
	advertisement.ability_100base_x_hdx = 0;
	advertisement.ability_10base_t_fdx = 0;
	advertisement.ability_10base_t_hdx = 0;

	memset(&masterslave_control,0,sizeof(vtss_phy_masterslave_control_t));
	vtss_phy_masterslave_control_get( port, &masterslave_control);
	masterslave_control.ability_1000base_t_fdx = 0;

	if ( speed  == MAC_SPEED_100 )
	{
		if ( duplexity == 1 ) 
			advertisement.ability_100base_x_fdx = 1 ; 
		else { 
			advertisement.ability_100base_t4 =  1;
			advertisement.ability_100base_x_hdx = 1;
		}
	} 
	else if( speed ==  MAC_SPEED_10) 
	{
		if ( duplexity == 1 ) 
			advertisement.ability_10base_t_fdx = 1;
		else 
			advertisement.ability_10base_t_hdx = 1;
	}
	else  
	{
		if ( duplexity ==  1 ) 
			masterslave_control.ability_1000base_t_fdx = 1;
	}

	control.reset = 0;
	control.loopback = 0;
	control.autoneg_enable = 1;
	control.powerdown = 0;
	control.isolate = 0;		
	control.autoneg_restart = 1;
	control.fdx = 0;
	control.collision_test = 0;
	vtss_phy_masterslave_control_set( port,&masterslave_control);
	vtss_phy_autoneg_advertisement_set( port, &advertisement);

	vtss_phy_control_set(port,&control );
	
	vtss_nlm_change_port_status( port, id,speed, duplexity,1);

	return 0;
}
	
static int is_autoneg_complete(int port)
{
        vtss_phy_status_t   status;
        port++;
        if ( VTSS_OK  != vtss_phy_status_get(port, &status) )
                return 0;
        if ( status.autoneg_complete == 1 )
                return 1;

        return 0;


}
void nlm_spi4_soft_reset( phoenix_reg_t * mmio) 
{

	phoenix_write_reg( mmio, CNTRL_REG, FIELD_VALUE(SPI4_RX_ENABLE,0) |
 					FIELD_VALUE(SPI4_TX_ENABLE,0)); 
	phoenix_write_reg( mmio, RX_CONTROL,FIELD_VALUE(RX_ENABLE,0));
	phoenix_write_reg (mmio, TX_CONTROL, FIELD_VALUE(TX_THRESHOLD,512) | FIELD_VALUE(TX_ENABLE,0)); 
	phoenix_write_reg (mmio, RX_CONTROL, FIELD_VALUE(SOFT_RESET,1));
        while ((GET_FIELD(phoenix_read_reg( mmio , RX_CONTROL),SOFT_RESET_DONE)) == 0) {}
        phoenix_write_reg (mmio, RX_CONTROL, FIELD_VALUE(SOFT_RESET,0));

}

int nlm_spi4_init( device_conf * current_dev )
{
	int daughter_card_id ;
	int i=0;
	port_info * internal_info = NULL;
	volatile uint32_t delay;
	int block = 0; 
	int auto_complete[ NLM_MAX_SPI4_PORT_PER_BLOCK ] = { [0 ... 9] = 0 }; 
	struct spi4_shm_vars *m;

	

	if (!xlr_board_atx_i()) {
		printf("SPI4 interface is not supported on this board");
		return ERR_DEVICE_NOT_SUPORTTED;
	}

	if ( current_dev == NULL ) 
		return ERR_INVALID_DEVICE_CONFIG;


	if((current_dev->shm = nlm_spi4_shm_init()) == NULL)
		return ERR_NO_MEM;
	m = current_dev->shm;

	spin_lock(&m->spi4_lock[current_dev->id]) ;

	if ( check_device_config( current_dev ) ==  ERR_INVALID_DEVICE_CONFIG ) {
		spin_unlock(&m->spi4_lock[current_dev->id]) ;
		return ERR_INVALID_DEVICE_CONFIG;
	}


	if ( m->spi4_dev_init[current_dev->id] == 1 ) {
		spin_unlock(&m->spi4_lock[current_dev->id]) ;
		return INIT_NOT_NEEDED;
	}

	internal_info = ( port_info *) 	current_dev->intrl;
	block = internal_info->block_id;

	m->spi4_free_desc[current_dev->id] = 0;

	daughter_card_id = current_dev->id + 1;

	if ( current_dev->mmio == NULL )
		current_dev->mmio = ( phoenix_reg_t *) (phoenix_io_base + spi4_offsets[current_dev->id]);

	if ( spi4_card_detect( daughter_card_id ) == 0) {
		printf("Daughter card not present.Skipping SPI4_%d\n",current_dev->id);
		spin_unlock(&m->spi4_lock[current_dev->id]) ;
		return ERR_INVALID_OPERATION;
	}

	if ( current_dev->max_tx_frame_len > current_dev->max_rx_frame_len )
		current_dev->max_rx_frame_len = current_dev->max_tx_frame_len;

	max_frame_length = current_dev->max_rx_frame_len;
	
	vtss_nlm_init(daughter_card_id);	

	nlm_spi4_soft_reset( current_dev->mmio);

	for( i =1; i < 11; i++ )
		vtss_port_set_enable(i,0,0);

	// pad calibration 
	phoenix_write_reg ( current_dev->mmio, PAD_CALIBRATION_0, FIELD_VALUE(LVDS_TENLMNATION,1) |
		                             FIELD_VALUE(HSTL_TENLMNATION,0) |
		                             FIELD_VALUE(CAL_PRESET,0) |
		                             FIELD_VALUE(TX_EN_COUNT,1) |
		                             FIELD_VALUE(RX_EN_COUNT,1) |
		                             FIELD_VALUE(TX_N_PRESET,0) |
		                             FIELD_VALUE(TX_P_PRESET,0) |
		                             FIELD_VALUE(RX_N_PRESET,0) |
		                             FIELD_VALUE(RX_P_PRESET,0)) ;

	// configure spi4 controller
	phoenix_write_reg( current_dev->mmio, CNTRL_REG,	FIELD_VALUE(RX_TRN_RESET,1) |
					FIELD_VALUE(FIFO_SW_RESET,1) |
					FIELD_VALUE(DDL_ENABLE,1) |
					FIELD_VALUE(SPI4_RX_ENABLE,0) |
					FIELD_VALUE(SPI4_RX_SW_RESET,1) |
					FIELD_VALUE(SPI4_TX_ENABLE,0) |
					FIELD_VALUE(SPI4_TX_SW_RESET,1)) ;


	// configure spi4 thresholds for 16 symetric channels 
	phoenix_write_reg( current_dev->mmio, SPI_HNGY_0, FIELD_VALUE(EG_HNGY_THRESH_0,1) |
		                FIELD_VALUE(EG_HNGY_THRESH_1,1) |
		                FIELD_VALUE(EG_HNGY_THRESH_2,1) |
		                FIELD_VALUE(EG_HNGY_THRESH_3,1)) ;

	phoenix_write_reg( current_dev->mmio, SPI_HNGY_1, FIELD_VALUE(EG_HNGY_THRESH_4,1) |
		                FIELD_VALUE(EG_HNGY_THRESH_5,1) |
		                FIELD_VALUE(EG_HNGY_THRESH_6,1) |
		                FIELD_VALUE(EG_HNGY_THRESH_7,1)) ;
	phoenix_write_reg( current_dev->mmio, SPI_HNGY_2, FIELD_VALUE(EG_HNGY_THRESH_8,1) |
		                FIELD_VALUE(EG_HNGY_THRESH_9,1) |
		                FIELD_VALUE(EG_HNGY_THRESH_10,1) |
		                FIELD_VALUE(EG_HNGY_THRESH_11,1)) ;
	phoenix_write_reg( current_dev->mmio, SPI_HNGY_3, FIELD_VALUE(EG_HNGY_THRESH_12,1) |
		                FIELD_VALUE(EG_HNGY_THRESH_13,1) |
		                FIELD_VALUE(EG_HNGY_THRESH_14,1) |
		                FIELD_VALUE(EG_HNGY_THRESH_15,1)) ;
	phoenix_write_reg( current_dev->mmio, SPI_STRV_0, FIELD_VALUE(EG_STRV_THRESH_0,3) |
		                FIELD_VALUE(EG_STRV_THRESH_1,3) |
		                FIELD_VALUE(EG_STRV_THRESH_2,3) |
		                FIELD_VALUE(EG_STRV_THRESH_3,3)) ;
	phoenix_write_reg( current_dev->mmio, SPI_STRV_1, FIELD_VALUE(EG_STRV_THRESH_4,3) |
		                FIELD_VALUE(EG_STRV_THRESH_5,3) |
		                FIELD_VALUE(EG_STRV_THRESH_6,3) |
		                FIELD_VALUE(EG_STRV_THRESH_7,3)) ;
	phoenix_write_reg( current_dev->mmio, SPI_STRV_2, FIELD_VALUE(EG_STRV_THRESH_8,3) |
		                FIELD_VALUE(EG_STRV_THRESH_9,3) |
		                FIELD_VALUE(EG_STRV_THRESH_10,3) |
		                FIELD_VALUE(EG_STRV_THRESH_11,3)) ;
	phoenix_write_reg( current_dev->mmio, SPI_STRV_3, FIELD_VALUE(EG_STRV_THRESH_12,3) |
		                FIELD_VALUE(EG_STRV_THRESH_13,3) |
		                FIELD_VALUE(EG_STRV_THRESH_14,3) |
		                FIELD_VALUE(EG_STRV_THRESH_15,3)) ;

	// configure the DMA credit registers
	phoenix_write_reg( current_dev->mmio, DMA_CR_0, FIELD_VALUE(WR0_CREDITS,7) |
		                FIELD_VALUE(RD0_CREDITS,7) |
		                FIELD_VALUE(WR1_CREDITS,7) |
		                FIELD_VALUE(RD1_CREDITS,7) |
		                FIELD_VALUE(WR2_CREDITS,7) |
		                FIELD_VALUE(RD2_CREDITS,7) |
		                FIELD_VALUE(WR3_CREDITS,7) |
		                FIELD_VALUE(RD3_CREDITS,7) |
		                FIELD_VALUE(WR4_CREDITS,7) |
		                FIELD_VALUE(RD4_CREDITS,7)) ;

	phoenix_write_reg( current_dev->mmio, DMA_CR_1, FIELD_VALUE(WR5_CREDITS,7) |
		                FIELD_VALUE(RD5_CREDITS,7) |
		                FIELD_VALUE(WR6_CREDITS,7) |
		                FIELD_VALUE(RD6_CREDITS,7) |
		                FIELD_VALUE(WR7_CREDITS,7) |
		                FIELD_VALUE(RD7_CREDITS,7) |
		                FIELD_VALUE(WR8_CREDITS,7) |
		                FIELD_VALUE(RD8_CREDITS,7) |
		                FIELD_VALUE(WR9_CREDITS,7) |
		                FIELD_VALUE(RD9_CREDITS,7)) ;

	phoenix_write_reg( current_dev->mmio, DMA_CR_2, FIELD_VALUE(WR10_CREDITS,7) |
		                FIELD_VALUE(RD10_CREDITS,7) |
		                FIELD_VALUE(WR11_CREDITS,7) |
		                FIELD_VALUE(RD11_CREDITS,7) |
		                FIELD_VALUE(WR12_CREDITS,7) |
		                FIELD_VALUE(RD12_CREDITS,7) |
		                FIELD_VALUE(WR13_CREDITS,7) |
		                FIELD_VALUE(RD13_CREDITS,7) |
		                FIELD_VALUE(WR14_CREDITS,7) |
		                FIELD_VALUE(RD14_CREDITS,7)) ;

	phoenix_write_reg( current_dev->mmio, DMA_CR_3, FIELD_VALUE(WR15_CREDITS,7) |
		                FIELD_VALUE(RD15_CREDITS,7) |
		                FIELD_VALUE(SP_CLASS_WR_CREDITS,2) |
		                FIELD_VALUE(SP_CLASS_RD_CREDITS,2) |
		                FIELD_VALUE(REG_FR_IN_WR_CREDITS,2) |
		                FIELD_VALUE(REG_FR_IN_RD_CREDITS,2) |
		                FIELD_VALUE(FR_OUT_WR_CREDITS,2) |
		                FIELD_VALUE(FR_OUT_RD_CREDITS,2) |
		                FIELD_VALUE(P2P_CHANNEL_RD_CREDITS, 2)
		                );

	// turn on l2 alloc
	phoenix_write_reg (current_dev->mmio, L2_ALLOC_CTRL,
		FIELD_VALUE(RX_L2_ALLOCATE,current_dev->l2_alloc_rx) |
		FIELD_VALUE(TX_L2_ALLOCATE,current_dev->l2_alloc_tx));
	
	nlm_config_spill(current_dev);

	

	if ( current_dev->rx_regular_buf_size & 0x1f ) {
			/* the frame size given is not aligned to cache line( 32 bit)*/
			current_dev->rx_regular_buf_size = (current_dev->rx_regular_buf_size & ~(0x01f)) + 32;
	}

	if ( current_dev->enable_prepad == 1 )
		current_dev->rx_regular_buf_size = current_dev->rx_regular_buf_size + 32;

	phoenix_write_reg ( current_dev->mmio, TX_CONTROL, FIELD_VALUE(TX_THRESHOLD,512) |
			FIELD_VALUE(TX_ENABLE,1)) ;

	phoenix_write_reg (current_dev->mmio, DESC_PACK_CTRL,
			   FIELD_VALUE(BYTE_OFFSET, current_dev->byte_offset[0])                 |// Byte offset of packet data
			   FIELD_VALUE(PREPAD_ENABLE,current_dev->enable_prepad)                |
			   FIELD_VALUE(REGULAR_SIZE,current_dev->rx_regular_buf_size)              // Size of regular buffer
			      );

	phoenix_write_reg (current_dev->mmio,BYTE_OFFSET_0_7,
			FIELD_VALUE(BYTE_OFFSET_0, current_dev->byte_offset[0]) |
			FIELD_VALUE(BYTE_OFFSET_1, current_dev->byte_offset[1]) |
			FIELD_VALUE(BYTE_OFFSET_2, current_dev->byte_offset[2]) |
			FIELD_VALUE(BYTE_OFFSET_3, current_dev->byte_offset[3]) |
			FIELD_VALUE(BYTE_OFFSET_4, current_dev->byte_offset[4]) |
			FIELD_VALUE(BYTE_OFFSET_5, current_dev->byte_offset[5]) |
			FIELD_VALUE(BYTE_OFFSET_6, current_dev->byte_offset[6]) |
			FIELD_VALUE(BYTE_OFFSET_7, current_dev->byte_offset[7]) 
			);
			
	
	phoenix_write_reg (current_dev->mmio,BYTE_OFFSET_8_F,
			FIELD_VALUE(BYTE_OFFSET_8, current_dev->byte_offset[8]) |
			FIELD_VALUE(BYTE_OFFSET_9, current_dev->byte_offset[9]) |
			FIELD_VALUE(BYTE_OFFSET_10, current_dev->byte_offset[10]) |
			FIELD_VALUE(BYTE_OFFSET_11, current_dev->byte_offset[11]) |
			FIELD_VALUE(BYTE_OFFSET_12, current_dev->byte_offset[12]) |
			FIELD_VALUE(BYTE_OFFSET_13, current_dev->byte_offset[13]) |
			FIELD_VALUE(BYTE_OFFSET_14, current_dev->byte_offset[14]) |
			FIELD_VALUE(BYTE_OFFSET_15, current_dev->byte_offset[15]) 
			);

	 
	// setup fast message network credit registers
	for(i = 0; i < 16; i++) {
		phoenix_write_reg( current_dev->mmio,XGS_TX0_BUCKET_SIZE+i,blk_conf[block].bucket_size[i] );
        }

	phoenix_write_reg( current_dev->mmio,XGS_RFR_BUCKET_SIZE,blk_conf[block].bucket_size[17] );

	for(i = 0; i < NLM_MAX_CREDIT_COUNTER; i++) {
                phoenix_write_reg( current_dev->mmio,CC_CPU0_0 + i,blk_conf[block].credit_counter[i] );
        }

	// enable packet recieve
	phoenix_write_reg( current_dev->mmio, RX_CONTROL, FIELD_VALUE(RX_ENABLE,1) |
				FIELD_VALUE(ERROR_STATUS, current_dev->error_mask));
			
	spi4_arizona_setup(current_dev->id+1);

	nlm_setup_pde(current_dev);
	for( i = 0; i < NLM_MAX_SPI4_PORT_PER_BLOCK; i++ ) {
		if ( current_dev->phy_conf[i].auto_neg == 1) {
			megis_phoenix_init( daughter_card_id );
			auto_complete[i] = is_autoneg_complete(i);
			enable_spi4_autonegotiation(i + (NLM_MAX_SPI4_PORT_PER_BLOCK * current_dev->id));
		}
		else {
			//disable_auto_neg_spi4(current_dev,i);
			disable_spi4_autonegotiation( (i + (NLM_MAX_SPI4_PORT_PER_BLOCK * current_dev->id)),
				current_dev->phy_conf[i].speed,current_dev->phy_conf[i].duplex );
		}
	}

	for(i =0; i < NLM_MAX_SPI4_PORT_PER_BLOCK; i++) {
		if ( auto_complete[i] == 1 ) {
			int try = 0;
			vtss_phy_status_t   status;
			do {
				for( delay =0 ; delay<0xfffff; delay++);
				vtss_phy_status_get((i+1) , &status);
				try++;
				if ( try == 1000)
					break;
			} while ( status.autoneg_complete == 0);
		}
	}
	
	for( i =0; i < NLM_MAX_SPI4_PORT_PER_BLOCK; i++ ) { 
		if ( current_dev->phy_conf[i].auto_neg == 1)
			check_spi4_link_status(current_dev, i + (NLM_MAX_SPI4_PORT_PER_BLOCK * current_dev->id));
			
	}

	SET_PORT_INIT(internal_info->state);
	m->spi4_dev_init[current_dev->id] = 1;
	spin_unlock(&m->spi4_lock[current_dev->id]) ;
			
	return 0;
}

int nlm_spi4_port_close( device_conf * current_dev, int port)
{

	port_info * internal_info = NULL;
	struct spi4_shm_vars *m;


	if (!xlr_board_atx_i()) {
		printf("SPI4 interface is not supported on this board");
		return ERR_DEVICE_NOT_SUPORTTED;
	}

	
	if ( current_dev == NULL ) {
		return ERR_INVALID_OPERATION;
        }
	if((m = current_dev->shm) == NULL)
		return ERR_NO_MEM;


	spin_lock(&m->spi4_lock[current_dev->id]) ;

	if ( (port > 9) ||  (port < 0 )){
		spin_unlock(&m->spi4_lock[current_dev->id]) ;
		return ERR_INVALID_OPERATION;
	}

	port++;

	internal_info =  ( port_info *)current_dev->intrl;

	if ( internal_info == NULL ) {
		spin_unlock(&m->spi4_lock[current_dev->id]) ;
                return ERR_INVALID_OPERATION;
        }


	megis_phoenix_init(current_dev->id + 1);

	vtss_port_set_enable(port,0,0);

	spin_unlock(&m->spi4_lock[current_dev->id]) ;

	return 1;

}

int nlm_spi4_inject_free_desc( device_conf * current_dev)
{
	int desc_bucket = MSGRNG_STNID_XMAC1RFR;
	int i;
	struct msgrng_msg free_msg;
	int sent_cnt =0;
	port_info * internal = ( port_info *)current_dev->intrl;
	int block = internal->block_id;
	int num_desc = blk_conf[block].max_frin_desc;
	void *pbuf = 0;
	void * (*alloc_rxbuf)(void *arg) = blk_conf[block].pkt_mem->alloc_rx_buf;
	void * arg =  blk_conf[block].pkt_mem->arg;
	struct spi4_shm_vars *m;

	if((m = current_dev->shm) == NULL)
		return ERR_NO_MEM;

	
	if ( m->spi4_free_desc[current_dev->id] == 1 )
		return -1;

	if ( current_dev->id == 0 )
		desc_bucket  = MSGRNG_STNID_XMAC0RFR;

	printf("Spi4 inject %d free descriptors, stid=%d\n", num_desc, desc_bucket);
	
	free_msg.msg1 = free_msg.msg2 = free_msg.msg3 = 0;
	for (i = 0; i < num_desc; i++) {
		pbuf =  alloc_rxbuf(arg);
		if ( pbuf  == NULL )
			break;
		else
			sent_cnt++;
		(void )mac_make_desc_rfr(&free_msg, 0, 0, virt_to_phys(pbuf));
		while(message_send(1, 2, desc_bucket, &free_msg));
	}
	printf("Free packets Request=%d, Sent=%d\n",num_desc,sent_cnt);
	m->spi4_free_desc[current_dev->id] = 1;
	return num_desc;

}

int nlm_spi4_port_open( device_conf * current_dev, int port)
{
	port_info * internal_info = NULL;
	uint32_t crc_add = 0;
	struct spi4_shm_vars *m;


	if (!xlr_board_atx_i()){
		printf("SPI4 interface is not supported on this board");
		return ERR_DEVICE_NOT_SUPORTTED;
	}

	
	if ( current_dev == NULL ) {
		return ERR_INVALID_OPERATION;
        }
	if((m = current_dev->shm) == NULL)
		return ERR_NO_MEM;


	spin_lock(&m->spi4_lock[current_dev->id]) ;

	
	if ( (port > 9) ||  (port < 0 )){
		spin_unlock(&m->spi4_lock[current_dev->id]) ;
		return ERR_INVALID_OPERATION;
	}
	port++;

	internal_info =  ( port_info *)current_dev->intrl;

	if ( internal_info == NULL ) {
		printf("internal_info NULL\n");
		spin_unlock(&m->spi4_lock[current_dev->id]) ;
                return ERR_INVALID_OPERATION;
        }

	megis_phoenix_init(current_dev->id + 1);


	if ( !PORT_OPEN(internal_info->state)) {
		nlm_spi4_inject_free_desc( current_dev);

	SET_PORT_OPEN(internal_info->state);
	}

	/* Enable crc append option */
	crc_add = vtss_io_read(M2_BLK_MACS, port, M2_TRI_DENORM);
	crc_add &= ~(1<<5); // clear 5th bit crc_upd
	crc_add |= 1<<4;  // set 4th bit crc_add
	vtss_io_write(M2_BLK_MACS, port, M2_TRI_DENORM,crc_add);

	
	vtss_port_set_enable(port,1,1);
	spin_unlock(&m->spi4_lock[current_dev->id]) ;
	return 1;


}

