/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/
#include <stdio.h>
#include <stdlib.h>

#include "spill.h"
#include "mm.h"
#include "gmac.h"
#include "atomic.h"
#include "mac.h"
#include "packet.h"
#include "shmvars.h"

// spills for gmac
struct fr_desc     *gmac0_frin_spill;
struct fr_desc     *gmac0_frout_spill;
struct size_2_desc *gmac0_rx_spill0;
struct size_2_desc *gmac0_rx_spill1;
struct size_2_desc *gmac0_rx_spill2;
struct size_2_desc *gmac0_rx_spill3;

struct fr_desc     *gmac1_frin_spill;
struct fr_desc     *gmac1_frout_spill;
struct size_2_desc *gmac1_rx_spill0;
struct size_2_desc *gmac1_rx_spill1;
struct size_2_desc *gmac1_rx_spill2;
struct size_2_desc *gmac1_rx_spill3;
extern struct block_config  blk_conf[4];


// spills for xgs0 (used by both xgmac & spi4)
struct fr_desc    *xgs0_frin_spill;
struct fr_desc    *xgs0_frout_spill;
struct size_2_desc *xgs0_rx_spill0;
struct size_2_desc *xgs0_rx_spill1;
struct size_2_desc *xgs0_rx_spill2;
struct size_2_desc *xgs0_rx_spill3;

// spills for xgs1 (used by both xgmac & spi4)
struct fr_desc    *xgs1_frin_spill;
struct fr_desc    *xgs1_frout_spill;
struct size_2_desc *xgs1_rx_spill0;
struct size_2_desc *xgs1_rx_spill1;
struct size_2_desc *xgs1_rx_spill2;
struct size_2_desc *xgs1_rx_spill3;

extern void *cacheline_aligned_malloc(int size);

static void *spill_alloc(int size)
{
	return cacheline_aligned_malloc(size);
}

static void nlm_spill_configure ( phoenix_reg_t * mmio, int reg_start_0, 
				int reg_start_1,int reg_size, int size) {

	uint64_t phys_addr;
	void * virt_spill_mem = spill_alloc( size);

	if ( virt_spill_mem ==  NULL ){
		printf("spill_alloc FAILED!!\n");
		return;
	}

	phys_addr = virt_to_phys( virt_spill_mem);


	phoenix_write_reg( mmio,reg_start_0,(phys_addr >> 5) & 0xffffffff);
	phoenix_write_reg( mmio,reg_start_1, (phys_addr >> 37) & 0x07);
	phoenix_write_reg( mmio, reg_size, size);

}

static int check_on_chip_free_in( int user_entry)
{
	int on_chip_free_in = 128;
	if ( user_entry <= on_chip_free_in ) {
		on_chip_free_in = on_chip_free_in - user_entry;
		return 0;
	}
	return user_entry;

}
static int check_on_chip_class_fifo(int user_entry)
{
	int on_chip_class_fifo = 128;
	if ( user_entry <= on_chip_class_fifo ) {
		on_chip_class_fifo  = on_chip_class_fifo - user_entry;
		return 0;
	}
	return user_entry;
}

static int check_on_chip_frout( int user_entry)
{
	int on_chip_frout = 128;
	if ( user_entry <=  on_chip_frout ) {
		on_chip_frout = on_chip_frout - user_entry;
		return 0;
	}
	return user_entry;
}
	

void nlm_config_spill(device_conf  * current_dev )
{
	int frin_count = 0;
	int max_tx_desc = 0;
	int class_count = 0;
	port_info * internal = ( port_info *)current_dev->intrl;
	int block = internal->block_id;
	struct netblk_shm_vars *m;

	if((m = nlm_netblk_shm_init()) == NULL)
		return;

	if ( current_dev->type == TYPE_NLM_GMAC ) {
		frin_count  = blk_conf[block].max_frin_desc << 2;
		class_count = blk_conf[block].max_frin_desc << 2; 
		max_tx_desc = blk_conf[block].max_tx_desc << 2;
		max_tx_desc = max_tx_desc * blk_conf[block].tx_desc_size;
	}
	else {
		frin_count  = blk_conf[block].max_frin_desc ;
		class_count = blk_conf[block].max_frin_desc ; 
		max_tx_desc = blk_conf[block].max_tx_desc ;
		max_tx_desc = max_tx_desc * blk_conf[block].tx_desc_size;
	}

	if ( !atomic_cmpset_int(&m->nlm_spill_init[block], 0, 1))
		return ;

	

	frin_count = check_on_chip_free_in(frin_count);
	class_count = check_on_chip_class_fifo( class_count );
	max_tx_desc = check_on_chip_frout( max_tx_desc);


	if ( blk_conf[block].max_frin_desc ) {
		nlm_spill_configure( current_dev->mmio, REG_FRIN_SPILL_MEM_START_0,
			REG_FRIN_SPILL_MEM_START_1,REG_FRIN_SPILL_MEM_SIZE,
			(  frin_count * sizeof(struct fr_desc)));

		nlm_spill_configure( current_dev->mmio, CLASS0_SPILL_MEM_START_0,
			CLASS0_SPILL_MEM_START_1, CLASS0_SPILL_MEM_SIZE,
			(  class_count * sizeof(struct fr_desc) ));
		nlm_spill_configure( current_dev->mmio, CLASS1_SPILL_MEM_START_0,
			CLASS1_SPILL_MEM_START_1,CLASS1_SPILL_MEM_SIZE,
			(  class_count * sizeof(struct fr_desc) ));
		
		nlm_spill_configure( current_dev->mmio, CLASS2_SPILL_MEM_START_0,
			CLASS2_SPILL_MEM_START_1, CLASS2_SPILL_MEM_SIZE,
			(  class_count * sizeof(struct fr_desc) ));

		nlm_spill_configure( current_dev->mmio, CLASS3_SPILL_MEM_START_0,
			CLASS3_SPILL_MEM_START_1, CLASS3_SPILL_MEM_SIZE,
			(  class_count * sizeof(struct fr_desc) ));
	}

	 if ( blk_conf[block].max_tx_desc) {
		nlm_spill_configure( current_dev->mmio, FROUT_SPILL_MEM_START_0,
			FROUT_SPILL_MEM_START_1, FROUT_SPILL_MEM_SIZE,
			(max_tx_desc * sizeof( struct size_1_desc)));
			
		
	}
	
	phoenix_write_reg( current_dev->mmio, REG_FRIN_1_SPILL_MEM_START_0, 0);
	phoenix_write_reg( current_dev->mmio, REG_FRIN_1_SPILL_MEM_START_1, 0);
	phoenix_write_reg( current_dev->mmio, REG_FRIN_1_SPILL_MEM_SIZE, 0);

}

	

void nlm_spill_setup(struct net_config_struct *net_config) 
{
	int size = 0;  
	if(net_config->gmac0_frin_spill){
		size = sizeof(struct fr_desc) * net_config->gmac0_frin_spill_entries;
		gmac0_frin_spill = (struct fr_desc*) spill_alloc(size);
	}
	if(net_config->gmac1_frin_spill){
		size = sizeof(struct fr_desc) * net_config->gmac1_frin_spill_entries;
		gmac1_frin_spill = (struct fr_desc*) spill_alloc(size);
	}
	if(net_config->xgs0_frin_spill){
		size = sizeof(struct fr_desc) * net_config->xgs0_frin_spill_entries;
		xgs0_frin_spill = (struct fr_desc*) spill_alloc(size);
	}
	if(net_config->xgs1_frin_spill){
		size = sizeof(struct fr_desc) * net_config->xgs1_frin_spill_entries;
		xgs1_frin_spill = (struct fr_desc*) spill_alloc(size); 
	}
	 
	// setup free out spills
	if(net_config->gmac0_frout_spill){
		size = sizeof(struct fr_desc) * net_config->gmac0_frout_spill_entries;
		gmac0_frout_spill = (struct fr_desc*) spill_alloc(size);
	}
	if(net_config->gmac1_frout_spill){
		size = sizeof(struct fr_desc) * net_config->gmac1_frout_spill_entries;
		gmac1_frout_spill = (struct fr_desc*) spill_alloc(size);
	}
	if(net_config->xgs0_frout_spill){
		size = sizeof(struct fr_desc) * net_config->xgs0_frout_spill_entries;
		xgs0_frout_spill = (struct fr_desc*) spill_alloc(size);
	}
	if(net_config->xgs1_frout_spill){
		size = sizeof(struct fr_desc) * net_config->xgs1_frout_spill_entries;
		xgs1_frout_spill = (struct fr_desc*) spill_alloc(size);
	}
	   
	// setup rx spill entries
	if(net_config->gmac0_rx_spill0){
		size = sizeof(struct size_2_desc) * net_config->gmac0_rx_spill0_entries;
		gmac0_rx_spill0 = (struct size_2_desc*) spill_alloc(size); 
	}
	if(net_config->gmac0_rx_spill1){
		size = sizeof(struct size_2_desc) * net_config->gmac0_rx_spill1_entries;
		gmac0_rx_spill1 = (struct size_2_desc*) spill_alloc(size);
	}
	if(net_config->gmac0_rx_spill2){
		size = sizeof(struct size_2_desc) * net_config->gmac0_rx_spill2_entries;
		gmac0_rx_spill2 = (struct size_2_desc*) spill_alloc(size);
	}
	if(net_config->gmac0_rx_spill3){
		size = sizeof(struct size_2_desc) * net_config->gmac0_rx_spill3_entries;
		gmac0_rx_spill3 = (struct size_2_desc*) spill_alloc(size);
	}
	if(net_config->gmac1_rx_spill0){
		size = sizeof(struct size_2_desc) * net_config->gmac1_rx_spill0_entries;
		gmac1_rx_spill0 = (struct size_2_desc*) spill_alloc(size); 
	}
	if(net_config->gmac1_rx_spill1){
		size = sizeof(struct size_2_desc) * net_config->gmac1_rx_spill1_entries;
		gmac1_rx_spill1 = (struct size_2_desc*) spill_alloc(size);
	}
	if(net_config->gmac1_rx_spill2){
		size = sizeof(struct size_2_desc) * net_config->gmac1_rx_spill2_entries;
		gmac1_rx_spill2 = (struct size_2_desc*) spill_alloc(size);
	}
	if(net_config->gmac1_rx_spill3){
		size = sizeof(struct size_2_desc) * net_config->gmac1_rx_spill3_entries;
		gmac1_rx_spill3 = (struct size_2_desc*) spill_alloc(size);
	}


	if(net_config->xgs0_rx_spill0){
		size = sizeof(struct size_2_desc) * net_config->xgs0_rx_spill0_entries;
		xgs0_rx_spill0 = (struct size_2_desc*) spill_alloc(size);
	}
	if(net_config->xgs0_rx_spill1){
		size = sizeof(struct size_2_desc) * net_config->xgs0_rx_spill1_entries;
		xgs0_rx_spill1 = (struct size_2_desc*) spill_alloc(size);
	}
	if(net_config->xgs0_rx_spill2){
		size = sizeof(struct size_2_desc) * net_config->xgs0_rx_spill2_entries;
		xgs0_rx_spill2 = (struct size_2_desc*) spill_alloc(size);
	}
	if(net_config->xgs0_rx_spill3){
		size = sizeof(struct size_2_desc) * net_config->xgs0_rx_spill3_entries;
		xgs0_rx_spill3 = (struct size_2_desc*) spill_alloc(size);
	}

	if(net_config->xgs1_rx_spill0){
		size = sizeof(struct size_2_desc) * net_config->xgs1_rx_spill0_entries;
		xgs1_rx_spill0 = (struct size_2_desc*) spill_alloc(size);
	}
	if(net_config->xgs1_rx_spill1){
		size = sizeof(struct size_2_desc) * net_config->xgs1_rx_spill1_entries;
		xgs1_rx_spill1 = (struct size_2_desc*) spill_alloc(size);
	}
	if(net_config->xgs1_rx_spill2){
		size = sizeof(struct size_2_desc) * net_config->xgs1_rx_spill2_entries;
		xgs1_rx_spill2 = (struct size_2_desc*) spill_alloc(size);
	}
	if(net_config->xgs1_rx_spill3){
		size = sizeof(struct size_2_desc) * net_config->xgs1_rx_spill3_entries;
		xgs1_rx_spill3 = (struct size_2_desc*) spill_alloc(size);
	}
}
