/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

/*****************************************************************************
 gdbshared.h - shared memory details


 MODIFICATION HISTORY
 --------------------
 v1.0,26dec05,Suresh DS   written
 v1.1,15mar06,Suresh DS   support for shared code segment (kuseg & kseg0)


 DESCRIPTION
 -----------
 This file assigns the shared memory segment referred by standard OS (currently
 VxWorks/Linux/NETOS) in vCPU 0 and NETOS in other vCPUs, and declares the 
 related macros; The shared memory segment starts at 0x80040000
******************************************************************************/

#ifndef __GDBSHARED_H__
#define __GDBSHARED_H__

/* --- macros for multi session support (with UART1) --- */
#ifndef TOTAL_VCPU
#define TOTAL_VCPU 32
#endif
#ifndef CACHE_LINE_SIZE
#define CACHE_LINE_SIZE 32
#endif
#define OS_NETOS_TX_BUF_SIZE    1600 * 8
#define OS_NETOS_RX_BUF_SIZE    256 * 8
#define OS_NETOS_TX_PKTLEN_NUMBER 32

/* kseg0 address for the shared memory starting point */
#define OS_NETOS_SHARED_MEM_START 0xffffffff80040000ULL 
#define OS_NETOS_RX_BUF_BASE OS_NETOS_SHARED_MEM_START
#define OS_NETOS_TX_BUF_BASE  (OS_NETOS_RX_BUF_BASE + (OS_NETOS_RX_BUF_SIZE * TOTAL_VCPU))
#define OS_NETOS_CPU_ONLINE_MAP_LOCK  (OS_NETOS_TX_BUF_BASE + (OS_NETOS_TX_BUF_SIZE * TOTAL_VCPU))
#define OS_NETOS_CPU_ONLINE_MAP_LOCK_INIT_DONE  (OS_NETOS_CPU_ONLINE_MAP_LOCK + CACHE_LINE_SIZE)
#define OS_NETOS_CPU_ONLINE_MAP (OS_NETOS_CPU_ONLINE_MAP_LOCK_INIT_DONE + CACHE_LINE_SIZE)
#define OS_NETOS_RXFIFO    (OS_NETOS_CPU_ONLINE_MAP + CACHE_LINE_SIZE)
#define OS_NETOS_TXFIFO    (OS_NETOS_RXFIFO + 12 * CACHE_LINE_SIZE)
#define OS_NETOS_TX_PKTLEN_FIFO    (OS_NETOS_TXFIFO + 12 * CACHE_LINE_SIZE)
#define OS_NETOS_TX_PKTLEN         (OS_NETOS_TX_PKTLEN_FIFO + 12 * CACHE_LINE_SIZE)
#define OS_NETOS_TBH_LOCK     (OS_NETOS_TX_PKTLEN + 4 * OS_NETOS_TX_PKTLEN_NUMBER * CACHE_LINE_SIZE)
#define OS_NETOS_CONTINUEALL_MASK (OS_NETOS_TBH_LOCK + CACHE_LINE_SIZE)
#define OS_NETOS_CPKT_NOT_PROCESSED (OS_NETOS_CONTINUEALL_MASK + CACHE_LINE_SIZE)
/* TODO: for future */
/*#define OS_NETOS_GDB_ONLINE_MAP (OS_NETOS_CPU_ONLINE_MAP + CACHE_LINE_SIZE)*/
/*#define OS_NETOS_GDB_ONLINE_MAP_LOCK (OS_NETOS_GDB_ONLINE_MAP + CACHE_LINE_SIZE)*/

#define LOCK_INIT_DONE 0x900ddeed

#ifdef NETOS_SLAVE_PRINT
#define OS_NETOS_PRINT_BASE (OS_NETOS_CPKT_NOT_PROCESSED + CACHE_LINE_SIZE)
#define OS_NETOS_PRINT_VCPU 32
#define OS_NETOS_PRINT_BUF_SIZE 200
#endif // #ifdef NETOS_SLAVE_PRINT

#endif // #ifndef __GDBSHARED_H__
