/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

/***************************************************************************
 gdbstub.h - the header file for gdb stub

 MODIFICATION HISTORY
 --------------------
 v1.0,19sep05,Suresh DS  written (with contributions from Prabhath/Harsha)
 v1.1,26dec05,Suresh DS  included "gdbshared.h" 
 v1.2,15mar06,Suresh DS  support for integration with VxWorks in shared code 
                         segment (kseg0) mode, support for "breakall" feature
 v1.3,05jul06,Suresh DS	 allocation of KSEG0 memory for FIFO support,
                         increase of some existing KSEG0 blocks
 v1.4,28jan08,Suresh DS  sign extension, indentation, cleanup
 v1.5,16apr08,Suresh DS  KUSEG_MODE,KSEG0_MODE redefinitions removed,
                         MASTER_VCPU changed to TUNNELER_VCPU
 v1.6,09feb09,Suresh DS  infrastructure changes

 DESCRIPTION
 -----------
 This file includes the various header files in NETOS needed by gdb
 stub and also defines the common macros used in both serial and network 
 communication modes. 
*******************************************************************************/

#ifndef __GDBSTUB_H__
#define __GDBSTUB_H__

#ifndef TOTAL_VCPU
#define TOTAL_VCPU 32
#endif

#include <stdio.h>
#include <stdlib.h>

#include "string.h"
#include "hexutil.h"
#include "mipsregs.h"
#include "spinlock.h"
#include "cache.h"
#include "asmmacros.h"
#include "mips-exts.h"
#include "types.h"
#include "iomap.h"
#include "mips-opcode.h"
#include "gdbserial.h"
#include "gdbshared.h"
#include "irq.h"
#include "compiler.h"
#include "ctype.h"
#include "mm.h"
#include "circular_fifo.h"

#define NETOS_Index_Invalidate_I Index_Invalidate_I

/* This is referred in serial mode */
#define TUNNELER_VCPU 0

/* No. of attempts by stub to put the packet to the host */
#define MAX_GDB_PUTPACKET_TRIES 10

/* No. of clusters in pool memory */
#define GDB_NUM_CLUSTERS 1024

/* Size of each cluster in pool memory */
#define GDB_CLUSTER_SIZE 8

/* Enables output messages (for debugging) */
/* Used only in development phase */
/*#define GDB_DEBUG_MSG*/
/*#define OS_DEBUG */

/*
 * Instructions with specal significance to debuggers.
 */
				     /*#define BREAK_INSTR 0x0000000d *//* instruction code for break */
#define BREAK_INSTR 0x000001cd	/* instruction code for break */
#define NOP_INSTR 0x00000000	/* instruction code for no-op */
#define INITIAL_BREAK_INSTR 0x0000008d	/* code for initial break */

/* Macros used in monitor command processing */
#define TOTAL_BREAKPOINTS 64

/* Run-time macros and variable for serial/network modes */
enum {
    GDB_SERIAL,
    GDB_NETWORK
};

/* Some global variables */
enum {
    GDB_MAP,
    CPU_MAP
};

enum {
    ADD_MAP_ENTRY,
    REMOVE_MAP_ENTRY
};

enum {
    STOPPED_BY_IC,
    STOPPED_BY_IB,
    STOPPED_BY_BP,
    STOPPED_BY_SS,
    STOPPED_BY_CTRLC,
    STOPPED_BY_BA,
    STOPPED_BY_SM,
    STOPPED_BY_CA,
    RUNNING
};

enum {
    NORMAL_BKPT,
    SINGLESTEP_BKPT,
    INITIAL_BKPT,
    TBH_BKPT
};

/* Normal breakpoint */
struct gdb_bpnode {
    volatile uint64_t *addr; /* address at which to break */
    volatile uint32_t instr; /* original instruction at addr */
    volatile uint8_t valid;
};

/* Single step breakpoint */
struct gdb_ssnode {
    volatile uint64_t *targetAddr; /* address at which to break */
    volatile uint32_t savedInstr; /* original instruction at targetAddr */
};

/* TBH breakpoint */
struct gdb_tbhnode {
    volatile uint64_t *addr; /* address which contains some other's break */
    volatile uint32_t instr; /* original instruction at addr */
    volatile uint64_t *targetAddr; /* single-step next address at which to break */
    volatile uint32_t savedInstr; /* original instruction at single-step's next address */
};

struct byte_u {
	volatile unsigned char nibble0:4;
	volatile unsigned char nibble1:4;
};

union mem_u {
	volatile struct byte_u byte[8];
	volatile uint8_t c_arr[8];
	volatile uint16_t s_arr[4];
	volatile uint32_t i_arr[2];
	volatile uint64_t val64;
	volatile uint32_t val32;
	volatile uint16_t val16;
	volatile uint8_t val8;
};

#endif /* #ifndef __GDBSTUB_H__ */
