/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _CACHE_H
#define _CACHE_H

#include "asmmacros.h"
#include "iomap.h"

#define CACHELINE_SIZE 32

/**** L1 MACROS *******/

#define L1_WAYS 8
#define L1_INDEX 128 
#define L1_INDEX_LOAD_TAG 0x5
#define L1_INDEX_INVALIDATE 0x1

/**** L2 Registers ****/

/* register defines assume this mapping: */
/* [39:20]  Phoenix_io_base              */
/* [19:12]  L2_config_space              */
/* [11:8]   L2 Register Set              */
/* [7:5]    L2 Bank                      */
/* [4:2]    L2 Register Number           */

#define L2_CONFIG_SPACE        0x1b000

#define L2_CFG_BANK_OFS  5

#define L2_BANK_SIZE           0x20

#define L2_BANK_0              0x00  
#define L2_BANK_1              0x20 
#define L2_BANK_2              0x40
#define L2_BANK_3              0x60  
#define L2_BANK_4              0x80
#define L2_BANK_5              0xa0
#define L2_BANK_6              0xc0
#define L2_BANK_7              0xe0

/* register set 0 registers: */   

#define L2_DEFEATURE_REG       0x0
#define L2_CMD_REG             0x4
#define L2_DISABLE_WLB_ENTRIES 0x8
#define L2_DISABLE_DOB_ENTRIES 0xc
#define L2_TAG_VLD_ARRAY_REG   0x14
#define L2_DATA_ECC_REG        0x18
#define L2_PHNX_IO_SHDOW_REG   0x1c

/* register set 1 registers: */

#define L2_DATA_REG_7   0x100  /* Bits [255:224] */
#define L2_DATA_REG_6   0x104  /* Bits [223:192] */
#define L2_DATA_REG_5   0x108  /* Bits [191:160] */
#define L2_DATA_REG_4   0x10c  /* Bits [159:128] */
#define L2_DATA_REG_3   0x110  /* Bits [127:96]  */
#define L2_DATA_REG_2   0x114  /* Bits [95:64]   */
#define L2_DATA_REG_1   0x118  /* Bits [63:32]   */
#define L2_DATA_REG_0   0x11c  /* Bits [31:0]    */
 
/* register set 2 registers: */

#define L2_CORR_ECC_LOG_REG   0x200
#define L2_UNCORR_ECC_LOG_REG 0x204
#define L2_PERF_COUNTER_0     0x208
#define L2_PERF_COUNTER_1     0x20c
#define L2_DIS_WAYS_REG_0     0x210
#define L2_DIS_WAYS_REG_1     0x214
#define L2_DIS_WAYS_REG_2     0x218

/* ****** L2 Register Definitions ***** */

/* bits definitions of L2_DEFEATURE_REG: */

#define L2_DIS_VLD_ARR_PARITY 0x001
#define L2_DIS_TAG_RAM_ECC    0x002
#define L2_DIS_DATA_RAM_ECC   0x004
#define L2_DIS_DATA_POISON    0x008
#define L2_INJ_VLD_ARR_PERR   0x010
#define L2_INJ_TAG_RAM_SBE    0x020
#define L2_INJ_TAG_RAM_DBE    0x040
#define L2_INJ_DATA_RAM_SBE   0x080
#define L2_INJ_DATA_RAM_DBE   0x100
#define L2_ALLOW_DROP_WLBFULL 0x200
#define L2_BANK_ENABLE        0x400
#define L2_GLB_DIS_WAYS_OFS      24

/* bit definitions of L2_CMD_REG: */
/* mappings:                      */
/*
   [3:0]    CMD OP FIELD:
   -----
   [18:16]  VBITS:  Valid Bits [Lock,Dirty,Clean]
   [21:19]  WAY:    Way Bits
   [31:22]  SET:    Set Bits   [17:8]
   -----
   [31:28]  PBITS:  Performance Counter Select Bits
*/

/* Decode CMD OP FIELD: [3:0]: */
#define L2_READ_VLD_ARR   0x0       /* [SET]           */
#define L2_WRITE_VLD_ARR  0x1       /* [SET,WAY,VBITS] */
#define L2_READ_TAG_RAM   0x2       /* [SET,WAY]       */
#define L2_WRITE_TAG_RAM  0x3       /* [SET,WAY]       */
#define L2_READ_DATA_RAM  0x4       /* [SET,WAY]       */
#define L2_WRITE_DATA_RAM 0x5       /* [SET,WAY]       */
#define L2_PERF_CNT_0_SEL 0x6       /* [PBits]         */
#define L2_PERF_CNT_1_SEL 0x7       /* [PBits]         */
#define L2_EVICT_LINE     0x8       /* [SET,WAY]       */
#define L2_REL_UC_ERR_LOG_LOCK 0x9

#define L2_CMD_OP_FLD_VBIT_LOCK_MASK  0x4
#define L2_CMD_OP_FLD_VBIT_DTY_MASK   0x2
#define L2_CMD_OP_FLD_VBIT_CLN_MASK   0x1
#define L2_CMD_OP_FLD_VBITS_OFS        16
#define L2_CMD_OP_FLD_WAY_OFS          19
#define L2_CMD_OP_FLD_SET_OFS          22
#define L2_CMD_OP_FLD_PBITS_OFS        28

/* definitions for L2_TAG_VLD_ARRAY_REG */
#define L2_TAG_VLD_ARRAY_WAY0_OFS     0
#define L2_TAG_VLD_ARRAY_WAY1_OFS     3
#define L2_TAG_VLD_ARRAY_WAY2_OFS     6
#define L2_TAG_VLD_ARRAY_WAY3_OFS     9
#define L2_TAG_VLD_ARRAY_WAY4_OFS    12
#define L2_TAG_VLD_ARRAY_WAY5_OFS    15
#define L2_TAG_VLD_ARRAY_WAY6_OFS    18
#define L2_TAG_VLD_ARRAY_WAY7_OFS    21
#define L2_TAG_VLD_ARRAY_WAY_MASK   0x7
#define L2_TAG_VLD_ARRAY_LRU_OFS     24
#define L2_TAG_VLD_ARRAY_LRU_MASK  0x2f
#define L2_TAG_VLD_ARRAY_PAR_OFS     31
#define L2_TAG_VLD_ARRAY_PAR_MASK   0x1

#define L2_TAG_VLD_WR_TAG_ADR_OFS    10

/*
 * L2 Performance Counter Events
 */

#define L2_PERF_EVENT_VLD_OPS           0x0
#define L2_PERF_EVENT_CPU_OPS           0x1
#define L2_PERF_EVENT_IO_OPS      		0x2
#define L2_PERF_EVENT_HITS        		0x3
#define L2_PERF_EVENT_SH_HITS     		0x4
#define L2_PERF_EVENT_MOD_HITS    		0x5
#define L2_PERF_EVENT_RDX_HITS    		0x6
#define L2_PERF_EVENT_UPG_HITS    		0x7
#define L2_PERF_EVENT_WR_HITS   		0x8
#define L2_PERF_EVENT_WR_ALLOCS   		0x9
#define L2_PERF_EVENT_EV_ALLOCS         0xa
#define L2_PERF_EVENT_RETRIES   		0xb
#define L2_PERF_EVENT_GUARD_RETS  		0xc
#define L2_PERF_EVENT_WLB_FULL_RETS 	0xd
#define L2_PERF_EVENT_DOB_FULL_RETS     0xe
#define L2_PERF_EVENT_ACCFIFO_FULL_RET  0xf

#define CACHE_ENABLE   1
#define CACHE_DISABLE  0

/*
 * Cache Operations available on all MIPS processors with R4000-style caches
 */

#define Index_Invalidate_I      0x00
#define Index_Writeback_Inv_D   0x01
#define Index_Load_Tag_I	    0x04
#define Index_Load_Tag_D	    0x05
#define Index_Store_Tag_I	    0x08
#define Index_Store_Tag_D	    0x09
#define Hit_Invalidate_I	    0x10
#define Hit_Invalidate_D	    0x11
#define Hit_Writeback_Inv_D	    0x15

#define Pref_Load		        0
#define Pref_Store		        1
#define Pref_Load_Streamed	    Pref_Load
#define Pref_Store_Streamed 	Pref_Store
#define Pref_Load_Retained	    Pref_Load
#define Pref_Store_Retained 	Pref_Store
#define Pref_WB_Invalidate	    25
#define Pref_Prep_For_Store	    30

// ------------------------------------
// L2 performace counters functionality
// ------------------------------------

#define L2_NUM_BANKS 8

#ifndef __ASSEMBLY__

#define L2_BANK_REG(bank, reg) ((((bank) << 5) | (reg)) >> 2)

static __inline__ void perf_ctr_l2_start(int event, int ctr)
{
	phoenix_reg_t *mmio = phoenix_io_mmio(PHOENIX_IO_L2_OFFSET);
	int i, reg;
	uint32_t value = (event << 28) | (ctr << 0);

	for(i = 0; i < L2_NUM_BANKS; i++) {
		reg = L2_BANK_REG(i, L2_CMD_REG);
		phoenix_write_reg(mmio, reg, value); 
	}
}

static __inline__ uint64_t perf_ctr_l2_read(int reg_ctr)
{
	phoenix_reg_t *mmio = phoenix_io_mmio(PHOENIX_IO_L2_OFFSET);
	int i, reg;
	uint64_t total = 0;

	for(i = 0; i < L2_NUM_BANKS; i++) {
		reg = L2_BANK_REG(i, reg_ctr);
		total += phoenix_read_reg(mmio, reg);
	}

	return total;
}

static __inline__ void perf_ctr_l2_write(int reg_ctr, unsigned long value)
{
	phoenix_reg_t *mmio = phoenix_io_mmio(PHOENIX_IO_L2_OFFSET);
	int i, reg;

	for(i = 0; i < L2_NUM_BANKS; i++) {
		reg = L2_BANK_REG(i, reg_ctr);
		phoenix_write_reg(mmio, reg, value);
	}
}

#endif

#endif
