/***********************************************************************

Copyright 2003-2006 Raza Microelectronics, Inc.(RMI). All rights
reserved.

Use of this software shall be governed in all respects by the terms and
conditions of the RMI software license agreement ("SLA") that was
accepted by the user as a condition to opening the attached files. 

Without limiting the foregoing, use of this software in source and
binary code forms, with or without modification, and subject to all
other SLA terms and conditions, is permitted.

Any transfer or redistribution of the source code, with or without
modification, IS PROHIBITED, unless specifically allowed by the SLA.

Any transfer or redistribution of the binary code, with or without
modification, is permitted, provided that the following condition is
met:

Redistributions in binary form must reproduce the above copyright
notice, the SLA, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the distribution:


THIS SOFTWARE IS PROVIDED BY Raza Microelectronics, Inc. `AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL RMI OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

*****************************#RMI_3#***********************************/

/* contig_alloc.h -- header file for memory routines.  */

#ifndef _CONTIG_ALLOC_H_
#define _CONTIG_ALLOC_H_

#ifdef NLM_LINUXU
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include "spinlock.h"

#define NAV             128   /* number of bins */
#define DEFAULT_TRIM_THRESHOLD (128L * 1024L)
#define DEFAULT_TOP_PAD 0
#define MBIN_SZ 4096

#define     _PTR             void *
#define     _VOID	     void
#ifndef _PARAMS
#define _PARAMS(paramlist)  paramlist
#endif
struct _reent {
	int _errno;
};
extern struct _reent *reentptr;
#define _REENT reentptr
#define _stderr_r(x) stderr
# define _REENT_SMALL_CHECK_INIT(ptr) /* nothing */
#define       _DEFUN(name, arglist, args)     name(args)
#define       _AND            ,
//#define       ptrdiff_t   int
/* linux userspace cannot access cop0 */
#define SPIN_LOCK_IRQ_SAVE(x, y) { spin_lock(x); y = 0; }
#define SPIN_UNLOCK_IRQ_RESTORE(x, y) spin_unlock(x)
#define _POINTER_INT long


/* SVID2/XPG mallinfo structure */
struct mallinfo {
	int arena;    /* total space allocated from system */
	int ordblks;  /* number of non-inuse chunks */
	int smblks;   /* unused -- always zero */
	int hblks;    /* number of mmapped regions */
	int hblkhd;   /* total space in mmapped regions */
	int usmblks;  /* unused -- always zero */
	int fsmblks;  /* unused -- always zero */
	int uordblks; /* total allocated space */
	int fordblks; /* total non-inuse space */
	int keepcost; /* top-most, releasable (via malloc_trim) space */
};

struct contig_mem {
	spinlock_t lock;
	int initdone;
	char  *membase;
	char  *sbrkaddr;
	unsigned long long    size;
	unsigned long long    paddr;
	unsigned long trim_threshold;
	unsigned long top_pad;
	char *sbrk_base;
	unsigned long max_sbrked_mem;
	unsigned long max_total_mem;
	struct mallinfo current_mallinfo;
	struct _reent   reent;
	char av[MBIN_SZ];
	
};

extern struct contig_mem *contig_mem_ptr;


static inline void *contig_alloc(size_t nbytes)
{
	struct contig_mem *m = contig_mem_ptr;
	void *ret;
	unsigned int flags;
	extern _PTR _smp_malloc_r _PARAMS ((struct _reent *, size_t));

	if(m == NULL || !m->initdone)
		return NULL;
	SPIN_LOCK_IRQ_SAVE(&m->lock, flags);
	ret = _smp_malloc_r (_REENT, nbytes);
	SPIN_UNLOCK_IRQ_RESTORE(&m->lock, flags);

	return ret;
}

static inline void contig_free(void *ptr)
{
	struct contig_mem *m = contig_mem_ptr;
	extern _VOID _smp_free_r _PARAMS ((struct _reent *, _PTR));
	unsigned int flags;

	if(m == NULL || !m->initdone)
		return;

	SPIN_LOCK_IRQ_SAVE(&m->lock, flags);
	_smp_free_r (_REENT, ptr);
	SPIN_UNLOCK_IRQ_RESTORE(&m->lock, flags);

	return;
}

static inline void *contig_calloc(size_t nmemb, size_t size)
{
	struct contig_mem *m = contig_mem_ptr;
	void *ret;
	unsigned int flags;
	extern _PTR _smp_calloc_r _PARAMS ((struct _reent *, size_t, size_t));

	if(m == NULL || !m->initdone)
		return NULL;

	SPIN_LOCK_IRQ_SAVE(&m->lock, flags);
	ret = _smp_calloc_r (_REENT, nmemb, size);
	SPIN_UNLOCK_IRQ_RESTORE(&m->lock, flags);

	return ret;
}

static inline void *contig_realloc(void *src, size_t size)
{
	struct contig_mem *m = contig_mem_ptr;
	void *ret;
	unsigned int flags;
	extern _PTR _smp_realloc_r _PARAMS ((struct _reent *, _PTR, size_t));

	if(m == NULL || !m->initdone)
		return NULL;

	SPIN_LOCK_IRQ_SAVE(&m->lock, flags);
	ret = _smp_realloc_r (_REENT, src, size);
	SPIN_UNLOCK_IRQ_RESTORE(&m->lock, flags);

	return ret;
}

static inline void *contig_malign(size_t align, size_t size)
{
	struct contig_mem *m = contig_mem_ptr;
	void *ret;
	unsigned int flags;
	extern _PTR _smp_memalign_r _PARAMS ((struct _reent *, size_t, size_t));

	if(m == NULL || !m->initdone)
		return NULL;

	SPIN_LOCK_IRQ_SAVE(&m->lock, flags);
	ret = _smp_memalign_r (_REENT, align, size);
	SPIN_UNLOCK_IRQ_RESTORE(&m->lock, flags);

	return ret;
}

static inline int  contig_pinfo(unsigned long long *virtstart, 
		unsigned long long *physstart)
{
	struct contig_mem *m = contig_mem_ptr;
	if(m != NULL) {
		*virtstart = (long)m->membase;
		*physstart = m->paddr;
		return 0;
	}
	return -1;
}
	

#else /* NLM_LINUXU */
void *contig_alloc(size_t size);
void *contig_calloc(size_t nelem, size_t elsize);
void *contig_realloc(void *obj, size_t size);
void *contig_malign(size_t align, size_t size);
void *contig_free(void *ptr);
int  contig_pinfo(unsigned long long *virtstart, unsigned long long *physsart);
#endif

#endif
