/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/
#ifndef __HASH_H__
#define __HASH_H__

#define NETL_MAX_HASH_BITS 15

#include "spinlock.h"

struct netl_hash_entry {
	unsigned int hash; /* hash value for the above key */
	struct netl_hash_entry	*next;
};

struct netl_hash_table {
	struct netl_hash_entry **hash_table;
	spinlock_t hash_table_lock;
	int size;
	unsigned int mask;
	int (*compare)(struct netl_hash_entry *cur, struct netl_hash_entry *entry);
	unsigned int (*hash)(const char *str, int len);
};

int netl_hash_table_init(struct netl_hash_table *t, int hash_bits, 
	int (*compare)(struct netl_hash_entry *cur, struct netl_hash_entry *entry));
struct netl_hash_entry *
	netl_hash_insert(struct netl_hash_table *t, struct netl_hash_entry *e, unsigned int hash);
struct netl_hash_entry *
	netl_hash_lookup(struct netl_hash_table *t, struct netl_hash_entry *e, unsigned int hash);
struct netl_hash_entry *
	netl_hash_remove(struct netl_hash_table *t, struct netl_hash_entry *e, unsigned int hash);


#endif
