/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _MAC_H
#define _MAC_H

#include "types.h"
#include "msgring.h"
#include "spinlock.h"
#include "cache.h"
#include "mips-exts.h"

#define MAC_BYTE_OFFSET 0

enum mac {
	GMAC=0,
	XGMAC,
	SPI4
};

enum ctrl {
	CTRL_RES0,
	CTRL_RES1,
	CTRL_REG_FREE,
	CTRL_JUMBO_FREE,
	CTRL_CONT,
	CTRL_EOP,
	CTRL_START,
	CTRL_SNGL,
};

enum xgmac_config {
	XG_CONFIG_0                = 0x00,
	XG_CONFIG_1,
	XG_CONFIG_2,
	XG_CONFIG_3,
	XG_SA_LS_WORD,
	XG_SA_MS_WORD,
	XG_MAX_FRAME_LENGTH        = 0x08,
	XG_REV_LEVEL               = 0x0B,
	XG_AMIIM_COMMAND           = 0x10,
	XG_AMIIM_FIELD,
	XG_AMIIM_CONFIG,
	XG_AMIIM_LINK_FAIL_VECTOR,
	XG_AMIIM_INDICATOR
};

enum mac_glue_config {
	TX_CONTROL                      = 0xA0,
	RX_CONTROL,
	DESC_PACK_CTRL,
	STAT_CTRL,
	L2_ALLOC_CTRL,
	INT_MASK,
	INT_REG,

	CORE_CONTROL                    = 0xA8,
	BYTE_OFFSET_0_7,
	BYTE_OFFSET_8_F,

	PAUSE_FRAME                     = 0xAB,
	FIFO_MODE_CTRL                  = 0xAF,

	L2_TYPE                         = 0xF0,

	PARSER_CONFIG                   = 0x100,
	PARSE_DEPTH                     = 0x101,
	STRING_HASH_MASK0,
	STRING_HASH_MASK1,
	STRING_HASH_MASK2,
	STRING_HASH_MASK3,
	L3_L4_PROTO_MASK,
	L3_CTABLE                       = 0x140,
	/* L3 Cam table is 16 entries deep and each entry is 2 regs wide */
	L4_CTABLE                       = 0x160,
	/* L3 Cam table is 8 entries deep and each entry is 2 regs wide */
	CAM_4X_128_TABLE                = 0x172,
	CAM_4X_128_KEY                  = 0x180,
        CAM_4X_128_MASK                 = 0x184,
	TRANSLATE_TABLE                 = 0x1A0,
	TRANSLATE_TABLE_BASE            = 0x1A0,
	L3C_TABLE_MASK			= 0x1E0,
	L4C_TABLE_MASK			= 0x1E4,
	HASH7_BASE_MASK			= 0x1E8,
	HASH7_BASE_MASK_CAM_KEYS	= 0x1EC,

	 
	/* Translate Table is 128 entries deep and each entry is 16-bit wide
	 * (2 entries per reg) */

	// (Not applicable to gigi macs) The following four regs are for specifying
	// read and write credits (synonymous with packet descriptors ?). It says for
	// 'Interface X Rx/Tx duplex channel. What are those channels ?
	DMA_CR_0                        = 0x200,
	DMA_CR_1,
	DMA_CR_2,
	DMA_CR_3,

	REG_FRIN_SPILL_MEM_START_0,
	REG_FRIN_SPILL_MEM_START_1,
	REG_FRIN_SPILL_MEM_SIZE,

	FROUT_SPILL_MEM_START_0,
	FROUT_SPILL_MEM_START_1,
	FROUT_SPILL_MEM_SIZE,

	CLASS0_SPILL_MEM_START_0,
	CLASS0_SPILL_MEM_START_1,
	CLASS0_SPILL_MEM_SIZE,

	JUM_FRIN_SPILL_MEM_START_0,
	JUM_FRIN_SPILL_MEM_START_1,
	JUM_FRIN_SPILL_MEM_SIZE,

	CLASS1_SPILL_MEM_START_0,
	CLASS1_SPILL_MEM_START_1,
	CLASS1_SPILL_MEM_SIZE,

	CLASS2_SPILL_MEM_START_0,
	CLASS2_SPILL_MEM_START_1,
	CLASS2_SPILL_MEM_SIZE,

	CLASS3_SPILL_MEM_START_0,
	CLASS3_SPILL_MEM_START_1,
	CLASS3_SPILL_MEM_SIZE,

	SPI_HNGY_0,
	SPI_HNGY_1,
	SPI_HNGY_2,
	SPI_HNGY_3,
	SPI_STRV_0,
	SPI_STRV_1,
	SPI_STRV_2,
	SPI_STRV_3,

	REG_FRIN_1_SPILL_MEM_START_0     = 0x219,
	REG_FRIN_1_SPILL_MEM_START_1,
	REG_FRIN_1_SPILL_MEM_SIZE,

	JUM_FRIN_1_SPILL_MEM_START_0,
	JUM_FRIN_1_SPILL_MEM_START_1,
	JUM_FRIN_1_SPILL_MEM_SIZE,  

	TX_DATA_FIFO_0                   = 0x221,
	TX_DATA_FIFO_1,
	TX_DATA_FIFO_2,
	TX_DATA_FIFO_3,
	TX_DATA_FIFO_4,
	TX_DATA_FIFO_5,
	TX_DATA_FIFO_6,
	TX_DATA_FIFO_7,
	RX_DATA_FIFO_0,
	RX_DATA_FIFO_1,
	RX_DATA_FIFO_2,
	RX_DATA_FIFO_3,
	RX_DATA_FIFO_4,
	RX_DATA_FIFO_5,
	RX_DATA_FIFO_6,
	RX_DATA_FIFO_7,

	PAD_CALIBRATION_0,
	PAD_CALIBRATION_1,

	FREEQ_CARVE                      = 0x233,

	DBG_DESC_WORD_COUNT		= 0x23C,
	DBG_DESC_WORD_COUNT_SEL,
	DBG_RX_DESC_IN_MAC,
	DBG_TX_DESC_IN_MAC,

	SPI4_STATIC_DELAY_0              = 0x240,
	SPI4_STATIC_DELAY_1,
	SPI4_STATIC_DELAY_STATUS,
	SPI4_CONTROL,

	CLASS_WATERMARKS,
	RX_WATERMARKS_0,
	RX_WATERMARKS_1,
	RX_WATERMARKS_2,
	RX_WATERMARKS_3,
	FREE_WATERMARKS,

	FIFO_CARVING_SLOTS0,
	FIFO_CARVING_SLOTS1,
	
	//ROUND_ROBIN_TABLE,
	PDE_CLASS_0                      = 0x300,
	PDE_CLASS_1                      = 0x302,
	PDE_CLASS_2                      = 0x304,
	PDE_CLASS_3                      = 0x306,

	G_JFR0_BUCKET_SIZE                = 0x320,
	G_RFR0_BUCKET_SIZE,
	G_TX0_BUCKET_SIZE,
	G_TX1_BUCKET_SIZE,
	G_TX2_BUCKET_SIZE,
	G_TX3_BUCKET_SIZE,
	G_JFR1_BUCKET_SIZE,
	G_RFR1_BUCKET_SIZE,

	XGS_TX0_BUCKET_SIZE              = 0x320,
	XGS_TX1_BUCKET_SIZE,
	XGS_TX2_BUCKET_SIZE,
	XGS_TX3_BUCKET_SIZE,
	XGS_TX4_BUCKET_SIZE,
	XGS_TX5_BUCKET_SIZE,
	XGS_TX6_BUCKET_SIZE,
	XGS_TX7_BUCKET_SIZE,
	XGS_TX8_BUCKET_SIZE,
	XGS_TX9_BUCKET_SIZE,
	XGS_TX10_BUCKET_SIZE,
	XGS_TX11_BUCKET_SIZE,
	XGS_TX12_BUCKET_SIZE,
	XGS_TX13_BUCKET_SIZE,
	XGS_TX14_BUCKET_SIZE,
	XGS_TX15_BUCKET_SIZE,
	XGS_JFR_BUCKET_SIZE,
	XGS_RFR_BUCKET_SIZE,

	CC_CPU0_0                        = 0x380,
	CC_CPU1_0                        = 0x388,
	CC_CPU2_0                        = 0x390,
	CC_CPU3_0                        = 0x398,
	CC_CPU4_0                        = 0x3a0,
	CC_CPU5_0                        = 0x3a8,
	CC_CPU6_0                        = 0x3b0,
	CC_CPU7_0                        = 0x3b8
};


enum {
	GMAC_0 = 0,
	GMAC_1,
	GMAC_2,
	GMAC_3,
        GMAC_4,
        GMAC_5,
        GMAC_6,
        GMAC_7,
	XGMAC_0,
	XGMAC_1,
	SPI4_0,
	SPI4_1,
	PHOENIX_MAX_MACS
};





struct driver_data {
	struct net_device    *dev;
	spinlock_t            lock;
	volatile uint32_t    *mmio;
	uint8_t               hwaddr[6];
	int                   id, type, iface;
};

static inline int mac_make_desc_rfr(struct msgrng_msg *msg, int id, int type,
				    unsigned long long addr)
{
	int stid = 0;

	if ((type == XGMAC) || (type == SPI4))
	  stid = msgrng_xgmac_stid_rfr(id);
	else 
	  stid = msgrng_gmac_stid_rfr(id);
	
        msg->msg0 = ( ((uint64_t)0  << 63) |
                    ((uint64_t)0    << 54) |
                    ((uint64_t)0    << 40) |
                    ((uint64_t)addr));


                                                                    
	return stid;
}

#define MAC_TX_DESC_ALIGNMENT (CACHELINE_SIZE - 1)
static inline int mac_make_desc_tx(struct msgrng_msg *msg, int id, int type,
				   unsigned long long addr, int len) {
	int tx_stid = 0;
	int fr_stid = 0;

	if (type == XGMAC) {
		tx_stid = msgrng_xgmac_stid_tx(id);
		fr_stid = msgrng_xgmac_stid_rfr(id);
	}
	else {
		tx_stid = msgrng_gmac_stid_tx(id);
		fr_stid = msgrng_cpu_to_bucket(processor_id());
	}

	msg->msg0 = ( ((uint64_t)1  << 63) |
                    ((uint64_t)fr_stid << 54) |
                    ((uint64_t)len << 40)  |
                    ((uint64_t)addr));
	
		                                                     

	return tx_stid;
}



static inline void make_gmac_desc_td(struct msgrng_msg *msg, int fbid, unsigned long long addr, int len) {
  msg->msg0 = ( ((uint64_t)1  << 63)    | 
                 ((uint64_t)fbid << 54) |
                 ((uint64_t)len << 40)  |
                 ((uint64_t)addr));
  
}

static inline void make_gmac_desc_p2p(struct msgrng_msg *msg, int fbid, unsigned long long addr, int len) {
  msg->msg0 = ( ((uint64_t)1  << 62)    | 
                 ((uint64_t)fbid << 54) |
                 ((uint64_t)len << 43)  |
                 ((uint64_t)addr));  
}


struct ip_2_stdid_rec {
    char    *ip;    
    int      st_id;  
};

struct phnx_mac {
    int     instance;
    int     type;
    char   *name;
    unsigned long io_offset;

    // int irq;
};

struct config_macs {

    int     gmac0; // Set 1 if gmac 0 should be initialized
    int     gmac1; // Set 1 if gmac 1 should be initialized
    int     gmac2; // Set 1 if gmac 2 should be initialized
    int     gmac3; // Set 1 if gmac 3 should be initialized
    int     gmac4; // Set 1 if gmac 4 should be initialized
    int     gmac5; // Set 1 if gmac 5 should be initialized
    int     gmac6; // Set 1 if gmac 6 should be initialized
    int     gmac7; // Set 1 if gmac 7 should be initialized
    int     split_mode; // Set 1 if gmacs are in split mode
    int     gmac_max_free_descriptors_0; // Max number of free descriptors that are
                                         // allocated to gmacs. In case of split mode
                                         // this is for context containing gmac0 and
                                         // gmac1.
    int     gmac_max_free_descriptors_1; // In case of split mode this is for context 
                                         // containing gmac2 and gmac3.

    /* 
     * For interface 0 or 1, xgmac and spi4 are exclusive.
     * i.e. for the same interface, both xgmac and spi4 can-not
     * be set. These settings also have to match with board
     * settings.
     * ATX-I board both interfaces are SPI4 on the board
     * ATX-II board both interfaces are XGMAC on the board
     * ATX-III (or the PCI-X daughter card), does not have any of
     * these interfaces.
     */

    int     spi0; // Set 1 if spi4 interface 0 should be initialized
    int     xgmac0; // Set 1 if xgmac interface 0 should be initialized
    int     ten_gig_max_free_descriptors_0; // Max number of free descriptors for
                                            // 10 gig interface 0

    int     spi1; // Set 1 if spi4 interface 1 should be initialized
    int     xgmac1; // Set 1 if xgmac interface 1 should be initialized
    int     ten_gig_max_free_descriptors_1; // Max number of free descriptors for
                                            // 10 gig interface 1
    int     abm; // Set Automatic memory management by default

};

extern struct config_macs    conf_netdev;

extern struct ip_2_stdid_rec ip_2_stdids_tbl [];
extern struct phnx_mac phnx_mac_devices[];

void dstip_msb_classifier(void);

void mac_config_spill_area (struct net_device *dev);
int nlm_phnx_mac_init_module();
void mac_config_pde(struct net_device *dev, uint32_t cpumask);

extern int net_conf_error_status;
extern int net_conf_bucket_offset;

#define READ_COMMAND 1
#define MGMT_CLOCK_SELECT 0x07 // divisor = 28

#define PHY_STATUS_RETRIES 25000

#endif
