/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _MAC_DEFINES_H
#define _MAC_DEFINES_H

#include "types.h"
#include "msgring.h"
#include "spinlock.h"
#include "cache.h"
#include "mips-exts.h"

#define ONE_BIT              0x0000000000000001ULL
#define TWO_BITS             0x0000000000000003ULL
#define THREE_BITS           0x0000000000000007ULL
#define FOUR_BITS            0x000000000000000fULL
#define FIVE_BITS            0x000000000000001fULL
#define SIX_BITS             0x000000000000003fULL
#define SEVEN_BITS           0x000000000000007fULL
#define EIGHT_BITS           0x00000000000000ffULL
#define NINE_BITS            0x00000000000001ffULL
#define ELEVEN_BITS          0x00000000000007ffULL
#define TWELVE_BITS          0x0000000000000fffULL
#define FOURTEEN_BITS        0x0000000000003fffULL
#define SIXTEEN_BITS         0x000000000000ffffULL
#define THIRTY_TWO_BITS      0x00000000ffffffffULL
#define FORTY_BITS           0x000000ffffffffffULL

#define GET_FIELD(word,field) \
 ((word) & (field ## _MASK)) >> (field ## _LSB)

#define FIELD_VALUE(field,value) (((value) & (field ## _BITS)) << (field ## _LSB))

/*
 * NOTE: this macro expects 'word' to be uninitialized (i.e. zeroed)
 */
#define SET_FIELD(word,field,value) \
 { (word) |=  (((value) & (field ## _BITS)) << (field ## _LSB)); }

/*
 * This macro clears 'word', then sets the value
 */
#define CLEAR_SET_FIELD(word,field,value) \
 { (word) &= ~((field ## _BITS) << (field ## _LSB)); \
   (word) |=  (((value) & (field ## _BITS)) << (field ## _LSB)); }

/*
 * NOTE: May be used to build value specific mask 
 *        (e.g.  GEN_MASK(CTL_DSC_CPHR_3DES,CTL_DSC_CPHR_LSB)
 */
#define GEN_MASK(bits,lsb) ((bits) << (lsb))


/*
 * Define all the Network Accelerator Registers
 * ============================================
 */

/************************************
 * TX_CONTROL (Address Offset 0xA0)
 ***********************************/

/* #define TX15_HALT  ;Only present in SPI4    */
#define TX15_HALT_LSB   31
#define TX15_HALT_BITS  ONE_BIT
#define TX15_HALT_MASK  (TX15_HALT_BITS << TX15_HALT_LSB)

/* #define TX14_HALT  ;Only present in SPI4    */
#define TX14_HALT_LSB   30
#define TX14_HALT_BITS  ONE_BIT
#define TX14_HALT_MASK  (TX14_HALT_BITS << TX14_HALT_LSB)

/* #define TX13_HALT  ;Only present in SPI4    */
#define TX13_HALT_LSB   29
#define TX13_HALT_BITS  ONE_BIT
#define TX13_HALT_MASK  (TX13_HALT_BITS << TX13_HALT_LSB)

/* #define TX12_HALT  ;Only present in SPI4    */
#define TX12_HALT_LSB   28
#define TX12_HALT_BITS  ONE_BIT
#define TX12_HALT_MASK  (TX12_HALT_BITS << TX12_HALT_LSB)

/* #define TX11_HALT  ;Only present in SPI4    */
#define TX11_HALT_LSB   27
#define TX11_HALT_BITS  ONE_BIT
#define TX11_HALT_MASK  (TX11_HALT_BITS << TX11_HALT_LSB)

/* #define TX10_HALT  ;Only present in SPI4    */
#define TX10_HALT_LSB   26
#define TX10_HALT_BITS  ONE_BIT
#define TX10_HALT_MASK  (TX10_HALT_BITS << TX10_HALT_LSB)

/* #define TX9_HALT   ;Only present in SPI4   */
#define TX9_HALT_LSB   25
#define TX9_HALT_BITS  ONE_BIT
#define TX9_HALT_MASK  (TX9_HALT_BITS << TX9_HALT_LSB)

/* #define TX8_HALT   ;Only present in SPI4   */
#define TX8_HALT_LSB   24
#define TX8_HALT_BITS  ONE_BIT
#define TX8_HALT_MASK  (TX8_HALT_BITS << TX8_HALT_LSB)

/* #define TX7_HALT   ;Only present in SPI4   */
#define TX7_HALT_LSB   23
#define TX7_HALT_BITS  ONE_BIT
#define TX7_HALT_MASK  (TX7_HALT_BITS << TX7_HALT_LSB)

/* #define TX6_HALT   ;Only present in SPI4   */
#define TX6_HALT_LSB   22
#define TX6_HALT_BITS  ONE_BIT
#define TX6_HALT_MASK  (TX6_HALT_BITS << TX6_HALT_LSB)

/* #define TX5_HALT   ;Only present in SPI4   */
#define TX5_HALT_LSB   21
#define TX5_HALT_BITS  ONE_BIT
#define TX5_HALT_MASK  (TX5_HALT_BITS << TX5_HALT_LSB)

/* #define TX4_HALT   ;Only present in SPI4   */
#define TX4_HALT_LSB   20
#define TX4_HALT_BITS  ONE_BIT
#define TX4_HALT_MASK  (TX4_HALT_BITS << TX4_HALT_LSB)

/* #define TX3_HALT   ;Only present in SPI4   */
#define TX3_HALT_LSB   19
#define TX3_HALT_BITS  ONE_BIT
#define TX3_HALT_MASK  (TX3_HALT_BITS << TX3_HALT_LSB)

/* #define TX2_HALT   ;Only present in SPI4   */
#define TX2_HALT_LSB   18
#define TX2_HALT_BITS  ONE_BIT
#define TX2_HALT_MASK  (TX2_HALT_BITS << TX2_HALT_LSB)

/* #define TX1_HALT   ;Only present in SPI4   */
#define TX1_HALT_LSB   17
#define TX1_HALT_BITS  ONE_BIT
#define TX1_HALT_MASK  (TX1_HALT_BITS << TX1_HALT_LSB)

/* #define TX0_HALT   ;Only present in SPI4 or XGMAC   */
#define TX0_HALT_LSB   16
#define TX0_HALT_BITS  ONE_BIT
#define TX0_HALT_MASK  (TX0_HALT_BITS << TX0_HALT_LSB)

/* #define TX_IDLE    ; Transmit side is idle   */
#define TX_IDLE_LSB   15
#define TX_IDLE_BITS  ONE_BIT
#define TX_IDLE_MASK  (TX_IDLE_BITS << TX_IDLE_LSB)

/* #define TX_ENABLE     */
#define TX_ENABLE_LSB   14
#define TX_ENABLE_BITS  ONE_BIT
#define TX_ENABLE_MASK  (TX_ENABLE_BITS << TX_ENABLE_LSB)

/* #define TX_THRESHOLD   ; Number of bytes that must be fetched from memory before transmission of packet  */
#define TX_THRESHOLD_LSB    0
#define TX_THRESHOLD_BITS  FOURTEEN_BITS
#define TX_THRESHOLD_MASK  (TX_THRESHOLD_BITS << TX_THRESHOLD_LSB)

/************************************
 * RX_CONTROL (Address Offset 0xA1)
 ***********************************/

/* #define ERROR_STATUS  ;Should be set for error reporting in the Receive Descriptors    */
#define ERROR_STATUS_LSB  6
#define ERROR_STATUS_BITS  FOUR_BITS
#define ERROR_STATUS_MASK  (ERROR_STATUS_BITS << ERROR_STATUS_LSB)

/* #define SINGLE_PORT  ;Should be set for XGMAC mode and if doing single channel in SPI4 mode    */
#define SINGLE_PORT_LSB  10 
#define SINGLE_PORT_BITS  ONE_BIT
#define SINGLE_PORT_MASK  (SINGLE_PORT_BITS << SINGLE_PORT_LSB)

/* #define IP_RESET  ;Hard-Reset the MAC IP    */
#define IP_RESET_LSB  5 
#define IP_RESET_BITS  ONE_BIT
#define IP_RESET_MASK  (IP_RESET_BITS << IP_RESET_LSB)

/* #define DEBUG_INTERRUPT  ;Setting this bit will cause an interrupt if the appropriate interrupt mask bit is set.
 *                           Check the Interrupt Mask register bit 6  */
#define DEBUG_INTERRUPT_LSB  4 
#define DEBUG_INTERRUPT_BITS  ONE_BIT
#define DEBUG_INTERRUPT_MASK  (DEBUG_INTERRUPT_BITS << DEBUG_INTERRUPT_LSB)

/* #define SOFT_RESET_DONE  ;Software should poll this bit to detenlmne when soft reset is complete    */
#define SOFT_RESET_DONE_LSB  3 
#define SOFT_RESET_DONE_BITS  ONE_BIT
#define SOFT_RESET_DONE_MASK  (SOFT_RESET_DONE_BITS << SOFT_RESET_DONE_LSB)

/* #define SOFT_RESET  ;Software sets this bit to effect  software reset of this Network Interface    */
#define SOFT_RESET_LSB  2 
#define SOFT_RESET_BITS  ONE_BIT
#define SOFT_RESET_MASK  (SOFT_RESET_BITS << SOFT_RESET_LSB)

/* #define RX_HALT  ;Software can poll this bit to detenlmne when Rx has entered the Halted state    */
#define RX_HALT_LSB  1 
#define RX_HALT_BITS  ONE_BIT
#define RX_HALT_MASK  (RX_HALT_BITS << RX_HALT_LSB)

/* #define RX_ENABLE  ;Should be set for enabling Rx    */
#define RX_ENABLE_LSB  0 
#define RX_ENABLE_BITS  ONE_BIT
#define RX_ENABLE_MASK  (RX_ENABLE_BITS << RX_ENABLE_LSB)

/* #define RX_RGMII  ;Should be set for enabling RGMII on port 0    */
#define RX_RGMII_LSB  10
#define RX_RGMII_BITS  ONE_BIT
#define RX_RGMII_MASK  (RX_RGMII_BITS << RX_RGMII_LSB)

/************************************
 * DESC_PACK_CTRL (Address Offset 0xA2)
 ***********************************/

/* #define MAX_DESC_HOLD  ; This is the maximum number of descriptors for which a retry will occur; this
 *                          should not exceed the TmpFIFO depth for the corresponding port.
 *                          For GMACs this value is 16. For Xgmac 64 and for SPI4 the max value is
 *                          detenlmned by the FIFO Carving Slots Register*/
#define MAX_DESC_HOLD_LSB  20
#define MAX_DESC_HOLD_BITS  SIX_BITS
#define MAX_DESC_HOLD_MASK  (MAX_DESC_HOLD_BITS << MAX_DESC_HOLD_LSB)

/* #define BYTE_OFFSET ; Byte Offset from which GMAC Packet Data is written to memory. For XGMAC and SPI4
 *                       use register 0xA9-0xAA
 */
#define BYTE_OFFSET_LSB  17
#define BYTE_OFFSET_BITS  THREE_BITS
#define BYTE_OFFSET_MASK  (BYTE_OFFSET_BITS << BYTE_OFFSET_LSB)

/* #define PREPAD_ENABLE ; Byte Offset from which GMAC Packet Data is written to memory. For XGMAC and SPI4
 *                       use register 0xA9-0xAA
 */
#define PREPAD_ENABLE_LSB  16
#define PREPAD_ENABLE_BITS  ONE_BIT
#define PREPAD_ENABLE_MASK  (PREPAD_ENABLE_BITS << PREPAD_ENABLE_LSB)

#define MAX_ENTRY_LSB  14
#define MAX_ENTRY_BITS    TWO_BITS
#define MAX_ENTRY_MASK    (MAX_ENTRY_BITS << MAX_ENTRY_LSB)
//#endif

/* #define REGULAR_SIZE ; Size of Packet Buffer in Bytes. Shoul be multiple of cache line (32)
 */
#define REGULAR_SIZE_LSB 0 
#define REGULAR_SIZE_BITS    FOURTEEN_BITS
#define REGULAR_SIZE_MASK    (REGULAR_SIZE_BITS << REGULAR_SIZE_LSB)


/************************************
 * STAT_CTRL (Address Offset 0xA3)
 ***********************************/

/* #define OVERFLOW_ENABLE  ; Carry Oferflow Interrupt Enable
 */
#define OVERFLOW_ENABLE_LSB  4
#define OVERFLOW_ENABLE_BITS    ONE_BIT
#define OVERFLOW_ENABLE_MASK    (OVERFLOW_ENABLE_BITS << OVERFLOW_ENABLE_LSB)

/* #define GIG  ; Indicates Statistics Block driven by GMAC Core
 */
#define GIG_LSB  3
#define GIG_BITS    ONE_BIT
#define GIG_MASK    (GIG_BITS << GIG_LSB)

/* #define STATS_ENABLE  ; Statistics Block Enable
 */
#define STATS_ENABLE_LSB  2
#define STATS_ENABLE_BITS    ONE_BIT
#define STATS_ENABLE_MASK    (STATS_ENABLE_BITS << STATS_ENABLE_LSB)

/* #define CLEAR_COUNTERS  ; Clear Statistics Block Counters
 */
#define CLEAR_COUNTERS_LSB  1
#define CLEAR_COUNTERS_BITS    ONE_BIT
#define CLEAR_COUNTERS_MASK    (CLEAR_COUNTERS_BITS << CLEAR_COUNTERS_LSB)

/* #define AUTOZ  ; Automatically zero addressed statistics counters
 */
#define AUTOZ_LSB  0
#define AUTOZ_BITS    ONE_BIT
#define AUTOZ_MASK    (AUTOZ_BITS << AUTOZ_LSB)


/************************************
 * L2_ALLOC_CTRL (Address Offset 0xA4)
 ***********************************/

/* #define TX_L2_ALLOCATE  ; Number of Cache Lines to be marked L2 Allocate for TX
 */
#define TX_L2_ALLOCATE_LSB  9
#define TX_L2_ALLOCATE_BITS    NINE_BITS
#define TX_L2_ALLOCATE_MASK    (TX_L2_ALLOCATE_BITS << TX_L2_ALLOCATE_LSB)

/* #define RX_L2_ALLOCATE  ; Number of Cache Lines to be marked L2 Allocate for RX
 */
#define RX_L2_ALLOCATE_LSB  0
#define RX_L2_ALLOCATE_BITS    NINE_BITS
#define RX_L2_ALLOCATE_MASK    (RX_L2_ALLOCATE_BITS << RX_L2_ALLOCATE_LSB)


/*********************************************
 * INT_MASK (Address Offset 0xA5)
 * INT_REG  (Address Offset 0xA6)
 * Bit definitions are same for both registers
 *********************************************/

/* #define TX_SPI4_INT  ; Transmit interrupt from SPI4
 */
#define TX_SPI4_INT_LSB  28
#define TX_SPI4_INT_BITS    ONE_BIT
#define TX_SPI4_INT_MASK    (TX_SPI4_INT_BITS << TX_SPI4_INT_LSB)

/* #define COLL_INT  ; Collision occurred in half-duplex mode
 */
#define COLL_INT_LSB  27
#define COLL_INT_BITS    ONE_BIT
#define COLL_INT_MASK    (COLL_INT_BITS << COLL_INT_LSB)

/* #define COLL_RX_SPI4_INT  ; Receive interrupt from SPI4
 */
#define RX_SPI4_INT_LSB  27
#define RX_SPI4_INT_BITS    ONE_BIT
#define RX_SPI4_INT_MASK    (RX_SPI4_INT_BITS << RX_SPI4_INT_LSB)

/* #define ABORT_INT  ; packet was aborted
 */
#define ABORT_INT_LSB  26
#define ABORT_INT_BITS    ONE_BIT
#define ABORT_INT_MASK    (ABORT_INT_BITS << ABORT_INT_LSB)

/* #define UNDERRUN_INT  ; Underrun occurred while fetching packet data from memory.
 */
#define UNDERRUN_INT_LSB  25
#define UNDERRUN_INT_BITS    ONE_BIT
#define UNDERRUN_INT_MASK    (UNDERRUN_INT_BITS << UNDERRUN_INT_LSB)

/* #define MAC_DISCARD_INT  ; MAC Discard
 */
#define MAC_DISCARD_INT_LSB  24
#define MAC_DISCARD_INT_BITS    ONE_BIT
#define MAC_DISCARD_INT_MASK    (MAC_DISCARD_INT_BITS << MAC_DISCARD_INT_LSB)

/* #define ASYNC_FIFO_OVERFLOW_INT  ; Internal Error in MAC (asynchronous FIFO Overflow)
 */
#define ASYNC_FIFO_OVERFLOW_INT_LSB  23
#define ASYNC_FIFO_OVERFLOW_INT_BITS    ONE_BIT
#define ASYNC_FIFO_OVERFLOW_INT_MASK    (ASYNC_FIFO_OVERFLOW_INT_BITS << ASYNC_FIFO_OVERFLOW_INT_LSB)

/* #define TAG_FULL_INT  ; All Tags used up (May happen in SPI4 16 channels)
 */
#define TAG_FULL_INT_LSB  22
#define TAG_FULL_INT_BITS    ONE_BIT
#define TAG_FULL_INT_MASK    (TAG_FULL_INT_BITS << TAG_FULL_INT_LSB)

/* #define CLASS3_FULL_INT  ; Class 3 FIFO is full
 */
#define CLASS3_FULL_INT_LSB  21
#define CLASS3_FULL_INT_BITS    ONE_BIT
#define CLASS3_FULL_INT_MASK    (CLASS3_FULL_INT_BITS << CLASS3_FULL_INT_LSB)

/* #define CLASS3_EARLY_FULL_INT  ; Class 3 FIFO is > watermark
 */
#define CLASS3_EARLY_FULL_INT_LSB  20
#define CLASS3_EARLY_FULL_INT_BITS    ONE_BIT
#define CLASS3_EARLY_FULL_INT_MASK    (CLASS3_EARLY_FULL_INT_BITS << CLASS3_EARLY_FULL_INT_LSB)

/* #define CLASS2_FULL_INT  ; Class 2 FIFO is full
 */
#define CLASS2_FULL_INT_LSB  19
#define CLASS2_FULL_INT_BITS    ONE_BIT
#define CLASS2_FULL_INT_MASK    (CLASS2_FULL_INT_BITS << CLASS2_FULL_INT_LSB)

/* #define CLASS2_EARLY_FULL_INT  ; Class 2 FIFO is > watermark
 */
#define CLASS2_EARLY_FULL_INT_LSB  18
#define CLASS2_EARLY_FULL_INT_BITS    ONE_BIT
#define CLASS2_EARLY_FULL_INT_MASK    (CLASS2_EARLY_FULL_INT_BITS << CLASS2_EARLY_FULL_INT_LSB)

/* #define CLASS1_FULL_INT  ; Class 1 FIFO is full
 */
#define CLASS1_FULL_INT_LSB  17
#define CLASS1_FULL_INT_BITS    ONE_BIT
#define CLASS1_FULL_INT_MASK    (CLASS1_FULL_INT_BITS << CLASS1_FULL_INT_LSB)

/* #define CLASS1_EARLY_FULL_INT  ; Class 1 FIFO is > watermark
 */
#define CLASS1_EARLY_FULL_INT_LSB  16
#define CLASS1_EARLY_FULL_INT_BITS    ONE_BIT
#define CLASS1_EARLY_FULL_INT_MASK    (CLASS1_EARLY_FULL_INT_BITS << CLASS1_EARLY_FULL_INT_LSB)

/* #define CLASS0_FULL_INT  ; Class 0 FIFO is full
 */
#define CLASS0_FULL_INT_LSB  15
#define CLASS0_FULL_INT_BITS    ONE_BIT
#define CLASS0_FULL_INT_MASK    (CLASS0_FULL_INT_BITS << CLASS0_FULL_INT_LSB)

/* #define CLASS0_EARLY_FULL_INT  ; Class 0 FIFO is > watermark
 */
#define CLASS0_EARLY_FULL_INT_LSB  14
#define CLASS0_EARLY_FULL_INT_BITS    ONE_BIT
#define CLASS0_EARLY_FULL_INT_MASK    (CLASS0_EARLY_FULL_INT_BITS << CLASS0_EARLY_FULL_INT_LSB)

/* #define RX_DATA_FULL_INT  ; Rx Data Fifo is Full
 */
#define RX_DATA_FULL_INT_LSB  13
#define RX_DATA_FULL_INT_BITS    ONE_BIT
#define RX_DATA_FULL_INT_MASK    (RX_DATA_FULL_INT_BITS << RX_DATA_FULL_INT_LSB)

/* #define RX_EARLY_FULL_INT  ; Rx Data Fifo is > watermark
 */
#define RX_EARLY_FULL_INT_LSB  12
#define RX_EARLY_FULL_INT_BITS    ONE_BIT
#define RX_EARLY_FULL_INT_MASK    (RX_EARLY_FULL_INT_BITS << RX_EARLY_FULL_INT_LSB)

/* #define REGULAR_FREE_EMPTY_INT  ; Free In FIFO for Regular is EMPTY
 */
#define REGULAR_FREE_EMPTY_INT_LSB     9
#define REGULAR_FREE_EMPTY_INT_BITS    ONE_BIT
#define REGULAR_FREE_EMPTY_INT_MASK    (REGULAR_FREE_EMPTY_INT_BITS << REGULAR_FREE_EMPTY_INT_LSB)

/* #define REGULAR_FREE_EARLY_EMPTY_INT  ; Free In FIFO for Regular is < watermark
 */
#define REGULAR_FREE_EARLY_EMPTY_INT_LSB     8
#define REGULAR_FREE_EARLY_EMPTY_INT_BITS    ONE_BIT
#define REGULAR_FREE_EARLY_EMPTY_INT_MASK    (REGULAR_FREE_EARLY_EMPTY_INT_BITS << REGULAR_FREE_EARLY_EMPTY_INT_LSB)

/* #define SPILL_P2P_ERROR_INT  ; ECC or Address error on Spill Channel or P2P channel
 */
#define SPILL_P2P_ERROR_INT_LSB     7
#define SPILL_P2P_ERROR_INT_BITS    ONE_BIT
#define SPILL_P2P_ERROR_INT_MASK    (SPILL_P2P_ERROR_INT_BITS << SPILL_P2P_ERROR_INT_LSB)

/* #define DEBUG_INT  ; Interrupt generated by setting the debug interrupt bit in RX_CONTROL register
 */
#define DEBUG_INT_LSB     6
#define DEBUG_INT_BITS    ONE_BIT
#define DEBUG_INT_MASK    (DEBUG_INT_BITS << DEBUG_INT_LSB)

/* #define FREE_OUT_FULL_INT  ; Interrupt generated when Free Out descriptor is full
 */
#define FREE_OUT_FULL_INT_LSB     5
#define FREE_OUT_FULL_INT_BITS    ONE_BIT
#define FREE_OUT_FULL_INT_MASK    (FREE_OUT_FULL_INT_BITS << FREE_OUT_FULL_INT_LSB)

/* #define FREE_OUT_EARLY_FULL_INT  ; Interrupt generated when Free Out descriptor > watermark
 */
#define FREE_OUT_EARLY_FULL_INT_LSB     4
#define FREE_OUT_EARLY_FULL_INT_BITS    ONE_BIT
#define FREE_OUT_EARLY_FULL_INT_MASK    (FREE_OUT_EARLY_FULL_INT_BITS << FREE_OUT_EARLY_FULL_INT_LSB)

/* #define TX_FETCH_ERROR_INT  ; Interrupt generated when Bus Error on Data
 */
#define TX_FETCH_ERROR_INT_LSB     3
#define TX_FETCH_ERROR_INT_BITS    ONE_BIT
#define TX_FETCH_ERROR_INT_MASK    (TX_FETCH_ERROR_INT_BITS << TX_FETCH_ERROR_INT_LSB)

/* #define STATS_CARRY_INT  ; Interrupt generated when MAC Statistics counter overflows
 */
#define STATS_CARRY_INT_LSB     2
#define STATS_CARRY_INT_BITS    ONE_BIT
#define STATS_CARRY_INT_MASK    (STATS_CARRY_INT_BITS << STATS_CARRY_INT_LSB)

/* #define MDINT_INT  ; MDINT from PHY
 */
#define MDINT_INT_LSB     1
#define MDINT_INT_BITS    ONE_BIT
#define MDINT_INT_MASK    (MDINT_INT_BITS << MDINT_INT_LSB)

/* #define TX_ILLEGAL  ; Illegal Tx Descriptor received
 */
#define TX_ILLEGAL_LSB     0
#define TX_ILLEGAL_BITS    ONE_BIT
#define TX_ILLEGAL_MASK    (TX_ILLEGAL_BITS << TX_ILLEGAL_LSB)

/*********************************************
 * CORE_CONTROL (Address Offset 0xA8)
 *********************************************/

/* #define ERROR_ID  ; See USE_ERROR_ID
 */
#define ERROR_ID_LSB     3
#define ERROR_ID_BITS    SEVEN_BITS
#define ERROR_ID_MASK    (ERROR_ID_BITS << ERROR_ID_LSB)

/* #define USE_ERROR_ID  ; See USE_ERROR_ID
 */
#define USE_ERROR_ID_LSB     2
#define USE_ERROR_ID_BITS    ONE_BIT
#define USE_ERROR_ID_MASK    (USE_ERROR_ID_BITS << USE_ERROR_ID_LSB)

/* #define SPEED  ; Speed Setting (N/A in XGMAC or SPI4)
 *                  00: 125 MHz (1000 Mbps)
 *                  01:  25 MHz (100 Mbps)
 *                  10: 2.5 MHz (10 Mbps)
 *                  11: Reserved
 */
#define SPEED_LSB     0
#define SPEED_BITS    TWO_BITS
#define SPEED_MASK    (SPEED_BITS << SPEED_LSB)
#define FREQ_125MHz   0
#define FREQ_25MHz    1
#define FREQ_2p5MHz   2

/*********************************************
 * BYTE_OFFSET_0_7 (Address Offset 0xA9) Used only for SPI4 and XGMAC
 *********************************************/

/* #define BYTE_OFFSET_7  ; Byte Offset on Rx Side
 */
#define BYTE_OFFSET_7_LSB     21
#define BYTE_OFFSET_7_BITS    THREE_BITS
#define BYTE_OFFSET_7_MASK    (BYTE_OFFSET_7_BITS << BYTE_OFFSET_7_LSB)

/* #define BYTE_OFFSET_6  ; Byte Offset on Rx
 */
#define BYTE_OFFSET_6_LSB     18
#define BYTE_OFFSET_6_BITS    THREE_BITS
#define BYTE_OFFSET_6_MASK    (BYTE_OFFSET_6_BITS << BYTE_OFFSET_6_LSB)

/* #define BYTE_OFFSET_5  ; Byte Offset on Rx
 */
#define BYTE_OFFSET_5_LSB     15
#define BYTE_OFFSET_5_BITS    THREE_BITS
#define BYTE_OFFSET_5_MASK    (BYTE_OFFSET_5_BITS << BYTE_OFFSET_5_LSB)

/* #define BYTE_OFFSET_4  ; Byte Offset on Rx
 */
#define BYTE_OFFSET_4_LSB     12
#define BYTE_OFFSET_4_BITS    THREE_BITS
#define BYTE_OFFSET_4_MASK    (BYTE_OFFSET_4_BITS << BYTE_OFFSET_4_LSB)

/* #define BYTE_OFFSET_3  ; Byte Offset on Rx
 */
#define BYTE_OFFSET_3_LSB     9
#define BYTE_OFFSET_3_BITS    THREE_BITS
#define BYTE_OFFSET_3_MASK    (BYTE_OFFSET_3_BITS << BYTE_OFFSET_3_LSB)

/* #define BYTE_OFFSET_2  ; Byte Offset on Rx
 */
#define BYTE_OFFSET_2_LSB     6
#define BYTE_OFFSET_2_BITS    THREE_BITS
#define BYTE_OFFSET_2_MASK    (BYTE_OFFSET_2_BITS << BYTE_OFFSET_2_LSB)

/* #define BYTE_OFFSET_1  ; Byte Offset on Rx
 */
#define BYTE_OFFSET_1_LSB     3
#define BYTE_OFFSET_1_BITS    THREE_BITS
#define BYTE_OFFSET_1_MASK    (BYTE_OFFSET_1_BITS << BYTE_OFFSET_1_LSB)

/* #define BYTE_OFFSET_0  ; Byte Offset on Rx
 */
#define BYTE_OFFSET_0_LSB     0
#define BYTE_OFFSET_0_BITS    THREE_BITS
#define BYTE_OFFSET_0_MASK    (BYTE_OFFSET_0_BITS << BYTE_OFFSET_0_LSB)

/*********************************************
 * BYTE_OFFSET_8_15 (Address Offset 0xAA)
 *********************************************/

/* #define BYTE_OFFSET_15  ; Byte Offset on Rx
 */
#define BYTE_OFFSET_15_LSB     21
#define BYTE_OFFSET_15_BITS    THREE_BITS
#define BYTE_OFFSET_15_MASK    (BYTE_OFFSET_15_BITS << BYTE_OFFSET_15_LSB)

/* #define BYTE_OFFSET_14  ; Byte Offset on Rx
 */
#define BYTE_OFFSET_14_LSB     18
#define BYTE_OFFSET_14_BITS    THREE_BITS
#define BYTE_OFFSET_14_MASK    (BYTE_OFFSET_14_BITS << BYTE_OFFSET_14_LSB)

/* #define BYTE_OFFSET_13  ; Byte Offset on Rx
 */
#define BYTE_OFFSET_13_LSB     15
#define BYTE_OFFSET_13_BITS    THREE_BITS
#define BYTE_OFFSET_13_MASK    (BYTE_OFFSET_13_BITS << BYTE_OFFSET_13_LSB)

/* #define BYTE_OFFSET_12  ; Byte Offset on Rx
 */
#define BYTE_OFFSET_12_LSB     12
#define BYTE_OFFSET_12_BITS    THREE_BITS
#define BYTE_OFFSET_12_MASK    (BYTE_OFFSET_12_BITS << BYTE_OFFSET_12_LSB)

/* #define BYTE_OFFSET_11  ; Byte Offset on Rx
 */
#define BYTE_OFFSET_11_LSB     9
#define BYTE_OFFSET_11_BITS    THREE_BITS
#define BYTE_OFFSET_11_MASK    (BYTE_OFFSET_11_BITS << BYTE_OFFSET_11_LSB)

/* #define BYTE_OFFSET_10  ; Byte Offset on Rx
 */
#define BYTE_OFFSET_10_LSB     6
#define BYTE_OFFSET_10_BITS    THREE_BITS
#define BYTE_OFFSET_10_MASK    (BYTE_OFFSET_10_BITS << BYTE_OFFSET_10_LSB)

/* #define BYTE_OFFSET_9  ; Byte Offset on Rx
 */
#define BYTE_OFFSET_9_LSB     3
#define BYTE_OFFSET_9_BITS    THREE_BITS
#define BYTE_OFFSET_9_MASK    (BYTE_OFFSET_9_BITS << BYTE_OFFSET_9_LSB)

/* #define BYTE_OFFSET_8  ; Byte Offset on Rx
 */
#define BYTE_OFFSET_8_LSB     0
#define BYTE_OFFSET_8_BITS    THREE_BITS
#define BYTE_OFFSET_8_MASK    (BYTE_OFFSET_8_BITS << BYTE_OFFSET_8_LSB)

/*********************************************
 * PAUSE_FRAME (Address Offset 0xAB)
 *********************************************/

/* #define SENDING_PAUSE_FRAME  ; This bit can be polled to find out is pause frame has been sent
 */
#define SENDING_PAUSE_FRAME_LSB     16
#define SENDING_PAUSE_FRAME_BITS    ONE_BIT
#define SENDING_PAUSE_FRAME_MASK    (SENDING_PAUSE_FRAME_BITS << SENDING_PAUSE_FRAME_LSB)

/* #define PAUSE_FRAME_TIMER  ; Writing to this bit will cause a pause frame to be sent out.
 */
#define PAUSE_FRAME_TIMER_LSB     0
#define PAUSE_FRAME_TIMER_BITS    SIXTEEN_BITS
#define PAUSE_FRAME_TIMER_MASK    (PAUSE_FRAME_TIMER_BITS << PAUSE_FRAME_TIMER_LSB)

/*********************************************
 * FIFO_MODE_CTRL (Address Offset 0xAF) -- Applies to GMAC only
 *********************************************/

/* #define BACKPRESSURE_PERIODIC_SEL  ; The XLR transmitter transmits an idle (or back pressure state) every
 *                                      X cycles to refresh the link partner's state
 *                                      00 : 64
 *                                      01 : 512
 *                                      10 : 4K
 *                                      11 : infinite
 */
#define BACKPRESSURE_PERIODIC_SEL_LSB     17
#define BACKPRESSURE_PERIODIC_SEL_BITS    TWO_BITS
#define BACKPRESSURE_PERIODIC_SEL_MASK    (BACKPRESSURE_PERIODIC_SEL_BITS << BACKPRESSURE_PERIODIC_SEL_LSB)

/* #define FMODE  ; FIFO Mode Enable
 */
#define FMODE_LSB     16
#define FMODE_BITS    ONE_BIT
#define FMODE_MASK    (FMODE_BITS << FMODE_LSB)

/* #define FMODE_HI_WATERMARK  ; When RxData FIFO is above this watermark BP is generated
 */
#define FMODE_HI_LSB     8 
#define FMODE_HI_BITS    EIGTH_BITS
#define FMODE_HI_MASK    (FMODE_HI_BITS << FMODE_HI_LSB)

/* #define FMODE_LO_WATERMARK  ; When RxData FIFO is below this watermark BP is removed
 */
#define FMODE_LO_LSB     0
#define FMODE_LO_BITS    EIGHT_BITS
#define FMODE_LO_MASK    (FMODE_LO_BITS << FMODE_LO_LSB)

/*********************************************
 * DMA_CR_0 (Address Offset 0x200)
 *********************************************/

/* #define WR0_CREDITS  ; Write Credits for Rx Channel 0
 */
#define XGS_WR0_CREDITS_LSB     28
#define XGS_WR0_CREDITS_BITS    THREE_BITS
#define XGS_WR0_CREDITS_MASK    (XGS_WR0_CREDITS_BITS << XGS_WR0_CREDITS_LSB)

/* #define RD0_CREDITS  ; Read Credits for Tx Channel 0
 * For Channel 0 SPI4 and XGMAC has 4 bits instead of 3
 */
#define XGS_RD0_CREDITS_LSB     24
#define XGS_RD0_CREDITS_BITS    FOUR_BITS
#define XGS_RD0_CREDITS_MASK    (XGS_RD0_CREDITS_BITS << XGS_RD0_CREDITS_LSB)

/* #define WR0_CREDITS  ; Write Credits for Rx Channel 0
 */
#define WR0_CREDITS_LSB     27
#define WR0_CREDITS_BITS    THREE_BITS
#define WR0_CREDITS_MASK    (WR0_CREDITS_BITS << WR0_CREDITS_LSB)

/* #define RD0_CREDITS  ; Read Credits for Tx Channel 0
 */
#define RD0_CREDITS_LSB     24
#define RD0_CREDITS_BITS    THREE_BITS
#define RD0_CREDITS_MASK    (RD0_CREDITS_BITS << RD0_CREDITS_LSB)

/* #define WR1_CREDITS  ; Write Credits for Rx Channel 1
 */
#define WR1_CREDITS_LSB     21
#define WR1_CREDITS_BITS    THREE_BITS
#define WR1_CREDITS_MASK    (WR1_CREDITS_BITS << WR1_CREDITS_LSB)

/* #define RD1_CREDITS  ; Read Credits for Tx Channel 1
 */
#define RD1_CREDITS_LSB     18
#define RD1_CREDITS_BITS    THREE_BITS
#define RD1_CREDITS_MASK    (RD1_CREDITS_BITS << RD1_CREDITS_LSB)

/* #define WR2_CREDITS  ; Write Credits for Rx Channel 2
 */
#define WR2_CREDITS_LSB     15
#define WR2_CREDITS_BITS    THREE_BITS
#define WR2_CREDITS_MASK    (WR2_CREDITS_BITS << WR2_CREDITS_LSB)

/* #define RD2_CREDITS  ; Read Credits for Tx Channel 2
 */
#define RD2_CREDITS_LSB     12
#define RD2_CREDITS_BITS    THREE_BITS
#define RD2_CREDITS_MASK    (RD2_CREDITS_BITS << RD2_CREDITS_LSB)

/* #define WR3_CREDITS  ; Write Credits for Rx Channel 3
 */
#define WR3_CREDITS_LSB     9
#define WR3_CREDITS_BITS    THREE_BITS
#define WR3_CREDITS_MASK    (WR3_CREDITS_BITS << WR3_CREDITS_LSB)

/* #define RD3_CREDITS  ; Read Credits for Tx Channel 3
 */
#define RD3_CREDITS_LSB     6
#define RD3_CREDITS_BITS    THREE_BITS
#define RD3_CREDITS_MASK    (RD3_CREDITS_BITS << RD3_CREDITS_LSB)

/* #define WR4_CREDITS  ; Write Credits for Rx Channel 4
 */
#define WR4_CREDITS_LSB     3
#define WR4_CREDITS_BITS    THREE_BITS
#define WR4_CREDITS_MASK    (WR4_CREDITS_BITS << WR4_CREDITS_LSB)

/* #define RD4_CREDITS  ; Read Credits for Tx Channel 4
 */
#define RD4_CREDITS_LSB     0
#define RD4_CREDITS_BITS    THREE_BITS
#define RD4_CREDITS_MASK    (RD4_CREDITS_BITS << RD4_CREDITS_LSB)

/*********************************************
 * DMA_CR_1 (Address Offset 0x201)
 *********************************************/

/* #define WR5_CREDITS  ; Write Credits for Rx Channel 5
 */
#define WR5_CREDITS_LSB     27
#define WR5_CREDITS_BITS    THREE_BITS
#define WR5_CREDITS_MASK    (WR5_CREDITS_BITS << WR5_CREDITS_LSB)

/* #define RD5_CREDITS  ; Read Credits for Tx Channel 5
 */
#define RD5_CREDITS_LSB     24
#define RD5_CREDITS_BITS    THREE_BITS
#define RD5_CREDITS_MASK    (RD5_CREDITS_BITS << RD5_CREDITS_LSB)

/* #define WR6_CREDITS  ; Write Credits for Rx Channel 6
 */
#define WR6_CREDITS_LSB     21
#define WR6_CREDITS_BITS    THREE_BITS
#define WR6_CREDITS_MASK    (WR6_CREDITS_BITS << WR6_CREDITS_LSB)

/* #define RD6_CREDITS  ; Read Credits for Tx Channel 6
 */
#define RD6_CREDITS_LSB     18
#define RD6_CREDITS_BITS    THREE_BITS
#define RD6_CREDITS_MASK    (RD6_CREDITS_BITS << RD6_CREDITS_LSB)

/* #define WR7_CREDITS  ; Write Credits for Rx Channel 7
 */
#define WR7_CREDITS_LSB     15
#define WR7_CREDITS_BITS    THREE_BITS
#define WR7_CREDITS_MASK    (WR7_CREDITS_BITS << WR7_CREDITS_LSB)

/* #define RD7_CREDITS  ; Read Credits for Tx Channel 7
 */
#define RD7_CREDITS_LSB     12
#define RD7_CREDITS_BITS    THREE_BITS
#define RD7_CREDITS_MASK    (RD7_CREDITS_BITS << RD7_CREDITS_LSB)

/* #define WR8_CREDITS  ; Write Credits for Rx Channel 8
 */
#define WR8_CREDITS_LSB     9
#define WR8_CREDITS_BITS    THREE_BITS
#define WR8_CREDITS_MASK    (WR8_CREDITS_BITS << WR8_CREDITS_LSB)

/* #define RD8_CREDITS  ; Read Credits for Tx Channel 8
 */
#define RD8_CREDITS_LSB     6
#define RD8_CREDITS_BITS    THREE_BITS
#define RD8_CREDITS_MASK    (RD8_CREDITS_BITS << RD8_CREDITS_LSB)

/* #define WR9_CREDITS  ; Write Credits for Rx Channel 9
 */
#define WR9_CREDITS_LSB     3
#define WR9_CREDITS_BITS    THREE_BITS
#define WR9_CREDITS_MASK    (WR9_CREDITS_BITS << WR9_CREDITS_LSB)

/* #define RD9_CREDITS  ; Read Credits for Tx Channel 9
 */
#define RD9_CREDITS_LSB     0
#define RD9_CREDITS_BITS    THREE_BITS
#define RD9_CREDITS_MASK    (RD9_CREDITS_BITS << RD9_CREDITS_LSB)

/*********************************************
 * DMA_CR_2 (Address Offset 0x202)
 *********************************************/

/* #define WR10_CREDITS  ; Write Credits for Rx Channel 10
 */
#define WR10_CREDITS_LSB     27
#define WR10_CREDITS_BITS    THREE_BITS
#define WR10_CREDITS_MASK    (WR10_CREDITS_BITS << WR10_CREDITS_LSB)

/* #define RD10_CREDITS  ; Read Credits for Tx Channel 10
 */
#define RD10_CREDITS_LSB     24
#define RD10_CREDITS_BITS    THREE_BITS
#define RD10_CREDITS_MASK    (RD10_CREDITS_BITS << RD10_CREDITS_LSB)

/* #define WR11_CREDITS  ; Write Credits for Rx Channel 11
 */
#define WR11_CREDITS_LSB     21
#define WR11_CREDITS_BITS    THREE_BITS
#define WR11_CREDITS_MASK    (WR11_CREDITS_BITS << WR11_CREDITS_LSB)

/* #define RD11_CREDITS  ; Read Credits for Tx Channel 11
 */
#define RD11_CREDITS_LSB     18
#define RD11_CREDITS_BITS    THREE_BITS
#define RD11_CREDITS_MASK    (RD11_CREDITS_BITS << RD11_CREDITS_LSB)

/* #define WR12_CREDITS  ; Write Credits for Rx Channel 12
 */
#define WR12_CREDITS_LSB     15
#define WR12_CREDITS_BITS    THREE_BITS
#define WR12_CREDITS_MASK    (WR12_CREDITS_BITS << WR12_CREDITS_LSB)

/* #define RD12_CREDITS  ; Read Credits for Tx Channel 12
 */
#define RD12_CREDITS_LSB     12
#define RD12_CREDITS_BITS    THREE_BITS
#define RD12_CREDITS_MASK    (RD12_CREDITS_BITS << RD12_CREDITS_LSB)

/* #define WR13_CREDITS  ; Write Credits for Rx Channel 13
 */
#define WR13_CREDITS_LSB     9
#define WR13_CREDITS_BITS    THREE_BITS
#define WR13_CREDITS_MASK    (WR13_CREDITS_BITS << WR13_CREDITS_LSB)

/* #define RD13_CREDITS  ; Read Credits for Tx Channel 13
 */
#define RD13_CREDITS_LSB     6
#define RD13_CREDITS_BITS    THREE_BITS
#define RD13_CREDITS_MASK    (RD13_CREDITS_BITS << RD13_CREDITS_LSB)

/* #define WR14_CREDITS  ; Write Credits for Rx Channel 14
 */
#define WR14_CREDITS_LSB     3
#define WR14_CREDITS_BITS    THREE_BITS
#define WR14_CREDITS_MASK    (WR14_CREDITS_BITS << WR14_CREDITS_LSB)

/* #define RD14_CREDITS  ; Read Credits for Tx Channel 14
 */
#define RD14_CREDITS_LSB     0
#define RD14_CREDITS_BITS    THREE_BITS
#define RD14_CREDITS_MASK    (RD14_CREDITS_BITS << RD14_CREDITS_LSB)

/*********************************************
 * DMA_CR_3 (Address Offset 0x203)
 *********************************************/

/* #define WR15_CREDITS  ; Write Credits for Rx Channel 15
 */
#define WR15_CREDITS_LSB     27
#define WR15_CREDITS_BITS    THREE_BITS
#define WR15_CREDITS_MASK    (WR15_CREDITS_BITS << WR15_CREDITS_LSB)

/* #define RD15_CREDITS  ; Read Credits for Tx Channel 15
 */
#define RD15_CREDITS_LSB     24
#define RD15_CREDITS_BITS    THREE_BITS
#define RD15_CREDITS_MASK    (RD15_CREDITS_BITS << RD15_CREDITS_LSB)

/* #define SP_CLASS_WR_CREDITS  ; Write Credits for Class Spill Channels
 */
#define SP_CLASS_WR_CREDITS_LSB     21
#define SP_CLASS_WR_CREDITS_BITS    THREE_BITS
#define SP_CLASS_WR_CREDITS_MASK    (SP_CLASS_WR_CREDITS_BITS << SP_CLASS_WR_CREDITS_LSB)

/* #define SP_CLASS_RD_CREDITS  ; Read Credits for Class Spill Channels
 */
#define SP_CLASS_RD_CREDITS_LSB     18
#define SP_CLASS_RD_CREDITS_BITS    THREE_BITS
#define SP_CLASS_RD_CREDITS_MASK    (SP_CLASS_RD_CREDITS_BITS << SP_CLASS_RD_CREDITS_LSB)

/* #define P2P_CHANNEL_RD_CREDITS  ; Read Credits for P2P Channel
 */
#define P2P_CHANNEL_RD_CREDITS_LSB     12
#define P2P_CHANNEL_RD_CREDITS_BITS    THREE_BITS
#define P2P_CHANNEL_RD_CREDITS_MASK    (P2P_CHANNEL_RD_CREDITS_BITS << P2P_CHANNEL_RD_CREDITS_LSB)

/* #define REG_FR_IN_WR_CREDITS  ; Write Credits for Regular Free In Channel
 */
#define REG_FR_IN_WR_CREDITS_LSB     9
#define REG_FR_IN_WR_CREDITS_BITS    THREE_BITS
#define REG_FR_IN_WR_CREDITS_MASK    (REG_FR_IN_WR_CREDITS_BITS << REG_FR_IN_WR_CREDITS_LSB)

/* #define REG_FR_IN_RD_CREDITS  ; Read Credits for Regular Free In Channel
 */
#define REG_FR_IN_RD_CREDITS_LSB     6
#define REG_FR_IN_RD_CREDITS_BITS    THREE_BITS
#define REG_FR_IN_RD_CREDITS_MASK    (REG_FR_IN_RD_CREDITS_BITS << REG_FR_IN_RD_CREDITS_LSB)

/* #define FR_OUT_WR_CREDITS  ; Write Credits for Free Out Channel
 */
#define FR_OUT_WR_CREDITS_LSB     3
#define FR_OUT_WR_CREDITS_BITS    THREE_BITS
#define FR_OUT_WR_CREDITS_MASK    (FR_OUT_WR_CREDITS_BITS << FR_OUT_WR_CREDITS_LSB)

/* #define FR_OUT_RD_CREDITS  ; Read Credits for Free Out Channel
 */
#define FR_OUT_RD_CREDITS_LSB     0
#define FR_OUT_RD_CREDITS_BITS    THREE_BITS
#define FR_OUT_RD_CREDITS_MASK    (FR_OUT_RD_CREDITS_BITS << FR_OUT_RD_CREDITS_LSB)

/**************************************************
 * REG_FRIN_SPILL_MEM_START_1 (Address Offset 0x205)
 * FROUT_SPILL_MEM_START_1 (Address Offset 0x208)
 * CLASS0_SPILL_MEM_START_1 (Address Offset 0x20B)
 * CLASS1_SPILL_MEM_START_1 (Address Offset 0x211)
 * CLASS2_SPILL_MEM_START_1 (Address Offset 0x214)
 * CLASS3_SPILL_MEM_START_1 (Address Offset 0x217)
 * REG_FRIN_1_SPILL_MEM_START_1 (Address Offset 0x21A)
 **************************************************/

/* #define RD_EXCLUSIVE  ; Cache Data will be evicted when read
 */
#define RD_EXCLUSIVE_LSB     5
#define RD_EXCLUSIVE_BITS    ONE_BIT
#define RD_EXCLUSIVE_MASK    (RD_EXCLUSIVE_BITS << RD_EXCLUSIVE_LSB)

/* #define L2_WR_ALLOC  ; L2 Allocate for write
 */
#define L2_WR_ALLOC_LSB     4
#define L2_WR_ALLOC_BITS    ONE_BIT
#define L2_WR_ALLOC_MASK    (L2_WR_ALLOC_BITS << L2_WR_ALLOC_LSB)

/* #define L2_RD_ALLOC  ; L2 Allocate for read 
 */
#define L2_RD_ALLOC_LSB     3
#define L2_RD_ALLOC_BITS    ONE_BIT
#define L2_RD_ALLOC_MASK    (L2_RD_ALLOC_BITS << L2_RD_ALLOC_LSB)

/* #define MEM_START_MSB  ; L2 Allocate for read 
 */
#define MEM_START_MSB_LSB     0
#define MEM_START_MSB_BITS    THREE_BITS
#define MEM_START_MSB_MASK    (MEM_START_MSB_BITS << MEM_START_MSB_LSB)

/***************************************************
 * SPI_HNGY_0 (Address Offset 0x219)
 **************************************************/

/* #define EG_HNGY_THRESH_0  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_0_LSB     24
#define EG_HNGY_THRESH_0_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_0_MASK    (EG_HNGY_THRESH_0_BITS << EG_HNGY_THRESH_0_LSB)

/* #define EG_HNGY_THRESH_1  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_1_LSB     16
#define EG_HNGY_THRESH_1_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_1_MASK    (EG_HNGY_THRESH_1_BITS << EG_HNGY_THRESH_1_LSB)

/* #define EG_HNGY_THRESH_2  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_2_LSB     8
#define EG_HNGY_THRESH_2_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_2_MASK    (EG_HNGY_THRESH_2_BITS << EG_HNGY_THRESH_2_LSB)

/* #define EG_HNGY_THRESH_3  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_3_LSB     0
#define EG_HNGY_THRESH_3_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_3_MASK    (EG_HNGY_THRESH_3_BITS << EG_HNGY_THRESH_3_LSB)

/***************************************************
 * SPI_HNGY_1 (Address Offset 0x21A)
 **************************************************/

/* #define EG_HNGY_THRESH_4  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_4_LSB     24
#define EG_HNGY_THRESH_4_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_4_MASK    (EG_HNGY_THRESH_4_BITS << EG_HNGY_THRESH_4_LSB)

/* #define EG_HNGY_THRESH_5  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_5_LSB     16
#define EG_HNGY_THRESH_5_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_5_MASK    (EG_HNGY_THRESH_5_BITS << EG_HNGY_THRESH_5_LSB)

/* #define EG_HNGY_THRESH_6  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_6_LSB     8
#define EG_HNGY_THRESH_6_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_6_MASK    (EG_HNGY_THRESH_6_BITS << EG_HNGY_THRESH_6_LSB)

/* #define EG_HNGY_THRESH_7  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_7_LSB     0
#define EG_HNGY_THRESH_7_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_7_MASK    (EG_HNGY_THRESH_7_BITS << EG_HNGY_THRESH_7_LSB)

/***************************************************
 * SPI_HNGY_2 (Address Offset 0x21B)
 **************************************************/

/* #define EG_HNGY_THRESH_8  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_8_LSB     24
#define EG_HNGY_THRESH_8_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_8_MASK    (EG_HNGY_THRESH_8_BITS << EG_HNGY_THRESH_8_LSB)

/* #define EG_HNGY_THRESH_9  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_9_LSB     16
#define EG_HNGY_THRESH_9_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_9_MASK    (EG_HNGY_THRESH_9_BITS << EG_HNGY_THRESH_9_LSB)

/* #define EG_HNGY_THRESH_10  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_10_LSB     8
#define EG_HNGY_THRESH_10_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_10_MASK    (EG_HNGY_THRESH_10_BITS << EG_HNGY_THRESH_10_LSB)

/* #define EG_HNGY_THRESH_11  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_11_LSB     0
#define EG_HNGY_THRESH_11_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_11_MASK    (EG_HNGY_THRESH_11_BITS << EG_HNGY_THRESH_11_LSB)

/***************************************************
 * SPI_HNGY_3 (Address Offset 0x21C)
 **************************************************/

/* #define EG_HNGY_THRESH_12  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_12_LSB     24
#define EG_HNGY_THRESH_12_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_12_MASK    (EG_HNGY_THRESH_12_BITS << EG_HNGY_THRESH_12_LSB)

/* #define EG_HNGY_THRESH_13  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_13_LSB     16
#define EG_HNGY_THRESH_13_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_13_MASK    (EG_HNGY_THRESH_13_BITS << EG_HNGY_THRESH_13_LSB)

/* #define EG_HNGY_THRESH_2  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_14_LSB     8
#define EG_HNGY_THRESH_14_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_14_MASK    (EG_HNGY_THRESH_14_BITS << EG_HNGY_THRESH_14_LSB)

/* #define EG_HNGY_THRESH_15  ; SPI4 Tx FIFO Hungry Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_HNGY_THRESH_15_LSB     0
#define EG_HNGY_THRESH_15_BITS    SEVEN_BITS
#define EG_HNGY_THRESH_15_MASK    (EG_HNGY_THRESH_15_BITS << EG_HNGY_THRESH_15_LSB)

/***************************************************
 * SPI_STRV_0 (Address Offset 0x21D)
 **************************************************/

/* #define EG_STRV_THRESH_0  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_0_LSB     24
#define EG_STRV_THRESH_0_BITS    SEVEN_BITS
#define EG_STRV_THRESH_0_MASK    (EG_STRV_THRESH_0_BITS << EG_STRV_THRESH_0_LSB)

/* #define EG_STRV_THRESH_1  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_1_LSB     16
#define EG_STRV_THRESH_1_BITS    SEVEN_BITS
#define EG_STRV_THRESH_1_MASK    (EG_STRV_THRESH_1_BITS << EG_STRV_THRESH_1_LSB)

/* #define EG_STRV_THRESH_2  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_2_LSB     8
#define EG_STRV_THRESH_2_BITS    SEVEN_BITS
#define EG_STRV_THRESH_2_MASK    (EG_STRV_THRESH_2_BITS << EG_STRV_THRESH_2_LSB)

/* #define EG_STRV_THRESH_3  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_3_LSB     0
#define EG_STRV_THRESH_3_BITS    SEVEN_BITS
#define EG_STRV_THRESH_3_MASK    (EG_STRV_THRESH_3_BITS << EG_STRV_THRESH_3_LSB)

/***************************************************
 * SPI_STRV_1 (Address Offset 0x21E)
 **************************************************/

/* #define EG_STRV_THRESH_4  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_4_LSB     24
#define EG_STRV_THRESH_4_BITS    SEVEN_BITS
#define EG_STRV_THRESH_4_MASK    (EG_STRV_THRESH_4_BITS << EG_STRV_THRESH_4_LSB)

/* #define EG_STRV_THRESH_5  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_5_LSB     16
#define EG_STRV_THRESH_5_BITS    SEVEN_BITS
#define EG_STRV_THRESH_5_MASK    (EG_STRV_THRESH_5_BITS << EG_STRV_THRESH_5_LSB)

/* #define EG_STRV_THRESH_6  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_6_LSB     8
#define EG_STRV_THRESH_6_BITS    SEVEN_BITS
#define EG_STRV_THRESH_6_MASK    (EG_STRV_THRESH_6_BITS << EG_STRV_THRESH_6_LSB)

/* #define EG_STRV_THRESH_7  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_7_LSB     0
#define EG_STRV_THRESH_7_BITS    SEVEN_BITS
#define EG_STRV_THRESH_7_MASK    (EG_STRV_THRESH_7_BITS << EG_STRV_THRESH_7_LSB)

/***************************************************
 * SPI_STRV_2 (Address Offset 0x21F)
 **************************************************/

/* #define EG_STRV_THRESH_8  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_8_LSB     24
#define EG_STRV_THRESH_8_BITS    SEVEN_BITS
#define EG_STRV_THRESH_8_MASK    (EG_STRV_THRESH_8_BITS << EG_STRV_THRESH_8_LSB)

/* #define EG_STRV_THRESH_9  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_9_LSB     16
#define EG_STRV_THRESH_9_BITS    SEVEN_BITS
#define EG_STRV_THRESH_9_MASK    (EG_STRV_THRESH_9_BITS << EG_STRV_THRESH_9_LSB)

/* #define EG_STRV_THRESH_10  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_10_LSB     8
#define EG_STRV_THRESH_10_BITS    SEVEN_BITS
#define EG_STRV_THRESH_10_MASK    (EG_STRV_THRESH_10_BITS << EG_STRV_THRESH_10_LSB)

/* #define EG_STRV_THRESH_11  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_11_LSB     0
#define EG_STRV_THRESH_11_BITS    SEVEN_BITS
#define EG_STRV_THRESH_11_MASK    (EG_STRV_THRESH_11_BITS << EG_STRV_THRESH_11_LSB)

/***************************************************
 * SPI_STRV_3 (Address Offset 0x220)
 **************************************************/

/* #define EG_STRV_THRESH_12  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_12_LSB     24
#define EG_STRV_THRESH_12_BITS    SEVEN_BITS
#define EG_STRV_THRESH_12_MASK    (EG_STRV_THRESH_12_BITS << EG_STRV_THRESH_12_LSB)

/* #define EG_STRV_THRESH_13  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_13_LSB     16
#define EG_STRV_THRESH_13_BITS    SEVEN_BITS
#define EG_STRV_THRESH_13_MASK    (EG_STRV_THRESH_13_BITS << EG_STRV_THRESH_13_LSB)

/* #define EG_STRV_THRESH_2  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_14_LSB     8
#define EG_STRV_THRESH_14_BITS    SEVEN_BITS
#define EG_STRV_THRESH_14_MASK    (EG_STRV_THRESH_14_BITS << EG_STRV_THRESH_14_LSB)

/* #define EG_STRV_THRESH_15  ; SPI4 Tx FIFO Starving Threshold. These are related to 0x08C in the SPI4 Interface
 */
#define EG_STRV_THRESH_15_LSB     0
#define EG_STRV_THRESH_15_BITS    SEVEN_BITS
#define EG_STRV_THRESH_15_MASK    (EG_STRV_THRESH_15_BITS << EG_STRV_THRESH_15_LSB)

/***************************************************
 * TX_DATA_FIFO_0 (Address Offset 0x221)
 **************************************************/

/* #define TX0_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX0_DATA_FIFO_START_LSB     24
#define TX0_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX0_DATA_FIFO_START_MASK    (TX0_DATA_FIFO_START_BITS << TX0_DATA_FIFO_START_LSB)

/* #define TX0_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX0_DATA_FIFO_SIZE_LSB     16
#define TX0_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX0_DATA_FIFO_SIZE_MASK    (TX0_DATA_FIFO_SIZE_BITS << TX0_DATA_FIFO_SIZE_LSB)

/* #define TX1_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX1_DATA_FIFO_START_LSB     8
#define TX1_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX1_DATA_FIFO_START_MASK    (TX1_DATA_FIFO_START_BITS << TX1_DATA_FIFO_START_LSB)

/* #define TX1_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX1_DATA_FIFO_SIZE_LSB     0
#define TX1_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX1_DATA_FIFO_SIZE_MASK    (TX1_DATA_FIFO_SIZE_BITS << TX1_DATA_FIFO_SIZE_LSB)

/***************************************************
 * TX_DATA_FIFO_1 (Address Offset 0x222)
 **************************************************/

/* #define TX2_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX2_DATA_FIFO_START_LSB     24
#define TX2_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX2_DATA_FIFO_START_MASK    (TX2_DATA_FIFO_START_BITS << TX2_DATA_FIFO_START_LSB)

/* #define TX2_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX2_DATA_FIFO_SIZE_LSB     16
#define TX2_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX2_DATA_FIFO_SIZE_MASK    (TX2_DATA_FIFO_SIZE_BITS << TX2_DATA_FIFO_SIZE_LSB)

/* #define TX3_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX3_DATA_FIFO_START_LSB     8
#define TX3_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX3_DATA_FIFO_START_MASK    (TX3_DATA_FIFO_START_BITS << TX3_DATA_FIFO_START_LSB)

/* #define TX3_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX3_DATA_FIFO_SIZE_LSB     0
#define TX3_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX3_DATA_FIFO_SIZE_MASK    (TX3_DATA_FIFO_SIZE_BITS << TX3_DATA_FIFO_SIZE_LSB)

/***************************************************
 * TX_DATA_FIFO_2 (Address Offset 0x223)
 **************************************************/

/* #define TX4_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX4_DATA_FIFO_START_LSB     24
#define TX4_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX4_DATA_FIFO_START_MASK    (TX4_DATA_FIFO_START_BITS << TX4_DATA_FIFO_START_LSB)

/* #define TX4_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX4_DATA_FIFO_SIZE_LSB     16
#define TX4_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX4_DATA_FIFO_SIZE_MASK    (TX4_DATA_FIFO_SIZE_BITS << TX4_DATA_FIFO_SIZE_LSB)

/* #define TX5_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX5_DATA_FIFO_START_LSB     8
#define TX5_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX5_DATA_FIFO_START_MASK    (TX5_DATA_FIFO_START_BITS << TX5_DATA_FIFO_START_LSB)

/* #define TX5_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX5_DATA_FIFO_SIZE_LSB     0
#define TX5_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX5_DATA_FIFO_SIZE_MASK    (TX5_DATA_FIFO_SIZE_BITS << TX5_DATA_FIFO_SIZE_LSB)

/***************************************************
 * TX_DATA_FIFO_3 (Address Offset 0x224)
 **************************************************/

/* #define TX6_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX6_DATA_FIFO_START_LSB     24
#define TX6_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX6_DATA_FIFO_START_MASK    (TX6_DATA_FIFO_START_BITS << TX6_DATA_FIFO_START_LSB)

/* #define TX6_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX6_DATA_FIFO_SIZE_LSB     16
#define TX6_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX6_DATA_FIFO_SIZE_MASK    (TX6_DATA_FIFO_SIZE_BITS << TX6_DATA_FIFO_SIZE_LSB)

/* #define TX7_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX7_DATA_FIFO_START_LSB     8
#define TX7_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX7_DATA_FIFO_START_MASK    (TX7_DATA_FIFO_START_BITS << TX7_DATA_FIFO_START_LSB)

/* #define TX7_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX7_DATA_FIFO_SIZE_LSB     0
#define TX7_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX7_DATA_FIFO_SIZE_MASK    (TX7_DATA_FIFO_SIZE_BITS << TX7_DATA_FIFO_SIZE_LSB)

/***************************************************
 * TX_DATA_FIFO_4 (Address Offset 0x225)
 **************************************************/

/* #define TX8_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX8_DATA_FIFO_START_LSB     24
#define TX8_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX8_DATA_FIFO_START_MASK    (TX8_DATA_FIFO_START_BITS << TX8_DATA_FIFO_START_LSB)

/* #define TX8_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX8_DATA_FIFO_SIZE_LSB     16
#define TX8_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX8_DATA_FIFO_SIZE_MASK    (TX8_DATA_FIFO_SIZE_BITS << TX8_DATA_FIFO_SIZE_LSB)

/* #define TX9_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX9_DATA_FIFO_START_LSB     8
#define TX9_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX9_DATA_FIFO_START_MASK    (TX9_DATA_FIFO_START_BITS << TX9_DATA_FIFO_START_LSB)

/* #define TX9_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX9_DATA_FIFO_SIZE_LSB     0
#define TX9_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX9_DATA_FIFO_SIZE_MASK    (TX9_DATA_FIFO_SIZE_BITS << TX9_DATA_FIFO_SIZE_LSB)

/***************************************************
 * TX_DATA_FIFO_5 (Address Offset 0x226)
 **************************************************/

/* #define TX10_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX10_DATA_FIFO_START_LSB     24
#define TX10_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX10_DATA_FIFO_START_MASK    (TX10_DATA_FIFO_START_BITS << TX10_DATA_FIFO_START_LSB)

/* #define TX10_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX10_DATA_FIFO_SIZE_LSB     16
#define TX10_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX10_DATA_FIFO_SIZE_MASK    (TX10_DATA_FIFO_SIZE_BITS << TX10_DATA_FIFO_SIZE_LSB)

/* #define TX11_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX11_DATA_FIFO_START_LSB     8
#define TX11_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX11_DATA_FIFO_START_MASK    (TX11_DATA_FIFO_START_BITS << TX11_DATA_FIFO_START_LSB)

/* #define TX11_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX11_DATA_FIFO_SIZE_LSB     0
#define TX11_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX11_DATA_FIFO_SIZE_MASK    (TX11_DATA_FIFO_SIZE_BITS << TX11_DATA_FIFO_SIZE_LSB)

/***************************************************
 * TX_DATA_FIFO_6 (Address Offset 0x227)
 **************************************************/

/* #define TX6_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX12_DATA_FIFO_START_LSB     24
#define TX12_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX12_DATA_FIFO_START_MASK    (TX12_DATA_FIFO_START_BITS << TX12_DATA_FIFO_START_LSB)

/* #define TX6_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX12_DATA_FIFO_SIZE_LSB     16
#define TX12_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX12_DATA_FIFO_SIZE_MASK    (TX12_DATA_FIFO_SIZE_BITS << TX12_DATA_FIFO_SIZE_LSB)

/* #define TX13_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX13_DATA_FIFO_START_LSB     8
#define TX13_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX13_DATA_FIFO_START_MASK    (TX13_DATA_FIFO_START_BITS << TX13_DATA_FIFO_START_LSB)

/* #define TX13_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX13_DATA_FIFO_SIZE_LSB     0
#define TX13_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX13_DATA_FIFO_SIZE_MASK    (TX13_DATA_FIFO_SIZE_BITS << TX13_DATA_FIFO_SIZE_LSB)

/***************************************************
 * TX_DATA_FIFO_7 (Address Offset 0x228)
 **************************************************/

/* #define TX14_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX14_DATA_FIFO_START_LSB     24
#define TX14_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX14_DATA_FIFO_START_MASK    (TX14_DATA_FIFO_START_BITS << TX14_DATA_FIFO_START_LSB)

/* #define TX14_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX14_DATA_FIFO_SIZE_LSB     16
#define TX14_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX14_DATA_FIFO_SIZE_MASK    (TX14_DATA_FIFO_SIZE_BITS << TX14_DATA_FIFO_SIZE_LSB)

/* #define TX15_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define TX15_DATA_FIFO_START_LSB     8
#define TX15_DATA_FIFO_START_BITS    SEVEN_BITS
#define TX15_DATA_FIFO_START_MASK    (TX15_DATA_FIFO_START_BITS << TX15_DATA_FIFO_START_LSB)

/* #define TX15_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define TX15_DATA_FIFO_SIZE_LSB     0
#define TX15_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define TX15_DATA_FIFO_SIZE_MASK    (TX15_DATA_FIFO_SIZE_BITS << TX15_DATA_FIFO_SIZE_LSB)

/***************************************************
 * RX_DATA_FIFO_0 (Address Offset 0x229)
 **************************************************/

/* #define RX0_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX0_DATA_FIFO_START_LSB     24
#define RX0_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX0_DATA_FIFO_START_MASK    (RX0_DATA_FIFO_START_BITS << RX0_DATA_FIFO_START_LSB)

/* #define RX0_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX0_DATA_FIFO_SIZE_LSB     16
#define RX0_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX0_DATA_FIFO_SIZE_MASK    (RX0_DATA_FIFO_SIZE_BITS << RX0_DATA_FIFO_SIZE_LSB)

/* #define RX1_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX1_DATA_FIFO_START_LSB     8
#define RX1_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX1_DATA_FIFO_START_MASK    (RX1_DATA_FIFO_START_BITS << RX1_DATA_FIFO_START_LSB)

/* #define RX1_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX1_DATA_FIFO_SIZE_LSB     0
#define RX1_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX1_DATA_FIFO_SIZE_MASK    (RX1_DATA_FIFO_SIZE_BITS << RX1_DATA_FIFO_SIZE_LSB)

/***************************************************
 * RX_DATA_FIFO_1 (Address Offset 0x22A)
 **************************************************/

/* #define RX2_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX2_DATA_FIFO_START_LSB     24
#define RX2_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX2_DATA_FIFO_START_MASK    (RX2_DATA_FIFO_START_BITS << RX2_DATA_FIFO_START_LSB)

/* #define RX2_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX2_DATA_FIFO_SIZE_LSB     16
#define RX2_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX2_DATA_FIFO_SIZE_MASK    (RX2_DATA_FIFO_SIZE_BITS << RX2_DATA_FIFO_SIZE_LSB)

/* #define RX3_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX3_DATA_FIFO_START_LSB     8
#define RX3_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX3_DATA_FIFO_START_MASK    (RX3_DATA_FIFO_START_BITS << RX3_DATA_FIFO_START_LSB)

/* #define RX3_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX3_DATA_FIFO_SIZE_LSB     0
#define RX3_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX3_DATA_FIFO_SIZE_MASK    (RX3_DATA_FIFO_SIZE_BITS << RX3_DATA_FIFO_SIZE_LSB)

/***************************************************
 * RX_DATA_FIFO_2 (Address Offset 0x22B)
 **************************************************/

/* #define RX4_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX4_DATA_FIFO_START_LSB     24
#define RX4_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX4_DATA_FIFO_START_MASK    (RX4_DATA_FIFO_START_BITS << RX4_DATA_FIFO_START_LSB)

/* #define RX4_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX4_DATA_FIFO_SIZE_LSB     16
#define RX4_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX4_DATA_FIFO_SIZE_MASK    (RX4_DATA_FIFO_SIZE_BITS << RX4_DATA_FIFO_SIZE_LSB)

/* #define RX5_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX5_DATA_FIFO_START_LSB     8
#define RX5_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX5_DATA_FIFO_START_MASK    (RX5_DATA_FIFO_START_BITS << RX5_DATA_FIFO_START_LSB)

/* #define RX5_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX5_DATA_FIFO_SIZE_LSB     0
#define RX5_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX5_DATA_FIFO_SIZE_MASK    (RX5_DATA_FIFO_SIZE_BITS << RX5_DATA_FIFO_SIZE_LSB)

/***************************************************
 * RX_DATA_FIFO_3 (Address Offset 0x22C)
 **************************************************/

/* #define RX6_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX6_DATA_FIFO_START_LSB     24
#define RX6_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX6_DATA_FIFO_START_MASK    (RX6_DATA_FIFO_START_BITS << RX6_DATA_FIFO_START_LSB)

/* #define RX6_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX6_DATA_FIFO_SIZE_LSB     16
#define RX6_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX6_DATA_FIFO_SIZE_MASK    (RX6_DATA_FIFO_SIZE_BITS << RX6_DATA_FIFO_SIZE_LSB)

/* #define RX7_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX7_DATA_FIFO_START_LSB     8
#define RX7_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX7_DATA_FIFO_START_MASK    (RX7_DATA_FIFO_START_BITS << RX7_DATA_FIFO_START_LSB)

/* #define RX7_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX7_DATA_FIFO_SIZE_LSB     0
#define RX7_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX7_DATA_FIFO_SIZE_MASK    (RX7_DATA_FIFO_SIZE_BITS << RX7_DATA_FIFO_SIZE_LSB)

/***************************************************
 * RX_DATA_FIFO_4 (Address Offset 0x22D)
 **************************************************/

/* #define RX8_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX8_DATA_FIFO_START_LSB     24
#define RX8_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX8_DATA_FIFO_START_MASK    (RX8_DATA_FIFO_START_BITS << RX8_DATA_FIFO_START_LSB)

/* #define RX8_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX8_DATA_FIFO_SIZE_LSB     16
#define RX8_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX8_DATA_FIFO_SIZE_MASK    (RX8_DATA_FIFO_SIZE_BITS << RX8_DATA_FIFO_SIZE_LSB)

/* #define RX9_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX9_DATA_FIFO_START_LSB     8
#define RX9_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX9_DATA_FIFO_START_MASK    (RX9_DATA_FIFO_START_BITS << RX9_DATA_FIFO_START_LSB)

/* #define RX9_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX9_DATA_FIFO_SIZE_LSB     0
#define RX9_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX9_DATA_FIFO_SIZE_MASK    (RX9_DATA_FIFO_SIZE_BITS << RX9_DATA_FIFO_SIZE_LSB)

/***************************************************
 * RX_DATA_FIFO_5 (Address Offset 0x22E)
 **************************************************/

/* #define RX10_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX10_DATA_FIFO_START_LSB     24
#define RX10_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX10_DATA_FIFO_START_MASK    (RX10_DATA_FIFO_START_BITS << RX10_DATA_FIFO_START_LSB)

/* #define RX10_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX10_DATA_FIFO_SIZE_LSB     16
#define RX10_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX10_DATA_FIFO_SIZE_MASK    (RX10_DATA_FIFO_SIZE_BITS << RX10_DATA_FIFO_SIZE_LSB)

/* #define RX11_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX11_DATA_FIFO_START_LSB     8
#define RX11_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX11_DATA_FIFO_START_MASK    (RX11_DATA_FIFO_START_BITS << RX11_DATA_FIFO_START_LSB)

/* #define RX11_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX11_DATA_FIFO_SIZE_LSB     0
#define RX11_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX11_DATA_FIFO_SIZE_MASK    (RX11_DATA_FIFO_SIZE_BITS << RX11_DATA_FIFO_SIZE_LSB)

/***************************************************
 * RX_DATA_FIFO_6 (Address Offset 0x22F)
 **************************************************/

/* #define RX6_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX12_DATA_FIFO_START_LSB     24
#define RX12_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX12_DATA_FIFO_START_MASK    (RX12_DATA_FIFO_START_BITS << RX12_DATA_FIFO_START_LSB)

/* #define RX6_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX12_DATA_FIFO_SIZE_LSB     16
#define RX12_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX12_DATA_FIFO_SIZE_MASK    (RX12_DATA_FIFO_SIZE_BITS << RX12_DATA_FIFO_SIZE_LSB)

/* #define RX13_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX13_DATA_FIFO_START_LSB     8
#define RX13_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX13_DATA_FIFO_START_MASK    (RX13_DATA_FIFO_START_BITS << RX13_DATA_FIFO_START_LSB)

/* #define RX13_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX13_DATA_FIFO_SIZE_LSB     0
#define RX13_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX13_DATA_FIFO_SIZE_MASK    (RX13_DATA_FIFO_SIZE_BITS << RX13_DATA_FIFO_SIZE_LSB)

/***************************************************
 * RX_DATA_FIFO_7 (Address Offset 0x230)
 **************************************************/

/* #define RX14_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX14_DATA_FIFO_START_LSB     24
#define RX14_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX14_DATA_FIFO_START_MASK    (RX14_DATA_FIFO_START_BITS << RX14_DATA_FIFO_START_LSB)

/* #define RX14_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX14_DATA_FIFO_SIZE_LSB     16
#define RX14_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX14_DATA_FIFO_SIZE_MASK    (RX14_DATA_FIFO_SIZE_BITS << RX14_DATA_FIFO_SIZE_LSB)

/* #define RX15_DATA_FIFO_START  ; SPI4 Tx Data FIFO Start address 
 */
#define RX15_DATA_FIFO_START_LSB     8
#define RX15_DATA_FIFO_START_BITS    SEVEN_BITS
#define RX15_DATA_FIFO_START_MASK    (RX15_DATA_FIFO_START_BITS << RX15_DATA_FIFO_START_LSB)

/* #define RX15_DATA_FIFO_SIZE  ; SPI4 Tx Daa FIFO size 
 */
#define RX15_DATA_FIFO_SIZE_LSB     0
#define RX15_DATA_FIFO_SIZE_BITS    SEVEN_BITS
#define RX15_DATA_FIFO_SIZE_MASK    (RX15_DATA_FIFO_SIZE_BITS << RX15_DATA_FIFO_SIZE_LSB)

/***************************************************
 * PAD_CALIBRATION_0 (Address Offset 0x231)
 **************************************************/

/* #define LVDS_TENLMNATION  ; Set LVDS Tenlmnation on
 */
#define LVDS_TENLMNATION_LSB     13
#define LVDS_TENLMNATION_BITS    ONE_BIT
#define LVDS_TENLMNATION_MASK    (LVDS_TENLMNATION_BITS << LVDS_TENLMNATION_LSB)

/* #define HSTL_TENLMNATION  ; Set HSTL Tenlmnation on
 */
#define HSTL_TENLMNATION_LSB     12
#define HSTL_TENLMNATION_BITS    ONE_BIT
#define HSTL_TENLMNATION_MASK    (HSTL_TENLMNATION_BITS << HSTL_TENLMNATION_LSB)

/* #define CAL_PRESET  ; Drive Strength on Pads preset to this value
 */
#define CAL_PRESET_LSB     6
#define CAL_PRESET_BITS    SIX_BITS
#define CAL_PRESET_MASK    (CAL_PRESET_BITS << CAL_PRESET_LSB)

/* #define TX_EN_COUNT  ; This will allow Tx Signals to get automatic drive strength
 */
#define TX_EN_COUNT_LSB     5
#define TX_EN_COUNT_BITS    ONE_BIT
#define TX_EN_COUNT_MASK    (TX_EN_COUNT_BITS << TX_EN_COUNT_LSB)

/* #define RX_EN_COUNT  ; This will allow Rx Signals to get automatic drive strength
 */
#define RX_EN_COUNT_LSB     4
#define RX_EN_COUNT_BITS    ONE_BIT
#define RX_EN_COUNT_MASK    (RX_EN_COUNT_BITS << RX_EN_COUNT_LSB)

/* #define TX_N_PRESET  ; This will override automatic drive strength with CAL_PRESET
 */
#define TX_N_PRESET_LSB     3
#define TX_N_PRESET_BITS    ONE_BIT
#define TX_N_PRESET_MASK    (TX_N_PRESET_BITS << TX_N_PRESET_LSB)

/* #define TX_P_PRESET  ; This will override automatic drive strength with CAL_PRESET
 */
#define TX_P_PRESET_LSB     2
#define TX_P_PRESET_BITS    ONE_BIT
#define TX_P_PRESET_MASK    (TX_P_PRESET_BITS << TX_P_PRESET_LSB)

/* #define RX_N_PRESET  ; This will override automatic drive strength with CAL_PRESET
 */
#define RX_N_PRESET_LSB     1
#define RX_N_PRESET_BITS    ONE_BIT
#define RX_N_PRESET_MASK    (RX_N_PRESET_BITS << RX_N_PRESET_LSB)

/* #define RX_P_PRESET  ; This will override automatic drive strength with CAL_PRESET
 */
#define RX_P_PRESET_LSB     0
#define RX_P_PRESET_BITS    ONE_BIT
#define RX_P_PRESET_MASK    (RX_P_PRESET_BITS << RX_P_PRESET_LSB)

/***************************************************
 * PAD_CALIBRATION_1 (Address Offset 0x232)
 **************************************************/

/* #define NF_CODE  ; Read Back Drive Strength
 */
#define NF_CODE_LSB     18
#define NF_CODE_BITS    SIX_BITS
#define NF_CODE_MASK    (NF_CODE_BITS << NF_CODE_LSB)

/* #define PF_CODE  ; Read Back Drive Strength
 */
#define PF_CODE_LSB     12
#define PF_CODE_BITS    SIX_BITS
#define PF_CODE_MASK    (PF_CODE_BITS << PF_CODE_LSB)

/* #define N_TERM_CODE  ; Read Back Tenlmnation Resistance
 */
#define N_TERM_CODE_LSB     6
#define N_TERM_CODE_BITS    SIX_BITS
#define N_TERM_CODE_MASK    (N_TERM_CODE_BITS << N_TERM_CODE_LSB)

/* #define P_TERM_CODE  ; Read Back Tenlmnation Resistance
 */
#define P_TERM_CODE_LSB     0
#define P_TERM_CODE_BITS    SIX_BITS
#define P_TERM_CODE_MASK    (P_TERM_CODE_BITS << P_TERM_CODE_LSB)

/***************************************************
 * FREEQ_CARVE (Address Offset 0x233)
 **************************************************/

/* #define MAX_MTU_SIZE  ; Used only for SPI4, for XGMII check register 0x08 and RGMII check register 0x04
 */
#define MAX_MTU_SIZE_LSB     5
#define MAX_MTU_SIZE_BITS    FOURTEEN_BITS
#define MAX_MTU_SIZE_MASK    (MAX_MTU_SIZE_BITS << MAX_MTU_SIZE_LSB)

/* #define FREEOUTQ_CARVING_MODE  ; Used only for GMACs
 */
#define FREEOUTQ_CARVING_MODE_LSB     6
#define FREEOUTQ_CARVING_MODE_BITS    ONE_BIT
#define FREEOUTQ_CARVING_MODE_MASK    (FREEOUTQ_CARVING_MODE_BITS << FREEOUTQ_CARVING_MODE_LSB)

/* #define FREEQ_CARVING_MODE  ; Used only for GMACs
 */
#define FREEQ_CARVING_MODE_LSB     5
#define FREEQ_CARVING_MODE_BITS    ONE_BIT
#define FREEQ_CARVING_MODE_MASK    (FREEQ_CARVING_MODE_BITS << FREEQ_CARVING_MODE_LSB)

/* #define MAX_UNDERRUN_RETRY_COUNT  ; Max number of Retries after which packet is aborted
 */
#define MAX_UNDERRUN_RETRY_COUNT_LSB     0
#define MAX_UNDERRUN_RETRY_COUNT_BITS    FIVE_BITS
#define MAX_UNDERRUN_RETRY_COUNT_MASK    (MAX_UNDERRUN_RETRY_COUNT_BITS << MAX_UNDERRUN_RETRY_COUNT_LSB)

/***************************************************
 * SPI4_STATIC_DELAY_0 (Address Offset 0x240)
 **************************************************/

/* #define DATA_LINE_7_DELAY  ; Static Delay Value
 */
#define DATA_LINE_7_DELAY_LSB     28
#define DATA_LINE_7_DELAY_BITS    FOUR_BITS
#define DATA_LINE_7_DELAY_MASK    (DATA_LINE_7_DELAY_BITS << DATA_LINE_7_DELAY_LSB)

/* #define DATA_LINE_6_DELAY  ; Static Delay Value
 */
#define DATA_LINE_6_DELAY_LSB     24
#define DATA_LINE_6_DELAY_BITS    FOUR_BITS
#define DATA_LINE_6_DELAY_MASK    (DATA_LINE_6_DELAY_BITS << DATA_LINE_6_DELAY_LSB)

/* #define DATA_LINE_5_DELAY  ; Static Delay Value
 */
#define DATA_LINE_5_DELAY_LSB     20
#define DATA_LINE_5_DELAY_BITS    FOUR_BITS
#define DATA_LINE_5_DELAY_MASK    (DATA_LINE_5_DELAY_BITS << DATA_LINE_5_DELAY_LSB)

/* #define DATA_LINE_4_DELAY  ; Static Delay Value
 */
#define DATA_LINE_4_DELAY_LSB     16
#define DATA_LINE_4_DELAY_BITS    FOUR_BITS
#define DATA_LINE_4_DELAY_MASK    (DATA_LINE_4_DELAY_BITS << DATA_LINE_4_DELAY_LSB)

/* #define DATA_LINE_3_DELAY  ; Static Delay Value
 */
#define DATA_LINE_3_DELAY_LSB     12
#define DATA_LINE_3_DELAY_BITS    FOUR_BITS
#define DATA_LINE_3_DELAY_MASK    (DATA_LINE_3_DELAY_BITS << DATA_LINE_3_DELAY_LSB)

/* #define DATA_LINE_2_DELAY  ; Static Delay Value
 */
#define DATA_LINE_2_DELAY_LSB     8
#define DATA_LINE_2_DELAY_BITS    FOUR_BITS
#define DATA_LINE_2_DELAY_MASK    (DATA_LINE_2_DELAY_BITS << DATA_LINE_2_DELAY_LSB)

/* #define DATA_LINE_1_DELAY  ; Static Delay Value
 */
#define DATA_LINE_1_DELAY_LSB     4
#define DATA_LINE_1_DELAY_BITS    FOUR_BITS
#define DATA_LINE_1_DELAY_MASK    (DATA_LINE_1_DELAY_BITS << DATA_LINE_1_DELAY_LSB)

/* #define DATA_LINE_0_DELAY  ; Static Delay Value
 */
#define DATA_LINE_0_DELAY_LSB     0
#define DATA_LINE_0_DELAY_BITS    FOUR_BITS
#define DATA_LINE_0_DELAY_MASK    (DATA_LINE_0_DELAY_BITS << DATA_LINE_0_DELAY_LSB)

/***************************************************
 * SPI4_STATIC_DELAY_1 (Address Offset 0x241)
 **************************************************/

/* #define DATA_LINE_15_DELAY  ; Static Delay Value
 */
#define DATA_LINE_15_DELAY_LSB     28
#define DATA_LINE_15_DELAY_BITS    FOUR_BITS
#define DATA_LINE_15_DELAY_MASK    (DATA_LINE_15_DELAY_BITS << DATA_LINE_15_DELAY_LSB)

/* #define DATA_LINE_14_DELAY  ; Static Delay Value
 */
#define DATA_LINE_14_DELAY_LSB     24
#define DATA_LINE_14_DELAY_BITS    FOUR_BITS
#define DATA_LINE_14_DELAY_MASK    (DATA_LINE_14_DELAY_BITS << DATA_LINE_14_DELAY_LSB)

/* #define DATA_LINE_13_DELAY  ; Static Delay Value
 */
#define DATA_LINE_13_DELAY_LSB     20
#define DATA_LINE_13_DELAY_BITS    FOUR_BITS
#define DATA_LINE_13_DELAY_MASK    (DATA_LINE_13_DELAY_BITS << DATA_LINE_13_DELAY_LSB)

/* #define DATA_LINE_12_DELAY  ; Static Delay Value
 */
#define DATA_LINE_12_DELAY_LSB     16
#define DATA_LINE_12_DELAY_BITS    FOUR_BITS
#define DATA_LINE_12_DELAY_MASK    (DATA_LINE_12_DELAY_BITS << DATA_LINE_12_DELAY_LSB)

/* #define DATA_LINE_11_DELAY  ; Static Delay Value
 */
#define DATA_LINE_11_DELAY_LSB     12
#define DATA_LINE_11_DELAY_BITS    FOUR_BITS
#define DATA_LINE_11_DELAY_MASK    (DATA_LINE_11_DELAY_BITS << DATA_LINE_11_DELAY_LSB)

/* #define DATA_LINE_10_DELAY  ; Static Delay Value
 */
#define DATA_LINE_10_DELAY_LSB     8
#define DATA_LINE_10_DELAY_BITS    FOUR_BITS
#define DATA_LINE_10_DELAY_MASK    (DATA_LINE_10_DELAY_BITS << DATA_LINE_10_DELAY_LSB)

/* #define DATA_LINE_9_DELAY  ; Static Delay Value
 */
#define DATA_LINE_9_DELAY_LSB     4
#define DATA_LINE_9_DELAY_BITS    FOUR_BITS
#define DATA_LINE_9_DELAY_MASK    (DATA_LINE_9_DELAY_BITS << DATA_LINE_9_DELAY_LSB)

/* #define DATA_LINE_8_DELAY  ; Static Delay Value
 */
#define DATA_LINE_8_DELAY_LSB     0
#define DATA_LINE_8_DELAY_BITS    FOUR_BITS
#define DATA_LINE_8_DELAY_MASK    (DATA_LINE_8_DELAY_BITS << DATA_LINE_8_DELAY_LSB)

/***************************************************
 * SPI4_STATIC_DELAY_STATUS (Address Offset 0x242)
 **************************************************/

/* #define TX_STAT_1_DELAY  ; Static Delay Value
 */
#define TX_STAT_1_DELAY_LSB     8
#define TX_STAT_1_DELAY_BITS    FOUR_BITS
#define TX_STAT_1_DELAY_MASK    (TX_STAT_1_DELAY_BITS << TX_STAT_1_DELAY_LSB)

/* #define TX_STAT_0_DELAY  ; Static Delay Value
 */
#define TX_STAT_0_DELAY_LSB     4
#define TX_STAT_0_DELAY_BITS    FOUR_BITS
#define TX_STAT_0_DELAY_MASK    (TX_STAT_0_DELAY_BITS << TX_STAT_0_DELAY_LSB)

/* #define RX_CONTROL_DELAY  ; Static Delay Value
 */
#define RX_CONTROL_DELAY_LSB     0
#define RX_CONTROL_DELAY_BITS    FOUR_BITS
#define RX_CONTROL_DELAY_MASK    (RX_CONTROL_DELAY_BITS << RX_CONTROL_DELAY_LSB)

/***************************************************
 * SPI4_CONTROL (Address Offset 0x243)
 **************************************************/

/* #define STATIC_DELAY  ; Static Delay Enable
 */
#define STATIC_DELAY_LSB     2
#define STATIC_DELAY_BITS    ONE_BIT
#define STATIC_DELAY_MASK    (STATIC_DELAY_BITS << STATIC_DELAY_LSB)

/* #define LVDS_LVTTL  ; LVDS mode, this is read only
 */
#define LVDS_LVTTL_LSB     1
#define LVDS_LVTTL_BITS    ONE_BIT
#define LVDS_LVTTL_MASK    (LVDS_LVTTL_BITS << LVDS_LVTTL_LSB)

/* #define SPI4_ENABLE  ; Spi4 Enable, this is read only
 */
#define SPI4_ENABLE_LSB     0
#define SPI4_ENABLE_BITS    ONE_BIT
#define SPI4_ENABLE_MASK    (SPI4_ENABLE_BITS << SPI4_ENABLE_LSB)

/***************************************************
 * CLASS_WATERMARKS (Address Offset 0x244)
 **************************************************/

/* #define CLASS_0_WATERMARK  ; Watermark for Class FIFO in units of Descriptors (64 bit entries)
 */
#define CLASS_0_WATERMARK_LSB     24
#define CLASS_0_WATERMARK_BITS    FIVE_BITS
#define CLASS_0_WATERMARK_MASK    (CLASS_0_WATERMARK_BITS << CLASS_0_WATERMARK_LSB)

/* #define CLASS_1_WATERMARK  ; Watermark for Class FIFO in units of Descriptors (64 bit entries)
 */
#define CLASS_1_WATERMARK_LSB     16
#define CLASS_1_WATERMARK_BITS    FIVE_BITS
#define CLASS_1_WATERMARK_MASK    (CLASS_1_WATERMARK_BITS << CLASS_1_WATERMARK_LSB)

/* #define CLASS_2_WATERMARK  ; Watermark for Class FIFO in units of Descriptors (64 bit entries)
 */
#define CLASS_2_WATERMARK_LSB     8
#define CLASS_2_WATERMARK_BITS    FIVE_BITS
#define CLASS_2_WATERMARK_MASK    (CLASS_2_WATERMARK_BITS << CLASS_2_WATERMARK_LSB)

/* #define CLASS_3_WATERMARK  ; Watermark for Class FIFO in units of Descriptors (64 bit entries)
 */
#define CLASS_3_WATERMARK_LSB     0
#define CLASS_3_WATERMARK_BITS    FIVE_BITS
#define CLASS_3_WATERMARK_MASK    (CLASS_3_WATERMARK_BITS << CLASS_3_WATERMARK_LSB)

/***************************************************
 * RX_WATERMARKS_0 (Address Offset 0x245)
 **************************************************/

/* #define RX_0_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_0_DATA_WATERMARK_LSB     24
#define RX_0_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_0_DATA_WATERMARK_MASK    (RX_0_DATA_WATERMARK_BITS << RX_0_DATA_WATERMARK_LSB)

/* #define RX_1_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_1_DATA_WATERMARK_LSB     16
#define RX_1_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_1_DATA_WATERMARK_MASK    (RX_1_DATA_WATERMARK_BITS << RX_1_DATA_WATERMARK_LSB)

/* #define RX_2_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_2_DATA_WATERMARK_LSB     8
#define RX_2_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_2_DATA_WATERMARK_MASK    (RX_2_DATA_WATERMARK_BITS << RX_2_DATA_WATERMARK_LSB)

/* #define RX_3_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_3_DATA_WATERMARK_LSB     0
#define RX_3_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_3_DATA_WATERMARK_MASK    (RX_3_DATA_WATERMARK_BITS << RX_3_DATA_WATERMARK_LSB)

/***************************************************
 * RX_WATERMARKS_1 (Address Offset 0x246)
 **************************************************/

/* #define RX_4_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_4_DATA_WATERMARK_LSB     24
#define RX_4_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_4_DATA_WATERMARK_MASK    (RX_4_DATA_WATERMARK_BITS << RX_4_DATA_WATERMARK_LSB)

/* #define RX_5_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_5_DATA_WATERMARK_LSB     16
#define RX_5_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_5_DATA_WATERMARK_MASK    (RX_5_DATA_WATERMARK_BITS << RX_5_DATA_WATERMARK_LSB)

/* #define RX_6_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_6_DATA_WATERMARK_LSB     8
#define RX_6_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_6_DATA_WATERMARK_MASK    (RX_6_DATA_WATERMARK_BITS << RX_6_DATA_WATERMARK_LSB)

/* #define RX_7_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_7_DATA_WATERMARK_LSB     0
#define RX_7_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_7_DATA_WATERMARK_MASK    (RX_7_DATA_WATERMARK_BITS << RX_7_DATA_WATERMARK_LSB)

/***************************************************
 * RX_WATERMARKS_2 (Address Offset 0x247)
 **************************************************/

/* #define RX_8_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_8_DATA_WATERMARK_LSB     24
#define RX_8_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_8_DATA_WATERMARK_MASK    (RX_8_DATA_WATERMARK_BITS << RX_8_DATA_WATERMARK_LSB)

/* #define RX_9_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_9_DATA_WATERMARK_LSB     16
#define RX_9_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_9_DATA_WATERMARK_MASK    (RX_9_DATA_WATERMARK_BITS << RX_9_DATA_WATERMARK_LSB)

/* #define RX_10_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_10_DATA_WATERMARK_LSB     8
#define RX_10_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_10_DATA_WATERMARK_MASK    (RX_10_DATA_WATERMARK_BITS << RX_10_DATA_WATERMARK_LSB)

/* #define RX_11_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_11_DATA_WATERMARK_LSB     0
#define RX_11_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_11_DATA_WATERMARK_MASK    (RX_11_DATA_WATERMARK_BITS << RX_11_DATA_WATERMARK_LSB)

/***************************************************
 * RX_WATERMARKS_3 (Address Offset 0x248)
 **************************************************/

/* #define RX_12_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_12_DATA_WATERMARK_LSB     24
#define RX_12_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_12_DATA_WATERMARK_MASK    (RX_12_DATA_WATERMARK_BITS << RX_12_DATA_WATERMARK_LSB)

/* #define RX_13_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_13_DATA_WATERMARK_LSB     16
#define RX_13_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_13_DATA_WATERMARK_MASK    (RX_13_DATA_WATERMARK_BITS << RX_13_DATA_WATERMARK_LSB)

/* #define RX_14_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_14_DATA_WATERMARK_LSB     8
#define RX_14_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_14_DATA_WATERMARK_MASK    (RX_14_DATA_WATERMARK_BITS << RX_14_DATA_WATERMARK_LSB)

/* #define RX_15_DATA_WATERMARK  ; Watermark for RxData FIFO in units of Cacheline (32 byte entries)
 */
#define RX_15_DATA_WATERMARK_LSB     0
#define RX_15_DATA_WATERMARK_BITS    SEVEN_BITS
#define RX_15_DATA_WATERMARK_MASK    (RX_15_DATA_WATERMARK_BITS << RX_15_DATA_WATERMARK_LSB)

/***************************************************
 * FREE_WATERMARKS (Address Offset 0x249)
 **************************************************/

/* #define FREEOUT_WATERMARKS  ; Watermark for FreeOut FIFO in units of Descriptors (64 bit entries)
 */
#define FREEOUT_WATERMARKS_LSB     16
#define FREEOUT_WATERMARKS_BITS    SEVEN_BITS
#define FREEOUT_WATERMARKS_MASK    (FREEOUT_WATERMARKS_BITS << FREEOUT_WATERMARKS_LSB)

/* #define REG_FREE_IN_WATERMARKS  ; Watermark for Regular FreeIn FIFO in units of Descriptors (64 bit entries)
 */
#define REG_FREE_IN_LSB     0
#define REG_FREE_IN_BITS    SEVEN_BITS
#define REG_FREE_IN_MASK    (REG_FREE_IN_BITS << REG_FREE_IN_LSB)

/***************************************************
 * FIFO_CARVING_SLOTS0 (Address Offset 0x24A) Allocation ratios of channel bandwidth among all channels.
 *                                           The Sum of all entries (0x24A and 0x24B) should be <= 16.
 **************************************************/

/* #define CHANNEL_7_SLOT  ;  Channel ratio
 */
#define CHANNEL_7_SLOT_LSB     28
#define CHANNEL_7_SLOT_BITS    FOUR_BITS
#define CHANNEL_7_SLOT_MASK    (CHANNEL_7_SLOT_BITS << CHANNEL_7_SLOT_LSB)

/* #define CHANNEL_6_SLOT  ; Channel ratio
 */
#define CHANNEL_6_SLOT_LSB     24
#define CHANNEL_6_SLOT_BITS    FOUR_BITS
#define CHANNEL_6_SLOT_MASK    (CHANNEL_6_SLOT_BITS << CHANNEL_6_SLOT_LSB)

/* #define CHANNEL_5_SLOT  ; Channel ratio
 */
#define CHANNEL_5_SLOT_LSB     20
#define CHANNEL_5_SLOT_BITS    FOUR_BITS
#define CHANNEL_5_SLOT_MASK    (CHANNEL_5_SLOT_BITS << CHANNEL_5_SLOT_LSB)

/* #define CHANNEL_4_SLOT  ; Channel ratio
 */
#define CHANNEL_4_SLOT_LSB     16
#define CHANNEL_4_SLOT_BITS    FOUR_BITS
#define CHANNEL_4_SLOT_MASK    (CHANNEL_4_SLOT_BITS << CHANNEL_4_SLOT_LSB)

/* #define CHANNEL_3_SLOT  ; Channel ratio
 */
#define CHANNEL_3_SLOT_LSB     12
#define CHANNEL_3_SLOT_BITS    FOUR_BITS
#define CHANNEL_3_SLOT_MASK    (CHANNEL_3_SLOT_BITS << CHANNEL_3_SLOT_LSB)

/* #define CHANNEL_2_SLOT  ; Channel ratio
 */
#define CHANNEL_2_SLOT_LSB     8
#define CHANNEL_2_SLOT_BITS    FOUR_BITS
#define CHANNEL_2_SLOT_MASK    (CHANNEL_2_SLOT_BITS << CHANNEL_2_SLOT_LSB)

/* #define CHANNEL_1_SLOT  ; Channel ratio
 */
#define CHANNEL_1_SLOT_LSB     4
#define CHANNEL_1_SLOT_BITS    FOUR_BITS
#define CHANNEL_1_SLOT_MASK    (CHANNEL_1_SLOT_BITS << CHANNEL_1_SLOT_LSB)

/* #define CHANNEL_0_SLOT  ; Channel ratio
 */
#define CHANNEL_0_SLOT_LSB     0
#define CHANNEL_0_SLOT_BITS    FOUR_BITS
#define CHANNEL_0_SLOT_MASK    (CHANNEL_0_SLOT_BITS << CHANNEL_0_SLOT_LSB)

/***************************************************
 * FIFO_CARVING_SLOTS1 (Address Offset 0x24A) Allocation ratios of channel bandwidth among all channels.
 *                                           The Sum of all entries (0x24A and 0x24B) should be <= 16.
 **************************************************/

/* #define CHANNEL_15_SLOT  ;  Channel ratio
 */
#define CHANNEL_15_SLOT_LSB     28
#define CHANNEL_15_SLOT_BITS    FOUR_BITS
#define CHANNEL_15_SLOT_MASK    (CHANNEL_15_SLOT_BITS << CHANNEL_15_SLOT_LSB)

/* #define CHANNEL_14_SLOT  ;  Channel ratio
 */
#define CHANNEL_14_SLOT_LSB     24
#define CHANNEL_14_SLOT_BITS    FOUR_BITS
#define CHANNEL_14_SLOT_MASK    (CHANNEL_14_SLOT_BITS << CHANNEL_14_SLOT_LSB)

/* #define CHANNEL_13_SLOT  ;  Channel ratio
 */
#define CHANNEL_13_SLOT_LSB     20
#define CHANNEL_13_SLOT_BITS    FOUR_BITS
#define CHANNEL_13_SLOT_MASK    (CHANNEL_13_SLOT_BITS << CHANNEL_13_SLOT_LSB)

/* #define CHANNEL_12_SLOT  ;  Channel ratio
 */
#define CHANNEL_12_SLOT_LSB     16
#define CHANNEL_12_SLOT_BITS    FOUR_BITS
#define CHANNEL_12_SLOT_MASK    (CHANNEL_12_SLOT_BITS << CHANNEL_12_SLOT_LSB)

/* #define CHANNEL_11_SLOT  ;  Channel ratio
 */
#define CHANNEL_11_SLOT_LSB     12
#define CHANNEL_11_SLOT_BITS    FOUR_BITS
#define CHANNEL_11_SLOT_MASK    (CHANNEL_11_SLOT_BITS << CHANNEL_11_SLOT_LSB)

/* #define CHANNEL_10_SLOT  ;  Channel ratio
 */
#define CHANNEL_10_SLOT_LSB     8
#define CHANNEL_10_SLOT_BITS    FOUR_BITS
#define CHANNEL_10_SLOT_MASK    (CHANNEL_10_SLOT_BITS << CHANNEL_10_SLOT_LSB)

/* #define CHANNEL_9_SLOT  ;  Channel ratio
 */
#define CHANNEL_9_SLOT_LSB     4
#define CHANNEL_9_SLOT_BITS    FOUR_BITS
#define CHANNEL_9_SLOT_MASK    (CHANNEL_9_SLOT_BITS << CHANNEL_9_SLOT_LSB)

/* #define CHANNEL_8_SLOT  ;  Channel ratio
 */
#define CHANNEL_8_SLOT_LSB     0
#define CHANNEL_8_SLOT_BITS    FOUR_BITS
#define CHANNEL_8_SLOT_MASK    (CHANNEL_8_SLOT_BITS << CHANNEL_8_SLOT_LSB)

/**************************************************
 * Registers Specific to GMAC Interface           *
 *                                                *
 * Address Space 0x00 - 0x0EF                     *
 *************************************************/

/***************************************************
 * G_CONFIG_1 (Address Offset 0x00) Configuration Register #1
 **************************************************/

/* #define G_SOFT_RESET  ;  1: Resets the GMAC
 */
#define G_SOFT_RESET_LSB     31
#define G_SOFT_RESET_BITS    ONE_BIT
#define G_SOFT_RESET_MASK    (G_SOFT_RESET_BITS << G_SOFT_RESET_LSB)

/* #define RESET_RX_MAC_CONTROL  ;  1: Resets the RX_CONTROL block
 */
#define RESET_RX_MAC_CONTROL_LSB     19
#define RESET_RX_MAC_CONTROL_BITS    ONE_BIT
#define RESET_RX_MAC_CONTROL_MASK    (RESET_RX_MAC_CONTROL_BITS << RESET_RX_MAC_CONTROL_LSB)

/* #define RESET_TX_MAC_CONTROL  ;  1: Resets the TX_CONTROL block
 */
#define RESET_TX_MAC_CONTROL_LSB     18
#define RESET_TX_MAC_CONTROL_BITS    ONE_BIT
#define RESET_TX_MAC_CONTROL_MASK    (RESET_TX_MAC_CONTROL_BITS << RESET_TX_MAC_CONTROL_LSB)

/* #define RESET_RX_FUNCTION  ;  1: Resets the RX_FUNCTION block
 */
#define RESET_RX_FUNCTION_LSB     17
#define RESET_RX_FUNCTION_BITS    ONE_BIT
#define RESET_RX_FUNCTION_MASK    (RESET_RX_FUNCTION_BITS << RESET_RX_FUNCTION_LSB)

/* #define RESET_TX_FUNCTION  ;  1: Resets the TX_FUNCTION block
 */
#define RESET_TX_FUNCTION_LSB     16
#define RESET_TX_FUNCTION_BITS    ONE_BIT
#define RESET_TX_FUNCTION_MASK    (RESET_TX_FUNCTION_BITS << RESET_TX_FUNCTION_LSB)

/* #define LOOP_BACK  ;  1: MAC is in loopback mode
 */
#define LOOP_BACK_LSB     8
#define LOOP_BACK_BITS    ONE_BIT
#define LOOP_BACK_MASK    (LOOP_BACK_BITS << LOOP_BACK_LSB)

/* #define RECEIVE_FLOW_CONTROL_ENABLE  ;  1: detects and act on PAUSE frames
 */
#define RECEIVE_FLOW_CONTROL_ENABLE_LSB     5
#define RECEIVE_FLOW_CONTROL_ENABLE_BITS    ONE_BIT
#define RECEIVE_FLOW_CONTROL_ENABLE_MASK    (RECEIVE_FLOW_CONTROL_ENABLE_BITS << RECEIVE_FLOW_CONTROL_ENABLE_LSB)

/* #define TRANSMIT_FLOW_CONTROL_ENABLE  ;  1: allows transmit of PAUSE Frames when requested by software
 */
#define TRANSMIT_FLOW_CONTROL_ENABLE_LSB     4
#define TRANSMIT_FLOW_CONTROL_ENABLE_BITS    ONE_BIT
#define TRANSMIT_FLOW_CONTROL_ENABLE_MASK    (TRANSMIT_FLOW_CONTROL_ENABLE_BITS << TRANSMIT_FLOW_CONTROL_ENABLE_LSB)

/* #define SYNC_RECEIVE_ENABLE  ;  1: read only syncronized receive enable
 */
#define SYNC_RECEIVE_ENABLE_LSB     3
#define SYNC_RECEIVE_ENABLE_BITS    ONE_BIT
#define SYNC_RECEIVE_ENABLE_MASK    (SYNC_RECEIVE_ENABLE_BITS << SYNC_RECEIVE_ENABLE_LSB)

/* #define RECEIVE_ENABLE  ;  1: receive enable
 */
#define RECEIVE_ENABLE_LSB     2
#define RECEIVE_ENABLE_BITS    ONE_BIT
#define RECEIVE_ENABLE_MASK    (RECEIVE_ENABLE_BITS << RECEIVE_ENABLE_LSB)

/* #define SYNC_TRANSMIT_ENABLE  ;  1: read only syncronized transmit enable
 */
#define SYNC_TRANSMIT_ENABLE_LSB     1
#define SYNC_TRANSMIT_ENABLE_BITS    ONE_BIT
#define SYNC_TRANSMIT_ENABLE_MASK    (SYNC_TRANSMIT_ENABLE_BITS << SYNC_TRANSMIT_ENABLE_LSB)

/* #define TRANSMIT_ENABLE  ;  1: transmit enable
 */
#define TRANSMIT_ENABLE_LSB     0
#define TRANSMIT_ENABLE_BITS    ONE_BIT
#define TRANSMIT_ENABLE_MASK    (TRANSMIT_ENABLE_BITS << TRANSMIT_ENABLE_LSB)

/***************************************************
 * G_CONFIG_2 (Address Offset 0x01) Configuration Register #2
 **************************************************/

/* #define PREAMBLE_LENGTH  ; Length of Preamble in bytes
 */
#define PREAMBLE_LENGTH_LSB     12
#define PREAMBLE_LENGTH_BITS    FOUR_BITS
#define PREAMBLE_LENGTH_MASK    (PREAMBLE_LENGTH_BITS << PREAMBLE_LENGTH_LSB)

/* #define INTERFACE_MODE  ; Type of interface
 *                           00 : Reserved
 *                           01 : Nibble mode (10/100)
 *                           10 : Byte mode (1000)
 *                           11 : Reserved
 */
#define INTERFACE_MODE_LSB     8
#define INTERFACE_MODE_BITS    TWO_BITS
#define INTERFACE_MODE_MASK    (INTERFACE_MODE_BITS << INTERFACE_MODE_LSB)
#define NIBBLE_MODE   1
#define BYTE_MODE     2

/* #define LENGTH_FIELD_CHECKING  ; causes MAC to check if length field in packet matches actual length of packet
 */
#define LENGTH_FIELD_CHECKING_LSB     4
#define LENGTH_FIELD_CHECKING_BITS    ONE_BIT
#define LENGTH_FIELD_CHECKING_MASK    (LENGTH_FIELD_CHECKING_BITS << LENGTH_FIELD_CHECKING_LSB)

/* #define PAD_CRC_ENABLE  ; MAC will pad all short frames and append a CRC on all frames
 */
#define PAD_CRC_ENABLE_LSB     2
#define PAD_CRC_ENABLE_BITS    ONE_BIT
#define PAD_CRC_ENABLE_MASK    (PAD_CRC_ENABLE_BITS << PAD_CRC_ENABLE_LSB)

/* #define CRC_ENABLE  ; MAC appends a CRC on all frames, PAD_CRC_ENABLE takes precedence if set
 */
#define CRC_ENABLE_LSB     1
#define CRC_ENABLE_BITS    ONE_BIT
#define CRC_ENABLE_MASK    (CRC_ENABLE_BITS << CRC_ENABLE_LSB)

/* #define FULL_DUPLEX  ; 1: Full duplex 0: Half duplex
 */
#define FULL_DUPLEX_LSB     0
#define FULL_DUPLEX_BITS    ONE_BIT
#define FULL_DUPLEX_MASK    (FULL_DUPLEX_BITS << FULL_DUPLEX_LSB)


/*********************************************
 * MAX_FRM ( Address offset 0x04 )
**********************************************/

#define MAX_FM_LEN_LSB	   0
#define MAX_FM_LEN_BITS    SIXTEEN_BITS 
#define MAX_FM_LEN_MASK    ( MAX_FM_LEN_BITS << MAX_FM_LEN_LSB)

/***************************************************
 * G_IPG (Address Offset 0x02) IPG/IFG
 **************************************************/

/* #define N_B2B_IPG_1  ; NON-BACK-TO-BACK INTER-PACKET-GAP PART 1
 */
#define N_B2B_IPG_1_LSB     24
#define N_B2B_IPG_1_BITS    SEVEN_BITS
#define N_B2B_IPG_1_MASK    (N_B2B_IPG_1_BITS << N_B2B_IPG_1_LSB)

/* #define N_B2B_IPG_2  ; NON-BACK-TO-BACK INTER-PACKET-GAP PART 2
 */
#define N_B2B_IPG_2_LSB     16
#define N_B2B_IPG_2_BITS    SEVEN_BITS
#define N_B2B_IPG_2_MASK    (N_B2B_IPG_2_BITS << N_B2B_IPG_2_LSB)

/* #define MIN_IFG_ENFORCE  ; Minimum IFG Enforcement on Ingress. Non-confonlmng Packets are dropped
 */
#define MIN_IFG_ENFORCE_LSB     8
#define MIN_IFG_ENFORCE_BITS    EIGHT_BITS
#define MIN_IFG_ENFORCE_MASK    (MIN_IFG_ENFORCE_BITS << MIN_IFG_ENFORCE_LSB)

/* #define B2B_IPG  ; BACK-TO-BACK INTER-PACKET-GAP
 */
#define B2B_IPG_LSB     0
#define B2B_IPG_BITS    SEVEN_BITS
#define B2B_IPG_MASK    (B2B_IPG_BITS << B2B_IPG_LSB)

/***************************************************
 * G_MAC_FILTER_CONFIG (Address Offset 0x5C) MAC Packet Filter
 **************************************************/

/* #define BROADCAST_EN  ; 
 */
#define BROADCAST_EN_LSB     10
#define BROADCAST_EN_BITS    ONE_BIT
#define BROADCAST_EN_MASK    (BROADCAST_EN_BITS << BROADCAST_EN_LSB)

/* #define PAUSE_FRAME_EN  ; 
 */
#define PAUSE_FRAME_EN_LSB     9
#define PAUSE_FRAME_EN_BITS    ONE_BIT
#define PAUSE_FRAME_EN_MASK    (PAUSE_FRAME_EN_BITS << PAUSE_FRAME_EN_LSB)

/* #define MCAST_EN  ; 
 */
#define MCAST_EN_LSB     8
#define MCAST_EN_BITS    ONE_BIT
#define MCAST_EN_MASK    (MCAST_EN_BITS << MCAST_EN_LSB)

/* #define UCAST_EN  ; 
 */
#define UCAST_EN_LSB     7
#define UCAST_EN_BITS    ONE_BIT
#define UCAST_EN_MASK    (UCAST_EN_BITS << UCAST_EN_LSB)

/* #define HASH_MCAST_EN  ; 
 */
#define HASH_MCAST_EN_LSB     6
#define HASH_MCAST_EN_BITS    ONE_BIT
#define HASH_MCAST_EN_MASK    (HASH_MCAST_EN_BITS << HASH_MCAST_EN_LSB)

/* #define HASH_UCAST_EN  ; 
 */
#define HASH_UCAST_EN_LSB     5
#define HASH_UCAST_EN_BITS    ONE_BIT
#define HASH_UCAST_EN_MASK    (HASH_UCAST_EN_BITS << HASH_UCAST_EN_LSB)

/* #define ADDR_MATCH_DISCARD  ; Discard if Address Matches any of the 4 entries
 */
#define ADDR_MATCH_DISCARD_LSB     4
#define ADDR_MATCH_DISCARD_BITS    ONE_BIT
#define ADDR_MATCH_DISCARD_MASK    (ADDR_MATCH_DISCARD_BITS << ADDR_MATCH_DISCARD_LSB)

/* #define MAC_ADDR3_VALID  ; 
 */
#define MAC_ADDR3_VALID_LSB     3
#define MAC_ADDR3_VALID_BITS    ONE_BIT
#define MAC_ADDR3_VALID_MASK    (MAC_ADDR3_VALID_BITS << MAC_ADDR3_VALID_LSB)

/* #define MAC_ADDR2_VALID  ; 
 */
#define MAC_ADDR2_VALID_LSB     2
#define MAC_ADDR2_VALID_BITS    ONE_BIT
#define MAC_ADDR2_VALID_MASK    (MAC_ADDR2_VALID_BITS << MAC_ADDR2_VALID_LSB)

/* #define MAC_ADDR1_VALID  ; 
 */
#define MAC_ADDR1_VALID_LSB     1
#define MAC_ADDR1_VALID_BITS    ONE_BIT
#define MAC_ADDR1_VALID_MASK    (MAC_ADDR1_VALID_BITS << MAC_ADDR1_VALID_LSB)

/* #define MAC_ADDR0_VALID  ; 
 */
#define MAC_ADDR0_VALID_LSB     0
#define MAC_ADDR0_VALID_BITS    ONE_BIT
#define MAC_ADDR0_VALID_MASK    (MAC_ADDR0_VALID_BITS << MAC_ADDR0_VALID_LSB)

/***************************************************
 * G_INTERFACE_CONTROL (Address Offset 0x0E) Interface Control
 **************************************************/

/* #define TBI_MODE  ; 
 */
#define TBI_MODE_LSB     27
#define TBI_MODE_BITS    ONE_BIT
#define TBI_MODE_MASK    (TBI_MODE_BITS << TBI_MODE_LSB)

/* #define SGMIISPEED  ; Legacy Half Duplex mode
 */
#define SGMIISPEED_LSB     25
#define SGMIISPEED_BITS    TWO_BITS
#define SGMIISPEED_MASK    (SGMIISPEED_BITS << SGMIISPEED_LSB)

/* #define LHDMODE  ; Legacy Half Duplex mode
 */
#define LHDMODE_LSB     25
#define LHDMODE_BITS    ONE_BIT
#define LHDMODE_MASK    (LHDMODE_BITS << LHDMODE_LSB)

/* #define PHY_MODE  ; Legacy Half Duplex mode
 */
#define PHY_MODE_LSB     24
#define PHY_MODE_BITS    ONE_BIT
#define PHY_MODE_MASK    (PHY_MODE_BITS << PHY_MODE_LSB)

/* #define JAB_PROTECTION_EN  ; 
 */
#define JAB_PROTECTION_EN_LSB     0
#define JAB_PROTECTION_EN_BITS    ONE_BIT
#define JAB_PROTECTION_EN_MASK    (JAB_PROTECTION_EN_BITS << JAB_PROTECTION_EN_LSB)



/**************************************************
 * Registers Specific to XGMAC Interface           *
 *                                                *
 * Address Space 0x00 - 0x0EF                     *
 *************************************************/
/***************************************************
 * XG_CONFIG_0 (Address Offset 0x00) Configuration Register #0
 **************************************************/
/* #define RSTRCTL  ;  1: resets Receive Control module
 */
#define MACRST_LSB	31
#define MACRST_BITS     ONE_BIT
#define MACRST_MASK    (MACRST_BITS << MACRST_LSB)

#define RSTRCTL_LSB     22
#define RSTRCTL_BITS    ONE_BIT
#define RSTRCTL_MASK    (RSTRCTL_BITS << RSTRCTL_LSB)

#define RSTRFN_LSB      21
#define RSTRFN_BITS     ONE_BIT
#define RSTRFN_MASK     (RSTRFN_BITS << RSTRFN_LSB)

#define RSTTCTL_LSB     16
#define RSTTCTL_BITS    ONE_BIT
#define RSTTCTL_MASK    (RSTTCTL_BITS << RSTTCTL_LSB)

#define RSTTFN_LSB      15
#define RSTTFN_BITS     ONE_BIT
#define RSTTFN_MASK    (RSTTFN_BITS << RSTTFN_LSB)

/***************************************************
 * XG_CONFIG_1 (Address Offset 0x01) Configuration Register #1
 **************************************************/

/* #define TCTLEN  ;  1: Allows transmission of Pause Frames
 */
#define TCTLEN_LSB     31
#define TCTLEN_BITS    ONE_BIT
#define TCTLEN_MASK    (TCTLEN_BITS << TCTLEN_LSB)

/* #define TFEN  ;  1: Allows transmission of Frames
 */
#define TFEN_LSB     30
#define TFEN_BITS    ONE_BIT
#define TFEN_MASK    (TFEN_BITS << TFEN_LSB)

/* #define RCTLEN  ;  1: Allows receiver to detect and act on Pause Frames
 */
#define RCTLEN_LSB     29
#define RCTLEN_BITS    ONE_BIT
#define RCTLEN_MASK    (RCTLEN_BITS << RCTLEN_LSB)

/* #define RFEN  ;  1: Allows reception of Frames
 */
#define RFEN_LSB     28
#define RFEN_BITS    ONE_BIT
#define RFEN_MASK    (RFEN_BITS << RFEN_LSB)


/* #define SYNC_TFEN  ;  1: Read Only TFEN SYNCED
 */
#define SYNC_TFEN_LSB     26
#define SYNC_TFEN_BITS    ONE_BIT
#define SYNC_TFEN_MASK    (SYNC_TFEN_BITS << SYNC_TFEN_LSB)

/* #define SYNC_RFEN  ;  1: Read Only RFEN SYNCED
 */
#define SYNC_RFEN_LSB     24
#define SYNC_RFEN_BITS    ONE_BIT
#define SYNC_RFEN_MASK    (SYNC_RFEN_BITS << SYNC_RFEN_LSB)

/* #define RCTLSHRTP  ;  1: XOFF PAUSE Frames will pause the transmit for 64 slot times,
 *                          thereby shortening the XOFF control frame pause time
 */
#define RCTLSHRTP_LSB     12
#define RCTLSHRTP_BITS    ONE_BIT
#define RCTLSHRTP_MASK    (RCTLSHRTP_BITS << RCTLSHRTP_LSB)

/* #define DLYFCSTX  ;  Delay the computation of CRC by X number of 4 byte words
 *                      Useful if there are extra words before the Ethernet Header
 *                      and if we do not want CRC to be computed over these words    
 */
#define DLYFCSTX_LSB     10
#define DLYFCSTX_BITS    TWO_BITS
#define DLYFCSTX_MASK    (DLYFCSTX_BITS << DLYFCSTX_LSB)

/* #define DLYFCSRX  ;  Delay the computation of CRC by X number of 4 byte words
 *                      Useful if there are extra words before the Ethernet Header
 *                      and if we do not want CRC to be computed over these words    
 */
#define DLYFCSRX_LSB     8
#define DLYFCSRX_BITS    ONE_BIT
#define DLYFCSRX_MASK    (DLYFCSRX_BITS << DLYFCSRX_LSB)

/* #define PPEN      ;  Setting this bit enables the use of per-packet settings on transmit frames
 *                      Should be always set to 0 in Revision B
 */
#define PPEN_LSB     7
#define PPEN_BITS    ONE_BIT
#define PPEN_MASK    (PPEN_BITS << PPEN_LSB)


/* #define BYTSWP    ;  Bytes endianness are swapped. Should be used for test pusposes only
 */
#define BYTSWP_LSB     6
#define BYTSWP_BITS    ONE_BIT
#define BYTSWP_MASK    (BYTSWP_BITS << BYTSWP_LSB)

/* #define DRPLT64    ;  
 */
#define DRPLT64_LSB     5
#define DRPLT64_BITS    ONE_BIT
#define DRPLT64_MASK    (DRPLT64_BITS << DRPLT64_LSB)

/* #define PRMSCRX    ;  
 */
#define PRMSCRX_LSB     4
#define PRMSCRX_BITS    ONE_BIT
#define PRMSCRX_MASK    (PRMSCRX_BITS << PRMSCRX_LSB)

/* #define LENCHK    ;  MAC will check if length field in packet matches the packet length
 */
#define LENCHK_LSB     3
#define LENCHK_BITS    ONE_BIT
#define LENCHK_MASK    (LENCHK_BITS << LENCHK_LSB)

/* #define GENFCS    ;  Set this bit of generating and appending CRC of all frames OR
 *                      if any of the PADMODE bits are one
 */
#define GENFCS_LSB     2
#define GENFCS_BITS    ONE_BIT
#define GENFCS_MASK    (GENFCS_BITS << GENFCS_LSB)

/* #define PADMODE    ; 00 -Do not pad
 *                      01 -pad to 64 bytes. Append CRC 
 *                      10 -IF VLAN pad to 68 or else pad to 64. Append CRC
 *                      11 -pad to 68. Append CRC
 */
#define PADMODE_LSB     0
#define PADMODE_BITS    TWO_BITS
#define PADMODE_MASK    (PADMODE_BITS << PADMODE_LSB)

/***************************************************
 * XG_MAX_FRAME_LENGTH (Address Offset 0x08) Maximum Frame length
 **************************************************/

/* #define MXFRMWCTX  ;  Maximum frame length in (4 byte) words for transmission
 */
#define MXFRMWCTX_LSB     16
#define MXFRMWCTX_BITS    FOURTEEN_BITS
#define MXFRMWCTX_MASK    (MXFRMWCTX_BITS << MXFRMWCTX_LSB)

/* #define MXFRMBCRX  ;  Maximum frame length in bytes for reception
 */
#define MXFRMBCRX_LSB     0
#define MXFRMBCRX_BITS    SIXTEEN_BITS
#define MXFRMBCRX_MASK    (MXFRMBCRX_BITS << MXFRMBCRX_LSB)

/***************************************************************************
 * XGMAC_CLK_DIV (Address Offset 0x12) MII Management Configuration Register
 **************************************************************************/

/* #define NOPRAM  ;  1: Bypass Preamble portion of the Management Frame format
 */
#define NOPRAM_LSB     7
#define NOPRAM_BITS    ONE_BIT
#define NOPRAM_MASK    (NOPRAM_BITS << NOPRAM_LSB)

/* #define XGMAC_MDIO_CLKDIV  ;  1: Bypass Preamble portion of the Management Frame format
 */
#define XGMAC_MDIO_CLKDIV_LSB     0
#define XGMAC_MDIO_CLKDIV_BITS    SEVEN_BITS
#define XGMAC_MDIO_CLKDIV_MASK    (XGMAC_MDIO_CLKDIV_BITS << XGMAC_MDIO_CLKDIV_LSB)

          
          

/***************************************************
 * SPI_RESET_CONFIG (Address Offset 0x6C) SPI4 Interface Config
 **************************************************/

/* #define MORE_TRN_EN  ; 
 */
#define MORE_TRN_EN_LSB     14
#define MORE_TRN_EN_BITS    ONE_BIT
#define MORE_TRN_EN_MASK    (MORE_TRN_EN_BITS << MORE_TRN_EN_LSB)

/* #define TX_FRM_ERR_EN  ; 
 */
#define TX_FRM_ERR_EN_LSB     13
#define TX_FRM_ERR_EN_BITS    ONE_BIT
#define TX_FRM_ERR_EN_MASK    (TX_FRM_ERR_EN_BITS << TX_FRM_ERR_EN_LSB)

/* #define RX_TRN_LOS  ; 
 */
#define RX_TRN_LOS_LSB     12
#define RX_TRN_LOS_BITS    ONE_BIT
#define RX_TRN_LOS_MASK    (RX_TRN_LOS_BITS << RX_TRN_LOS_LSB)

/* #define RX_TRN_RESET  ; 
 */
#define RX_TRN_RESET_LSB     11
#define RX_TRN_RESET_BITS    ONE_BIT
#define RX_TRN_RESET_MASK    (RX_TRN_RESET_BITS << RX_TRN_RESET_LSB)

/* #define FIFO_SW_RESET  ; 
 */
#define FIFO_SW_RESET_LSB     10
#define FIFO_SW_RESET_BITS    ONE_BIT
#define FIFO_SW_RESET_MASK    (FIFO_SW_RESET_BITS << FIFO_SW_RESET_LSB)

/* #define DDL_ENABLE  ; 
 */
#define DDL_ENABLE_LSB     9
#define DDL_ENABLE_BITS    ONE_BIT
#define DDL_ENABLE_MASK    (DDL_ENABLE_BITS << DDL_ENABLE_LSB)

/* #define SPI_DEP_LPB  ; 
 */
#define SPI_DEP_LPB_LSB     8
#define SPI_DEP_LPB_BITS    ONE_BIT
#define SPI_DEP_LPB_MASK    (SPI_DEP_LPB_BITS << SPI_DEP_LPB_LSB)

/* #define SPI_SHAL_LPB  ; 
 */
#define SPI_SHAL_LPB_LSB     7
#define SPI_SHAL_LPB_BITS    ONE_BIT
#define SPI_SHAL_LPB_MASK    (SPI_SHAL_LPB_BITS << SPI_SHAL_LPB_LSB)

/* #define USR_DEEP_LPB  ; 
 */
#define USR_DEEP_LPB_LSB     6
#define USR_DEEP_LPB_BITS    ONE_BIT
#define USR_DEEP_LPB_MASK    (USR_DEEP_LPB_BITS << USR_DEEP_LPB_LSB)

/* #define USR_SHAL_LPB  ; 
 */
#define USR_SHAL_LPB_LSB     5
#define USR_SHAL_LPB_BITS    ONE_BIT
#define USR_SHAL_LPB_MASK    (USR_SHAL_LPB_BITS << USR_SHAL_LPB_LSB)

/* #define SPI4_RX_ENABLE  ; 
 */
#define SPI4_RX_ENABLE_LSB     3
#define SPI4_RX_ENABLE_BITS    ONE_BIT
#define SPI4_RX_ENABLE_MASK    (SPI4_RX_ENABLE_BITS << SPI4_RX_ENABLE_LSB)

/* #define SPI4_RX_SW_RESET  ; 
 */
#define SPI4_RX_SW_RESET_LSB     2
#define SPI4_RX_SW_RESET_BITS    ONE_BIT
#define SPI4_RX_SW_RESET_MASK    (SPI4_RX_SW_RESET_BITS << SPI4_RX_SW_RESET_LSB)

/* #define SPI4_TX_ENABLE  ; 
 */
#define SPI4_TX_ENABLE_LSB     1
#define SPI4_TX_ENABLE_BITS    ONE_BIT
#define SPI4_TX_ENABLE_MASK    (SPI4_TX_ENABLE_BITS << SPI4_TX_ENABLE_LSB)

/* #define SPI4_TX_SW_RESET  ; 
 */
#define SPI4_TX_SW_RESET_LSB     0
#define SPI4_TX_SW_RESET_BITS    ONE_BIT
#define SPI4_TX_SW_RESET_MASK    (SPI4_TX_SW_RESET_BITS << SPI4_TX_SW_RESET_LSB)

/*************************************************************************
 * SPI_INT (Address Offset 0x70) SPI4 Interface Interrupt Status
 * SPI_INT_MASK (Address Offset 0x74) SPI4 Interface Interrupt Status Mask
 ************************************************************************/

/* #define INVALID_CW  ; 
 */
#define INVALID_CW_LSB     3
#define INVALID_CW_BITS    ONE_BIT
#define INVALID_CW_MASK    (INVALID_CW_BITS << INVALID_CW_LSB)

/* #define DIP2_ERROR  ; 
 */
#define DIP2_ERROR_LSB     2
#define DIP2_ERROR_BITS    ONE_BIT
#define DIP2_ERROR_MASK    (DIP2_ERROR_BITS << DIP2_ERROR_LSB)

/* #define DIP4_ERROR  ; 
 */
#define DIP4_ERROR_LSB     1
#define DIP4_ERROR_BITS    ONE_BIT
#define DIP4_ERROR_MASK    (DIP4_ERROR_BITS << DIP4_ERROR_LSB)

/*************************************************************************
 * SPI_TX_STATUS (Address Offset 0x7C) SPI4 Interface Tx Status
 ************************************************************************/

/* #define TX_SYNC  ; 
 */
#define TX_SYNC_LSB     2
#define TX_SYNC_BITS    ONE_BIT
#define TX_SYNC_MASK    (TX_SYNC_BITS << TX_SYNC_LSB)

/* #define TX_TRAIN_DETECT  ; 
 */
#define TX_TRAIN_DETECT_LSB     1
#define TX_TRAIN_DETECT_BITS    ONE_BIT
#define TX_TRAIN_DETECT_MASK    (TX_TRAIN_DETECT_BITS << TX_TRAIN_DETECT_LSB)

/* #define TX_CAL_ERROR  ; 
 */
#define TX_CAL_ERROR_LSB     0
#define TX_CAL_ERROR_BITS    ONE_BIT
#define TX_CAL_ERROR_MASK    (TX_CAL_ERROR_BITS << TX_CAL_ERROR_LSB)

/*************************************************************************
 * SPI_RX_STATUS (Address Offset 0x80) SPI4 Interface Rx Status
 ************************************************************************/

/* #define MISSING_EOP  ; 
 */
#define MISSING_EOP_LSB     4
#define MISSING_EOP_BITS    ONE_BIT
#define MISSING_EOP_MASK    (MISSING_EOP_BITS << MISSING_EOP_LSB)

/* #define MISSING_SOP  ; 
 */
#define MISSING_SOP_LSB     3
#define MISSING_SOP_BITS    ONE_BIT
#define MISSING_SOP_MASK    (MISSING_SOP_BITS << MISSING_SOP_LSB)

/* #define RX_SYNC  ; 
 */
#define RX_SYNC_LSB     1
#define RX_SYNC_BITS    ONE_BIT
#define RX_SYNC_MASK    (RX_SYNC_BITS << RX_SYNC_LSB)

/* #define RX_TRAIN_DETECT  ; 
 */
#define RX_TRAIN_DETECT_LSB     0
#define RX_TRAIN_DETECT_BITS    ONE_BIT
#define RX_TRAIN_DETECT_MASK    (RX_TRAIN_DETECT_BITS << RX_TRAIN_DETECT_LSB)



#endif
