/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

// MDIO Low level Access routines
// All Phy's accessed from GMAC0 base

#ifndef _MDIO_H
#define _MDIO_H

#include "mac_defines.h"
#include "gmac.h"

#define CICADA_PHY_ADDR                         0x00  /* This may have to be changed */
#define CICADA_MODE_CONTROL                     0x00 
#define CICADA_MODE_STATUS                      0x01 
#define CICADA_PHY_IDENTIFIER_1                 0x02 
#define CICADA_PHY_IDENTIFIER_2                 0x03 
#define CICADA_AUTO_NEGOTIATION_ADVT            0x04 
#define CICADA_AUTO_NEGO_LNK_PRT_ABILITY        0x05 
#define CICADA_AUTO_NEGO_EXPAN                  0x06 
#define CICADA_AUTO_NEGO_NXT_PG_TR              0x07 
#define CICADA_AUTO_NEGO_LNK_PRT_NXT_PG_REC     0x08 
#define CICADA_REG_1000BASE_T_CONTROL           0x09 
#define CICADA_REG_1000BASE_T_STATUS            0x0A 

/* 0x0B - 0x0E RESERVED */

#define CICADA_REG_1000BASE_T_STATUS_EXTEN_1    0x0F 
#define CICADA_REG_100BASE_T_STATUS_EXTEN       0x10 
#define CICADA_REG_1000BASE_T_STATUS_EXTEN_2    0x11 
#define CICADA_BYPASS_CONTROL                   0x12 

/* 0x13 - 0x15 RESERVED */

#define CICADA_EXTEN_CONTROL_STATUS             0x16 
#define CICADA_EXTEN_PHY_CONTROL_1              0x17 
#define CICADA_EXTEN_PHY_CONTROL_2              0x18 
#define CICADA_INTERRUPT_MASK                   0x19 
#define CICADA_INTERRUPT_STATUS                 0x1A 
#define CICADA_LED_CONTROL                      0x1B 
#define CICADA_AUX_CONTROL_STATUS               0x1C 

/* 0x1D RESERVED */

#define CICADA_MAC_INTERFACE_AUTONEG_CON_STA    0x1E 
#define CICADA_EXTENDED_PAGE_ACCESS             0x1F 


static inline void xmdio_address (volatile unsigned int *_mmio, 
								  uint32_t phy_addr, 
                                  uint32_t dev_ad, 
	                              uint32_t address) 
{
	uint32_t st_field = 0x0;
	uint32_t op_type  = 0x0 ; // address operation
	uint32_t ta_field = 0x2 ; // ta field

	phoenix_write_reg( _mmio, 0x11, (((st_field & 0x3) << 30) | 
				((op_type & 0x3) << 28) | ((phy_addr & 0x1F) << 23) | 
				((dev_ad  & 0x1F) << 18) | ((ta_field & 0x3) << 16) | 
				((address  & 0xffff) <<  0)));

	phoenix_write_reg( _mmio,0x10, ((0x0 << 3) | 0x5));
	phoenix_write_reg( _mmio,0x10, ((0x1 << 3) | 0x5));
	phoenix_write_reg( _mmio,0x10, ((0x0 << 3) | 0x5));

	/* wait for dev_ad cycle to complete */

	while ( phoenix_read_reg( _mmio,0x14 ) & 0x1)
		;
}
 
static inline int xmdio_read(volatile unsigned int *_mmio, 
	                         uint32_t phy_addr, uint32_t address) 
{
	uint32_t st_field = 0x0;
	uint32_t op_type  = 0x3 ; // read operation
	uint32_t ta_field = 0x2 ; // ta field
	uint32_t data = 0;

	xmdio_address (_mmio, phy_addr, 5, address);
	phoenix_write_reg( _mmio,0x11,(((st_field & 0x3) << 30) | ((op_type  & 0x3) << 28) | 
					((phy_addr & 0x1F) << 23) | ((5  & 0x1F   ) << 18) | 
					((ta_field & 0x3) << 16) | ((data & 0xffff) <<  0)));

	phoenix_write_reg( _mmio,0x10,( (0x0 << 3) | 0x5));
	phoenix_write_reg( _mmio,0x10,( (0x1 << 3) | 0x5));
	phoenix_write_reg( _mmio,0x10,( (0x0 << 3) | 0x5));

	/* wait for write cycle to complete */

	while (phoenix_read_reg( _mmio,0x14) & 0x1)
		; 

	data = (uint32_t)phoenix_read_reg( _mmio,0x11) & 0xffff;

	return data;
}
 
static inline void xmdio_write (volatile unsigned int *_mmio, 
		                        uint32_t phy_addr, 
                                uint32_t address, 
                                uint32_t data) 
{
	uint32_t st_field = 0x0;
	uint32_t op_type  = 0x1 ; // write operation
	uint32_t ta_field = 0x2 ; // ta field

    xmdio_address ( _mmio, phy_addr, 5, address);
	phoenix_write_reg( _mmio ,0x11,(((st_field & 0x3) << 30) | ((op_type & 0x3) << 28) | 
				   ((phy_addr & 0x1F) << 23) | ((5 & 0x1F) << 18) | 
				   ((ta_field & 0x3) << 16) | ((data & 0xffff) <<  0)));

	phoenix_write_reg(_mmio ,0x10,((0x0 << 3) | 0x5));
	phoenix_write_reg(_mmio,0x10,( (0x1 << 3) | 0x5));
	phoenix_write_reg(_mmio,0x10, ((0x0 << 3) | 0x5));

	/* wait for write cycle to complete */
	while (phoenix_read_reg(_mmio,0x14) & 0x1)
		; 
}

#endif
