/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _MIPSREGS_H
#define _MIPSREGS_H

#include "compiler.h"

/*
 * Coprocessor 0 register names
 */

#define  CP0_INDEX          $0
#define  CP0_RANDOM         $1
#define  CP0_ENTRYLO0       $2
#define  CP0_ENTRYLO1       $3
#define  CP0_CONF           $3
#define  CP0_CONTEXT        $4
#define  CP0_PAGEMASK       $5
#define  CP0_WIRED          $6
#define  CP0_INFO           $7
#define  CP0_BADVADDR       $8
#define  CP0_COUNT          $9
#define  CP0_ENTRYHI        $10
#define  CP0_COMPARE        $11
#define  CP0_STATUS         $12
#define  CP0_CAUSE          $13
#define  CP0_EPC            $14
#define  CP0_PRID           $15
#define  CP0_CONFIG         $16
#define  CP0_LLADDR         $17
#define  CP0_WATCHLO        $18
#define  CP0_WATCHHI        $19
#define  CP0_XCONTEXT       $20
#define  CP0_FRAMEMASK      $21
#define  CP0_DIAGNOSTIC     $22
#define  CP0_DEBUG          $23
#define  CP0_DEPC           $24
#define  CP0_PERFORMANCE    $25
#define  CP0_ECC            $26
#define  CP0_CACHEERR       $27
#define  CP0_TAGLO          $28
#define  CP0_TAGHI          $29
#define  CP0_ERROREPC       $30
#define  CP0_DESAVE         $31

#define  CP0_OSSCRATCH      $22
#define  CP0_EBASE          $15     
#define  CP0_PERF_CTR       $25

#define CP1_FIR             $0
#define CP1_FCCR            $25
#define CP1_FEXR            $26
#define CP1_FENR            $28
#define CP1_FCSR            $31

/*
 * Status register bits available in all MIPS CPUs.
 */

#define  ST0_KX             0x00000080
#define  ST0_IM             0x0000ff00
#define  STATUSB_IP0        8
#define  STATUSF_IP0        (1 <<  8)
#define  STATUSB_IP1        9
#define  STATUSF_IP1        (1 <<  9)
#define  STATUSB_IP2        10
#define  STATUSF_IP2        (1 << 10)
#define  STATUSB_IP3        11
#define  STATUSF_IP3        (1 << 11)
#define  STATUSB_IP4        12
#define  STATUSF_IP4        (1 << 12)
#define  STATUSB_IP5        13
#define  STATUSF_IP5        (1 << 13)
#define  STATUSB_IP6        14
#define  STATUSF_IP6        (1 << 14)
#define  STATUSB_IP7        15
#define  STATUSF_IP7        (1 << 15)
#define  STATUSB_IP8        0
#define  STATUSF_IP8        (1 << 0)
#define  STATUSB_IP9        1
#define  STATUSF_IP9        (1 << 1)
#define  STATUSB_IP10       2
#define  STATUSF_IP10       (1 << 2)
#define  STATUSB_IP11       3
#define  STATUSF_IP11       (1 << 3)
#define  STATUSB_IP12       4
#define  STATUSF_IP12       (1 << 4)
#define  STATUSB_IP13       5
#define  STATUSF_IP13       (1 << 5)
#define  STATUSB_IP14       6
#define  STATUSF_IP14       (1 << 6)
#define  STATUSB_IP15       7
#define  STATUSF_IP15       (1 << 7)

#define ST0_CH              0x00040000
#define ST0_NMI             0x00080000
#define ST0_SR              0x00100000
#define ST0_TS              0x00200000
#define ST0_BEV             0x00400000
#define ST0_RE              0x02000000
#define ST0_FR              0x04000000
#define ST0_CU              0xf0000000
#define ST0_CU0             0x10000000
#define ST0_CU1             0x20000000
#define ST0_CU2             0x40000000
#define ST0_CU3             0x80000000
#define ST0_XX              0x80000000      /* MIPS IV naming */

/*
 * Bitfields and bit numbers in the coprocessor 0 
 * cause register.  * Refer to your MIPS R4xx0 
 * manual, chapter 5 for explanation.
 */

#define  CAUSEB_EXCCODE     2
#define  CAUSEF_EXCCODE     (31 <<  2)
#define  CAUSEB_IP          8
#define  CAUSEF_IP          (255 <<  8)
#define  CAUSEB_IP0         8
#define  CAUSEF_IP0         (1 <<  8)
#define  CAUSEB_IP1         9
#define  CAUSEF_IP1         (1 <<  9)
#define  CAUSEB_IP2         10
#define  CAUSEF_IP2         (1 << 10)
#define  CAUSEB_IP3         11
#define  CAUSEF_IP3         (1 << 11)
#define  CAUSEB_IP4         12
#define  CAUSEF_IP4         (1 << 12)
#define  CAUSEB_IP5         13
#define  CAUSEF_IP5         (1 << 13)
#define  CAUSEB_IP6         14
#define  CAUSEF_IP6         (1 << 14)
#define  CAUSEB_IP7         15
#define  CAUSEF_IP7         (1 << 15)
#define  CAUSEB_IV          23
#define  CAUSEF_IV          (1 << 23)
#define  CAUSEB_CE          28
#define  CAUSEF_CE          (3 << 28)
#define  CAUSEB_BD          31
#define  CAUSEF_BD          (1 << 31)

/*
 * Symbolic register names for 32 bit ABI
 */

#define zero    $0      /* wired zero */
#define AT      $1      /* assembler temp  - uppercase because of ".set at" */
#define v0      $2      /* return value */
#define v1      $3
#define a0      $4      /* argument registers */
#define a1      $5
#define a2      $6
#define a3      $7
#define t0      $8      /* caller saved */
#define t1      $9
#define t2      $10
#define t3      $11
#define t4      $12
#define t5      $13
#define t6      $14
#define t7      $15
#define s0      $16     /* callee saved */
#define s1      $17
#define s2      $18
#define s3      $19
#define s4      $20
#define s5      $21
#define s6      $22
#define s7      $23
#define t8      $24     /* caller saved */
#define t9      $25
#define jp      $25     /* PIC jump register */
#define k0      $26     /* kernel scratch */
#define k1      $27
#define gp      $28     /* global pointer */
#define sp      $29     /* stack pointer */
#define fp      $30     /* frame pointer */
#define s8	    $30		/* same like fp! */
#define ra      $31     /* return address */

#endif
