/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _MM_H
#define _MM_H

#define PAGE_SIZE (1 << 12)
#define KSEG0     (0xffffffff80000000ULL)
#define KSEG1     (0xffffffffA0000000ULL)
#define KSEG2     (0xffffffffC0000000ULL)
#define KSEG3     (0xffffffffE0000000ULL)
#define MAX_CPUS       32
#define KSEG0_PHY_BOUNDARY 0x10000000ULL    // 256 MB

#ifndef __ASSEMBLY__

#include "types.h"
#include "mips-exts.h"
#include "xlr_boot_lib.h"
#include "tlb.h"

extern struct tlb_struct *pte;
extern volatile struct xlr_vcpu_wakeup_info *xlr_vcpu_wakeup_info;

struct segment_info {
    unsigned long vaddr;  
    uint64_t paddr;  
    uint64_t size;
};

extern int ntlb;
extern struct segment_info netos_app_tlbs[MAX_TLB_MAPPINGS];

static inline uint64_t virt_to_phys(volatile void * address)
{
	unsigned long addr = (unsigned long)address;
	long long retval;
	int pagemask;

    	if (addr < (unsigned long)KSEG0) {
		if (addr > MAX_VIRT_ADDR)
			return 0;

		/* Assumption... lo0 and lo1 are physically contiguous */
		retval = pte[addr >> (MIN_PAGE_BITS + 1)].entrylo0;

		/* No tlb mapped... return 0 */
		if (!retval)
			return 0;

		pagemask = (retval >> 34) | 0x1fff;
		retval = retval << 30;
		retval = retval >> 36 << 12;
		retval |= addr & pagemask;
		return retval;
	}
	    if (addr < (unsigned long)KSEG1) {
    	    return addr - (unsigned long)KSEG0;
    	}
	    if (addr < (unsigned long)KSEG2) {
    	    return addr - (unsigned long)KSEG1;
    	}
	return 0;
}

static inline void *phys_to_virt(uint64_t addr)
{
	long long retval;

	if (addr < KSEG0_PHY_BOUNDARY)
		return (void *) ((unsigned long) (addr + KSEG0));

	if (addr >= 1ULL << MAX_VIRT_PHYS_BITS)
		return (void *)(unsigned long)0;

	/* Assumption... lo0 and lo1 are physically contiguous */
	retval = pte[addr >> (MIN_PAGE_BITS + 1)].entrylo1 >> 34 <<
						(MIN_PAGE_BITS + 1);

	/* No tlb mapped... return 0 */
	if (!retval)
		return 0;

	retval |= addr & ((1 << (MIN_PAGE_BITS + 1)) - 1);
	return (void *)(long)retval;
}


static inline uint64_t kseg0_virt_to_phys(volatile void * addr) {
    return (unsigned long)addr - (unsigned long)KSEG0;
}

static inline void *kseg0_phys_to_virt(uint64_t addr) {
    return (void *) ((unsigned long) (addr + KSEG0));
}


#endif

#endif
