/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _NETDEVICE_H
#define _NETDEVICE_H

#include "packet.h"
#include "types.h"
#include "msgring.h"
#include "net_config.h"

#define IFNAMSIZ 20

struct net_device {

    char                name[IFNAMSIZ];
    struct net_device  *next;
    unsigned char       dev_addr[20];
    int                 speed;
    int                 fdx;
    int                 abm;
    void               *driver_data;
    void               *priv;
    void               *frin_spill_packets;
    void               *frout_spill_packets;
    void               *rx_spill_packets;

    int                (*init) (struct net_device *dev);
    int                (*open) (struct net_device *dev);
    void               (*close) (struct net_device *dev);
    int                (*transmit) (struct packet *pbuf, struct net_device *dev, int len, int port);
    int                (*transmit_free) (struct packet *pbuf, struct net_device *dev, int len, int port);
    int                (*transmit_port) (struct packet *pbuf, struct net_device *dev, 
		                         int len, int port);
    int                (*receive) (void (*)(struct packet *), struct net_device *dev, int bucket);
    struct packet *    (*receive_wait) (uint32_t *cycles, struct net_device *dev, 
                                        unsigned int *len, unsigned int *seq_num, int *port, int bucket);
    struct packet *    (*receive_callback) (struct net_device *dev, struct msgrng_msg *msg,
                                            unsigned int *len, unsigned int *seq_num);
    struct packet *    (*receive_nowait) (uint32_t *cycles, struct net_device *dev, 
                                          unsigned int *len, unsigned int *seq_num, int *port, int bucket);
    int                (*set_mac_address) (struct net_device *dev);
    uint32_t           (*read_reg) (struct net_device *dev, int reg);
    void               (*write_reg) (struct net_device *dev, int reg, uint32_t value);
    int                (*soft_reset) (struct net_device *dev);
    void               (*init_msgring) (struct net_device *dev);
    void               (*change_mac_hwaddr) (struct net_device *dev, uint64_t mac_addr);
};

extern struct net_device *dev_get_by_name(const char *name);

extern struct net_device *netdev;

extern int xlr_dev_init(net_config *net_conf);

#endif
