/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _NETOSCRF_H
#define _NETOSCRF_H

#include "msgring.h"
#include "net_config.h"
#include "irq.h"
#include "nlmcrf/crflib.h"

#define NLM_MAX_SPI4_PORT_PER_CTRL 10

/* FDT functions */
extern int fdt_get_core_bucket_conf(int core, char **buckets, char **credits);
extern int fdt_get_gmac_bucket_conf(int gmac_id, bucket_t buckets[], int bklen, 
		bucket_t credits[], int crlen);
extern int fdt_get_gmac_ppties(unsigned int *gmac_list, unsigned long *gmac_addr, int *gmac_own);
extern int fdt_get_owned_bucket_list(int *start, int *end, int *mask);
extern int fdt_get_cpu_info(unsigned int *online_map);
extern int fdt_is_gmac0_in_sgmii(void);
extern int fdt_open();
extern int  fdt_get_gmac_pde_reginfo(int);
extern int fdt_get_xgs_ppties(int type, unsigned int *gmac_list, 
		unsigned long *gmac_addr, int *gmac_own);
extern int fdt_get_xgs_bucket_conf(int type, int id, bucket_t buckets[], int bklen, 
		bucket_t credits[], int crlen);
extern int  fdt_get_xgs_pde_reginfo(int type, int id);
extern uint64_t fdt_get_vuart_fifo_addr(int tx, int instance);
extern int fdt_get_per_cpu_bucket_map(uint32_t cpu_map, int *bkt_map);
extern uint64_t fdt_get_heap_info(uint64_t *heapvaddr, uint64_t *heapsize,
                    uint64_t *sh_heapvaddr, uint64_t *sh_heapsize);
extern int fdt_get_console_string(char *str, int len);
extern uint64_t fdt_get_uart_lock(int);
extern int fdt_gmac_enabled(int gmacid);
extern int fdt_xgs_enabled(int xgsid, int *type);
extern int fdt_get_uart_baud(int uartno, uint32_t *baud);

extern unsigned int nlmk_en;
extern spinlock_t  nlmk_lock;
extern unsigned long nlmk_ebase;
extern void **nlm_call_table;
extern char   *g_envp[];
extern struct nlm_resource *nlm_this_domres;
struct netos_list { 
	struct netos_list *prev; 
	struct netos_list *next;
};
struct arp_entry {
	uint32_t ipaddr;   
	uint32_t domid;
	uint16_t ptype;
	uint8_t  port;
	uint8_t  stid;
	uint64_t macaddr;
	uint32_t valid;
	struct netos_list list;
};
enum arp_op { NLMCRF_ARP_ADD = 0, NLMCRF_ARP_DEL, NLMCRF_ARP_LOOKUP };
enum ifc_op { NLMCRF_IFC_UP = 0, NLMCRF_IFC_DOWN, NLMCRF_IFC_LOOKUP };
extern int nlm_arp_entry_manage(uint32_t ipaddr, uint64_t macaddr, uint32_t domid, 
		int ptype, int port, int op, void **data);
extern int nlm_ifc_entry_manage(uint32_t ipaddr, uint64_t macaddr, uint32_t domid, 
		int ptype, int port, int op, void **data);
static inline int nlm_arp_entry_lookup(uint32_t dstip, void **data)
{
	  return nlm_arp_entry_manage(dstip, 0ULL, 0,
                                  0, 0, NLMCRF_ARP_LOOKUP, data);
}
static inline int nlm_ifc_entry_lookup(int ptype, int port, void **data)
{
	  return nlm_ifc_entry_manage(0, 0ULL, 0,
                                  ptype, port, NLMCRF_IFC_LOOKUP, data);
}

extern void nlmk_spill_setup(); 
extern int nlmk_wakeup_cpus(void *fn, volatile void *args, uint32_t cpu_mask);
extern void nlmk_config_pde(phoenix_reg_t *mmio, int type, int id);
extern void nlmk_message_ring_cpu_init();
extern void nlmk_setup_heap();
extern irqreturn_t nlmk_eventq_ipi_handler(int irq, struct pt_regs *ptregs);
extern void nlmk_net_reset(void);
extern int nlmk_cpu_to_cpu_msgring_handler(int code, int size, struct msgrng_msg *msg);
extern volatile int event_fmnq_polling_enabled;
extern volatile void *netos_thr_wakeup_pointer[];
extern void netos_init_secondary_thread(void *args);
extern int nlm_route_ipv4_data_packet(struct msgrng_msg *msg_out, int fbid, int byteoffset);
extern int nlm_get_ptype_from_stid(int stid, int *ptype, int *fbstid);
extern void nlmk_default_packet_init(void);
extern void nlm_dump_packet_info(struct msgrng_msg *msg_out);
extern int nlmk_create_cl_pools(void);
#endif

