/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _PERF_H
#define _PERF_H


// includes
#include <stdio.h>
#include <stdlib.h>
#include <types.h>
#include "netdevice.h"
#include "mac.h"
#include "cpu.h"


// setup l1 counters and clear
void init_l1_counters() {
  int value;

  // setup perf counters
  value = (0<<11) | (33<<5) | (1<<1);   // LsuPerfLoadOpWrbP
  write_32bit_cp0_register_sel(CP0_PERFORMANCE, value, 0);
  value = (0<<11) | (35<<5) | (1<<1);   // LsuPerfLoadHitWrbP
  write_32bit_cp0_register_sel(CP0_PERFORMANCE, value, 2);
  
  write_32bit_cp0_register_sel(CP0_PERFORMANCE, 0, 1);
  write_32bit_cp0_register_sel(CP0_PERFORMANCE, 0, 3);
}


// setup l2 counters and init
void init_l2_counters() {
  unsigned int *l2cfg;
  
  l2cfg = (unsigned int *)(phoenix_io_base + 0x1b000);
  
  l2cfg[0x004 >> 2] = (3<<28) | 6;   // L2P0 select, L2Hit - B0
  l2cfg[0x004 >> 2] = (0<<28) | 7;   // L2P1 select, L2 Ops -B0
  l2cfg[0x024 >> 2] = (3<<28) | 6;   // L2P0 select, L2Hit - B1
  l2cfg[0x024 >> 2] = (0<<28) | 7;   // L2P1 select, L2 Ops -B1
  l2cfg[0x044 >> 2] = (3<<28) | 6;   // L2P0 select, L2Hit - B2
  l2cfg[0x044 >> 2] = (0<<28) | 7;   // L2P1 select, L2 Ops -B2
  l2cfg[0x064 >> 2] = (3<<28) | 6;   // L2P0 select, L2Hit - B3
  l2cfg[0x064 >> 2] = (0<<28) | 7;   // L2P1 select, L2 Ops -B3
  l2cfg[0x084 >> 2] = (3<<28) | 6;   // L2P0 select, L2Hit - B4
  l2cfg[0x084 >> 2] = (0<<28) | 7;   // L2P1 select, L2 Ops -B4
  l2cfg[0x0a4 >> 2] = (3<<28) | 6;   // L2P0 select, L2Hit - B5
  l2cfg[0x0a4 >> 2] = (0<<28) | 7;   // L2P1 select, L2 Ops -B5
  l2cfg[0x0c4 >> 2] = (3<<28) | 6;   // L2P0 select, L2Hit - B6
  l2cfg[0x0c4 >> 2] = (0<<28) | 7;   // L2P1 select, L2 Ops -B6
  l2cfg[0x0e4 >> 2] = (3<<28) | 6;   // L2P0 select, L2Hit - B7
  l2cfg[0x0e4 >> 2] = (0<<28) | 7;   // L2P1 select, L2 Ops -B7

  
  // zero out perf counters
  l2cfg[0x208 >> 2] = 0x0;	// L2P0 - B0
  l2cfg[0x20c >> 2] = 0x0;	// L2P1 - B0
  l2cfg[0x228 >> 2] = 0x0;	// L2P0 - B1
  l2cfg[0x22c >> 2] = 0x0;	// L2P1 - B1
  l2cfg[0x248 >> 2] = 0x0;	// L2P0 - B2
  l2cfg[0x24c >> 2] = 0x0;	// L2P1 - B2
  l2cfg[0x268 >> 2] = 0x0;	// L2P0 - B3
  l2cfg[0x26c >> 2] = 0x0;	// L2P1 - B3
  l2cfg[0x288 >> 2] = 0x0;	// L2P0 - B4
  l2cfg[0x28c >> 2] = 0x0;	// L2P1 - B4
  l2cfg[0x2a8 >> 2] = 0x0;	// L2P0 - B5
  l2cfg[0x2ac >> 2] = 0x0;	// L2P1 - B5
  l2cfg[0x2c8 >> 2] = 0x0;	// L2P0 - B6
  l2cfg[0x2cc >> 2] = 0x0;	// L2P1 - B6
  l2cfg[0x2e8 >> 2] = 0x0;	// L2P0 - B7
  l2cfg[0x2ec >> 2] = 0x0;	// L2P1 - B7
}


// read l1 access counter
static inline unsigned long long read_l1_access_counter() {
  return(read_32bit_cp0_register_sel(CP0_PERFORMANCE, 1));
}


// read l1 hit counter
static inline unsigned long long read_l1_hit_counter() {
  return(read_32bit_cp0_register_sel(CP0_PERFORMANCE, 3));
}




// read l2 access counter
static inline unsigned long long read_l2_access_counter() {
  unsigned int *l2cfg;
  unsigned long long l2_load_counter;
  l2cfg = (uint32_t *)(phoenix_io_base + 0x1b000);

  l2_load_counter   = l2cfg[0x20c >> 2]; // B0   
  l2_load_counter  += l2cfg[0x22c >> 2]; // B1  
  l2_load_counter  += l2cfg[0x24c >> 2]; // B2   
  l2_load_counter  += l2cfg[0x26c >> 2]; // B3   
  l2_load_counter  += l2cfg[0x28c >> 2]; // B4   
  l2_load_counter  += l2cfg[0x2ac >> 2]; // B5  
  l2_load_counter  += l2cfg[0x2cc >> 2]; // B6   
  l2_load_counter  += l2cfg[0x2ec >> 2]; // B7
  return (l2_load_counter);
}



// read l2 hit counter
static inline unsigned long long read_l2_hit_counter() {
  unsigned int *l2cfg;
  unsigned long long l2_hit_counter;
  l2cfg = (uint32_t *)(phoenix_io_base + 0x1b000);
  l2_hit_counter    = l2cfg[0x208 >> 2]; // B0
  l2_hit_counter   += l2cfg[0x228 >> 2]; // B1
  l2_hit_counter   += l2cfg[0x248 >> 2]; // B2
  l2_hit_counter   += l2cfg[0x268 >> 2]; // B3
  l2_hit_counter   += l2cfg[0x288 >> 2]; // B4
  l2_hit_counter   += l2cfg[0x2a8 >> 2]; // B5
  l2_hit_counter   += l2cfg[0x2c8 >> 2]; // B6
  l2_hit_counter   += l2cfg[0x2e8 >> 2]; // B7
  return (l2_hit_counter);
}

#endif
