/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _PKT_GEN_H
#define _PKT_GEN_H

#include <stdio.h>
#include "packet.h"
#define MAX_PKT_SIZE  1536
#define SEQ_SIGNATURE 0x01dfaced
#define CRC32_POLY    0x04c11db7

typedef struct packet *p_packet, c_packet ;

/* 
 * CRC Table
 */

extern unsigned long crc32_table[256];
extern unsigned long crc32_table_ttable[256];

/* 
 * public functions
 */

extern p_packet   pkt_new ();
extern void       pkt_payload        ( p_packet pkt, int _length);
extern void       pkt_print          ( p_packet pkt );
extern void       pkt_ethernet_encap ( p_packet pkt, long long da, long long sa, int type );
extern void       pkt_ethernet_encap_notype ( p_packet pkt, long long da, long long sa );
extern void       pkt_crc_encap      ( p_packet pkt );
extern void       pkt_delete         ( p_packet pkt );
extern void       pkt_insert_seq     ( p_packet pkt, int seq_num );
extern void       pkt_encap_byte     ( p_packet pkt, uint8_t add_byte );
extern void       pkt_encap_2byte    ( p_packet pkt, uint16_t add_2bytes );
extern int        pkt_get_length     ( p_packet pkt );
extern void       pkt_set_length     ( p_packet pkt, int _length );
extern uint8_t    pkt_get_data       ( p_packet pkt, int index );


/*
 * private functions
 */

extern p_packet  get_l2_eth_pkt(long long da, long long sa, int type, int pkt_len, int seq_num);
extern unsigned long  crc32(unsigned char *buf, int len);
extern unsigned long  crc32_for_hash(unsigned char *buf, int len);
extern void  space(p_packet pkt, int size);
extern void  init_crc32();
extern int   flip_byte(int pkt_data);
extern int   flip_32(int pkt_data);
extern void  pkt_ipv4_encap(p_packet pkt, uint32_t src, uint32_t dest, uint32_t protocol,
							uint32_t ttl, uint32_t checksum);
extern void  pkt_ipv4_options_encap(p_packet pkt, uint32_t src, uint32_t dest, uint32_t protocol,
									uint32_t ttl, uint32_t checksum, uint32_t hdrLen );
extern void  pkt_ipv6_encap(p_packet pkt, uint32_t src, uint32_t dest, uint32_t flow_lable, int protocol, int ttl);
extern void  pkt_tcp_encap(p_packet pkt, uint16_t sport, uint16_t dport, uint32_t seq, uint32_t ack, uint8_t hlen,
						   uint8_t flags, uint16_t window_size, uint16_t urg_pointer) ;
extern void  pkt_udp_encap(p_packet pkt, uint16_t src, uint16_t dest);
extern void  pkt_tcp_checksum(p_packet pkt, uint32_t src, uint32_t dest, uint32_t good_checksum);
extern void  pkt_udp_checksum(p_packet pkt, uint32_t src, uint32_t dest, uint32_t good_checksum);
extern void  pkt_udp_checksum_ipv6(p_packet pkt, uint32_t src, uint32_t dest, uint32_t good_checksum);
extern void  pkt_tdp_checksum_ipv6(p_packet pkt, uint32_t src, uint32_t dest, uint32_t good_checksum);
extern void  pkt_custom_l4_checksum(p_packet pkt, uint32_t src, uint32_t dest, uint32_t l4_protocol, uint32_t good_checksum);
extern void  pkt_vlan_encap(p_packet pkt, uint16_t vlan_tag);
extern void  pkt_copy(p_packet new_packet, p_packet old_packet);
extern void  pkt_encap_snap(p_packet pkt, uint16_t type);
extern void  pkt_encap_llc(p_packet pkt, uint8_t dsap, uint8_t ssap, uint8_t ctrl);
extern void  pkt_encap_ppp_uncomp(p_packet pkt, uint16_t hdlc, uint16_t protocol);
extern void  pkt_encap_ppp_comp(p_packet pkt, uint16_t protocol);
extern unsigned long crc32_for_ttable(unsigned char *buf, int len, int hash_poly);
 
#endif
