/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _POD_H
#define _POD_H

#include "types.h"
#include "msgring.h"
#include "netdevice.h"
#include "mac.h"
#include "spinlock.h"

/* 17 bits */
#define MAX_SEQ_NUM (0x20000)

extern struct msgrng_msg pod_msgs[];
extern void pod_start(void);

struct pod_stats {
  uint32_t tx_packets;
  uint32_t dropped_packets;
};

extern volatile struct pod_stats pod_stats;

static inline struct msgrng_msg *pod_get_msg(int seq)
{
  if (seq < 0 || seq >= MAX_SEQ_NUM) return 0;
  return &pod_msgs[seq];
}

static inline void pod_add(int size, int stid, struct msgrng_msg *msg)
{
  //int seq = (msg->msg0 >> 40) & 0xfff;
  uint64_t cmd = 0;
  int code = 0x01;

  //if (seq < 0 || seq >= MAX_SEQ_NUM) return;

  cmd = ( (size-1)<<16) | (code<<8) |stid;
  /* may have to do a sd on 32bit machine */
  msg->msg2 = (msg->msg2 & ~((uint64_t)0xfffff<<40)) | (cmd << 40);
  barrier();
}

static inline void pod_ignore(int seq)
{
  uint64_t cmd = 0x03<<8;
  struct msgrng_msg *msg = pod_get_msg(seq);

  if (!msg) return;
  /* may have to do a sd on 32bit machine */
  barrier();
  msg->msg2 = (msg->msg2 & ~((uint64_t)0xfff<<40)) | ((uint64_t)cmd << 40);
}

#endif
