/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef __NETL_BARRIER_PARAM_H
#define __NETL_BARRIER_PARAM_H
#include <barrier.h>
#include <list.h>
#include <spinlock.h>
#include <sys/types.h>
#define BARRIER_MAGIC       0xdeeddeed

#define BARRIER_OPEN     0
#define BARRIER_CLOSE    1
#define BARRIER_UNLINK   2

struct barrier_struct
{
    barrier weight;  /*barrier weight */
    char *name;
    int magic;
    int oflag;
    struct list list;
    int unlink;
    int users;
    volatile int flag;
    spinlock_t lock;
};

typedef struct barrier_struct barrier_tag;

struct barrier_open_struct
{
    const char *name;
    int oflag;
    unsigned int weight;
    barrier *ret; /*return value*/
};

struct barrier_close_struct 
{
    barrier_tag *barrier;
    int ret; /*return value*/
};

struct barrier_init_struct 
{
    barrier_tag *barrier;
    int value;
    int ret; /*return value*/
};

struct barrier_unlink_struct 
{
    const char *name;
    int ret; /*return value*/
};

struct barrier_param
{
    int index;
    union {
        struct barrier_open_struct open;
        struct barrier_init_struct init;
        struct barrier_close_struct close;
        struct barrier_unlink_struct unlink;
    };
};

#endif
