/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef __XLR_BOOT_LIB_H
#define __XLR_BOOT_LIB_H
#include "spinlock.h"
#include "irq.h"
#include "mips-exts.h"
#include "boot_mgr.h"

#define CPU_RUNNING 0x1
#define CPU_STOOPED 0x2
#define MAX_ELF_SEGMENTS 16
#define MAX_TLB_MAPPINGS 16
#define MAX_WIRED_ENTRIES 14

#define XLR_ARGV_BUF_SIZE 256
#define XLR_ENV_BUF_SIZE 256
#define XLR_MAX_ARGV 32
#define XLR_MAX_ENV 32
#define XLR_NUM_CPU 32
#define MAX_FRAGMENTS 32
#define EXTENDED_BOOT_MAGIC 0xE88E8DED
#define MAX_PHYS 0xffffffffffULL
#define NMI_BL_SHARED_REGION 0x1f000000
#define XLR_PERSISTENT_MEMORY_MAGIC 0xdead900d
#define KSEG0_MEM_MAP 0x10000001
#define KUSEG_MEM_MAP 0x10000002
#define FREE_TLB_NONSHARED 1
#define FREE_TLB_ALL    2

#define CPU_RUNNING 0x1
#define CPU_STOPPED 0x2
#define CPU_WAIT_TO_RELEASE_RESOURCE 0x3
#define CPU_RESOURCE_RELEASED 0x4

#define IRQ_STOP_CPU_IPI                        51
#define IRQ_WAKEUP_CPU_IPI                      52
#define IRQ_RECLAIM_CPU_RSRC_IPI                53
#define IRQ_JOIN_CPU_IPI                        54


enum {KUSEG_APP, KSEG0_APP};

#define lib_spin_lock(x) smp_lock((spinlock_t *)x)
#define lib_spin_unlock(x) smp_unlock((spinlock_t *)x)

#define lib_spin_lock_irqsave(xlock,flags)\
		{	\
			unsigned int tmp;\
			unsigned int tmp1  = processor_id() + 1;\
			local_irq_save(flags);\
			__asm__ __volatile__(\
			    ".set\tnoreorder\t\t\t# spin_lock\n"\
			    "1:\tll\t%1, %2\n\t"\
			    "bnez\t%1, 1b\n\t"\
			    "move\t%1, %3\n\t"\
			    "sc\t%1, %0\n\t"\
			    "beqz\t%1, 1b\n\t"\
			    " sync\n\t"\
			    ".set\treorder"\
			    : "=m" (xlock->lock), "=&r" (tmp)\
			    : "m" (xlock->lock), "r"(tmp1)\
			    : "memory");\
		}


#define lib_spin_unlock_irqrestore(xlock,flags)\
		__asm__ __volatile__( \
	    ".set\tnoreorder\t\t\t# spin_unlock\n\t"\
	    "sync\n\t"\
	    "sw\t$0, %0\n\t"\
	    ".set\treorder"\
	    : "=m" (xlock->lock)\
	    :: "memory");\
	local_irq_restore(flags);


struct cpu_tlb_mapping {
    int page_size;
    int asid;
    int coherency;
    int attr;
    unsigned long virt;
    uint64_t phys;
};

struct core_segment_info
{
	unsigned long long vaddr;
	unsigned long long memsz;
	unsigned int flags;
};
typedef struct {
    volatile unsigned int lock;
} lib_spinlock_t;

struct proc_section_info
{
        int mode;
        uint32_t vaddr[MAX_ELF_SEGMENTS];
        uint32_t size[MAX_ELF_SEGMENTS];
        uint32_t tlb_vaddr[MAX_TLB_MAPPINGS];
        uint32_t tlb_size[MAX_TLB_MAPPINGS];
        uint64_t tlb_phys[MAX_TLB_MAPPINGS];
};

struct xlr_kuseg_mem_blk
{
        uint64_t phys;
        uint64_t size;
        struct xlr_kuseg_mem_blk *next;
};


struct per_cpu_info_loader {
	uint64_t num_tlbs;
	uint64_t ext_tlb_ptr;
};

struct per_cpu_info{
	uint64_t num_tlbs;
	struct ext_tlb *ext_tlb_ptr;
};

struct xlr_lib_shared_mem
{
	uint64_t entries;
	uint64_t tot_size;
	uint64_t addr[MAX_FRAGMENTS];
	uint64_t size[MAX_FRAGMENTS];
};

struct vcpu_extended_info{
	uint64_t magic;
	uint64_t ver_major;
	uint64_t ver_minor;
	uint64_t size;
	uint64_t per_cpu_info_size;
	struct per_cpu_info *per_cpu_info_ptr;
	spinlock_t *uart0_lock;
	spinlock_t *uart1_lock;
	spinlock_t *global_lock;
	spinlock_t *bridge_io_lock;
	spinlock_t *gmac_lock[8];
	spinlock_t *xgmac_lock[2];
	spinlock_t *spi4_lock[2];
	struct xlr_lib_shared_mem loader_shmem;
	int (*syscall)(int, void*);
	void *fdtptr;   /* use this to pass FDT to application */
	int dom_id; /* dom_id from NetOS->App */
};

struct ext_tlb{
	uint32_t page_size;
	uint32_t asid;
	uint32_t coherency;
	uint32_t attr;
	uint64_t virt;
	uint64_t phys;
};

struct xlr_vcpu_wakeup_info {

	volatile unsigned int cpu_status; /*RUNNING or STOPPED*/
 
	/*Validity for this structure.*/
	int            valid;
 
	/*For master vcpu this is an elf_entry, for buddy vcpus this points to the function pointer passed in "wakeup" call.*/
	unsigned long  func,args;

	/*Stack pointer for this cpu.*/
	unsigned long long sp, gp;

	/*Master CPU No.*/
	int master_cpu;

 	/*Master CPU Mask.*/
	unsigned int master_mask;

	/*Buddy CPU Mask*/
	unsigned int buddy_mask;
 
	unsigned int       psb_os_cpu_map; 
 
	
	/*Argv passed while launching application. These all are pointers in 
    argv_buf.*/
	int argc; /*No. of arg*/
	unsigned char *argv[XLR_MAX_ARGV]; 
	unsigned char argv_buf[XLR_ARGV_BUF_SIZE]; /*Actual Argv */
 
	/* No. Of Valid TLB Entries*/
	int            map_count;

	/* Valid only in case of kuseg applications. Kuseg master and buddy vcpus 
	both will have valid entries here
	*/
	struct cpu_tlb_mapping map[MAX_TLB_MAPPINGS];
	struct core_segment_info seg_info[MAX_ELF_SEGMENTS];
 
	/* Environment variables, envs is an array of pointers. All pointers points 
	   in to env_buf. Output io device is specfied in one of the environment 
	   variale as ttyS0 or ttyS1. Generic bootloader always sets this to ttyS0
	*/
	int env; /*No. Of env*/
	unsigned char *envs[XLR_MAX_ENV];   
	unsigned char env_buf[XLR_ENV_BUF_SIZE];

	/*Mode to decide Kuseg/Kseg applications*/
	unsigned int mode; 

	/* This points to same physical memory (kseg0), for all applications
	 * "printk" routine of netos_lib will hold this lock while dumping
	 */
	lib_spinlock_t *printk_lock; 

	/*Set when we are launching an kseg application on other than vcpu0*/
	int kseg_master;

	/*Reentry Function For KUSEG Applications, Each vcpu can have its own 
	reentry function.*/
	void (*reentry_function)(unsigned long);
	unsigned long reentry_args;

	/*Shared Memory between all application launched by bootloader*/
	uint64_t app_shared_mem_addr;
	uint64_t app_shared_mem_size;
	uint64_t app_shared_mem_orig;

	/*Lock used to avoid window between wakeup and stop*/
	lib_spinlock_t *loader_lock;

	/*This must contain same values in all xlr_vcpu_wakeup_info structures.
	  This is used to avoid window while freeing memory.
	*/
	unsigned long *global_wakeup_mask;

	/*This ll be set to 1 if memory being used by this vcpu can be freed*/
	struct vcpu_extended_info *vcpu_extended_info_ptr;
};

extern void set_print_lock(spinlock_t *lock);
extern void release_recursive_print_lock(void);
extern unsigned int acquire_print_lock(void);
extern void release_print_lock(unsigned int);

extern unsigned long get_shared_mem_start(unsigned long *start, uint64_t *size);

struct cpu_wakeup_info {
        int            master_cpu;
        int            map_count;
        int            valid;
        unsigned long  func;
        unsigned long  args;
        int            argc;
        uint32_t       buddy_mask;
        uint32_t       cpu_mask;
        char          *argv[32];
        char           buf[256];
        struct cpu_tlb_mapping map[MAX_TLB_MAPPINGS];
};

extern unsigned long psb_os_phys_base;
extern int cpu_index;
extern struct cpu_wakeup_info cpu_wakeup_tbl[32];

struct processor
{
        const char *name;
        int processor_id;
        struct processor_rev *processor_rev;

        struct cpu *cpu;
        uint64_t cpu_frequency;

        int nr_cores;
        int nr_cpus;

        unsigned long io_base;
        unsigned long io_size;
        struct component_base *component_base;

        /* For XLR, this is message_ring_cpu_init */
        void (*cpu_ipc_init)(struct processor *this_chip);

        void (*park_cpu)(void *cpu_data);
        void (*wakeup_cpu) (int cpu_id, void *stack);

        void (*processor_init)(struct processor *this_processor);
        void (*processor_reset)(struct processor *this_processor);

        struct board *this_board;
};
struct processor *get_processor_info(void);
static inline struct cpu *get_cpu_info()
{
        struct processor *this_processor = get_processor_info();
        return(this_processor->cpu);
}

struct cpu
{
        /* per_cpu_arch_init() */
        void (*cpu_init) (void *cpu_data);
        /* Nothing for now */
        void (*cpu_reset) (void *cpu_data);

        void *cpu_data;
};

#define NUM_WORDS 2048
#ifndef THREAD_SIZE
#define THREAD_SIZE (NUM_WORDS * sizeof(long))
#endif
struct thread_info {
        unsigned long stack[NUM_WORDS];
};

extern void xlr_lib_release_memory(uint64_t phys, uint64_t size);
extern int xlr_join_kuseg_app(struct bootfile_ *file, uint32_t master, uint32_t buddy, uint32_t parent, 
                                                uint64_t loadaddr, char *user_app_addr, int override, int nargs, char *cmdline_args[],
                                                char *env_args[], int verbose, uint64_t max_phys);
extern int xlr_load_kuseg_app(struct bootfile_ *file, uint32_t master, uint64_t loadaddr,
                              char *user_app_addr, uint32_t buddy, int override, int nargs,
                              char **cmdline_args, char **, int ,uint64_t);

extern void xlr_lib_free_memory(uint64_t start, uint64_t size);
extern uint64_t xlr_lib_alloc_memory(uint64_t *size);
extern void xlr_lib_entry(unsigned long args);
extern struct xlr_lib_shared_mem *xlr_lib_get_curr_shmem(void);
extern struct xlr_lib_shared_mem * xlr_lib_reserve_shmem(uint64_t,uint64_t, int);
extern int xlr_lib_stop_vcpu(unsigned long mask);
extern unsigned long tlb_lower_align_size(uint64_t size);
extern void xlr_load_init(void);
extern void xlr_launch_init(void);
extern void xlr_lib_intr_handler(int ipi);

#define psb_mem_map psb_mem_map
#define xlr_lib_phnx_io_mmio(offset) cpu_io_mmio(offset)
#define xlr_lib_phnx_write_reg(base,offset,val) phoenix_write_reg(base,offset,val)
#define xlr_lib_phnx_read_reg(base,offset) phoenix_read_reg(base,offset)

#define xlr_lib_boot_file bootfile_

#define read_elf_header(a,b,c)   memcpy_virt_phys((unsigned char *)a,(uint64_t)b,(int)c)
#define read_elf_segment(a,b,c)  memcpy_virt_phys((unsigned char *)a,(uint64_t)b,(int)c)
#define read_elf_section(a,b,c)  memcpy_virt_phys((unsigned char *)a,(uint64_t)b,(int)c)
#define read_elf_data(a,b,c)     memcpy_virt_phys((unsigned char *)a,(uint64_t)b,(int)c)

#define xlr_fill_cop0_reg_32(reg,sel,value) 	\
				__asm__ __volatile__(\
					".set\tpush\n\t"\
					".set mips32\n\t"\
					"mtc0\t%0,"STR(reg)", %1\n\t"\
					".set\tpop"\
					:: "r" (value), "i" (sel) );

#define xlr_fill_cop0_reg_64(reg,sel,value) 	\
				__asm__ __volatile__(\
					".set\tpush\n\t"\
					".set mips32\n\t"\
					"mtc0\t%0,"STR(reg)", %1\n\t"\
					".set\tpop"\
					:: "r" (value), "i" (sel) );

#define lib_spin_lock(x) smp_lock((spinlock_t *)x)
#define lib_spin_unlock(x) smp_unlock((spinlock_t *)x)

#endif
