/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#ifndef _XGMAC_H
#define _XGMAC_H

#include <netdevice.h>
#include <device.h>

// defines for registers
#define XGMAC_CLK_DIV              0x12

//#define FIELD_VALUE(field,value) (((value) & (field ## _BITS)) << (field ## _LSB))

/**************************************************
 * Registers Specific to XGMAC Interface           *
 *                                                *
 * Address Space 0x00 - 0x0EF                     *
 *************************************************/

/***************************************************
 * XG_CONFIG_1 (Address Offset 0x01) Configuration Register #1
 **************************************************/

/* #define TCTLEN  ;  1: Allows transmission of Pause Frames
 */
#define TCTLEN_LSB     31
#define TCTLEN_BITS    ONE_BIT
#define TCTLEN_MASK    (TCTLEN_BITS << TCTLEN_LSB)

/* #define TFEN  ;  1: Allows transmission of Frames
 */
#define TFEN_LSB     30
#define TFEN_BITS    ONE_BIT
#define TFEN_MASK    (TFEN_BITS << TFEN_LSB)

/* #define RCTLEN  ;  1: Allows receiver to detect and act on Pause Frames
 */
#define RCTLEN_LSB     29
#define RCTLEN_BITS    ONE_BIT
#define RCTLEN_MASK    (RCTLEN_BITS << RCTLEN_LSB)

/* #define RFEN  ;  1: Allows reception of Frames
 */
#define RFEN_LSB     28
#define RFEN_BITS    ONE_BIT
#define RFEN_MASK    (RFEN_BITS << RFEN_LSB)

/* #define SYNC_TFEN  ;  1: Read Only TFEN SYNCED
 */
#define SYNC_TFEN_LSB     26
#define SYNC_TFEN_BITS    ONE_BIT
#define SYNC_TFEN_MASK    (SYNC_TFEN_BITS << SYNC_TFEN_LSB)

/* #define SYNC_RFEN  ;  1: Read Only RFEN SYNCED
 */
#define SYNC_RFEN_LSB     24
#define SYNC_RFEN_BITS    ONE_BIT
#define SYNC_RFEN_MASK    (SYNC_RFEN_BITS << SYNC_RFEN_LSB)

/* #define RCTLSHRTP  ;  1: XOFF PAUSE Frames will pause the transmit for 64 slot times,
 *                          thereby shortening the XOFF control frame pause time
 */
#define RCTLSHRTP_LSB     12
#define RCTLSHRTP_BITS    ONE_BIT
#define RCTLSHRTP_MASK    (RCTLSHRTP_BITS << RCTLSHRTP_LSB)

/* #define DLYFCSTX  ;  Delay the computation of CRC by X number of 4 byte words
 *                      Useful if there are extra words before the Ethernet Header
 *                      and if we do not want CRC to be computed over these words    
 */
#define DLYFCSTX_LSB     10
#define DLYFCSTX_BITS    TWO_BITS
#define DLYFCSTX_MASK    (DLYFCSTX_BITS << DLYFCSTX_LSB)

/* #define DLYFCSRX  ;  Delay the computation of CRC by X number of 4 byte words
 *                      Useful if there are extra words before the Ethernet Header
 *                      and if we do not want CRC to be computed over these words    
 */
#define DLYFCSRX_LSB     8
#define DLYFCSRX_BITS    ONE_BIT
#define DLYFCSRX_MASK    (DLYFCSRX_BITS << DLYFCSRX_LSB)

/* #define PPEN      ;  Setting this bit enables the use of per-packet settings on transmit frames
 *                      Should be always set to 0 in Revision B
 */
#define PPEN_LSB     7
#define PPEN_BITS    ONE_BIT
#define PPEN_MASK    (PPEN_BITS << PPEN_LSB)

/* #define BYTSWP    ;  Bytes endianness are swapped. Should be used for test pusposes only
 */
#define BYTSWP_LSB     6
#define BYTSWP_BITS    ONE_BIT
#define BYTSWP_MASK    (BYTSWP_BITS << BYTSWP_LSB)

/* #define DRPLT64    ;  
 */
#define DRPLT64_LSB     5
#define DRPLT64_BITS    ONE_BIT
#define DRPLT64_MASK    (DRPLT64_BITS << DRPLT64_LSB)

/* #define PRMSCRX    ;  
 */
#define PRMSCRX_LSB     4
#define PRMSCRX_BITS    ONE_BIT
#define PRMSCRX_MASK    (PRMSCRX_BITS << PRMSCRX_LSB)

/* #define LENCHK    ;  MAC will check if length field in packet matches the packet length
 */
#define LENCHK_LSB     3
#define LENCHK_BITS    ONE_BIT
#define LENCHK_MASK    (LENCHK_BITS << LENCHK_LSB)

/* #define GENFCS    ;  Set this bit of generating and appending CRC of all frames OR
 *                      if any of the PADMODE bits are one
 */
#define GENFCS_LSB     2
#define GENFCS_BITS    ONE_BIT
#define GENFCS_MASK    (GENFCS_BITS << GENFCS_LSB)

/* #define PADMODE    ; 00 -Do not pad
 *                      01 -pad to 64 bytes. Append CRC 
 *                      10 -IF VLAN pad to 68 or else pad to 64. Append CRC
 *                      11 -pad to 68. Append CRC
 */
#define PADMODE_LSB     0
#define PADMODE_BITS    TWO_BITS
#define PADMODE_MASK    (PADMODE_BITS << PADMODE_LSB)

/***************************************************
 * XG_MAX_FRAME_LENGTH (Address Offset 0x08) Maximum Frame length
 **************************************************/

/* #define MXFRMWCTX  ;  Maximum frame length in (4 byte) words for transmission
 */
#define MXFRMWCTX_LSB     16
#define MXFRMWCTX_BITS    FOURTEEN_BITS
#define MXFRMWCTX_MASK    (MXFRMWCTX_BITS << MXFRMWCTX_LSB)

/* #define MXFRMBCRX  ;  Maximum frame length in bytes for reception
 */
#define MXFRMBCRX_LSB     0
#define MXFRMBCRX_BITS    SIXTEEN_BITS
#define MXFRMBCRX_MASK    (MXFRMBCRX_BITS << MXFRMBCRX_LSB)

/***************************************************************************
 * XGMAC_CLK_DIV (Address Offset 0x12) MII Management Configuration Register
 **************************************************************************/

/* #define NOPRAM  ;  1: Bypass Preamble portion of the Management Frame format
 */
#define NOPRAM_LSB     7
#define NOPRAM_BITS    ONE_BIT
#define NOPRAM_MASK    (NOPRAM_BITS << NOPRAM_LSB)

/* #define XGMAC_MDIO_CLKDIV  ;  1: Bypass Preamble portion of the Management Frame format
 */
#define XGMAC_MDIO_CLKDIV_LSB     0
#define XGMAC_MDIO_CLKDIV_BITS    SEVEN_BITS
#define XGMAC_MDIO_CLKDIV_MASK    (XGMAC_MDIO_CLKDIV_BITS << XGMAC_MDIO_CLKDIV_LSB)

int nlm_xgmac_init( device_conf *current_dev);
int nlm_xgmac_open( device_conf * current_dev );
int  nlm_xgmac_close( device_conf * current_dev);
int clr_enable_xgmac_stat(device_conf * current_dev);
#endif
