/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

/* XLR CPU specific */

#ifndef _XLR_XLS_CPU_H
#define _XLR_XLS_CPU_H

#define CHIP_PROCESSOR_ID_XLR       0x00
#define CHIP_PROCESSOR_ID_XLS_608   0x80  
#define CHIP_PROCESSOR_ID_XLS_408   0x88
#define CHIP_PROCESSOR_ID_XLS_404   0x8c
#define CHIP_PROCESSOR_ID_XLS_208   0x8e
#define CHIP_PROCESSOR_ID_XLS_204   0x8f
#define CHIP_PROCESSOR_ID_XLS_108   0xce
#define CHIP_PROCESSOR_ID_XLS_104   0xcf

/* Defines for 'XLS B0' */
#define CHIP_PID_XLS_616_B0         0x40
#define CHIP_PID_XLS_608_B0         0x4a
#define CHIP_PID_XLS_416_B0         0x44
#define CHIP_PID_XLS_412_B0         0x4c
#define CHIP_PID_XLS_408_B0         0x4e
#define CHIP_PID_XLS_404_B0         0x4f
    
/* The block below, is moved in from
 * on_chip.h; the idea is that no one
 * should have to include on_chip.h
 * separately. Just include <cpu.h> and
 * this will invoke xlr_xls_cpu.h, which
 * has all things XLR/XLS
 */
#define DEFAULT_XLR_IO_BASE         	0xffffffffbef00000ULL
#define DEFAULT_XLR_IO_BASE_VIRT    	0xbef00000 	/* IO_BASE for Assembly macros */
#define DEFAULT_CPU_IO_BASE         	DEFAULT_XLR_IO_BASE
#define DEFAULT_CPU_IO_BASE_VIRT    	DEFAULT_XLR_IO_BASE_VIRT
#define CPU_IO_SIZE                     (1<<20)     /* Phoenix IO Size 	   		   */
#define HDR_OFFSET                      0x00000		/* Null value for XLR/XLP */	

/* PIO */

#define BRIDGE        					0x00000
#define SRAM          					0x07000
#define PIC       						0x08000
#define PCIX          					0x09000
#define HT            					0x0A000
#define UART0   						0x14000
#define UART1   						0x15000
#define I2C0        					0x16000
#define I2C1        					0x17000
#define GPIO          					0x18000
#define PIOBUS         					0x19000
#define PCIE_0         					0x1E000
#define PCIE_1        					0x1F000
#define SRIO_1        					0x1F000
#define USB        						0x24000
#define L2            					0x1b000

/**/
#define XGMAC0     						0x11000
#define XGMAC1     						0x13000
#define SPI40        					0x10000
#define SPI41        					0x12000

#define SECURITY      					0x0B000
/**/

#define DDR2_CHN0     					0x01000
#define DDR2_CHN1     					0x02000
#define DDR2_CHN2     					0x03000
#define DDR2_CHN3    					0x04000

#define RLD2_CHN0     					0x05000
#define RLD2_CHN1     					0x06000

/* Networking Blocks */
#define GMAC0        					0x0C000
#define GMAC1        					0x0D000
#define GMAC2        					0x0E000
#define GMAC3        					0x0F000
#define GMAC4   						0x20000
#define GMAC5   						0x21000

#define PHOENIX_CPLD_OFFSET     		0xffffffffbd840000ULL
#define GMAC6  							0x22000
#define GMAC7   						0x23000

#define CPU_MMIO_OFFSET(x)              DEFAULT_CPU_IO_BASE_VIRT + (x) 

#ifndef __ASSEMBLY__


#define cpu_read_reg(base, offset) (__be32_to_cpu((base)[(offset)]))
#define cpu_write_reg(base, offset, value) ((base)[(offset)] = __cpu_to_be32((value)))

#define phoenix_read_reg(base, offset)  (be32_to_cpu((base)[(offset)]))
#define phoenix_write_reg(base, offset, value) \
                                        ((base)[(offset)] = cpu_to_be32((value)))

static inline unsigned int xlr_revision(void)
{
    return chip_id() & 0xff00ff;
}

/* Returns '1' if the processor id belongs
 * to any one of the family of XLS Processors;
 * viz. 608/408/404/208/204, else '0'.
 */
static __inline__ int chip_processor_id_xls(void) {

    int processor_id = ((chip_id()>>8) & 0xff);
    switch (processor_id) {

        case CHIP_PROCESSOR_ID_XLS_608:
        case CHIP_PROCESSOR_ID_XLS_408:
        case CHIP_PROCESSOR_ID_XLS_404:
        case CHIP_PROCESSOR_ID_XLS_208:
        case CHIP_PROCESSOR_ID_XLS_204:
        case CHIP_PROCESSOR_ID_XLS_108:
        case CHIP_PROCESSOR_ID_XLS_104:
        case CHIP_PID_XLS_616_B0:
        case CHIP_PID_XLS_608_B0:
        case CHIP_PID_XLS_416_B0:
        case CHIP_PID_XLS_412_B0:
        case CHIP_PID_XLS_408_B0:
        case CHIP_PID_XLS_404_B0:
            return 1;
        default:
            return 0;
    }
}

static __inline__ int chip_processor_id_xls4xx(void) {

    int processor_id = ((chip_id()>>8) & 0xff);
    switch (processor_id) {
        case CHIP_PROCESSOR_ID_XLS_408:
        case CHIP_PROCESSOR_ID_XLS_404:
            return 1;
        default:
            return 0;
    }
}

static __inline__ int chip_processor_id_xls2xx(void) {

    int processor_id = ((chip_id()>>8) & 0xff);
    switch (processor_id) {
        case CHIP_PROCESSOR_ID_XLS_208:
        case CHIP_PROCESSOR_ID_XLS_204:
            return 1;
        default:
            return 0;
    }
}

static __inline__ int chip_processor_id_xls1xx(void) {

    int processor_id = ((chip_id()>>8) & 0xff);
    switch (processor_id) {
        case CHIP_PROCESSOR_ID_XLS_108:
        case CHIP_PROCESSOR_ID_XLS_104:
            return 1;
        default:
            return 0;
    }
}

/* The B0 Stepping of the XLS family... */
static __inline__ int chip_pid_xls_b0(void) {

    int processor_id = ((chip_id()>>8) & 0xff);
    switch (processor_id) {
        case CHIP_PID_XLS_616_B0:
        case CHIP_PID_XLS_608_B0:
        case CHIP_PID_XLS_416_B0:
        case CHIP_PID_XLS_412_B0:
        case CHIP_PID_XLS_408_B0:
        case CHIP_PID_XLS_404_B0:
            return 1;
        default:
            return 0;
    }
}
        
static __inline__ int is_xlr_c_series(void)
{
    int processor_id = ((chip_id()>>8) & 0xff);;
    int revision_id  = chip_id() & 0xff;
    int retval=0;

    switch (processor_id) {
        /* These are the relevant PIDs for XLR
         * steppings (hawk and above). For these,
         * PIDs, Rev-Ids of [5-9] indicate 'C'.
         */
        case 0x00: 
        case 0x02:
        case 0x07: 
        case 0x08:
        case 0x0A: 
        case 0x0B:
		case 0x0F:
            {
                if (revision_id>=5 && revision_id<=9) {
                    retval=1;
                }
                else {
                    retval=0;
                }
            }
            break;
        default:
            retval=0;
    }
    return retval;
}
#endif				// __ASSEMBLY__
#endif
