/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#include "system.h"
#include "psb.h"
#include "timer.h"
#include "cpu.h"
#include "stdlib.h"
#include "iomap.h"
#include "pic.h"
#include "nlm_hal.h"
#include "shared_structs.h"

/*
 * 'current' points to the current task struct. The task
 * struct could be of different types. One using linux
 * source base might prefer to use linux task structure
 * while those using 'pth' library might want 'current'
 * to point to 'pth_current'. Hence 'current' must be
 * set by individual modules. It is defined and 
 * initialized to NULL in srcs/kernel/kernel/time.c
 */

void *current = NULL;

unsigned int netos_mips_hpt_frequency;

#ifndef XLP_SIM
#define GPIO_RESET_CFG                  21

static uint32_t xls_get_cpu_clk(uint32_t cfg)
{
	int adivq, adivf;
	float ref = 66.67;
	float res;

	/* REF/2 * DIVF / DIVQ = PLLOUT */
	adivf = (cfg & 0xff) + 1;
	adivq = (cfg >> 8) & 0x7 ; 
	adivq = (1 << adivq); /* adivq = 2 ^ divq */

	res = ((ref / 2) * ((float)adivf/adivq)); 
	return (uint32_t)res;
}

static void phoenix_get_cpu_clk(void)
{
	int freq ;
	phoenix_reg_t *mmio = phoenix_io_mmio(PHOENIX_IO_GPIO_OFFSET);
	uint32_t cfg_reg = phoenix_read_reg(mmio,GPIO_RESET_CFG);

	if(chip_processor_id_xls())
		freq = xls_get_cpu_clk(cfg_reg);
	else {
		freq = (((cfg_reg >> 2) & 0x7f) + 1) * 16.667;

		if (cfg_reg & 0x200)
			freq = freq >> 1;
	}
	if (!freq) {
		netos_mips_hpt_frequency = 1200 * 1000 * 1000;
	} else {
		netos_mips_hpt_frequency = freq * 1000 * 1000;
	}

}
#endif

int init_netos(void)
{
#ifdef XLP_SIM
	netos_mips_hpt_frequency = nlm_cpu_frequency / HZ;
#else
	if(xlr_fdt.cpu_frequency == 0) {
		phoenix_get_cpu_clk();
		netos_mips_hpt_frequency = netos_mips_hpt_frequency / HZ;
	} else {
		netos_mips_hpt_frequency = xlr_fdt.cpu_frequency / HZ;
	}
#endif
	return 0;
}

