/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#include <stdio.h>
#include <stdlib.h>

#include "mipsregs.h"
#include "asmmacros.h"
#include "mips-exts.h"
#include "system.h"
#include "irq.h"
#include "cache.h"
#include "psb.h"
#include "mm.h"
#include "nlm_hal_macros.h"

static unsigned int config1;

static unsigned int icache_size;
static unsigned int dcache_size;
	
static unsigned int icache_line_size;
static unsigned int dcache_line_size;

static unsigned int icache_index_mask;
	
static unsigned int icache_assoc;
static unsigned int dcache_assoc;
	
static unsigned int icache_sets;
static unsigned int dcache_sets;
	

#define cache_op(op, base) __asm__ __volatile__ ("cache %0, 0(%1)\n" : : "i"(op), "r"(base))

void per_cpu_tlb_init(void)
{
	return;
}

void per_cpu_arch_init(void)
{
	unsigned long long value = 0;
	int flags = 0;

	/* Iniitialize TLBs */
	per_cpu_tlb_init();

	/* Enable weak ordering: TODO use macros for the value*/
	value = read_phnx_ctrl(0x300);
	write_phnx_ctrl(0x300, (value & ~0x0f00));

	/* Read the cache configuration */
	config1 = xlr_read_32bit_cp0_config1();
	
	icache_line_size = 1 << (((config1 >> 19) & 0x7)+1);
	dcache_line_size = 1 << (((config1 >> 10) & 0x7)+1);
	
	icache_sets = 1 << (((config1 >> 22) & 0x7)+6);
	dcache_sets = 1 << (((config1 >> 13) & 0x7)+6);
	
	icache_assoc = ((config1 >> 16) & 0x7) + 1;
	dcache_assoc = ((config1 >> 7) & 0x7) + 1;
	
	icache_size = icache_line_size * icache_sets * icache_assoc;
	dcache_size = dcache_line_size * dcache_sets * dcache_assoc;
	
	icache_index_mask = (icache_sets - 1) * icache_line_size;

	enable_KX(flags);
}

/* Dump the current entry* and pagemask registers */
static inline void dump_cur_tlb_regs(void)
{
	unsigned int entryhihi, entryhilo, entrylo0hi, entrylo0lo, entrylo1hi;
	unsigned int entrylo1lo, pagemask;

	__asm__ __volatile__ (
			".set push             \n"
			".set noreorder        \n"
			".set mips64           \n"
			".set noat             \n"
			"     tlbr             \n"
			"     dmfc0  $1, $10   \n"
			"     dsrl32 %0, $1, 0 \n"
			"     sra    %1, $1, 0 \n"
			"     dmfc0  $1, $2    \n"
			"     dsrl32 %2, $1, 0 \n"
			"     sra    %3, $1, 0 \n"
			"     dmfc0  $1, $3    \n"
			"     dsrl32 %4, $1, 0 \n"
			"     sra    %5, $1, 0 \n"
			"     mfc0   %6, $5    \n"
			".set pop              \n"
			: "=r" (entryhihi),
			"=r" (entryhilo),
			"=r" (entrylo0hi),
			"=r" (entrylo0lo),
			"=r" (entrylo1hi),
			"=r" (entrylo1lo),
			"=r" (pagemask));
	printf("%08X%08X %08X%08X %08X%08X %08X",
	 entryhihi, entryhilo,
	 entrylo0hi, entrylo0lo,
	 entrylo1hi, entrylo1lo,
	 pagemask);
}

void dump_tlb(int argc, char *argv[])
{
	int entry;
	int tlbsize;
	unsigned int config1;

	config1 = xlr_read_32bit_cp0_config1();
	tlbsize = ((config1 >> 25) & 0x3f) + 1;

	printf("Current TLB registers state:\n"
	 "      EntryHi       EntryLo0          EntryLo1     PageMask  Index\n"
	 "--------------------------------------------------------------------\n");
	dump_cur_tlb_regs();
	printf(" %08X\n", read_32bit_cp0_register(CP0_INDEX));
	printf("\n\nFull TLB Dump:\n"
	 "Idx      EntryHi       EntryLo0          EntryLo1     PageMask\n"
	 "--------------------------------------------------------------\n");
	for (entry = 0; entry < tlbsize; entry++) {
		set_index(entry);
		printf("\n%02i ", entry);
		__asm__ __volatile__ (
			  ".set push             \n"
			  ".set mips64           \n"
			  "     tlbr             \n"
			  ".set pop              \n");
		dump_cur_tlb_regs();
	}
	printf("\n");

}

void flush_dcache_all(void)
{
	int    i;
	int    dcache_lines;
	unsigned long base;

	dcache_lines = dcache_sets * dcache_assoc;
	base = (unsigned long)KSEG0;

	for (i = 0; i < dcache_lines; i++) {
		cache_op(Index_Writeback_Inv_D, base);
		base += dcache_line_size;
	}
}

void flush_icache_all(void)
{
	int    i;
	int    icache_lines;
	unsigned long base;

	icache_lines = icache_sets * icache_assoc;
	base = (unsigned long)KSEG0;

	for (i = 0; i < icache_lines; i++) {
		cache_op(Index_Invalidate_I, base);
		base += icache_line_size;
	}
}

void dump_cpu_info(int argc, char *argv[])
{
	printf("CPU ID: %d\n", processor_id());

	printf("Config Register 1: %08x\n"
	 "\t icache line size = %d\n"
	 "\t dcache ling size = %d\n"
	 "\t icache sets      = %d\n"
	 "\t dcache sets      = %d\n"
	 "\t icache assoc     = %d\n"
	 "\t dcache assoc     = %d\n"
	 "\t icache size      = %d\n"
	 "\t dcache size      = %d\n"
	 , xlr_read_32bit_cp0_config1(), 
	 icache_line_size, dcache_line_size,  
	 icache_sets, dcache_sets, 
	 icache_assoc, dcache_assoc, 
	 icache_size, dcache_size
	 );
	
	printf("CP 0 Registers:\n");
	printf("\tCP0_INDEX       = %08x\n", read_32bit_cp0_register(CP0_INDEX));
	printf("\tCP0_RANDOM      = %08x\n", read_32bit_cp0_register(CP0_RANDOM));
	printf("\tCP0_ENTRYLO0    = %016llx\n", read_64bit_cp0_register_sel(CP0_ENTRYLO0, 0));
	printf("\tCP0_ENTRYLO1    = %016llx\n", read_64bit_cp0_register_sel(CP0_ENTRYLO1, 0));
	printf("\tCP0_CONF        = %08x\n", read_32bit_cp0_register(CP0_CONF));
	printf("\tCP0_CONTEXT     = %016llx\n", read_64bit_cp0_register_sel(CP0_CONTEXT, 0));
	printf("\tCP0_PAGEMASK    = %08x\n", read_32bit_cp0_register(CP0_PAGEMASK));
	printf("\tCP0_WIRED       = %08x\n", read_32bit_cp0_register(CP0_WIRED));
	printf("\tCP0_INFO        = %08x\n", read_32bit_cp0_register(CP0_INFO));
	printf("\tCP0_BADVADDR    = %016llx\n", read_64bit_cp0_register_sel(CP0_BADVADDR, 0));
	printf("\tCP0_COUNT       = %08x\n", read_32bit_cp0_register(CP0_COUNT));
	printf("\tCP0_ENTRYHI     = %016llx\n", read_64bit_cp0_register_sel(CP0_ENTRYHI, 0));
	printf("\tCP0_COMPARE     = %08x\n", read_32bit_cp0_register(CP0_COMPARE));
	printf("\tCP0_STATUS      = %08x\n", read_32bit_cp0_register(CP0_STATUS));
	printf("\tCP0_CAUSE       = %08x\n", read_32bit_cp0_register(CP0_CAUSE));
	printf("\tCP0_EPC         = %016llx\n", read_64bit_cp0_register_sel(CP0_EPC, 0));
	printf("\tCP0_PRID        = %08x\n", read_32bit_cp0_register(CP0_PRID));
	printf("\tCP0_CONFIG      = %08x\n", read_32bit_cp0_register(CP0_CONFIG));
	printf("\tCP0_LLADDR      = %016llx\n", read_64bit_cp0_register_sel(CP0_LLADDR, 0));
	printf("\tCP0_WATCHLO     = %016llx\n", read_64bit_cp0_register_sel(CP0_WATCHLO, 0));
	printf("\tCP0_WATCHHI     = %08x\n", read_32bit_cp0_register(CP0_WATCHHI));
	printf("\tCP0_XCONTEXT    = %016llx\n", read_64bit_cp0_register_sel(CP0_XCONTEXT, 0));
	printf("\tCP0_FRAMEMASK   = %08x\n", read_32bit_cp0_register(CP0_FRAMEMASK));
	printf("\tCP0_DIAGNOSTIC  = %08x\n", read_32bit_cp0_register(CP0_DIAGNOSTIC));
	printf("\tCP0_DEBUG       = %08x\n", read_32bit_cp0_register(CP0_DEBUG));
	printf("\tCP0_DEPC        = %08x\n", read_32bit_cp0_register(CP0_DEPC));
	printf("\tCP0_PERFORMANCE = %08x\n", read_32bit_cp0_register(CP0_PERFORMANCE));
	printf("\tCP0_ECC         = %08x\n", read_32bit_cp0_register(CP0_ECC));
	printf("\tCP0_CACHEERR    = %08x\n", read_32bit_cp0_register(CP0_CACHEERR));
	printf("\tCP0_TAGLO       = %016llx\n", read_64bit_cp0_register_sel(CP0_TAGLO, 0));
	printf("\tCP0_TAGHI       = %016llx\n", read_64bit_cp0_register_sel(CP0_TAGHI, 0));
	printf("\tCP0_ERROREPC    = %016llx\n", read_64bit_cp0_register_sel(CP0_ERROREPC, 0));
	printf("\tCP0_DESAVE      = %08x\n", read_32bit_cp0_register(CP0_DESAVE));
}
