/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#include "types.h"
#include "shared_structs.h"

#include "psb.h"
#include "system.h"
#include "mips-exts.h"
#include "spinlock.h"
#include "bridge.h"
#include "iomap.h"

#define BARS 19
typedef struct {
	char name[20]; /* preprogrammed */
	int regnum;
	int enable; /* is it enabled at runtime? */
	int mask_min; /* bits to detenlmne mask */
	int mask_max;
	int base_min; /*bits to locate base value */
	int base_shift; /* to detenlmne min size of bar */
	uint64_t base;
	uint64_t max;
} bar;

bar BRIDGE_BARS[BARS] = {
	{"DRAM0 ",0, 0,4,15,16,24,0,0},
	{"DRAM1 ",1, 0,4,15,16,24,0,0},
	{"DRAM2 ",2, 0,4,15,16,24,0,0},
	{"DRAM3 ",3, 0,4,15,16,24,0,0},
	{"DRAM4 ",4, 0,4,15,16,24,0,0},
	{"DRAM5 ",5, 0,4,15,16,24,0,0},
	{"DRAM6 ",6, 0,4,15,16,24,0,0},
	{"DRAM7 ",7, 0,4,15,16,24,0,0},
	{"FLASH ",26,0,5,15,16,24,0,0},
	{"SRAM  ",27,0,15,15,16,24,0,0},
	{"HTMEM ",28,0,1,15,16,24,0,0},
	{"HTINT ",29,0,1,8,16,24,0,0},
	{"HTPIC ",30,0,0,0,12,24,0,0},
	{"HTSM  ",31,0,0,0,12,24,0,0},
	{"HTIO  ",32,0,0,0,17,25,0,0},
	{"HTCFG ",33,0,0,0,17,25,0,0},
	{"PCICFG",34,0,0,0,17,25,0,0},
	{"PCIMEM",35,0,1,15,16,24,0,0},
	{"PCI IO",36,0,0,0,18,26,0,0},
};

static void insert_bar_val(int bnum, uint32_t val) {
	int i = 0, j =0;
	unsigned mask,size;
	uint64_t MASK = 0;

	mask = 0;
	for (i = 0; i < BARS; i++) {
		if (BRIDGE_BARS[i].regnum == bnum) { // insert value
			if (val & 1) {
				printf("Inserting %2d (%s) val %08x ",bnum,BRIDGE_BARS[i].name,val);
				BRIDGE_BARS[i].enable = 1;
				size = BRIDGE_BARS[i].base_shift;
				if (BRIDGE_BARS[i].mask_min) {
					mask = 0;
					for (j=BRIDGE_BARS[i].mask_min;j<=BRIDGE_BARS[i].mask_max;j++) 
						mask = (mask << 1) | 1;
	
					MASK = (uint64_t)((val >> BRIDGE_BARS[i].mask_min) & mask);
	
					if (MASK) {
						for (j = BRIDGE_BARS[i].mask_min; j <= BRIDGE_BARS[i].mask_max; j++) {
				  			if (val & (1<<j)) {
								size ++;
				  			}
						}
					}
				}
				printf("Mask bits available: %04x ",mask);
				printf("Actual mask = 0x%llx ", (unsigned long
								 long)MASK);
				BRIDGE_BARS[i].base = ~MASK & 
						((uint64_t)val >> BRIDGE_BARS[i].base_min) << BRIDGE_BARS[i].base_shift;
				BRIDGE_BARS[i].max = BRIDGE_BARS[i].base + ((uint64_t)1 << size) - 1;
				printf("Size %05d MB ", 1 << (size - 20));
				printf("Base = %012llX upper = %012llX\n",
				       (unsigned long long)BRIDGE_BARS[i].base ,
				       (unsigned long long)BRIDGE_BARS[i].max );
			} 
			else {
				printf("%s is not enabled\n",BRIDGE_BARS[i].name);
			}
		}
	}
}

int check_bar_conflicts () {
	unsigned     i,j;
	int          retval = 0;
	uint32_t* mmio = (uint32_t *)(phoenix_io_base + PHOENIX_IO_BRIDGE_OFFSET);
	
	for (i = 0; i < BARS; i++) {
		insert_bar_val(BRIDGE_BARS[i].regnum,mmio[BRIDGE_BARS[i].regnum]);
	}

	for (i = 0; i < BARS; i++) {
		for (j = 0; j < BARS; j++) {
			if ((i != j) && BRIDGE_BARS[i].enable && BRIDGE_BARS[j].enable) {
				if (BRIDGE_BARS[i].max > BRIDGE_BARS[j].base && BRIDGE_BARS[i].max < BRIDGE_BARS[j].max) {
					printf("ERROR: BAR %s max overlaps with %s\n", BRIDGE_BARS[i].name, BRIDGE_BARS[j].name);
					retval = -1;
				}
				if(BRIDGE_BARS[i].base > BRIDGE_BARS[j].base && BRIDGE_BARS[i].base < BRIDGE_BARS[j].max) {
					printf("ERROR: BAR %s min overlaps with %s\n", BRIDGE_BARS[i].name, BRIDGE_BARS[j].name);
					retval = -1;
				}
			}
		}
	}
	return retval;
}
