/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/
#include <stdarg.h>
#include <unistd.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <contig_alloc_param.h>
#include <sys_entry.h>


void *contig_alloc(size_t size)
{
	struct nlm_contig_alloc_call ctgalloc;
	struct nlm_contig_alloc *m;

	ctgalloc.cmd = NLM_CONTIGALLOC_CALL;
	m = &ctgalloc.alloc;
	m->size  = size;
	nlm_sys_call(NLM_CONTIGALLOC_SYS_CALL, (void *)&ctgalloc);
	return ctgalloc.rv != 0 ? NULL : (void *)(long)m->memptr;
}

void *contig_calloc(size_t nelem, size_t elsize)
{
	struct nlm_contig_alloc_call ctgalloc;
	struct nlm_contig_calloc *m;

	ctgalloc.cmd = NLM_CONTIGCALLOC_CALL;
	m = &ctgalloc.calloc;
	m->nelem = nelem;
	m->elsize  = elsize;
	nlm_sys_call(NLM_CONTIGALLOC_SYS_CALL, (void *)&ctgalloc);
	return ctgalloc.rv != 0 ? NULL : (void *)(long)m->memptr;
}

void *contig_realloc(void *ptr, size_t size)
{
	struct nlm_contig_alloc_call ctgalloc;
	struct nlm_contig_realloc *m;

	ctgalloc.cmd = NLM_CONTIGREALLOC_CALL;
	m = &ctgalloc.realloc;
	m->obj = (long)ptr;
	m->size  = size;
	nlm_sys_call(NLM_CONTIGALLOC_SYS_CALL, (void *)&ctgalloc);
	return ctgalloc.rv != 0 ? NULL : (void *)(long)m->memptr;
}

void *contig_malign(size_t align, size_t size)
{
	struct nlm_contig_alloc_call ctgalloc;
	struct nlm_contig_malign *m;

	ctgalloc.cmd = NLM_CONTIGMALIGN_CALL;
	m = &ctgalloc.malign;
	m->align = align;
	m->size  = size;
	nlm_sys_call(NLM_CONTIGALLOC_SYS_CALL, (void *)&ctgalloc);
	return ctgalloc.rv != 0 ? NULL : (void *)(long)m->memptr;
}

void contig_free(void *ptr)
{
	struct nlm_contig_alloc_call ctgalloc;
	struct nlm_contig_free *m;

	ctgalloc.cmd = NLM_CONTIGFREE_CALL;
	m = &ctgalloc.free;
	m->memptr = (long)ptr;
	nlm_sys_call(NLM_CONTIGALLOC_SYS_CALL, (void *)&ctgalloc);
	return;
}

int contig_pinfo(unsigned long long *virtstart, unsigned long long *physstart)
{
	struct nlm_contig_alloc_call ctgalloc;
	struct nlm_contig_pinfo *m;

	ctgalloc.cmd = NLM_CONTIGPINFO_CALL;
	m = &ctgalloc.pinfo;
	nlm_sys_call(NLM_CONTIGALLOC_SYS_CALL, (void *)&ctgalloc);
	*virtstart = m->virtstart;
	*physstart = m->physstart;
	return ctgalloc.rv;
}
