/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#include <stdio.h>
#include <stdlib.h>
#include <system.h>
#include <libfdt.h>
#include <smp.h>		/* cpu_online_map */
#include <spinlock.h>
#include <devices.h>
#include <nae.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <nlm_hal_nae.h>
#ifdef XLR
#include <shared_structs.h>
#endif
#include "uart.h"
#include "fdt_local.h"

/* Some defines for FDT
 */
#define NAE_CFG_NODE            "/soc/nae-cfg"
#define UCORE_CFG_NODE          "/soc/nae-cfg/ucore/src@1"

/* FDT-related Global variables
 */
int dom_id;
char arcs_cmdline[CMDLINE_SIZE];

int rx_vc, fb_vc;

extern uint32_t cpu_online_map;
extern uint64_t heap_size;
extern void uart_update_params(uint64_t sharedcfg, int id);

static __inline__ unsigned long long get_fdt_dword(unsigned int *prop)
{
	unsigned int *buf = prop;
#ifdef __MIPSEL__
    return ((uint64_t)buf[1] << 32 | buf[0]);
#else
	return ((uint64_t)buf[0] << 32 | buf[1]);
#endif
}
static int prop_len = 0;

int get_fdt_prop(void *fdt, const char* node_path, const char *prop, void *prop_buf, int type)
{
	const void *pbuf;

#ifdef __MIPSEL__ /* Little Endian */
	char * src, *dst;
	int i;
#endif

	int node = fdt_path_offset(fdt, node_path);

	if (node < 0) {
		return -1;
	}

	pbuf = (char *)fdt_getprop(fdt, node, prop, &prop_len);
	if (pbuf == NULL) {
		return -1;
	}
#ifdef __MIPSEL__ /* Little Endian */

	if (type == PROP_STR) {
		memcpy((void *) prop_buf, (void * )pbuf, prop_len);

	} else { /* Copy in reverse to account for host LE order */
		src = (char *) pbuf;
		dst = (char *) prop_buf;
		for (i = 0; i < prop_len; ++i) {
			dst[i] = src[prop_len - i - 1];
		}
	}

#else
	memcpy((void *) prop_buf, (void * )pbuf, prop_len);
#endif /* #ifdef (__MIPSEL__) */
	return 0;
}

#ifdef XLP_SIM
int process_fdt(void *fdt) 
{
	char dom_node_str[32];

	unsigned int regs[4];
	int uart_id, fdt_err;
	uint32_t uart_sharedcfg;

	printf("(Hyperexec): fdt = %p, dom_id = %d\n", fdt, dom_id);
	if (fdt == NULL) {
		printf("FDT_ERROR: Got NULL FDT Pointer, Aborting!\n");
		return -1;
	}

	/* Validate FDT
	 */
	fdt_err = fdt_check_header(fdt);
	if (fdt_err < 0) {
		printf("FDT_ERROR: Invalid FDT Detected: %s\n",
				fdt_strerror(fdt_err));
		return -1;
	}

	snprintf(dom_node_str, 32, "/doms/dom@%d/uart", dom_id);
	GET_FPROP(fdt, dom_node_str,"id", &uart_id);
	if (get_fdt_prop(fdt, dom_node_str, "sharedcfg", &uart_sharedcfg, PROP_INT32) != 0) {

		if (uart_id == 0)
			xlr_netos_uart = TTYS0;
		else
			xlr_netos_uart = TTYS1;
	}
	else {
		xlr_netos_uart = VIRT_UART;
		uart_update_params((uint64_t)uart_sharedcfg, dom_id);
	}

	snprintf(dom_node_str, 32, "/doms/dom@%d", dom_id);

	/* get the heap size */
	GET_FPROP(fdt, dom_node_str, "heap-size", regs);

	heap_size = get_fdt_dword(regs);
	if(heap_size <= MIN_HEAP_SIZE) {
		printf("Error: Application is booted with invalid heap size\n");
		return -1;
	}
	printf("%s heapsize %lx\n", __func__, heap_size);

	/* extract CPU online info and UART
	 */
	GET_FPROP_STR(fdt, "/chosen", 		 "bootargs", arcs_cmdline);
	GET_FPROP_STR(fdt, dom_node_str, 	         "device_type", arcs_cmdline);

	snprintf(dom_node_str, 32, "/doms/dom@%d/cpu", dom_id);

	return 0;
}

#else
xlr_fdt_info_t xlr_fdt;
int process_fdt(void *fdt) 
{
	int i;
	unsigned int regs[4];
	int uart_id, fdt_err;
	unsigned int iobase, global_shmem;
	uint8_t macaddr[6];

	if (fdt == NULL) {
		printf("FDT_ERROR: Got NULL FDT Pointer, Aborting!\n");
		return -1;
	}

	/* Validate FDT 
	 */
	fdt_err = fdt_check_header(fdt);
	if (fdt_err < 0) {
		printf("FDT_ERROR: Invalid FDT Detected: %s\n", 
				fdt_strerror(fdt_err));
		return -1;
	}

	/* get the heap size */
	GET_FPROP(fdt, "/doms/dom@0", "heap-size", regs);
	heap_size = get_fdt_dword(regs);
	if(heap_size <= MIN_HEAP_SIZE) {
		printf("Error: Application is booted with invalid heap size\n");
		return -1;
	}
	printf("%s heapsize %lx\n", __func__, heap_size);

	/* extract CPU online info and UART
	 */
	GET_FPROP_STR(fdt, "/chosen", 		 "bootargs", arcs_cmdline);
	GET_FPROP_STR(fdt, "/chosen", 		 "bootenv", xlr_fdt.boot_env);
	GET_FPROP(fdt, "/doms/dom@0", 	 "device_type", arcs_cmdline);
	GET_FPROP(fdt, "/doms/dom@0/uart","id", &uart_id);
	GET_FPROP(fdt, "/doms/dom@0/cpu", "onlinemask", &cpu_online_map);
	GET_FPROP(fdt, "/system", "boardmajor", &xlr_fdt.major);
	GET_FPROP(fdt, "/system", "boardminor", &xlr_fdt.minor);
	GET_FPROP(fdt, "/system", "cpufrequency", &xlr_fdt.cpu_frequency);
	GET_FPROP(fdt, "/system", "globalshmem", &global_shmem);
	GET_FPROP(fdt, "/system", "iobase", &iobase);
	GET_FPROP(fdt, "/system", "macaddress", &macaddr);

	xlr_fdt.global_shmem_addr = (unsigned long)(int)global_shmem;
	xlr_fdt.iobase = (unsigned long)(int)iobase;
	for(i=0; i < 6; i++) {
		xlr_fdt.mac_addr[i] = macaddr[i];
	}

	return 0;
}
#endif

int fdt_get_cpu_online_map(void *fdt, unsigned int *map)
{
	char dom_node_str[32];

	snprintf(dom_node_str, 32, "/doms/dom@%d/cpu", dom_id);
	GET_FPROP(fdt, dom_node_str, "onlinemask", map);
	return 0;
}

int fdt_get_shared_mem(void *fdt, unsigned long long *off, 
		unsigned long long *sz)
{
	unsigned int regs[4];
	char dom_node_str[32];

	snprintf(dom_node_str, 32, "/doms/dom@%d", dom_id);

	GET_FPROP(fdt, dom_node_str, "shared-mem", regs);

#ifndef __MIPSEL__
	*off = (unsigned long long) regs[0] << 32 | regs[1];
	*sz = (unsigned long long) regs[2] << 32 | regs[3];
#else
	*off = get_fdt_dword(&regs[2]);
	*sz = get_fdt_dword(regs);
#endif
	return 0;
}

