/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#include <stdio.h>
#include <stdlib.h>

#include "iomap.h"
#include "msgring.h"
#include "types.h"
#include "pci.h"
#include "bridge.h"
#include "mips-exts.h"
#include "gpio.h"
#include "system.h"

#define MSGRNG_CC_INIT_CPU_DEST(conf, dest) \
do { \
	   msgrng_write_cc(MSGRNG_CC_##dest##_REG, conf[dest][0], 0 ); \
	   msgrng_write_cc(MSGRNG_CC_##dest##_REG, conf[dest][1], 1 ); \
	   msgrng_write_cc(MSGRNG_CC_##dest##_REG, conf[dest][2], 2 ); \
	   msgrng_write_cc(MSGRNG_CC_##dest##_REG, conf[dest][3], 3 ); \
	   msgrng_write_cc(MSGRNG_CC_##dest##_REG, conf[dest][4], 4 ); \
	   msgrng_write_cc(MSGRNG_CC_##dest##_REG, conf[dest][5], 5 ); \
	   msgrng_write_cc(MSGRNG_CC_##dest##_REG, conf[dest][6], 6 ); \
	   msgrng_write_cc(MSGRNG_CC_##dest##_REG, conf[dest][7], 7 ); \
} while(0)

/* Initialized CC for cpu 0 to send to all buckets at 0-7 cpus */
#define MSGRNG_CC_INIT_CPU(conf) \
do { \
	MSGRNG_CC_INIT_CPU_DEST(conf,0); \
	MSGRNG_CC_INIT_CPU_DEST(conf,1); \
	MSGRNG_CC_INIT_CPU_DEST(conf,2); \
	MSGRNG_CC_INIT_CPU_DEST(conf,3); \
	MSGRNG_CC_INIT_CPU_DEST(conf,4); \
	MSGRNG_CC_INIT_CPU_DEST(conf,5); \
	MSGRNG_CC_INIT_CPU_DEST(conf,6); \
	MSGRNG_CC_INIT_CPU_DEST(conf,7); \
	MSGRNG_CC_INIT_CPU_DEST(conf,8); \
	MSGRNG_CC_INIT_CPU_DEST(conf,9); \
	MSGRNG_CC_INIT_CPU_DEST(conf,10); \
	MSGRNG_CC_INIT_CPU_DEST(conf,11); \
	MSGRNG_CC_INIT_CPU_DEST(conf,12); \
	MSGRNG_CC_INIT_CPU_DEST(conf,13); \
	MSGRNG_CC_INIT_CPU_DEST(conf,14); \
	MSGRNG_CC_INIT_CPU_DEST(conf,15); \
} while (0)

#define MSGRNG_BUCKETSIZE_INIT_CPU(conf) \
do { \
	msgrng_write_bucksize(0, conf[0]); \
	msgrng_write_bucksize(1, conf[1]); \
	msgrng_write_bucksize(2, conf[2]); \
	msgrng_write_bucksize(3, conf[3]); \
	msgrng_write_bucksize(4, conf[4]); \
	msgrng_write_bucksize(5, conf[5]); \
	msgrng_write_bucksize(6, conf[6]); \
	msgrng_write_bucksize(7, conf[7]); \
} while(0);

static void check_msgring_cfg_defs(void)
{
	if (is_xls()) {
		if (((unsigned long)&xls_bucket_sizes == 0) ||
		    ((unsigned long)&xls_cc_table_cpu_0 == 0) ||
		    ((unsigned long)&xls_cc_table_cpu_1 == 0) ||
		    ((unsigned long)&xls_cc_table_cpu_2 == 0) ||
		    ((unsigned long)&xls_cc_table_cpu_3 == 0)
			)
		    goto err;
	}
	else {
		if (((unsigned long)&bucket_sizes == 0) ||
		    ((unsigned long)&cc_table_cpu_0 == 0) ||
		    ((unsigned long)&cc_table_cpu_1 == 0) ||
		    ((unsigned long)&cc_table_cpu_2 == 0) ||
		    ((unsigned long)&cc_table_cpu_3 == 0) ||
		    ((unsigned long)&cc_table_cpu_4 == 0) ||
		    ((unsigned long)&cc_table_cpu_5 == 0) ||
		    ((unsigned long)&cc_table_cpu_6 == 0) ||
		    ((unsigned long)&cc_table_cpu_7 == 0)
			)
			goto err;
	}

	return ;
	err:
	printf("***ERROR***: Message Ring Configuration file is not compiled in????\n");
	cpu_halt();
}

void message_ring_cpu_init(void)
{
	int cpu = phnx_cpu_id();
	bucket_t *buckets;
    bucket_t (*credits)[8] = NULL;


	/* Only the thr_0 o:f any core configures the message station */
	//if (thr != 0) return;

	/* update config register - interrupts disabled */
	write_32bit_cp2_register(MSGRNG_MSG_CONFIG_REG, 0);

	check_msgring_cfg_defs();

	if(is_xls()) {
			buckets = &xls_bucket_sizes.bucket[(cpu * MAX_STNS_PER_CORE)];
			switch(cpu) {
				case 0:
					credits = xls_cc_table_cpu_0.counters;
					break;
				case 1:
					credits = xls_cc_table_cpu_1.counters;
					break;
				case 2:
					credits = xls_cc_table_cpu_2.counters;
					break;
				case 3:
					credits = xls_cc_table_cpu_3.counters;
					break;
				default:
					printf("***ERROR***: Invalid cpu id\n");
					break;
				}
	} else {
		buckets = &bucket_sizes.bucket[(cpu * MAX_STNS_PER_CORE)];
		switch(cpu) {
			case 0:
				credits = cc_table_cpu_0.counters;
				break;
			case 1:
				credits = cc_table_cpu_1.counters;
				break;
			case 2:
				credits = cc_table_cpu_2.counters;
				break;
			case 3:
				credits = cc_table_cpu_3.counters;
				break;
			case 4:
				credits = cc_table_cpu_4.counters;
				break;
			case 5:
				credits = cc_table_cpu_5.counters;
				break;
			case 6:
				credits = cc_table_cpu_6.counters;
				break;
			case 7:
				credits = cc_table_cpu_7.counters;
				break;
			default:
				printf("***ERROR***: Invalid cpu id\n");
				return;
		}
	}

	MSGRNG_BUCKETSIZE_INIT_CPU(buckets);
	MSGRNG_CC_INIT_CPU(credits);
}
