/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

// packet generator
// for net_c env
// supports ethernet l2 type +crc only. 
// future support planned for ipv4 ipv6 mpls atm ppp esp ah
// 
#include <stdio.h>
#include <stdlib.h>

#include "pkt_gen.h"
#include "types.h"

/* 
 * CRC Table
 */

unsigned long crc32_table[256];
unsigned long crc32_table_ttable[256];


//******************************************************************************** 
//* Function   : 
//* Desription : Allocate memory for a packet
//* Return     : 
//* Arguments  : 
//******************************************************************************** 

p_packet pkt_new () {
	p_packet npkt;
	// npkt = (p_packet)malloc((sizeof(c_packet))+32);
	// if ( npkt == 0 ) {
	//         printf("Null pointer to packet \n");
	// }
	// npkt = (p_packet)(((unsigned int)npkt & 0xffffffe0) + 32);

	npkt = alloc_pbuf();
	if ( npkt == 0 ) 
	  printf("PKT_NEW: Null pointer to packet \n");
	return (npkt);
}

//******************************************************************************** 
//* Function   : 
//* Desription : Fill up the packet with an incrementing payload
//* Return     : 
//* Arguments  : 
//******************************************************************************** 

void pkt_payload ( p_packet pkt, int _length ) {
	int i ;

	pkt->len = _length ;
	for ( i = 0 ; i < _length ; i ++ ) {
		pkt->data[i] = i ;
	}
}

//******************************************************************************** 
//* Function   : 
//* Desription : Print the contents of a packet
//* Return     : 
//* Arguments  : 
//******************************************************************************** 

void pkt_print (p_packet pkt) {
	int i ;

	printf("Packet Length %0d\n",pkt->len);
	for ( i = 0 ; i < pkt->len ; i ++ ) {
		if ( i%16 == 0 ) 
			printf ("\n");
		printf ("   %02x", pkt->data[i]);
	}
	printf ("\n");
}

//******************************************************************************** 
//* Function   : 
//* Desription : Append Ethernet L2 headers to packet
//* Return     : 
//* Arguments  : 
//******************************************************************************** 

void pkt_ethernet_encap ( p_packet pkt, long long da, long long sa, int type ) {

	space(pkt,14);

	pkt->data[0]  = ( da >> 40 ) & 0xff ;
	pkt->data[1]  = ( da >> 32 ) & 0xff ;
	pkt->data[2]  = ( da >> 24 ) & 0xff ;
	pkt->data[3]  = ( da >> 16 ) & 0xff ;
	pkt->data[4]  = ( da >> 8  ) & 0xff ;
	pkt->data[5]  = ( da >> 0  ) & 0xff ;

	pkt->data[6]  = ( sa >> 40 ) & 0xff ;
	pkt->data[7]  = ( sa >> 32 ) & 0xff ;
	pkt->data[8]  = ( sa >> 24 ) & 0xff ;
	pkt->data[9]  = ( sa >> 16 ) & 0xff ;
	pkt->data[10] = ( sa >> 8  ) & 0xff ;
	pkt->data[11] = ( sa >> 0  ) & 0xff ;

	pkt->data[12] = ( type >> 8  ) & 0xff ;
	pkt->data[13] = ( type >> 0  ) & 0xff ;
}


void pkt_ethernet_encap_notype ( p_packet pkt, long long da, long long sa ) {

	space(pkt,12);

	pkt->data[0]  = ( da >> 40 ) & 0xff ;
	pkt->data[1]  = ( da >> 32 ) & 0xff ;
	pkt->data[2]  = ( da >> 24 ) & 0xff ;
	pkt->data[3]  = ( da >> 16 ) & 0xff ;
	pkt->data[4]  = ( da >> 8  ) & 0xff ;
	pkt->data[5]  = ( da >> 0  ) & 0xff ;

	pkt->data[6]  = ( sa >> 40 ) & 0xff ;
	pkt->data[7]  = ( sa >> 32 ) & 0xff ;
	pkt->data[8]  = ( sa >> 24 ) & 0xff ;
	pkt->data[9]  = ( sa >> 16 ) & 0xff ;
	pkt->data[10] = ( sa >> 8  ) & 0xff ;
	pkt->data[11] = ( sa >> 0  ) & 0xff ;

}

//******************************************************************************** 
//* Function   : 
//* Desription : Append FCS to the end of the packet
//* Return     : 
//* Arguments  : 
//******************************************************************************** 

void pkt_crc_encap ( p_packet pkt ) {
	unsigned            pktCrc ;

	pktCrc = crc32(pkt->data,pkt->len);

	pkt->data [ pkt->len+0 ] = ~( pktCrc & 0xff ) ;
	pkt->data [ pkt->len+1 ] = ~( ( pktCrc & 0xff00 ) >> 8 )  ;
	pkt->data [ pkt->len+2 ] = ~( ( pktCrc & 0xff0000 ) >> 16 ) ;
	pkt->data [ pkt->len+3 ] = ~( ( pktCrc & 0xff000000 ) >> 24 ) ;

	pkt->len = pkt->len + 4 ;
}

//******************************************************************************** 
//* Function   : 
//* Desription : Free up space used by the packet
//* Return     : 
//* Arguments  : 
//******************************************************************************** 

void pkt_delete ( p_packet pkt ) {
	// free (pkt);
	      free_pbuf(pkt);	
}

//******************************************************************************** 
//* Function   : 
//* Desription : Create space for encap function
//* Return     : 
//* Arguments  : 
//******************************************************************************** 

void space ( p_packet pkt, int size ) {
	int i ;

	for ( i = pkt->len ; i > 0 ; i -- ) 
		pkt->data [ i + size - 1 ] = pkt->data [ i - 1 ];

	pkt->len = pkt->len + size ;

	for ( i = 0 ; i < size ; i ++ ) 
		pkt->data [ i ] = 0 ;
	       
}

//******************************************************************************** 
//* Function   : 
//* Desription : Insert the sequence number into the packet. The last few 
//*            : bytes will be over written by the anchoring pattern and the 
//*            : the sequence number
//* Return     : 
//* Arguments  : 
//******************************************************************************** 

void  pkt_insert_seq ( p_packet pkt, int seq_num ) {

	pkt->data [ (pkt->len)-5 ] = ( SEQ_SIGNATURE & 0xff ) ;
	pkt->data [ (pkt->len)-6 ] = ( ( SEQ_SIGNATURE & 0xff00 ) >> 8 )  ;
	pkt->data [ (pkt->len)-7 ] = ( ( SEQ_SIGNATURE & 0xff0000 ) >> 16 ) ;
	pkt->data [ (pkt->len)-8 ] = ( ( SEQ_SIGNATURE & 0xff000000 ) >> 24 ) ;

	pkt->data [ (pkt->len)-1 ] = ( seq_num & 0xff ) ;
	pkt->data [ (pkt->len)-2 ] = ( ( seq_num & 0xff00 ) >> 8 )  ;
	pkt->data [ (pkt->len)-3 ] = ( ( seq_num & 0xff0000 ) >> 16 ) ;
	pkt->data [ (pkt->len)-4 ] = ( ( seq_num & 0xff000000 ) >> 24 ) ;
}

//******************************************************************************** 
//* Function   : 
//* Desription : Calculate crc
//* Return     : 
//* Arguments  : 
//******************************************************************************** 

unsigned long crc32(unsigned char *buf, int len) {
	unsigned char *p;
	unsigned long  crc;

	if (!crc32_table[1])    /* if not already done, */
		init_crc32();   /* build table */
	crc = 0xffffffff;       /* preload shift register, per CRC-32 spec */
	for (p = buf; len > 0; ++p, --len) {
		crc = (crc << 8) ^ crc32_table[(crc >> 24) ^ flip_byte(*p)];
	} 

	return flip_32(crc);            /* transmit complement, per CRC-32 spec */
}

//******************************************************************************** 
//* Function   : 
//* Desription : Initialize the crc function
//* Return     : 
//* Arguments  : 
//******************************************************************************** 

void init_crc32(void) {
	int i, j;
	unsigned long c;

	for (i = 0; i < 256; ++i) {
		for (c = i << 24, j = 8; j > 0; --j)
			c = c & 0x80000000 ? (c << 1) ^ CRC32_POLY : (c << 1);
		crc32_table[i] = c;
	}
}


//******************************************************************************** 
//* Function   : 
//* Desription : Initialize the crc function for ttable. This has a programable
//             : polynomial
//* Return     : 
//* Arguments  : 
//******************************************************************************** 

void init_crc32_ttable(int hash_poly) {
	int i, j;
	unsigned long c;
	      unsigned long CRC32_POLY_TTABLE = ( hash_poly & 0x7F ) << 25 ;

	for (i = 0; i < 256; ++i) {
		for (c = i << 24, j = 8; j > 0; --j)
			c = c & 0x80000000 ? (c << 1) ^ CRC32_POLY_TTABLE : (c << 1);
		crc32_table_ttable[i] = c;
	}
}

//******************************************************************************** 
//* Function   : 
//* Desription : CRC function for the hash in the prepad
//* Return     : 
//* Arguments  : 
//******************************************************************************** 
//
unsigned long crc32_for_hash(unsigned char *buf, int len) {
	      unsigned char *p;
	      unsigned long  crc;

	      if (!crc32_table[1])    /* if not already done, */
	              init_crc32();   /* build table */
	      crc = 0xffffffff;       /* preload shift register, per CRC-32 spec */
	      for (p = buf; len > 0; ++p, --len) {
	              crc = (crc << 8) ^ crc32_table[(crc >> 24) ^ (*p)];
	      }

	// act_crc  = crc ;
	// flip_crc_1 = ~crc & 0xffffffff ;
	      // flip_crc = flip_32(crc);
	      // printf("2 .Before flip crc = %x, after flip crc = %x %llx \n",act_crc,flip_crc,flip_crc_1);
	      return (crc);            /* transmit complement, per CRC-32 spec */
}


//******************************************************************************** 
//* Function   : 
//* Desription : CRC function for the hash into the translate_table
//* Return     : 
//* Arguments  : 
//******************************************************************************** 
//
unsigned long crc32_for_ttable(unsigned char *buf, int len, int hash_poly) {
	unsigned char *p;
	unsigned long  crc;

	if (!crc32_table_ttable[1])    /* if not already done, */
		init_crc32_ttable(hash_poly);   /* build table */
	crc = 0xfe000000;       /* preload shift register, per CRC-32 spec */
	for (p = buf; len > 0; ++p, --len) {
		crc = (crc << 8) ^ crc32_table_ttable[(crc >> 24) ^ (*p)];
	}

	// act_crc  = crc ;
	// flip_crc_1 = ~crc & 0xffffffff ;
	// flip_crc = flip_32(crc);
	// printf("2 .Before flip crc = %x, after flip crc = %x %llx \n",act_crc,flip_crc,flip_crc_1);

	return (crc);            /* transmit complement, per CRC-32 spec */
}

//******************************************************************************** 
//* Function   : 
//* Desription : Flips around an byte. Used in the crc function
//* Return     : 
//* Arguments  : 
//******************************************************************************** 

int flip_byte (int pkt_data) {
	int pkt_data_r ;

	pkt_data_r =   (int) pkt_data ;

	pkt_data_r =   ( ( pkt_data_r & 0x0f ) << 4 ) |
	       	       ( ( pkt_data_r & 0xf0 ) >> 4 ) ;
	pkt_data_r =   ( ( pkt_data_r & 0x03 ) << 2 ) | 
		       ( ( pkt_data_r & 0x0c ) >> 2 ) |
	       	       ( ( pkt_data_r & 0x30 ) << 2 ) |
	       	       ( ( pkt_data_r & 0xc0 ) >> 2 ) ;
	pkt_data_r =   ( ( pkt_data_r & 0x02 ) >> 1 ) |
	      	       ( ( pkt_data_r & 0x01 ) << 1 ) |
	       	       ( ( pkt_data_r & 0x08 ) >> 1 ) |
	       	       ( ( pkt_data_r & 0x04 ) << 1 ) |
	       	       ( ( pkt_data_r & 0x20 ) >> 1 ) |
	       	       ( ( pkt_data_r & 0x10 ) << 1 ) |
	       	       ( ( pkt_data_r & 0x80 ) >> 1 ) |
	       	       ( ( pkt_data_r & 0x40 ) << 1 ) ;

	return ( pkt_data_r & 0xff );
}

//******************************************************************************** 
//* Function   : 
//* Desription : Flips around a word. Used in the crc calculation
//* Return     : 
//* Arguments  : 
//******************************************************************************** 

int flip_32 (int pkt_data) {
	unsigned x;

	x = pkt_data ;

	/* Reverse the bits of x (32 total). */
	x = ((x & 0xAAAAAAAA) >>  1) | ((x & 0x55555555) <<  1);
	x = ((x & 0xCCCCCCCC) >>  2) | ((x & 0x33333333) <<  2);
	x = ((x & 0xF0F0F0F0) >>  4) | ((x & 0x0F0F0F0F) <<  4);
	x = ((x & 0xFF00FF00) >>  8) | ((x & 0x00FF00FF) <<  8);
	x = (x >> 16) | (x << 16);

	return x ;
}


//******************************************************************************** 
//* Function   : 
//* Desription : 
//* Return     : Returns an ethernet packet with sequence number and fcs
//* Arguments  : 
//******************************************************************************** 

p_packet get_l2_eth_pkt ( long long da, long long sa, int type, int pkt_len, int seq_num ) {
	p_packet pktPtr ;   
	pktPtr = pkt_new() ; 

	pkt_payload        (pktPtr,pkt_len-18); 
	pkt_ethernet_encap (pktPtr, da, sa, type); 
	pkt_insert_seq     (pktPtr, seq_num) ;
	pkt_crc_encap      (pktPtr); 

	return ( pktPtr );
}

//******************************************************************************** 
//* Function   : 
//* Desription : 
//* Return     : Return a the length of the packet
//* Arguments  : 
//******************************************************************************** 

int pkt_get_length ( p_packet _pkt ) {
	return ( _pkt->len );
}

//******************************************************************************** 
//* Function   : 
//* Desription : 
//* Return     : set the length for the packet
//* Arguments  : 
//******************************************************************************** 

void pkt_set_length ( p_packet _pkt, int _length ) {
	_pkt->len = _length ;
}

//******************************************************************************** 
//* Function   : 
//* Desription : 
//* Return     : returns the data at the corresponding index 
//* Arguments  : 
//******************************************************************************** 

uint8_t pkt_get_data ( p_packet _pkt, int _index ) {
	return ( _pkt->data[_index] );
}

void pkt_ipv4_encap ( p_packet pkt, uint32_t src, uint32_t dest, uint32_t protocol,
	uint32_t ttl, uint32_t checksum ) {

	uint32_t i;
	uint32_t sum         = 0;
	uint32_t version     = 0x45;  // header format assumes no options present
	uint32_t tos         = 0x0;
	uint32_t length      = pkt->len + 20;
	uint32_t id          = 0x0;
	uint32_t frag        = 0x0; 
	uint32_t ip_checksum = 0x0;
	uint16_t word16      ;
     
	// assemble IP header
	space(pkt,20);
	pkt->data[0] = version;
	pkt->data[1] = tos;

	pkt->data[2] = ( length >> 8 ) & 0xff ;
 	pkt->data[3] = ( length >> 0 ) & 0xff ;

	pkt->data[4] = ( id >> 8 ) & 0xff ;
	pkt->data[5] = ( id >> 0 ) & 0xff ;

	pkt->data[6] = ( frag >> 8 ) & 0xff ;
	pkt->data[7] = ( frag >> 0 ) & 0xff ;

	pkt->data[8] = ttl;
	pkt->data[9] = protocol;

	pkt->data[10] = 0x0;
	pkt->data[11] = 0x0;

	pkt->data[12] = ( src >> 24 ) & 0xff ;
	pkt->data[13] = ( src >> 16 ) & 0xff ;
	pkt->data[14] = ( src >>  8 ) & 0xff ;
	pkt->data[15] = ( src >>  0 ) & 0xff ;

	pkt->data[16] = ( dest >> 24 ) & 0xff ;
	pkt->data[17] = ( dest >> 16 ) & 0xff ;
	pkt->data[18] = ( dest >>  8 ) & 0xff ;
	pkt->data[19] = ( dest >>  0 ) & 0xff ;

	// lifted from http://www.netfor2.com/ipsum.htm
	// make 16 bit words out of every two adjacent 8 bit words in the packet
	// and add them up
	for (i=0;i<20;i=i+2){
		word16 =((pkt->data[i]<<8)&0xFF00)+(pkt->data[i+1]&0xFF);
		sum = sum + (uint32_t) word16;	
		// printf("ip_sum\t=\t%x, index = %d, data[i] = %x, data[i+1] = %x \n",sum,i,pkt->data[i],pkt->data[i+1]); 
	}

	// take only 16 bits out of the 32 bit sum and add up the carries
	while (sum>>16)
		sum = (sum & 0xFFFF)+(sum >> 16);

	// one's complement the result
	ip_checksum = ~sum;

	// ip_checksum = ( sum ^ 0xffffffff ) & 0xffff ;
	
	// printf("from ipv4 checksum ip_checksum = %x, sum = %x, good_checksum = %x \n",ip_checksum,sum,checksum);
	      
	// Place ip checksum in IP header
	if (checksum == 1) {
		pkt->data[10] = ( ip_checksum >> 8 ) & 0xff ;
	    pkt->data[11] = ( ip_checksum >> 0 ) & 0xff ;
	} else {
		if ( ( sum == 0xFFFF ) || ( sum == 0x0000 ) ) {
			sum = 0xAAAA ;
		}
	    pkt->data[10] = ( sum >> 8 ) & 0xff ;
	    pkt->data[11] = ( sum >> 0 ) & 0xff ;
	}
}

void pkt_ipv4_options_encap ( p_packet pkt, uint32_t src, uint32_t dest, uint32_t protocol,
	uint32_t ttl, uint32_t checksum, uint32_t hdrLen ) {

	uint32_t i;
	uint32_t sum         = 0;
	uint32_t version     = 0x45;  // header format assumes no options present
	uint32_t tos         = 0x0;
	uint32_t length      = pkt->len + 20;
	uint32_t id          = 0x0;
	uint32_t frag        = 0x0; 
	uint32_t ip_checksum = 0x0;
	uint16_t word16      ;

	// assemble IP header
	space(pkt,hdrLen*4);
	pkt->data[0] = version;
	pkt->data[1] = tos;
	  
	pkt->data[2] = ( length >> 8 ) & 0xff ;
	pkt->data[3] = ( length >> 0 ) & 0xff ;
	  
	pkt->data[4] = ( id >> 8 ) & 0xff ;
	pkt->data[5] = ( id >> 0 ) & 0xff ;

	pkt->data[6] = ( frag >> 8 ) & 0xff ;
	pkt->data[7] = ( frag >> 0 ) & 0xff ;
	  
	pkt->data[8] = ttl;
	pkt->data[9] = protocol;
	  
	pkt->data[10] = 0x0;
	pkt->data[11] = 0x0;
	  
	pkt->data[12] = ( src >> 24 ) & 0xff ;
	pkt->data[13] = ( src >> 16 ) & 0xff ;
	pkt->data[14] = ( src >>  8 ) & 0xff ;
	pkt->data[15] = ( src >>  0 ) & 0xff ;
	  
	pkt->data[16] = ( dest >> 24 ) & 0xff ;
	pkt->data[17] = ( dest >> 16 ) & 0xff ;
	pkt->data[18] = ( dest >>  8 ) & 0xff ;
	pkt->data[19] = ( dest >>  0 ) & 0xff ;

	// add options field
	for ( i = 0 ; i < ((hdrLen*4)-20) ; i ++ ) {
		pkt->data[i+20] = hdrLen+i ;
	}

	// lifted from http://www.netfor2.com/ipsum.htm
	// make 16 bit words out of every two adjacent 8 bit words in the packet
	// and add them up
	for (i=0;i<(hdrLen*4);i=i+2){
		word16 =((pkt->data[i]<<8)&0xFF00)+(pkt->data[i+1]&0xFF);
		sum = sum + (uint32_t) word16;	
		// printf("ip_sum\t=\t%x, index = %d, data[i] = %x, data[i+1] = %x \n",sum,i,pkt->data[i],pkt->data[i+1]); 
	}
	
	// take only 16 bits out of the 32 bit sum and add up the carries
	while (sum >> 16) {
		sum = (sum & 0xFFFF)+(sum >> 16);
	}

	// one's complement the result
	ip_checksum = ~sum;

	// ip_checksum = ( sum ^ 0xffffffff ) & 0xffff ;
	
	// printf("from ipv4 checksum ip_checksum = %x, sum = %x, good_checksum = %x \n",ip_checksum,sum,checksum);
	      
	// Place ip checksum in IP header
	if (checksum == 1) {
		pkt->data[10] = ( ip_checksum >> 8 ) & 0xff ;
	    pkt->data[11] = ( ip_checksum >> 0 ) & 0xff ;
	} else {
		pkt->data[10] = ( sum >> 8 ) & 0xff ;
	    pkt->data[11] = ( sum >> 0 ) & 0xff ;
	}
}

// encap ipv6 header onto packet
void pkt_ipv6_encap(p_packet pkt, uint32_t src, uint32_t dest, uint32_t flow_lable, int protocol, int ttl) {
	char version = 0x66;
	int length = pkt->len;

	space(pkt,40);
	
	pkt->data[0] = version;
	pkt->data[1] = (flow_lable >> 16) & 0xff;
	pkt->data[2] = (flow_lable >> 8)  & 0xff;
	pkt->data[3] = (flow_lable)       & 0xff; 

	pkt->data[4] = (length >> 8) & 0xff;
	pkt->data[5] = length        & 0xff;
	
	pkt->data[6] = protocol & 0xff;
	pkt->data[7] = ttl      & 0xff;

	// fill in lower bytes of ipsrc for now only. 
	pkt->data[8]  =  (src >> 24) & 0xff;
	pkt->data[9]  =  (src >> 16) & 0xff;
	pkt->data[10] =  (src >> 8) & 0xff;
	pkt->data[11]  = (src >> 0) & 0xff;

	pkt->data[24] =  (dest >> 24) & 0xff;
	pkt->data[25] =  (dest >> 16) & 0xff;
	pkt->data[26] =  (dest >> 8)  & 0xff;
	pkt->data[27] =  (dest >> 0)  & 0xff;

}


void pkt_tcp_encap ( p_packet pkt, uint16_t sport, uint16_t dport, uint32_t seq, uint32_t ack, uint8_t hlen,  
		     uint8_t flags, uint16_t window_size, uint16_t urg_pointer) {

	space(pkt,20);

	pkt->data[0] = ( sport >> 8 ) & 0xff ;
	pkt->data[1] = ( sport >> 0 ) & 0xff ;

	pkt->data[2] = ( dport >> 8 ) & 0xff ;
	pkt->data[3] = ( dport >> 0 ) & 0xff ;

	pkt->data[4] = ( seq >> 24 ) & 0xff ;
	pkt->data[5] = ( seq >> 16 ) & 0xff ;
	pkt->data[6] = ( seq >>  8 ) & 0xff ;
	pkt->data[7] = ( seq >>  0 ) & 0xff ;

	pkt->data[8]  = ( ack >> 24 ) & 0xff ;
	pkt->data[9]  = ( ack >> 16 ) & 0xff ;
	pkt->data[10] = ( ack >>  8 ) & 0xff ;
	pkt->data[11] = ( ack >>  0 ) & 0xff ;

	pkt->data[12] = (hlen & 0xf) << 4 ;
	pkt->data[13] = ( flags << 2 ) ;

	pkt->data[14] = ( window_size >> 8 ) & 0xff ;
	pkt->data[15] = ( window_size >> 0 ) & 0xff ;

	pkt->data[16] = 0x0;  // checksum holding area
	pkt->data[17] = 0x0;  // checksum holding area

	pkt->data[18] = ( urg_pointer >> 8 ) & 0xff ;
	pkt->data[19] = ( urg_pointer >> 0 ) & 0xff ;

}


void pkt_udp_encap(p_packet pkt, uint16_t src, uint16_t dest) {
	 uint16_t  udp_length = pkt->len + 8;
	 space (pkt, 8);
	 
	 pkt->data[0] = (src >> 8)  & 0xff;
	 pkt->data[1] = (src)       & 0xff;   

	 pkt->data[2] = (dest >> 8) & 0xff;   
	 pkt->data[3] = (dest)      & 0xff;   

	 pkt->data[4] = (udp_length >> 8) & 0xff;   
	 pkt->data[5] = (udp_length     ) & 0xff;   
	 pkt->data[6] = 0;   
	 pkt->data[7] = 0;   
}


void pkt_udp_checksum (p_packet pkt, uint32_t src, uint32_t dest, uint32_t good_checksum){
	p_packet pseudo_packet;
	uint32_t sum, checksum;
	uint16_t udp_length;
	uint16_t total_length;
	uint16_t protocol = 0x11;
	uint32_t i;
	uint16_t word16 ;

	pseudo_packet = pkt_new();
	for ( i = 0 ; i < MAX_PKT_SIZE ; i ++ ) pseudo_packet->data[i] = 0 ;

	udp_length = pkt->len;
	total_length = udp_length;

	// create psudo packet
	for (i=0;i<pkt->len;i++)
		pseudo_packet->data[i+12] = pkt->data[i];

	pseudo_packet->data[0] = ( src >> 24 ) & 0xff ;
	pseudo_packet->data[1] = ( src >> 16 ) & 0xff ;
	pseudo_packet->data[2] = ( src >>  8 ) & 0xff ;
	pseudo_packet->data[3] = ( src >>  0 ) & 0xff ;

	pseudo_packet->data[4] = ( dest >> 24 ) & 0xff ;
	pseudo_packet->data[5] = ( dest >> 16 ) & 0xff ;
	pseudo_packet->data[6] = ( dest >>  8 ) & 0xff ;
	pseudo_packet->data[7] = ( dest >>  0 ) & 0xff ;

	pseudo_packet->data[8]  = ( protocol >> 8 ) & 0xff ;
	pseudo_packet->data[9]  = ( protocol >> 0 ) & 0xff ;
	pseudo_packet->data[10] = ( total_length >> 8 ) & 0xff ;
	pseudo_packet->data[11] = ( total_length >> 0 ) & 0xff ;

	// compute udp checksum
	sum = 0 ;
	for ( i=0;i<udp_length+12; i=i+2) {
		word16 =((pseudo_packet->data[i]<<8)&0xFF00)+(pseudo_packet->data[i+1]&0xFF);
		sum = sum + (uint32_t) word16;	
		// printf("tcp_sum\t=\t%x, index = %d, data[i] = %x, data[i+1] = %x \n",sum,i,pseudo_packet->data[i],pseudo_packet->data[i+1]); 
	}

	while (sum>>16)
		sum = (sum & 0xFFFF)+(sum >> 16);

	// one's complement the result
	checksum = ~sum;

	// printf("from tcp checksum : checksum = %x, sum = %x, tcp_length = %d, good_checksum = %x \n",checksum,sum,tcp_length,good_checksum);

	// insert checksum
	if (good_checksum) {
		pkt->data[6] =  ( checksum >> 8 ) & 0xff ;
		pkt->data[7] =  ( checksum >> 0 ) & 0xff ;
	}
	else {
		if ( ( sum == 0xFFFF ) || ( sum == 0x0000 ) ) {
			sum = 0xAAAA ;
		}
		pkt->data[6] =  ( sum  >> 8 ) & 0xff ;
		pkt->data[7] =  ( sum  >> 0 ) & 0xff ;
	}
	pkt_delete ( pseudo_packet );
}


void pkt_tcp_checksum (p_packet pkt, uint32_t src, uint32_t dest, uint32_t good_checksum){
	p_packet pseudo_packet;
	uint32_t sum, checksum;
	uint16_t tcp_length;
	uint16_t total_length;
	uint16_t protocol = 0x6;
	uint32_t i;
	uint16_t word16 ;

	pseudo_packet = pkt_new();
	for ( i = 0 ; i < MAX_PKT_SIZE ; i ++ ) pseudo_packet->data[i] = 0 ;

	tcp_length = pkt->len;
	total_length = pkt->len;

	// create psudo packet
	for (i=0;i<pkt->len;i++)
		pseudo_packet->data[i+12] = pkt->data[i];

	pseudo_packet->data[0] = ( src >> 24 ) & 0xff ;
	pseudo_packet->data[1] = ( src >> 16 ) & 0xff ;
	pseudo_packet->data[2] = ( src >>  8 ) & 0xff ;
	pseudo_packet->data[3] = ( src >>  0 ) & 0xff ;

	pseudo_packet->data[4] = ( dest >> 24 ) & 0xff ;
	pseudo_packet->data[5] = ( dest >> 16 ) & 0xff ;
	pseudo_packet->data[6] = ( dest >>  8 ) & 0xff ;
	pseudo_packet->data[7] = ( dest >>  0 ) & 0xff ;

	pseudo_packet->data[8]  = ( protocol >> 8 ) & 0xff ;
	pseudo_packet->data[9]  = ( protocol >> 0 ) & 0xff ;
	pseudo_packet->data[10] = ( total_length >> 8 ) & 0xff ;
	pseudo_packet->data[11] = ( total_length >> 0 ) & 0xff ;

	sum = 0 ;
	for ( i=0;i<tcp_length+12; i=i+2) {
		word16 =((pseudo_packet->data[i]<<8)&0xFF00)+(pseudo_packet->data[i+1]&0xFF);
		sum = sum + (uint32_t) word16;	
		// printf("tcp_sum\t=\t%x, index = %d, data[i] = %x, data[i+1] = %x \n",sum,i,pseudo_packet->data[i],pseudo_packet->data[i+1]); 
	}

	while (sum>>16)
		sum = (sum & 0xFFFF)+(sum >> 16);

	// one's complement the result
	checksum = ~sum;

	// printf("from tcp checksum : checksum = %x, sum = %x, tcp_length = %d, good_checksum = %x \n",checksum,sum,tcp_length,good_checksum);

	// insert checksum
	if (good_checksum) {
		pkt->data[16] =  ( checksum >> 8 ) & 0xff ;
		pkt->data[17] =  ( checksum >> 0 ) & 0xff ;
	}
	else {
		if ( ( sum == 0xFFFF ) || ( sum == 0x0000 ) ) {
			sum = 0xAAAA ;
		}
		pkt->data[16] =  ( sum  >> 8 ) & 0xff ;
		pkt->data[17] =  ( sum  >> 0 ) & 0xff ;
	}
	pkt_delete ( pseudo_packet );
}


void pkt_custom_l4_checksum (p_packet pkt, uint32_t src, uint32_t dest, uint32_t l4_protocol, uint32_t good_checksum){
	p_packet pseudo_packet;
	uint32_t sum, checksum;
	uint16_t tcp_length;
	uint16_t total_length;
	uint16_t protocol = l4_protocol;
	uint32_t i;
	uint16_t word16;

	pseudo_packet = pkt_new();
	for ( i = 0 ; i < MAX_PKT_SIZE ; i ++ ) pseudo_packet->data[i] = 0 ;

	tcp_length = pkt->len ;
	total_length = pkt->len + ( 5 * 4 ) ;

	// create psudo packet
	for (i=0;i<pkt->len;i++)
	  pseudo_packet->data[i+12] = pkt->data[i];

	pseudo_packet->data[0] = ( src >> 24 ) & 0xff ;
	pseudo_packet->data[1] = ( src >> 16 ) & 0xff ;
	pseudo_packet->data[2] = ( src >>  8 ) & 0xff ;
	pseudo_packet->data[3] = ( src >>  0 ) & 0xff ;

	pseudo_packet->data[4] = ( dest >> 24 ) & 0xff ;
	pseudo_packet->data[5] = ( dest >> 16 ) & 0xff ;
	pseudo_packet->data[6] = ( dest >>  8 ) & 0xff ;
	pseudo_packet->data[7] = ( dest >>  0 ) & 0xff ;

	pseudo_packet->data[8]  = ( protocol >> 8 ) & 0xff ;
	pseudo_packet->data[9]  = ( protocol >> 0 ) & 0xff ;
	pseudo_packet->data[10] = ( total_length >> 8 ) & 0xff ;
	pseudo_packet->data[11] = ( total_length >> 0 ) & 0xff ;

	sum = 0 ;
	for ( i=0;i<tcp_length+12; i=i+2) {
		word16 =((pseudo_packet->data[i]<<8)&0xFF00)+(pseudo_packet->data[i+1]&0xFF);
		sum = sum + (uint32_t) word16;	
		// printf("tcp_sum\t=\t%x, index = %d, data[i] = %x, data[i+1] = %x \n",sum,i,pseudo_packet->data[i],pseudo_packet->data[i+1]); 
	}

	while (sum>>16)
	  sum = (sum & 0xFFFF)+(sum >> 16);

	// one's complement the result
	checksum = ~sum;

	// printf("from tcp checksum : checksum = %x, sum = %x, tcp_length = %d, good_checksum = %x \n",checksum,sum,tcp_length,good_checksum);

	// insert checksum
	if (good_checksum) {
		pkt->data[16] =  ( checksum >> 8 ) & 0xff ;
		pkt->data[17] =  ( checksum >> 0 ) & 0xff ;
	}
	else {
		pkt->data[16] =  ( sum >> 8 ) & 0xff ;
		pkt->data[17] =  ( sum >> 0 ) & 0xff ;
	}
	pkt_delete ( pseudo_packet );
}


void pkt_udp_checksum_ipv6 (p_packet pkt, uint32_t src, uint32_t dest, uint32_t good_checksum){
	p_packet pseudo_packet;
	uint32_t sum, checksum;
	uint16_t udp_length;
	uint16_t total_length;
	uint16_t protocol = 0x11;
	uint32_t i;
	uint16_t word16 ;

	pseudo_packet = pkt_new();
	for ( i = 0 ; i < MAX_PKT_SIZE ; i ++ ) pseudo_packet->data[i] = 0 ;

	udp_length = pkt->len ;
	total_length = udp_length;

	// create psudo packet
	for (i=0;i<pkt->len;i++)
	  pseudo_packet->data[i+40] = pkt->data[i];

	pseudo_packet->data[0] = ( src >> 24 ) & 0xff ;
	pseudo_packet->data[1] = ( src >> 16 ) & 0xff ;
	pseudo_packet->data[2] = ( src >>  8 ) & 0xff ;
	pseudo_packet->data[3] = ( src >>  0 ) & 0xff ;

	pseudo_packet->data[4] = ( dest >> 24 ) & 0xff ;
	pseudo_packet->data[5] = ( dest >> 16 ) & 0xff ;
	pseudo_packet->data[6] = ( dest >>  8 ) & 0xff ;
	pseudo_packet->data[7] = ( dest >>  0 ) & 0xff ;

	pseudo_packet->data[8]  = ( protocol >> 8 ) & 0xff ;
	pseudo_packet->data[9]  = ( protocol >> 0 ) & 0xff ;
	pseudo_packet->data[10] = ( total_length >> 8 ) & 0xff ;
	pseudo_packet->data[11] = ( total_length >> 0 ) & 0xff ;

	// compute udp checksum
	sum = 0 ;
	for ( i=0;i<udp_length+40; i=i+2) {
		word16 =((pseudo_packet->data[i]<<8)&0xFF00)+(pseudo_packet->data[i+1]&0xFF);
		sum = sum + (uint32_t) word16;	
	}
	
	while (sum >> 16) {
		sum = (sum & 0xFFFF)+(sum >> 16);
	}
	
	// one's complement the result
	checksum = ~sum;
	
	// insert checksum
	if (good_checksum) {
		pkt->data[6] =  ( checksum >> 8 ) & 0xff ;
		pkt->data[7] =  ( checksum >> 0 ) & 0xff ;
	}
	else {
		if ( ( sum == 0xFFFF ) || ( sum == 0x0000 ) ) {
	    	sum = 0xAAAA ;
	    }
	    pkt->data[6] =  ( sum  >> 8 ) & 0xff ;
	    pkt->data[7] =  ( sum  >> 0 ) & 0xff ;
	}

	pkt_delete ( pseudo_packet );
}


void pkt_vlan_encap (p_packet pkt, uint16_t vlan_tag){
	uint16_t vlan_const = 0x8100 ;

	space(pkt,4);
	pkt->data[3] = ( vlan_tag >> 0 ) & 0xff ;
	pkt->data[2] = ( vlan_tag >> 8 ) & 0xff ;
	pkt->data[1] = ( vlan_const >> 0 ) & 0xff ;
	pkt->data[0] = ( vlan_const >> 8 ) & 0xff ;
}

void pkt_copy (p_packet new_packet, p_packet old_packet) {
	// just copies the contents of the old buffer to 
	// the new buffer
	
	int i ;

	new_packet->len = old_packet->len ;
	for ( i = 0 ; i < new_packet->len ; i ++ ) {
		new_packet->data[i] = old_packet->data[i] ;
	}
}

//********************************************************************************
//* Function   :
//* Desription : Adds a single byte onto a packet data struct
//* Return     :
//* Arguments  :
//********************************************************************************

void  pkt_encap_byte ( p_packet pkt, uint8_t add_byte ) {
	space(pkt,1);
	pkt->data[0] = add_byte ;
} 

void  pkt_encap_2byte ( p_packet pkt, uint16_t add_2bytes ) {
	space(pkt,2);
	pkt->data[0] = ( add_2bytes & 0xff00 ) >> 8  ;
	pkt->data[1] = ( add_2bytes & 0x00ff ) >> 0  ;
} 

void  pkt_encap_snap ( p_packet pkt, uint16_t type ) {
	space(pkt,5);
	pkt->data[0] = 0x00 ;
	pkt->data[1] = 0x00 ;
	pkt->data[2] = 0x00 ;
	pkt->data[3] = ( type & 0xff00 ) >> 8 ;
	pkt->data[4] = ( type & 0x00ff ) >> 0 ;
}

void  pkt_encap_llc ( p_packet pkt, uint8_t dsap, uint8_t ssap, uint8_t ctrl ) {
	space(pkt,3);
	pkt->data[2] = ctrl ;
	pkt->data[1] = ssap ;
	pkt->data[0] = dsap ;
}

void pkt_encap_ppp_uncomp (p_packet pkt, uint16_t hdlc, uint16_t protocol) {
	space(pkt,4);
	pkt->data[0] = ( hdlc & 0xff00 ) >> 8 ;
	pkt->data[1] = ( hdlc & 0x00ff ) >> 0 ;
	pkt->data[2] = ( protocol & 0xff00 ) >> 8 ;
	pkt->data[3] = ( protocol & 0x00ff ) >> 0 ;
}

void pkt_encap_ppp_comp (p_packet pkt, uint16_t protocol) {
	space(pkt,2);
	pkt->data[0] = ( protocol & 0xff00 ) >> 8 ;
	pkt->data[1] = ( protocol & 0x00ff ) >> 0 ;
}
