/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <unistd.h>
#include <string.h>
#include <inttypes.h>
#include <setjmp.h>

#include "system.h"
#include "cache.h"
#include "psb.h"
#include "iomap.h"
#include "mipsregs.h"
#include "mips-exts.h"
#include "spinlock.h"
#include "asmmacros.h"

#include "contig_alloc.h"
#include "mm.h"
#include "uart.h"
#include "cpu.h"
#include "env_mgr.h"

#include <ctype.h>

#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <sys/times.h>
#include <sys/time.h>
#include <signal.h>

#include "irq.h"
#include "nlm_hal.h"
#include "xlr_boot_lib.h"

#ifdef XLP_SIM
#include "nlm_hal_pic.h"
#endif

#include <timer_api.h>
#include <barrier.h>
#include "tlb.h"

extern int process_fdt(void *fdt);
extern int init_nae(void *fdt);
extern int init_dtr ();
static void *fdtp;
extern int dom_id;


/* Structure filled in by get_mem_info.  Only the size field is
   actually used (by sbrk), so the others aren't even filled in.
*/
struct s_mem
{
	unsigned int size;
	unsigned int icsize;
	unsigned int dcsize;
};

extern unsigned char _loadaddr[], _endaddr[], _end[];

struct ext_tlb *ext_tlb = NULL;
struct tlb_struct *pte;

uint32_t syscall_clone_mask = 0x0;
unsigned long long phoenix_io_base __attribute__((weak));

#ifdef XLP_SIM
uint32_t xlr_io_base = (uint32_t)(DEFAULT_XLP_IO_BASE);
#else
uint32_t xlr_io_base = (uint32_t)(DEFAULT_PHOENIX_IO_BASE);
#endif

static struct vcpu_extended_info local_vcpu_ext_info;
struct per_cpu_info local_per_cpu_info;

#ifdef NLM_BRIDGE_WKAROUND
#include "nlm_rw_lock.h"
#include "global_shmem.h"
nlm_rwlock_t *nlm_bridge_lock;
int nlm_enable_br_wrkaround = 0;
#endif

static void get_extended_cpu_info(struct vcpu_extended_info *ptr);
unsigned int xlr_netos_uart = TTYS0;
unsigned int nlm_cpu_frequency;
uint64_t heap_size = MIN_HEAP_SIZE;

void get_mem_info (mem)
struct s_mem *mem;
{
	mem->size = _endaddr - _loadaddr;
}

unsigned int get_heap_size(void)
{
	return heap_size;
}

struct xlr_vcpu_wakeup_info xlr_vcpu_wakeup_info_local_copy[32];
volatile struct xlr_vcpu_wakeup_info *launcher_vcpu_info_ptr;


/* From elfwb/hal.c
 */
int num_cpus_online = 0;
barrier *exit_barrier;
extern void syscall_init(int);

volatile struct xlr_vcpu_wakeup_info *xlr_vcpu_wakeup_info;

int master_cpu = 0;
uint32_t buddy_cpu_mask = 0x00;
uint32_t master_cpu_mask = 0x0;
uint32_t cpu_online_map;
int      run_mode;

spinlock_t update_cpu_mask = SPIN_LOCK_UNLOCKED;

int     g_argc;
char   *g_argv[MAX_ARGS + 1], **argv;
char   *g_envp[MAX_ENV + 1], **envp;

int ntlb;
struct segment_info netos_app_tlbs[MAX_TLB_MAPPINGS];

void loader_info(int argc, char *argv[])
{
	printf("master_cpu = %d, master_cpu_mask = 0x%08x, buddy_cpu_mask = 0x%08x\n",
		   master_cpu, master_cpu_mask, buddy_cpu_mask);

	printf("cpu_online mask = 0x%08x\n", cpu_online_map);
}

uint64_t *nlm_kernel_args = NULL;
unsigned int nlmk_en = 0;
struct nlm_domain;
struct nlm_domain  *nlm_this_domain;
struct nlm_pcpu_info;
int nlm_pcpu_info_size;
struct nlm_pcpu_info *nlm_pcpu_info;
void (*nlm_nlmk_entry)(int);
unsigned long nlmk_ebase = 0x0;
extern void nlmk_trap_init(void);
struct nlm_resource *nlm_this_domres;

/* netos debugger stuff */
int gdb_connection_mode = -1; /* 0 for serial, 1 for network */
int gdb_uart = -1; /* 0 for uart 0, 1 for uart 1 */
int gdb_initial_break_val = 1; /* 0 for no initial break */
extern void gdbstub_init(void); /* for multi session */
extern void gdbstub_exit(void); /* for multi session */

static void hardware_init_boot2(void)
{
	int i;
	struct cpu_tlb_mapping *map;
	struct xlr_vcpu_wakeup_info *dst;
	struct xlr_vcpu_wakeup_info *src, *t;

	src = (struct xlr_vcpu_wakeup_info *)(long)read_64bit_cp0_register_sel( CP0_OSSCRATCH, WAKEUP_SCRATCH_IDX);
	launcher_vcpu_info_ptr = src;
	dst = xlr_vcpu_wakeup_info_local_copy;
	for (i = 0; i < 32; i++) {
                *dst = *src;
                src++;
                dst++;
	}
	t = xlr_vcpu_wakeup_info_local_copy;

	/*Always Points to Shared Data Structure Of VCPU0.*/
	xlr_vcpu_wakeup_info = (volatile struct xlr_vcpu_wakeup_info *)t;
	t = t + processor_id();

	set_print_lock((spinlock_t *)t->printk_lock);

	master_cpu = t->master_cpu;
	buddy_cpu_mask = t->buddy_mask;
	master_cpu_mask = t->master_mask;
	run_mode = t->mode;

	syscall_clone_mask = buddy_cpu_mask & ~master_cpu_mask;

	g_argc = t->argc;
	argv = (char **)t->argv;
	envp = (char **)t->envs;

	for (i = 0; (*argv) && (i < 32); ++i, ++argv) {
		g_argv[i] = strdup(*argv);
	}
	g_argv[i] = NULL;

	for (i = 0; (*envp) && (i < 32); ++i, ++envp) {
		g_envp[i] = strdup(*envp);
		if(!strcmp(g_envp[i],"ttyS0")){
			xlr_netos_uart = TTYS0;
		}
		else if(!strcmp(g_envp[i],"ttyS1")){
			xlr_netos_uart = TTYS1;
		}
		else if(!strcmp(g_envp[i],"virt_uart")) {
			xlr_netos_uart = VIRT_UART;
		}
	}

	g_envp[i] = NULL;

	cpu_online_map = (1<<master_cpu) | buddy_cpu_mask;

	map = (struct cpu_tlb_mapping *)t->map;

	ntlb = 1;
	netos_app_tlbs[0].vaddr = map[0].virt;
	netos_app_tlbs[0].paddr = map[0].phys;
	netos_app_tlbs[0].size  = 2 * map[0].page_size;
	for (i = 1; i < t->map_count; ++i) {
	        unsigned long vaddr;
		uint64_t paddr;

		vaddr = netos_app_tlbs[ntlb-1].vaddr + netos_app_tlbs[ntlb-1].size;
		paddr = netos_app_tlbs[ntlb-1].paddr + netos_app_tlbs[ntlb-1].size;

		if ((vaddr == (map[i]).virt) && (paddr == (map[i]).phys)) {
		        netos_app_tlbs[ntlb-1].size += 2 * (map[i].page_size);
			continue;
		}
		netos_app_tlbs[ntlb].vaddr = map[i].virt;
		netos_app_tlbs[ntlb].paddr = map[i].phys;
		netos_app_tlbs[ntlb].size  = 2 * map[i].page_size;
		++ntlb;
	}

	get_extended_cpu_info((void *)t->vcpu_extended_info_ptr);

}

#define validate_per_cpu_field(field) { \
	if ((long)(&(((struct per_cpu_info_loader *)0)->field)) > per_cpu_size)\
		return;}

static void get_per_cpu_info(struct per_cpu_info_loader *ptr, int per_cpu_size)
{
#ifdef DEBUG
	int i;
#endif /* DEBUG */
	unsigned long temp = (unsigned long)ptr;

	temp += processor_id() * per_cpu_size;
	ptr = (struct per_cpu_info_loader *) temp;
	validate_per_cpu_field(num_tlbs);
	validate_per_cpu_field(ext_tlb_ptr);
	local_per_cpu_info.num_tlbs = ptr->num_tlbs;
	local_per_cpu_info.ext_tlb_ptr = (struct ext_tlb *)(long)
					ptr->ext_tlb_ptr;

	ext_tlb = (struct ext_tlb *)(long)ptr->ext_tlb_ptr;
#ifdef DEBUG
	for (i = 0; i < ptr->num_tlbs; i++) {
		printf("tlb #%d: virt = %llx, phys = %llx, size = %llx\n",i,
		       (long long)ext_tlb->virt,
		       (long long)ext_tlb->phys,
		       (long long)ext_tlb->page_size);
		ext_tlb++;
	}
	ext_tlb -= ptr->num_tlbs;
#endif /* DEBUG */
}

#ifndef NETOS_LOADER_TODO
#define validate_ext_field(field) { \
	if ((long)(&(((struct vcpu_extended_info *)0)->field)) > extended_size)\
		return;}
#else
#define validate_ext_field(x)
#endif

static void get_extended_cpu_info(struct vcpu_extended_info *ptr)
{
	int extended_size = 0,i;
	struct vcpu_extended_info *ext = &local_vcpu_ext_info;

	memset(&local_vcpu_ext_info, 0, sizeof(struct vcpu_extended_info));
	memset(&local_per_cpu_info, 0, sizeof(struct per_cpu_info));
	if (!ptr)
		return;
	if (ptr->magic != EXTENDED_BOOT_MAGIC)
		return;
	ext->size = ptr->size;
	extended_size = ext->size;
	validate_ext_field(per_cpu_info_size);
	ext->per_cpu_info_size = ptr->per_cpu_info_size;
	validate_ext_field(per_cpu_info_ptr);
	ext->per_cpu_info_ptr = (struct per_cpu_info *)(long)ptr->per_cpu_info_ptr;
	if (ext->per_cpu_info_size > 0)
		get_per_cpu_info((struct per_cpu_info_loader *)
				  ext->per_cpu_info_ptr,
				  ext->per_cpu_info_size);
	validate_ext_field(uart0_lock);
	ext->uart0_lock = (spinlock_t *)(long)ptr->uart0_lock;
	validate_ext_field(uart1_lock);
	ext->uart1_lock = (spinlock_t *)(long)ptr->uart1_lock;
	validate_ext_field(global_lock);
	ext->global_lock = (spinlock_t *)(long)ptr->global_lock;
	validate_ext_field(bridge_io_lock);
	ext->bridge_io_lock = (spinlock_t *)(long)ptr->bridge_io_lock;
	for ( i = 0; i < 8 ; i++) {
		validate_ext_field(gmac_lock[i]);
		ext->gmac_lock[i] = (spinlock_t *)(long)ptr->gmac_lock[i];
	}
	validate_ext_field(xgmac_lock[0]);
	ext->xgmac_lock[0] = (spinlock_t *)(long)ptr->xgmac_lock[0];
	validate_ext_field(xgmac_lock[1]);
	ext->xgmac_lock[1] = (spinlock_t *)(long)ptr->xgmac_lock[1];
	validate_ext_field(spi4_lock[0]);
	ext->spi4_lock[0]= (spinlock_t *)(long)ptr->spi4_lock[0];
	validate_ext_field(spi4_lock[1]);
	ext->spi4_lock[1]= (spinlock_t *)(long)ptr->spi4_lock[1];
	validate_ext_field(loader_shmem);
	ext->loader_shmem = ptr->loader_shmem;
	ext->syscall = ptr->syscall;
	ext->fdtptr = ptr->fdtptr;
	ext->dom_id = ptr->dom_id;

	/* Retrieve FDT Pointer Stored
	 * during loader stage here.
	 */
	fdtp = ext->fdtptr;
	dom_id = ext->dom_id;
}


void software_init_hook(void)
{
}
void hardware_hazard_hook(void)
{
}

void process_gdbstub_args (void)
{
        int i;

        /* Check if netos debug mode is passed from command line */
        for (i = 0; i < g_argc; i++) {

                if (    !strncmp (g_argv[i], "netos_debug_serial", strlen ("netos_debug_serial")) ||
                        !strncmp (g_argv[i], "rds", strlen ("rds"))
                ) {
                        gdb_connection_mode = 0;
                        break;
                }
                else if (       !strcmp (g_argv[i], "netos_debug_network") ||
                                !strcmp (g_argv[i], "rdn")
                ) {
                        gdb_connection_mode = 1;
                        break;
                }
        }

        if (gdb_connection_mode >= 0) {
                char * netos_debug_temp = NULL;

                /* Check for serial mode and uart port */
                if (gdb_connection_mode == 0) {
                        if (!strncmp (g_argv[i], "netos_debug_serial", strlen ("netos_debug_serial")))
                                netos_debug_temp = g_argv[i] + strlen ("netos_debug_serial");
                        else if (!strncmp (g_argv[i], "rds", strlen ("rds")))
                                netos_debug_temp = g_argv[i] + strlen ("rds");
                        if (*netos_debug_temp && (!strcmp(netos_debug_temp, "_uart0")))
                                gdb_uart = 0;
                        else
                                gdb_uart = 1;
                }

                /* Check for initial / no initial break */
                for (i = 0; i < g_argc; i++)

                        if(     !strcmp (g_argv[i], "netos_debug_no_initial_break") ||
                                !strcmp (g_argv[i], "rdnib")
                        ) {
                                gdb_initial_break_val = 0;
                                break;
                        }
        }
}

void per_cpu_boot_init(void)
{
	unsigned int cp0_status = 0;

#ifdef _ABI64
	cp0_status |= (0x000000E0);
#endif

#ifdef XLP_SIM
	cp0_status |= 0x74000000;
#else
	cp0_status |= 0x50000000;
#endif

	write_32bit_cp0_register(CP0_STATUS, cp0_status);
#ifdef XLP_SIM
	write_32bit_cp1_ctrl_register (CP1_FCSR, 0x400000);
#endif


	write_32bit_cp0_register(CP0_PAGEMASK, 0);

	write_32bit_cp0_register(CP0_CAUSE, 0);

	/* Count Compare */
	write_32bit_cp0_register(CP0_COMPARE, 0);

}

/* Early initialization routine,
 * called from crt0.S's __start
 */
void hardware_init_hook(void)
{
	int cpu = 0;
	int barrier_num_cpus = 0;

	/* Initialize the Status Register */
	per_cpu_boot_init();


#ifdef XLP_SIM
	nlm_hal_init();
	phoenix_io_base  = xlp_io_base;
#endif

	hardware_init_boot2();
	process_fdt(fdtp);

#ifndef XLP_SIM

	nlm_cpu_frequency = (unsigned int)xlr_fdt.cpu_frequency;
	phoenix_io_base  = xlr_fdt.iobase;

#else

	nlm_cpu_frequency = nlm_hal_cpu_freq();

	syscall_init(cpu);

#endif

	for(cpu = 0; cpu < 32; cpu++) {
		if (cpu_online_map & (1 << cpu))
			num_cpus_online++;
	}


#ifdef NLM_BRIDGE_WKAROUND
    if(xlr_fdt.global_shmem_addr != 0) {
        nlm_bridge_lock = (nlm_rwlock_t *)(unsigned long)
            (xlr_fdt.global_shmem_addr + BRIDGE_WKAROUND_AREA_OFFSET);
        nlm_enable_br_wrkaround = 1;
    }
#endif

#ifndef XLP_SIM
	init_env((struct environment *)xlr_fdt.boot_env);
#endif

	uart_init();
	init_netos();
	process_gdbstub_args();
	per_cpu_arch_init();

	trap_init();

	if (init_irq() != 0) {
		printf("Unable to init IRQs. CPU %d will be in halt state\n",
				processor_id());
		__asm__ volatile("wait\n");
	}

	for(cpu = 0; cpu < 32; cpu++) {
		if((buddy_cpu_mask|master_cpu_mask) & (1U<<cpu))
			barrier_num_cpus++;
	}

	exit_barrier = barrier_open("_exit_barrier",O_CREAT,barrier_num_cpus);
	if(!exit_barrier){
		printf("Unable to allocate exit barrier\n");
	}

	pte =  (void *)(long)read_64bit_cp0_register_sel(CP0_OSSCRATCH, HPEXEC_PTEBASE_SCRATCH_IDX);

	init_timers();

#if defined(XLP_SIM)
	init_nae(fdtp);
        init_dtr();
#endif
	/* Enter the gdb stub
	 */
	if (gdb_connection_mode >= 0)
		gdbstub_init ();

}

/* Exit back to monitor, with the given status code.  */
void hardware_exit_hook (int status)
{
	char buf[100];
        unsigned int flags = 0;
	int len = 0;
	int i = 0;

	if (gdb_connection_mode >= 0)
		gdbstub_exit();

	if(exit_barrier)
		barrier_wait(exit_barrier);

	if (processor_id() == master_cpu) {

		/* stdout/stdin/stderr file descriptors are already closed! */
		len = snprintf(buf, 100, "CPUs (mask=0x%08x num_cpus_online=%d) exited; Done.\n",
		       cpu_online_map, num_cpus_online);

		flags = acquire_print_lock();
		while(i < len){
			outbyte(buf[i]);
			i++;
		}
		release_print_lock(flags);
	}

	cpu_halt();
}

#include "isadef.h"
/* TODO: The below dummy stubs complement some of the routines in the soft float implementation of doubles
 * for 64-bit ABI. libgcc.a needs to be compiled with some macros turned on or newlib needs to be compiled
 * not to use double float arithmetic. But for now use these stubs as the bootloader shouldn't be using any
 * floating pointer arithmetic anyway.
 */
#if (_MIPS_SIM == MIPS_SIM_ABI64)

void __trunctfdf2 (void) { abort(); }

#if 0
typedef float TFtype __attribute__ ((mode (TF)));
typedef unsigned int UTItype __attribute__ ((mode (TI)));
typedef enum
{
	CLASS_SNAN,
	CLASS_QNAN,
	CLASS_ZERO,
	CLASS_NUMBER,
	CLASS_INFINITY
} fp_class_type;

TFtype __make_tp (fp_class_type class, unsigned int sign, int exp, UTItype frac)
{
	abort();
}
#endif

#endif

void phnx_initialize_getopt(void)
{
	optind = 0;
}

void *cacheline_aligned_malloc(int size)
{
	void *addr = 0;

	addr = contig_alloc(size + (CACHELINE_SIZE << 1));
	if (addr == NULL) {
		printf("[%s]: unable to allocate memory!\n", __FUNCTION__);
		return NULL;
	}

	/* align the data to the next cache line */
	return (void *) ((((unsigned long) addr) + CACHELINE_SIZE) & ~(CACHELINE_SIZE - 1));
}

unsigned char cpu_stacks[XLx_NUM_CPUS][CPU_STACK_SIZE];
#define CPU_STACK_START(cpu) \
	    ((unsigned long)&cpu_stacks[(cpu)][0] + CPU_STACK_SIZE - CPU_STACK_INITIAL_FRAME_SIZE)

extern int smp_setjmp(jmp_buf, unsigned long);
extern void smp_longjmp(jmp_buf);

static __inline__ unsigned int cpu_active_stack_size(int cpu)
{
	unsigned long sp = 0;

	__asm__ __volatile__ (
			"or %0, $29, $0\n"
			: "=&r"(sp));

	return (unsigned int)(CPU_STACK_START(cpu) - sp);
}



static lib_spinlock_t clone_lock;
int _clone(int (*fn) (void *), void *child_stack, int flags, void *arg, int *err)
{

	unsigned int iflag;
	int cpu;
	extern volatile void *netos_thr_stack_pointer[];
	extern uint32_t syscall_clone_mask;
	lib_spinlock_t *clock = &clone_lock;

	if(child_stack == NULL) {
		*err = EINVAL;
		return -1;
	}

	if(syscall_clone_mask == 0) {
		*err = EINVAL;
		return -1;
	}

	lib_spin_lock_irqsave(clock, iflag);
	for(cpu = 0; cpu < XLx_NUM_CPUS ; cpu++) {
		if (syscall_clone_mask & (1 << cpu)) {
			syscall_clone_mask &= ~(1 << cpu);
			break;
		}
	}
	lib_spin_unlock_irqrestore(clock, iflag);

	if (cpu == XLx_NUM_CPUS) {
		*err = EAGAIN;
		return -1;
	}

	netos_thr_stack_pointer[cpu] = child_stack - 64;
	psb_wakeup_cpu((void *)(unsigned long)fn, arg, 1<<cpu);
	return cpu;
}

#define ADDRESS_OF(x) ((unsigned long)(char *)(x))
unsigned long heap_start = ADDRESS_OF(_end);
unsigned long heap_ptr = ADDRESS_OF(_end);

void *_sbrk(ptrdiff_t nbytes)
{
	char        *base = 0;
	unsigned int avail = 0;
	unsigned int heap_size=0;

	heap_size = get_heap_size();

	if (((size_t)heap_ptr >= heap_start) && ((size_t)heap_ptr < (heap_start + heap_size))) {
		avail = (heap_start + heap_size) - (size_t)heap_ptr;
		base = (char *)heap_ptr;
	}

	if ((nbytes > avail) || (heap_ptr + nbytes < heap_start))
		base = (char *)-1;
	else
		heap_ptr += nbytes;
	return base;
}

void __extenddftf2(void)
{
	return;
}
