/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

#define FUSE_USE_VERSION 26
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <sched.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mount.h>
#include <dlfcn.h>
#include <syscall.h>
#include <errno.h>
#include <fuse.h>
#include <dirent.h>
#include <libfdt.h>
#include "fdt_local.h"
#include "spinlock.h"
#include "nae.h"

#define mntpnt "/sysconfig"
#define rootdir "/hpexec"
#define logfname "./hpexeclog"
#define nodedir "/sysconfig/node0"

static char hpexecp[PATH_MAX] = rootdir;
static int mkdirmode = 0777;
static FILE *logfile;
static int rootdirlen, mntdirlen;

#define PATH_LEN 256
#define MAX_NAE_INDEX 18


static spinlock_t mmaplock;
static int mmapdone = 0;
static int nae_num_ports, nae_start_port;

pid_t fused_pid = 0;

void nlm_fuse_exit(void *data);
extern void *read_dtb_file(char *dtbfname);

static int create_sub_dirs(char *path, int hplen)
{
	char *end;
	int rv;

	if(path[0] != '/')
		return -EINVAL;

	if(strlen(path) == 1) 
		return 0;
	path++;

	while(1) {
		if(path[0] == '\0')
			break;
		end = strchr(path, '/');
		hpexecp[hplen++] = '/';
		if(end != NULL) {
			strncat(&hpexecp[hplen], path, end - path);
			hplen += end - path;
			hpexecp[hplen] = '\0';
		} else
			strcat(&hpexecp[hplen], path);
		fprintf(logfile, "creating dir %s\n", hpexecp);
		if((rv = mkdir(hpexecp, mkdirmode)) != 0)
			return rv;
		if(end == NULL)
			break;

		path = end + 1;
	}
	return 0;
}

static int create_my_files(void)
{
	int hplen = strlen(hpexecp);
	int rv, i;
	char fname[PATH_LEN];
	FILE *f;
	
	if((rv = mkdir(rootdir, mkdirmode)) != 0) {
		fprintf(stderr, "mkdir of %s failed, err=%s\n", rootdir, strerror(errno));
		return rv;
	}

	if(create_sub_dirs("/node0/nae", hplen) != 0)
		return -EINVAL;
	
	/* create the files */
	for(i = 0; i < MAX_NAE_INDEX; i++) {
		sprintf(fname, "%s/node0/nae/if%d", rootdir, i);
		f = fopen(fname, "w+");
		if(f == NULL) {
			fprintf(stderr, "fopen of %s failed, err=%s\n", fname, strerror(errno));
			return -EINVAL;
		}
		fclose(f);
	}
	return 0;
}

static inline char *modify_path(const char *path, char *mpath, int mlen)
{
	if(strlen(path) >= mlen - rootdirlen) { /* ~space for rootdir */
		return NULL;
	}
	strncpy(mpath, rootdir, mlen);
	if(strlen(path) > 1)
		strncpy(&mpath[rootdirlen], path, mlen - rootdirlen);
	return mpath;
}

static inline char *get_mnt_path(const char *path, char *mpath, int mlen)
{
	if(strlen(path) >= mlen - mntdirlen) { /* ~space fo mount point */
		return NULL;
	}
	strncpy(mpath, mntpnt, mlen);
	if(strlen(path) > 1)
		strncpy(&mpath[mntdirlen], path, mlen - mntdirlen);
	return mpath;
}

static int linuxu_nae_mmap(void)
{
	void *fdt;
	
		
	/* Get the shared mem info */
	if((fdt = read_dtb_file(getenv("lddtbfname"))) == NULL) {
		fprintf(stderr, "Error : FDT file read failed\n");
		return -1;
	}

	GET_FPROP(fdt, NAE_CFG_NODE, "num-ports", &nae_num_ports);
	GET_FPROP(fdt, NAE_CFG_NODE, "start-port-id", &nae_start_port);
	
	if (alloc_nae_mem(nae_num_ports, nae_start_port) < 0) {
		fprintf(stderr, "Allocation of NAE interfaces Failed \n");
		free(fdt);
		return -1;
	}
	free(fdt);
	return 0;
}

static int my_getattr(const char *path, struct stat *buf)
{
	int rv;
	char mpath[PATH_LEN];

	if((path = modify_path(path, mpath, PATH_LEN)) == NULL)
		return -ENAMETOOLONG;

	rv = lstat(path, buf);
	if(rv < 0)
		return rv;
	return 0;
}

static int my_readdir(const char *path, void *buf, fuse_fill_dir_t filler, off_t off, 
		struct fuse_file_info *info)
{
	struct dirent *de;
	DIR *dp;
	struct stat stat;
	char mpath[PATH_LEN];

	if((path = modify_path(path, mpath, PATH_LEN)) == NULL)
		return -ENAMETOOLONG;

	if((dp = opendir(path)) == NULL)
		return -errno;

	for(; (de = readdir(dp)) != NULL;) {
		memset(&stat, 0, sizeof(stat));
		stat.st_ino = de->d_ino;
		stat.st_mode = de->d_type << 12;
		if(filler(buf, de->d_name, &stat, 0))
			break;
	}
	closedir(dp);
	return 0;
}

static void my_destroy(void *data)
{
	fprintf(logfile, "%s in\n", __FUNCTION__);
	nlm_fuse_exit(data);
	return;
}

static int my_read(const char *path, char *buf, size_t sz, off_t off, struct fuse_file_info *info)
{
	char mpath[PATH_LEN];

	if((path = modify_path(path, mpath, PATH_LEN)) == NULL)
		return -ENAMETOOLONG;

	fprintf(logfile, "%s in, path %s\n", __FUNCTION__, path);
	return 0;
}

static int my_open(const char *path, struct fuse_file_info *info)
{
	char mpath[PATH_LEN], naepath[PATH_LEN];
	FILE *f;
	int rv, port;
	char *mntpath;
	const char *opath = path;

	if((path = modify_path(opath, mpath, PATH_LEN)) == NULL)
		return -ENAMETOOLONG;

	if((mntpath = get_mnt_path(opath, naepath, PATH_LEN)) == NULL)
		return -ENAMETOOLONG;

	fprintf(logfile, "%s in, path=%s\n", __FUNCTION__, path);

	spin_lock(&mmaplock);
	if(!mmapdone) {
		if((rv = linuxu_nae_mmap()) != 0) {
			fprintf(stderr, "[%s in] linuxu_nae_mmap failed", __FUNCTION__);
			spin_unlock(&mmaplock);
			return -1;
		}
		mmapdone = 1;
	}
	spin_unlock(&mmaplock);

	/* Check for out of range */
	sscanf(mntpath, NAE_IF_FMT, &port);
	if((port < nae_start_port) || (port >= (nae_start_port + nae_num_ports)))
		return 0;

	f = fopen(path, "w+");
	if(f == NULL) {
		fprintf(stderr, "[%s in] File not found path %s", __FUNCTION__, path);
		return -EINVAL;
	}

	if(nae_open(NULL, mntpath, 0, 0666) < 0) {
		fclose(f);
		fprintf(stderr, "[%s in] nae open failed for path %s", __FUNCTION__, mntpath);
		return -EINVAL;
	}
	fclose(f);
	return 0;
}

static int my_release(const char *path, struct fuse_file_info *info)
{

	char mpath[PATH_LEN];
	FILE *f;

	if((path = modify_path(path, mpath, PATH_LEN)) == NULL)
		return -ENAMETOOLONG;

	fprintf(logfile, "%s in, path %s\n", __FUNCTION__, path);

	f = fopen(path, "w+");
	if(f == NULL) {
		fprintf(logfile, "File not found for %s", path);
		return -EINVAL;
	}
	fclose(f);
	return 0;
}

static int my_ioctl(const char *path, int cmd, void *arg, struct fuse_file_info *info,
		unsigned int flags, void *data)
{
	char mpath[PATH_LEN];
	FILE *f;

	if((path = modify_path(path, mpath, PATH_LEN)) == NULL)
		return -ENAMETOOLONG;

	fprintf(logfile, "%s in, path %s cmd %d\n", __FUNCTION__, path, cmd);

	f = fopen(path, "w+");
	if(f == NULL) {
		fprintf(logfile, "File not found for %s", path);
		return -EINVAL;
	}
	fclose(f);
	return 0;
}

static struct fuse_operations fuse_ops = {
	.getattr = my_getattr,
	.readdir = my_readdir,
	.open = my_open,
	.read = my_read,
	.destroy = my_destroy,
	.release = my_release,
	.ioctl = my_ioctl
};

void nlm_fuse_exit(void *data)
{
	int rv;
	char buf[50];

	fprintf(logfile, "[fuse]: fused exiting!\n");
	rv = rmdir(mntpnt);
	if(rv < 0)
		fprintf(logfile, "rmdir of %s failed, err = %s\n", mntpnt, strerror(errno));

	sprintf(buf, "rm -rf %s\n", rootdir);
	rv = system(buf);
	if(rv == -1)
		fprintf(logfile, "cmd %s failed, err=%s\n", buf, strerror(errno));

	fprintf(logfile, "fuse exit done\n");
	fflush(logfile);
}

int nlm_fuse_init(void)
{
	int rv = 0;
	int i;
	struct stat st;

	/* open the log file */
	logfile = fopen(logfname, "w");
	rootdirlen = strlen(rootdir);
	mntdirlen  = strlen(mntpnt);

	fprintf(logfile, "%s in\n", __FUNCTION__);

	/* unmount the fs if it already exist */
	if(stat(mntpnt, &st) == 0) {
		rv = umount2(mntpnt, 2);
		if(rv < 0) {
			fprintf(stderr, "Umount of %s failed, err = %s\n", mntpnt, strerror(errno));
			return rv;
		}
		/* Wait for rootdir to get deleted */
		for(i = 0; i < 400; i++) {
			if(stat(rootdir, &st) != 0)
				break;
			usleep(10);
		}
	}

	rv = mkdir(mntpnt, mkdirmode);
	if(rv < 0) {
		fprintf(stderr, "Unable to create mountpoint![%s]\n", strerror(errno));
		return rv;
	}
	printf("[fuse]: Created [%s] mount point\n", mntpnt);

	rv = create_my_files();
	if(rv < 0)
		return rv;

	/* fuse deomon will run as child  */
	fused_pid = fork();
	if(fused_pid == 0) {
		int argc = 3;
		char *argv[] = { "./hpexec", mntpnt, "-f" };
		if(fuse_main(argc, argv, &fuse_ops, NULL) != 0)
			nlm_fuse_exit(NULL);
		exit(0);
	}
	/* Wait for fuse to come up */
	for(i = 0; i < 400; i++) {
		if(stat(nodedir, &st) == 0)
			break;
		usleep(10);
	}
	if(i >= 400) {
		fprintf(stderr, "Error: Fuse init failed , err=%s\n",strerror(errno));
		return -1;
	}
	else
		printf("Fuse init done (fused pid=%d)\n", (int)fused_pid);

	return 0;
}


