ABI=64

ifeq ($(XLP_SIM), 1)
TARGET  = mipsisa64r2-nlm-elf
CFLAGS += -march=xlp
else
TARGET  = mipsisa64xlr-elf
CFLAGS += -march=xlr
ARCH = xlr
endif

AS       =  $(TARGET)-gcc
CC       =  $(TARGET)-gcc
CXX      =  $(TARGET)-g++
LD       =  $(TARGET)-ld
AR       =  $(TARGET)-ar
RANLIB   =  $(TARGET)-ranlib
OBJDUMP  =  $(TARGET)-objdump
NM       =  $(TARGET)-nm
OBJCOPY  =  $(TARGET)-objcopy
READELF  =  $(TARGET)-readelf 
AWK      =  awk

ifeq ("$(ABI)","64")
ASFLAGS  += -mabi=64
CFLAGS   += -mabi=64
CPPFLAGS += -mabi=64
LDFLAGS  += -melf64btsmip
endif

ifeq ("$(GDBDEBUGGER)","1")
  opt-level = 0
  CFLAGS += -g3 -DGDB_REMOTE_DEBUG=1
endif


#
# Optimization and misc information to C compiler
#

ifeq ($(opt-level),)
  opt-level = 3
endif

CFLAGS += -O$(opt-level)

ifneq ($(unroll-loops),N)
  CFLAGS += -funroll-loops
endif

ifeq ($(inline-limit),)
  inline-limit = 20000
endif

CFLAGS += -finline-limit=$(inline-limit)
CFLAGS += -fomit-frame-pointer -mno-branch-likely
CFLAGS += -Wall #-Werror
CFLAGS += -G 0 -mno-gpopt
 
ASFLAGS += $(CFLAGS) -D__ASSEMBLY__

LDFLAGS += -d -warn-common #--no-warn-mismatch

#
# header file locations
#

CPPFLAGS += -I$(NETOS_LIB)/include

NETOS_GENSYM = netos_gensym

#
# lib path
#

##### include Symbol table in the executable begins #####

# To include symbols in the backtrace, Remove followin two lines from Makefile
# of netos_app 
# all: $(LINKER_SCRIPT) $(OBJS)
#	$(LD) $(LDFLAGS) $(LIBRARIES_START) $(OBJS) $(LIBRARIES_END) -o $(OUTPUT)
#
# Set up the $(OBJS) and $(OUTPUT) properly, and use following line in Makefile of 
# netos_app instead of default all.
# all: $(LINKER_SCRIPT) $(OBJS) SYMTAB_OUTPUT
all:

#Setup the default rule as all
all:

SYMTAB_NM = symtable_nm
SYMTAB_C  = rmisym.output.c
SYMTAB_O  = rmisym.output.o


SYMTAB_O_0: OUTPUT_0
	$(NM) -n $(OUTPUT) > $(SYMTAB_NM)
	$(NETOS_LIB)/scripts/$(NETOS_GENSYM) $(SYMTAB_NM) $(SYMTAB_C)
	$(CC) $(CFLAGS) -c $(SYMTAB_C)

SYMTAB_O_1: SYMTAB_O_0 OUTPUT_1
	$(NM) -n $(OUTPUT) > $(SYMTAB_NM)
	$(NETOS_LIB)/scripts/$(NETOS_GENSYM) $(SYMTAB_NM) $(SYMTAB_C)
	$(CC) $(CFLAGS) -c $(SYMTAB_C)

OUTPUT_1: SYMTAB_O_0
	$(LD) $(LDFLAGS) $(LIBRARIES_START) $(OBJS) $(LIBRARIES_END) $(SYMTAB_O) -o $(OUTPUT) 

OUTPUT_0: 
	 $(LD) $(LDFLAGS) $(LIBRARIES_START) $(OBJS) $(LIBRARIES_END) -o $(OUTPUT)

NON_SYMTAB_OUTPUT: 
	$(LD) $(LDFLAGS) $(LIBRARIES_START) $(OBJS) $(LIBRARIES_END) $(SYMTAB_O) -o $(OUTPUT) 

SYMTAB_OUTPUT: SYMTAB_O_1
	$(LD) $(LDFLAGS) $(LIBRARIES_START) $(OBJS) $(LIBRARIES_END) $(SYMTAB_O) -o $(OUTPUT) 


###### include Symbol table in the executable ends #####

all:

msgring.c: $(MSGRING_CFG)
	$(NETOS_LIB)/scripts/msgring $(MSGRING_CFG)

msgring_xls.c: $(MSGRING_XLS_CFG)
	$(NETOS_LIB)/scripts/msgring_xls $(MSGRING_XLS_CFG)

ifneq ($(COMPILE_AS_MODULE),y)
  include $(NETOS_LIB)/scripts/Makefile.app.mk
endif
