/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

%{
#include <stdio.h>
#include <string.h>
#include "msgring_xls.yacc.h"


int yyerror(const char *str)
{
  fprintf(stderr, "%s\n", str);
  return 1;
}

%}

%x c_comment

%%

"/*"                    BEGIN(c_comment); /* c-style comment */
<c_comment>[^*\n]*        /* eat anything that is not a * */
<c_comment>"*"+[^*/\n]*   /* eat up * s not followed by a / */
<c_comment>\n
<c_comment>"*"+"/"      BEGIN(INITIAL);

"//"[^\n]*"\n"         ; /* eat up one-line (C++-style) commens */

bucket                  yylval = (unsigned long)"bucket"; return TOK_BUCKET;
size                    yylval = (unsigned long)"size"; return TOK_SIZE;
[a-zA-Z_][a-zA-Z_0-9]*    yylval = (unsigned long)strdup(yytext); return TOK_WORD;
[0-9]+                  yylval = atoi(yytext); return TOK_INTEGER;
\"                      yylval = (unsigned long)"\""; return TOK_QUOTE;
\{                      yylval = (unsigned long)"{"; return TOK_OPEN_BRACE;
\}                      yylval = (unsigned long)"}"; return TOK_CLOSE_BRACE;
;                       yylval = (unsigned long)";"; return TOK_SEMICOLON;
\n                      /* ignore newline */;
[ \t]+                  /* ignore whitespace */;
.                       yyerror("Unknown token\n");
%%

int yywrap()
{
  return 1;
}

