#include <stdio.h>

#include "processor.h"
#include "fast_syscall.h"

int processor_id = 0xffffffff;
int is_step_a, is_step_b, is_step_c;
int xlr_step_a, xlr_step_b, xlr_step_c;
int xls_step_a, xls_step_b;
int on_xlr, xlr_7xx, xlr_5xx, xlr_3xx;
int on_xls, xls_6xx, xls_4xx, xls_2xx, xls_1xx, xls_608;

int on_xlp = 0;

int XLR_STEP_A[] = { -1 };
int XLR_STEP_B[] = {
	CHIP_PROCESSOR_ID_XLR_308_STEP_B, CHIP_PROCESSOR_ID_XLR_508_STEP_B,
	CHIP_PROCESSOR_ID_XLR_516_STEP_B, CHIP_PROCESSOR_ID_XLR_532_STEP_B,
	CHIP_PROCESSOR_ID_XLR_716_STEP_B, CHIP_PROCESSOR_ID_XLR_732_STEP_B,
	-1
};
int XLR_STEP_C[] = {
	CHIP_PROCESSOR_ID_XLR_308_STEP_C, CHIP_PROCESSOR_ID_XLR_508_STEP_C,
	CHIP_PROCESSOR_ID_XLR_516_STEP_C, CHIP_PROCESSOR_ID_XLR_532_STEP_C,
	CHIP_PROCESSOR_ID_XLR_716_STEP_C, CHIP_PROCESSOR_ID_XLR_732_STEP_C,
	-1
};

int XLS_STEP_A[] = {
	CHIP_PROCESSOR_ID_XLS_608_STEP_A, CHIP_PROCESSOR_ID_XLS_408_STEP_A,
	CHIP_PROCESSOR_ID_XLS_404_STEP_A, CHIP_PROCESSOR_ID_XLS_208_STEP_A,
	CHIP_PROCESSOR_ID_XLS_204_STEP_A, CHIP_PROCESSOR_ID_XLS_108_STEP_A,
	CHIP_PROCESSOR_ID_XLS_104_STEP_A, -1
};

int XLS_STEP_B[] = {
	CHIP_PROCESSOR_ID_XLS_616_STEP_B,
	CHIP_PROCESSOR_ID_XLS_416_STEP_B,
	CHIP_PROCESSOR_ID_XLS_608_STEP_B,
	CHIP_PROCESSOR_ID_XLS_412_STEP_B,
	CHIP_PROCESSOR_ID_XLS_408_STEP_B,
	CHIP_PROCESSOR_ID_XLS_404_STEP_B, -1
};

int XLS_STEP_C[] = {-1};

static const int *proc_id_matrix[MAX_PROC_FAMILY][MAX_STEPPING] = {
	{ /* XLR */
		XLR_STEP_A,
		XLR_STEP_B,
		XLR_STEP_C
	},
	{ /* XLS */
		XLS_STEP_A,
		XLS_STEP_B,
		XLS_STEP_C
	}
};

/*
 * Uncached functions
 */
static inline int is_uncached_xlr_step_a(int x)
{
	int t = (x >> 8) & 0xff, i = 0;
	if(is_stepping_a()) {
		while(proc_id_matrix[PROC_ID_XLR][PROC_ID_STEP_A][i] != -1 &&
		      proc_id_matrix[PROC_ID_XLR][PROC_ID_STEP_A][i] != t) i++;
		if(proc_id_matrix[PROC_ID_XLR][PROC_ID_STEP_A][i] != -1) {
			return 1;
		}
	}
	return 0;
}

static inline int is_uncached_xlr_step_b(int x)
{
	int t = (x >> 8) & 0xff, i = 0;
	if(is_stepping_b()) {
		while(proc_id_matrix[PROC_ID_XLR][PROC_ID_STEP_B][i] != -1 &&
		      proc_id_matrix[PROC_ID_XLR][PROC_ID_STEP_B][i] != t) i++;
		if(proc_id_matrix[PROC_ID_XLR][PROC_ID_STEP_B][i] != -1) {
			return 1;
		}
	}
	return 0;
}

static inline int is_uncached_xlr_step_c(int x)
{
	int t = (x >> 8) & 0xff, i = 0;
	if(is_stepping_c()) {
		while(proc_id_matrix[PROC_ID_XLR][PROC_ID_STEP_C][i] != -1 &&
		      proc_id_matrix[PROC_ID_XLR][PROC_ID_STEP_C][i] != t) i++;
		if(proc_id_matrix[PROC_ID_XLR][PROC_ID_STEP_C][i] != -1) {
			return 1;
		}
	}
	return 0;
}

static inline int is_uncached_xls_step_a(int x)
{
	int t = (x >> 8) & 0xff, i = 0;
	if(is_stepping_a()) {
		while(proc_id_matrix[PROC_ID_XLS][PROC_ID_STEP_A][i] != -1 &&
		      proc_id_matrix[PROC_ID_XLS][PROC_ID_STEP_A][i] != t) i++;
		if(proc_id_matrix[PROC_ID_XLS][PROC_ID_STEP_A][i] != -1) {
			return 1;
		}
	}
	return 0;
}

static inline int is_uncached_xls_step_b(int x)
{
	int t = (x >> 8) & 0xff, i = 0;
	if(is_stepping_b()) {
		while(proc_id_matrix[PROC_ID_XLS][PROC_ID_STEP_B][i] != -1 &&
		      proc_id_matrix[PROC_ID_XLS][PROC_ID_STEP_B][i] != t) i++;
		if(proc_id_matrix[PROC_ID_XLS][PROC_ID_STEP_B][i] != -1) {
			return 1;
		}
	}
	return 0;
}

#define is_uncached_xlr_7xx(x)						\
	({	int t = (x >> 8) & 0xff;				\
		(t == CHIP_PROCESSOR_ID_XLR_716_STEP_B || t ==  CHIP_PROCESSOR_ID_XLR_732_STEP_B || \
		 t == CHIP_PROCESSOR_ID_XLR_716_STEP_C || t ==  CHIP_PROCESSOR_ID_XLR_732_STEP_C); \
	})

#define is_uncached_xlr_5xx(x)						\
	({	int t = (x >> 8) & 0xff;				\
		(t == CHIP_PROCESSOR_ID_XLR_516_STEP_B || t ==  CHIP_PROCESSOR_ID_XLR_532_STEP_B || \
		 t == CHIP_PROCESSOR_ID_XLR_516_STEP_C || t ==  CHIP_PROCESSOR_ID_XLR_532_STEP_C); \
	})

#define is_uncached_xlr_3xx(x)						\
	({	int t = (x >> 8) & 0xff;				\
		(t == CHIP_PROCESSOR_ID_XLR_308_STEP_B || t ==  CHIP_PROCESSOR_ID_XLR_308_STEP_C); \
	})

#define is_uncached_xls_6xx(x)						\
	({	int t = (x >> 8) & 0xff;				\
		(t == CHIP_PROCESSOR_ID_XLS_608_STEP_A || t ==  CHIP_PROCESSOR_ID_XLS_608_STEP_B || \
		 t == CHIP_PROCESSOR_ID_XLS_616_STEP_B);		\
	})

#define is_uncached_xls_4xx(x)						\
	({	int t = (x >> 8) & 0xff;				\
		(t == CHIP_PROCESSOR_ID_XLS_408_STEP_A || t ==  CHIP_PROCESSOR_ID_XLS_408_STEP_B || \
		 t == CHIP_PROCESSOR_ID_XLS_416_STEP_B);		\
	})

#define is_uncached_xls_2xx(x)						\
	({	int t = (x >> 8) & 0xff;				\
		(t == CHIP_PROCESSOR_ID_XLS_204_STEP_A || t ==  CHIP_PROCESSOR_ID_XLS_208_STEP_A); \
	})

#define is_uncached_xls_1xx(x)						\
	({	int t = (x >> 8) & 0xff;				\
		(t == CHIP_PROCESSOR_ID_XLS_104_STEP_A || t ==  CHIP_PROCESSOR_ID_XLS_108_STEP_A); \
	})

#define is_uncached_stepping_a(x)				\
	({ int t = x & 0xff00ff;				\
		(t >= NLM_REVISION_A0 && t < NLM_REVISION_B0);	\
	})

#define is_uncached_stepping_b(x)				\
	({ int t = x & 0xff00ff;				\
		(t > NLM_REVISION_A1 && t <= NLM_REVISION_B2);	\
	})

#define is_uncached_stepping_c(x)				\
	({ int t = x & 0xff00ff;				\
		(t > NLM_REVISION_B2 && t <= NLM_REVISION_C4);	\
	})

void init_cached_proc_id(void)
{
	int p = fs_processorId();

	processor_id = p;

	/* XLP, for now */
	if (processor_id == 0x0C0000) {
		printf("Running on XLP (PID=0x%08x)\n", processor_id);
		on_xlp = 1;
		return;
	}

	is_step_a = is_uncached_stepping_a(p);
	is_step_b = is_uncached_stepping_b(p);
	is_step_c = is_uncached_stepping_c(p);

	xlr_step_a  = xlr_step_b = xlr_step_c = xls_step_a = xls_step_b = 0;
	on_xlr = xlr_7xx = xlr_5xx = xlr_3xx = xls_6xx = xls_4xx = xls_2xx = 0;

	on_xls = xls_step_a = xls_step_b = xls_2xx = xls_4xx = xls_6xx = 0;

	xlr_step_a = is_uncached_xlr_step_a(p);
	xlr_step_b = is_uncached_xlr_step_b(p);
	xlr_step_c = is_uncached_xlr_step_c(p);
	on_xlr = (xlr_step_a || xlr_step_b || xlr_step_c);

	xlr_7xx = is_uncached_xlr_7xx(p);
	xlr_5xx = is_uncached_xlr_5xx(p);
	xlr_3xx = is_uncached_xlr_3xx(p);

	xls_step_a = is_uncached_xls_step_a(p);
	xls_step_b = is_uncached_xls_step_b(p);
	on_xls = (xls_step_a || xls_step_b);
	xls_6xx = is_uncached_xls_6xx(p);
	xls_4xx = is_uncached_xls_4xx(p);
	xls_2xx = is_uncached_xls_2xx(p);
	xls_1xx = is_uncached_xls_1xx(p);
	xls_608 = (p == CHIP_PROCESSOR_ID_XLS_608_STEP_B ||
		   p == CHIP_PROCESSOR_ID_XLS_608_STEP_A);
	return;
}
