#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sched.h>
#include <sys/mman.h>
#include <sys/wait.h>

#include "nlm_soclib.h"
#include "processor.h"
#include "fast_syscall.h"


#if !((_MIPS_SIM == _MIPS_SIM_ABI32) || defined(ABI_32))
static double cpu_freq;
#endif

static double seconds_per_tick;

void init_seconds_per_tick(void)
{
	if (is_xlp()) {
		/* XLP */
#if (_MIPS_SIM == _MIPS_SIM_ABI32) || defined(ABI_32)

		seconds_per_tick = 7.5 * 1.0e-9; /* 133.33 MHz */
#else
		cpu_freq = fs_cpu_max_freq();

		if(cpu_freq == 0) {
			printf("Unable to get CPU frequency\n");
			exit(1);
		}
		else
			printf("cpu_freq = 0x%016lld\n", (unsigned long long)cpu_freq);

		seconds_per_tick = (((double)1.0)/cpu_freq);
#endif
	} else {
		/* XLR/XLS */
		seconds_per_tick = 15 * 1.0e-9; /* 66.66 MHz */
	}
	printf("Seconds per tick=%g secs\n", seconds_per_tick);
	return;
}

#if defined(NLM_ARCH_XLR) || (_MIPS_SIM == _MIPS_SIM_ABI32) || defined(ABI_32)
static uint64_t read_timer_cycles(void)
{
	return fs_read_timer();
}
#else
static inline uint64_t read_timer_cycles(void)
{
	uint64_t ret;

	__asm__ volatile (
		".set push \n"
		".set arch=xlp\n"
		"dmfur %0, $16\n"
		".set pop\n"
		:"=r"(ret)
		);

	return ret;
}
#endif

void nlm_soclib_timer_init(struct nlm_soclib_timer *timer)
{
	timer->start_cycles = timer->stop_cycles = timer->time_accum = 0;
	return;
}

void nlm_soclib_timer_start(struct nlm_soclib_timer *timer)
{
	timer->start_cycles = read_timer_cycles();
}

void nlm_soclib_timer_stop(struct nlm_soclib_timer *timer)
{
	timer->stop_cycles = read_timer_cycles();

	timer->time_accum += (timer->stop_cycles - timer->start_cycles);
}

double nlm_soclib_timer_seconds(const struct nlm_soclib_timer *timer)
{
	return seconds_per_tick * timer->time_accum;
}

