#include <stdio.h>
#include <stdlib.h>

#include "mips_opcode.h"
#include "system.h"
#include "psb.h"
#include "timer.h"
#include "cpu.h"
#include "uart.h"
#include "stdlib.h"
#include "shell.h"
#include "limits.h"
#include "isadef.h"
#include "xlr_boot_lib.h"

#define	MIPS_JR_RA	0x03e00008	/* instruction code for jr ra */
#define	MIPS_JR_K0	0x03400008	/* instruction code for jr k0 */
#define	MIPS_ERET	0x42000018	/* instruction code for eret */

#ifndef LONG_LONG_MAX
#define LONG_LONG_MAX 9223372036854775807LL
#endif
/* start addr */
extern unsigned char _loadaddr[];
extern unsigned char _edata[];
extern struct netos_symtab_t {
	unsigned long long pos;
	unsigned int off;
} netos_symtab[];
extern char netos_symtab_string_t[];
extern unsigned int xlr_netos_uart;
extern void dump_tlb(int argc, char *argv[]);

void stacktrace(struct pt_regs *regs);

static void show_back_trace(int argc, char *argv[]);
static void show_panic_regs(int argc, char *argv[]);
static void read_mem(int argc, char *argv[]);
static void panic_exit(int argc, char *argv[]);
static void show_cp0(int argc, char *argv[]);
static int netos_search_symbol(long addr, char *buf);
#define NR_CPUS 32
static struct pt_regs panic_regs[NR_CPUS];
void stacktrace_subr(long a0, long a1, long a2, long a3, u_long pc, u_long sp, u_long fp, u_long ra, void (*printfn)(const char*, ...));
static char *shell_argv[] = { "shell", "panic-shell   >" };

static unsigned int panic_flags;

#define NR_CPUS 32

#ifdef DEBUG
static void show_sp(int argc, char *argv[])
{
	unsigned long long mysp;
	__asm__ __volatile__(
			".set push	\n"
			".set mips64	\n"
			"move %0, $sp   \n"
			".set pop       \n"
			:"=r" (mysp) );
	printf("SP = %llx\n",mysp);
}
#endif /* DEBUG */

#ifdef NETOS_TLB_REFILL_HANDLER 
extern void dump_page_table();
static void dump_pte(int argc, char *argv[])
{
	dump_page_table();
}
#endif /* NETOS_TLB_REFILL_HANDLER */

/* We enter here with printf lock held */
void start_panic_shell()
{
	extern void trap_init(void);
	extern int init_irq(void);
	trap_init();
	init_irq();
	sti();

	if(!(cpu_online_map & 0x1)) {
		/*thread 0, core 0 is alive!!*/
		if(xlr_netos_uart == TTYS0){
			/*Bootloader is still alive can't start shell
			  on uart-0*/
			release_print_lock(panic_flags);
			printf("\nCan not start panic shell cpu_%d on uart-0\n"
			       ,processor_id());
			return;
		}
	}
	/* Only one cpu drops into a shell */
	sprintf(shell_argv[1],"panic-shell%d>",processor_id());
	printf("Starting netos panic shell on thread %d \n",processor_id());
	shell_init();
	register_shell_cmd("bt", show_back_trace, "Print backtrace");
	register_shell_cmd("regs", show_panic_regs, "Print Regs");
	register_shell_cmd("rdmem", read_mem, "Read Memory");
	register_shell_cmd("cpu_halt", panic_exit, "Stop Cpu");
	register_shell_cmd("dump_tlb", dump_tlb, "Print TLB");
#ifdef NETOS_TLB_REFILL_HANDLER 
	register_shell_cmd("dump_page_table", dump_pte, "Print Page Table");
#endif /* NETOS_TLB_REFILL_HANDLER */
	register_shell_cmd("cop0", show_cp0, "Print Cop0 Regs");
	change_prompt(2, shell_argv);
	for(;;) {
		shell_run();
	}

}

static void show_cp0(int argc, char *argv[])
{
	printf("CP 0 Registers:\n");
	printf("\tCP0_INDEX       = %08x\n", read_32bit_cp0_register(CP0_INDEX));
	printf("\tCP0_RANDOM      = %08x\n", read_32bit_cp0_register(CP0_RANDOM));
	printf("\tCP0_ENTRYLO0    = %016llx\n", read_64bit_cp0_register_sel(CP0_ENTRYLO0, 0));
	printf("\tCP0_ENTRYLO1    = %016llx\n", read_64bit_cp0_register_sel(CP0_ENTRYLO1, 0));
	printf("\tCP0_CONF        = %08x\n", read_32bit_cp0_register(CP0_CONF));
	printf("\tCP0_CONTEXT     = %016llx\n", read_64bit_cp0_register_sel(CP0_CONTEXT, 0));
	printf("\tCP0_PAGEMASK    = %08x\n", read_32bit_cp0_register(CP0_PAGEMASK));
	printf("\tCP0_WIRED       = %08x\n", read_32bit_cp0_register(CP0_WIRED));
	printf("\tCP0_INFO        = %08x\n", read_32bit_cp0_register(CP0_INFO));
	printf("\tCP0_BADVADDR    = %016llx\n", read_64bit_cp0_register_sel(CP0_BADVADDR, 0));
	printf("\tCP0_COUNT       = %08x\n", read_32bit_cp0_register(CP0_COUNT));
	printf("\tCP0_ENTRYHI     = %016llx\n", read_64bit_cp0_register_sel(CP0_ENTRYHI, 0));
	printf("\tCP0_COMPARE     = %08x\n", read_32bit_cp0_register(CP0_COMPARE));
	printf("\tCP0_STATUS      = %08x\n", read_32bit_cp0_register(CP0_STATUS));
	printf("\tCP0_CAUSE       = %08x\n", read_32bit_cp0_register(CP0_CAUSE));
	printf("\tCP0_EPC         = %016llx\n", read_64bit_cp0_register_sel(CP0_EPC, 0));
	printf("\tCP0_PRID        = %08x\n", read_32bit_cp0_register(CP0_PRID));
	printf("\tCP0_CONFIG      = %08x\n", read_32bit_cp0_register(CP0_CONFIG));
	printf("\tCP0_LLADDR      = %016llx\n", read_64bit_cp0_register_sel(CP0_LLADDR, 0));
	printf("\tCP0_WATCHLO     = %016llx\n", read_64bit_cp0_register_sel(CP0_WATCHLO, 0));
	printf("\tCP0_WATCHHI     = %08x\n", read_32bit_cp0_register(CP0_WATCHHI));
	printf("\tCP0_XCONTEXT    = %016llx\n", read_64bit_cp0_register_sel(CP0_XCONTEXT, 0));
	printf("\tCP0_FRAMEMASK   = %08x\n", read_32bit_cp0_register(CP0_FRAMEMASK));
	printf("\tCP0_DIAGNOSTIC  = %08x\n", read_32bit_cp0_register(CP0_DIAGNOSTIC));
	printf("\tCP0_DEBUG       = %08x\n", read_32bit_cp0_register(CP0_DEBUG));
	printf("\tCP0_DEPC        = %08x\n", read_32bit_cp0_register(CP0_DEPC));
	printf("\tCP0_PERFORMANCE = %08x\n", read_32bit_cp0_register(CP0_PERFORMANCE));
	printf("\tCP0_ECC         = %08x\n", read_32bit_cp0_register(CP0_ECC));
	printf("\tCP0_CACHEERR    = %08x\n", read_32bit_cp0_register(CP0_CACHEERR));
	printf("\tCP0_TAGLO       = %016llx\n", read_64bit_cp0_register_sel(CP0_TAGLO, 0));
	printf("\tCP0_TAGHI       = %016llx\n", read_64bit_cp0_register_sel(CP0_TAGHI, 0));
	printf("\tCP0_ERROREPC    = %016llx\n", read_64bit_cp0_register_sel(CP0_ERROREPC, 0));
	printf("\tCP0_DESAVE      = %08x\n", read_32bit_cp0_register(CP0_DESAVE));
}

static void panic_exit(int argc, char *argv[])
{
	release_print_lock(panic_flags);
	cpu_halt();
}

static void show_panic_regs(int argc, char *argv[])
{
	//show_regs(&panic_regs[processor_id()]);
}

static void read_mem(int argc, char *argv[])
{
	long long *abc;
	long long addr;
	long num_cachelines = 1;
	int i;
	char *nnn;
	char lastchar;

	if (argc < 2) {
		printf("usage: rdmem <addr> <num_cachelines>\n");
		return;
	}

	if (argc > 2)
		num_cachelines = strtol(argv[2], NULL, 16);

	addr = (unsigned long long)strtoll(argv[1], &nnn, 16);
	if (addr == LONG_LONG_MAX) {
		/* If addr is long long max, we exceed the signed range of address.
		 * strip off last nibble from the string and get the address.
		 */
		nnn--;
		lastchar = *nnn;
		*nnn = '\0';
		addr = (unsigned long long)strtoll(argv[1], NULL, 16);
		addr = addr << 4;

	}

	if ((num_cachelines > 0x80) || (num_cachelines < 1)) {
		printf("cachelines should be between 1 and 0x80 inclusive\n");
		return;
	}

	if (!addr) {
		printf("enter correct address\n");
		return;
	}

	addr = addr & ~0x1fLL;

	if ((((long)addr == (long)KSEG0) || (virt_to_phys((void *)(long)addr))) &&
	    (virt_to_phys((void *)(long)(addr + num_cachelines * 32)))) {
		// if addr and (addr + cacheline) have virt_to_phys mapping,
		// we are sure that it belongs to our address space.

		abc = (long long *)((long)addr);
		for ( i = 0; i < num_cachelines; i++) {
			printf("%p: %016llx",abc, *abc);
			abc++;
			printf(" %016llx",*abc);
			abc++;
			printf(" %016llx",*abc);
			abc++;
			printf(" %016llx\n",*abc);
			abc++;
		}
		return;
	}

	printf("address out of range\n");
	return;
}

static void show_back_trace(int argc, char *argv[])
{
	stacktrace(&panic_regs[processor_id()]);
}

static void store_panic_regs(struct pt_regs *regs)
{
	int i;
	int id = processor_id();

	for (i = 0; i < 32; i++)
		panic_regs[id].regs[i] = regs->regs[i];
	panic_regs[id].cp0_status = regs->cp0_status;
	panic_regs[id].hi = regs->hi;
	panic_regs[id].lo = regs->lo;
	panic_regs[id].cp0_badvaddr = regs->cp0_badvaddr;
	panic_regs[id].cp0_cause = regs->cp0_cause;
	panic_regs[id].cp0_epc = regs->cp0_epc;

}

int get_netos_sym_tokens()
{
	int netos_sym_tokens;
	netos_sym_tokens = (long)netos_symtab_string_t - (long)netos_symtab;
	if (netos_sym_tokens < 0)
		netos_sym_tokens = -netos_sym_tokens;
	netos_sym_tokens = netos_sym_tokens/ (sizeof(struct netos_symtab_t));
	return netos_sym_tokens;
}

void stacktrace(struct pt_regs *regs)
{
	store_panic_regs(regs);
	/* store number of symbols in netos_sym_tokens */
	printf("Call Trace:\n");
	printf("PC:      func+offset (a0,a1,a2,a3) ra stacksize\n");
	stacktrace_subr(regs->regs[4], regs->regs[5], regs->regs[6],
			regs->regs[7], regs->cp0_epc, regs->regs[29], 
			regs->regs[30], regs->regs[31], 
			(void (*)(const char*, ...))printf);

}

static long long kdbpeek64(unsigned long addr)
{
	long long rc;

	if (addr & 7) {
		printf("kdbpeek64: unaligned address %lx\n", addr);
		rc = -1 ;
	} else if (addr == 0) {
		printf("kdbpeek64: NULL\n");
		rc = 0xdeadfeed;
	} else {
		rc = *(long long *)(long)addr;
	}
	return rc;
}

static int kdbpeek(unsigned long addr)
{
	int rc;

	if (addr & 3) {
		printf("kdbpeek: unaligned address %lx\n", addr);
		rc = -1 ;
	} else if (addr == 0) {
		printf("kdbpeek: NULL\n");
		rc = 0xdeadfeed;
	} else {
		rc = *(int *)addr;
	}
	return rc;
}

/*
 * Map a function address to a string name, if known; or a hex string.
 */
static char *fn_name(long addr)
{
	static char buf[100];

	if (!(netos_search_symbol(addr,buf)))
		return buf;

	sprintf(buf, "%lx", addr);
	return (buf);
}

static void get_sym_by_off(char *c, char* buf)
{
	if (!c) return;
	while (*c != '\0' && *c != ' ') {
		sprintf(buf,"%c",*c);
		buf++;
		c++;
	}
}

/* TODO: Make this efficient... Do binary search...*/
/* returns 0 for success */
static int get_sym_by_addr(unsigned long addr, char *buf)
{
	int i;

	for (i = 0; i < get_netos_sym_tokens(); i++) {
		if (addr == (unsigned long)netos_symtab[i].pos) { //found addr
			if (buf)
				get_sym_by_off(&netos_symtab_string_t
				       [netos_symtab[i].off],buf);

			return 0;
		}
		if (addr < (unsigned long)netos_symtab[i].pos)
			return -1;
	}
	return -1;
}

static int netos_search_symbol(long addr, char *buf)
{
	return get_sym_by_addr(addr, buf);
}

unsigned long get_preceding_sym(unsigned long addr)
{
	int i;
	for (i = 0; i < get_netos_sym_tokens(); i++) {
		if (addr == (unsigned long)netos_symtab[i].pos) { //found addr
			return addr;
		}
		if (addr < (unsigned long)netos_symtab[i].pos)
			return netos_symtab[i-1].pos;
	}
	return 0;
}

extern long symtab_cpu_halt;
extern void __start(int arg);
extern void main(int argc, char *argv[]);

/*
 * Do a stack backtrace.
 * (*printfn)()  prints the output 
 */
void stacktrace_subr(long a0, long a1, long a2, long a3, u_long pc, u_long sp, u_long fp, u_long ra, void (*printfn)(const char*, ...))
{
	unsigned long va, subr;
	unsigned instr, mask;
	InstFmt i;
	int more, stksize;
	unsigned int frames =  0;
	int foundframesize = 0;

/* Jump here when done with a frame, to start a new one */
loop:
	stksize = 0;
	subr = 0;
	if (frames++ > 100) {
		(*printfn)("\nstackframe count exceeded\n");
		/* return breaks stackframe-size heuristics with gcc -O2 */
		goto finish;	/*XXX*/
	}

	/* check for bad SP: could foul up next frame */
	if (sp & 3 ) {
		(*printfn)("SP not alligned to word boundary: 0x%x \n", sp);
		ra = 0;
		subr = 0;
		goto done;
	}

	if (pc & 3) {
		(*printfn)("PC not alligned to word boundary : 0x%x \n", pc);
		ra = 0;
		goto done;
	}
	/* Check for bad PC */
	if ((unsigned)pc >= (unsigned)(long)_edata) {
		(*printfn)("PC is outside the text segment : 0x%x \n", pc);
		ra = 0;
		goto done;
	}

	/*
	 * Find the beginning of the current subroutine by scanning backwards
	 * from the current PC for the end of the previous subroutine.
	 */
	va = pc;
	if (get_netos_sym_tokens()) {
		va = get_preceding_sym(pc);
		if (va) {
			va -= sizeof(int);
			goto mips3_eret;
		}
		else {
			printf("Error: Unable to get symbol from symbol ");
			printf("table\n returning...\n");
			return;
		}
	}
	if (va == (int)symtab_cpu_halt) {
		va -= sizeof(int);
		goto mips3_eret;
	}
loop_again:

	do {
		va -= sizeof(int);
		if (va <= (unsigned)(long)_loadaddr)
			goto finish;
		if (va == (int)(long)(void *)__start) {
			va -= sizeof(int);
			goto mips3_eret;
		}
		if (va == (int)(long)(void *)main) {
			va -= sizeof(int);
			goto mips3_eret;
		}
		instr = kdbpeek(va);
		if (instr == MIPS_ERET)
			goto mips3_eret;
		if ((instr >>16 == ADDIU_SP_SP) || 
			    (instr >> 16 == DADDIU_SP_SP)) {
			if ((short) instr > 0){
				// skip next jr ra...
				va -= sizeof(int);
				continue;
			} else {
				// Check prev 3 instr
				instr = kdbpeek(va - sizeof(int) * 2);
				instr >>= 26;
				if (instr == OP_J) {
					va -= sizeof(int);
					goto mips3_eret;
				}
				instr = kdbpeek(va - sizeof(int) * 3);
				instr >>= 26;
				if (instr == OP_J) {
					va -= sizeof(int);
					goto mips3_eret;
				}
			}
		}
	} while (instr != MIPS_JR_RA && instr != MIPS_JR_K0);
	if ( get_netos_sym_tokens() && get_sym_by_addr((long) va + 8, NULL)) {
#if (_MIPS_SIM == MIPS_SIM_ABI64)
	if ( get_netos_sym_tokens() && get_sym_by_addr((long) va + 12, NULL)) 
#endif
		goto loop_again;
	}

	/* skip back over branch & delay slot */
	va += sizeof(int);
mips3_eret:
	va += sizeof(int);

	/* skip over nulls which might separate .o files */
	while ((instr = kdbpeek(va)) == 0)
		va += sizeof(int);
	subr = va;

	/* scan forwards to find stack size and any saved registers */
	stksize = 0;
	more = 3;
	mask = 0;
	foundframesize = 0;
	for (va = subr; more; va += sizeof(int),
			      more = (more == 3) ? 3 : more - 1) {
		/* stop if hit our current position */
		if (va >= pc)
			break;
		instr = kdbpeek(va);
		i.word = instr;
		switch (i.JType.op) {
		case OP_SPECIAL:
			switch (i.RType.func) {
			case OP_JR:
			case OP_JALR:
				more = 2; /* stop after next instruction */
				break;

			case OP_SYSCALL:
			case OP_BREAK:
				more = 1; /* stop now */
			};
			break;

		case OP_BCOND:
		case OP_J:
		case OP_JAL:
		case OP_BEQ:
		case OP_BNE:
		case OP_BLEZ:
		case OP_BGTZ:
			more = 2; /* stop after next instruction */
			break;

		case OP_COP0:
		case OP_COP1:
		case OP_COP2:
		case OP_COP3:
			switch (i.RType.rs) {
			case OP_BCx:
			case OP_BCy:
				more = 2; /* stop after next instruction */
			};
			break;
		case OP_SD:
			/* look for saved registers on the stack */
			if (i.IType.rs != 29)
				break;
			/* only restore the first one */
			if (mask & (1 << i.IType.rt))
				break;
			mask |= (1 << i.IType.rt);
			switch (i.IType.rt) {
			case 4: /* a0 */
				a0 = kdbpeek64(sp + (short)i.IType.imm);
				break;

			case 5: /* a1 */
				a1 = kdbpeek64(sp + (short)i.IType.imm);
				break;

			case 6: /* a2 */
				a2 = kdbpeek64(sp + (short)i.IType.imm);
				break;

			case 7: /* a3 */
				a3 = kdbpeek64(sp + (short)i.IType.imm);
				break;

			case 30: /* fp */
				fp = kdbpeek64(sp + (short)i.IType.imm);
				break;

			case 31: /* ra */
				ra = kdbpeek64(sp + (short)i.IType.imm);
			}
			break;

		case OP_SW:
			/* look for saved registers on the stack */
			if (i.IType.rs != 29)
				break;
			/* only restore the first one */
			if (mask & (1 << i.IType.rt))
				break;
			mask |= (1 << i.IType.rt);
			switch (i.IType.rt) {
			case 4: /* a0 */
				a0 = kdbpeek(sp + (short)i.IType.imm);
				break;

			case 5: /* a1 */
				a1 = kdbpeek(sp + (short)i.IType.imm);
				break;

			case 6: /* a2 */
				a2 = kdbpeek(sp + (short)i.IType.imm);
				break;

			case 7: /* a3 */
				a3 = kdbpeek(sp + (short)i.IType.imm);
				break;

			case 30: /* fp */
				fp = kdbpeek(sp + (short)i.IType.imm);
				break;

			case 31: /* ra */
				ra = kdbpeek(sp + (short)i.IType.imm);
			}
			break;

		case OP_ADDI:
		case OP_ADDIU:
		case OP_DADDIU:
			/* look for stack pointer adjustment */
			if (i.IType.rs != 29 || i.IType.rt != 29)
				break;
			/* don't count pops for mcount */
			if (!foundframesize) {
				stksize = - ((short)i.IType.imm);
				foundframesize = 1;
			}
		}
	}
done:
	(*printfn)("%x: %s+%x (%x,%x,%x,%x) %x %d\n",
		pc, fn_name(subr), pc - subr, a0, a1, a2, a3, ra, stksize);

	if (ra) {
		if (pc == ra && stksize == 0)
			(*printfn)("stacktrace: loop!\n");
		else {
			pc = ra;
			sp += stksize;
			ra = 0;
			goto loop;
		}
	} else {
finish:
		return;
	}
}

struct stack_unwind_info{
	uint64_t a0;
	uint64_t a1;
	uint64_t a2;
	uint64_t a3;
	uint64_t cp0_epc;
	uint64_t sp;
	uint64_t fp;
	uint64_t ra;
};

void unwind_support_func(struct stack_unwind_info *regs)
{
	panic_flags = acquire_print_lock();
	printf("Call Trace:\n");
	printf("PC:      func+offset (a0,a1,a2,a3) ra stacksize\n");
	stacktrace_subr(regs->a0, regs->a1, regs->a2,
			regs->a3, regs->cp0_epc, regs->sp, 
			regs->fp, regs->ra, 
			(void (*)(const char*, ...))printf);
	release_print_lock(panic_flags);
}
