/***********************************************************************
Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.
*****************************#NETL_2#********************************/
#ifndef _NLM_CRYPTO_H_
#define _NLM_CRYPTO_H_


#define NLM_ENCRYPT 1
#define NLM_DECRYPT 0

/* Explicitly assigned numbers for readability */
typedef enum cipher_algo
{
  NLM_CIPHER_BYPASS = 0,
  NLM_CIPHER_DES = 1,
  NLM_CIPHER_3DES = 2,
  NLM_CIPHER_AES128 = 3,
  NLM_CIPHER_AES192 = 4,
  NLM_CIPHER_AES256 = 5,
  NLM_CIPHER_ARC4 = 6,
  NLM_CIPHER_KASUMI_F8 = 7,
  NLM_CIPHER_SNOW3G_F8 = 8,
  NLM_CIPHER_CAMELLIA128 = 9,
  NLM_CIPHER_CAMELLIA192 = 0xA,
  NLM_CIPHER_CAMELLIA256 = 0xB,
  NLM_CIPHER_MAX = 0xC,
} cipher_algo_t;

typedef enum cipher_mode
{
  NLM_CIPHER_MODE_ECB = 0,
  NLM_CIPHER_MODE_CBC = 1,
  NLM_CIPHER_MODE_CFB = 2,
  NLM_CIPHER_MODE_OFB = 3,
  NLM_CIPHER_MODE_CTR = 4,
  NLM_CIPHER_MODE_AES_F8 = 5,
  NLM_CIPHER_MODE_GCM = 6,
  NLM_CIPHER_MODE_CCM = 7,
  NLM_CIPHER_MODE_UNDEFINED1 = 8,
  NLM_CIPHER_MODE_UNDEFINED2 = 9,
  NLM_CIPHER_MODE_LRW = 0xA,
  NLM_CIPHER_MODE_XTS = 0xB,
  NLM_CIPHER_MODE_MAX = 0xC,
} cipher_mode_t;;

typedef enum auth_algo
{
  NLM_AUTH_BYPASS = 0,
  NLM_AUTH_MD5 = 1,
  NLM_AUTH_SHA = 2,
  NLM_AUTH_UNDEFINED = 3,
  NLM_AUTH_AES128 = 4,
  NLM_AUTH_AES192 = 5,
  NLM_AUTH_AES256 = 6,
  NLM_AUTH_KASUMI_F9 = 7,
  NLM_AUTH_SNOW3G_F9 = 8,
  NLM_AUTH_CAMELLIA128 = 9,
  NLM_AUTH_CAMELLIA192 = 0xA,
  NLM_AUTH_CAMELLIA256 = 0xB,
  NLM_AUTH_GHASH = 0xC,
  NLM_AUTH_MAX = 0xD
} auth_algo_t;

typedef enum auth_mode
{
  NLM_AUTH_MODE_SHA1 = 0,	/* Only SHA */
  NLM_AUTH_MODE_SHA224 = 1,	/* Only SHA */
  NLM_AUTH_MODE_SHA256 = 2,	/* Only SHA */
  NLM_AUTH_MODE_SHA384 = 3,	/* Only SHA */
  NLM_AUTH_MODE_SHA512 = 4,	/* Only SHA */
  NLM_AUTH_MODE_CMAC = 5,	/* AES and Camellia */
  NLM_AUTH_MODE_XCBC = 6,	/* AES and Camellia */
  NLM_AUTH_MODE_CBC_MAC = 7,	/* AES and Camellia */
  NLM_AUTH_MODE_CCM = 8,	/* AES */
  NLM_AUTH_MODE_GCM = 9,	/* AES */
  NLM_AUTH_MODE_MAX = 0xA,
} auth_mode_t;

void nlm_crypto_print_error_msg (int status);
int nlm_crypto_cipher_alg_has_mode (unsigned int cipher_alg);
int nlm_crypto_auth_alg_has_mode (unsigned int auth_alg);

#endif
