#include <reent.h>
#include <errno.h>
#include <smp.h>
#include <uart.h>
#include <spinlock.h>
#include <devices.h>

#include "xlr_boot_lib.h"

static int stdio_open(struct _reent *ptr, const char *file, 
		int flags, int mode) 
{
	ptr->_errno = EIO;
	return -1;
}

/* read -- read bytes from serial port. 
 */
static _ssize_t stdio_read(struct _reent *ptr, int fd, 
		char *buf, size_t nbytes)
{
	int i = 0;
	unsigned int flags = 0;

	flags = acquire_print_lock();
	for (i = 0; i < nbytes; i++) {
		*(buf + i) = inbyte();
		if ((*(buf + i) == '\n') || (*(buf + i) == '\r')) {
			i++;
			break;
		}
	}
	release_print_lock(flags);

	return i;
}

/* write -- write bytes to the serial port. Ignore
 * fd, since stdout and stderr are the same.
 */
static _ssize_t stdio_write(struct _reent *ptr, int fd, 
		const char *buf, size_t nbytes)
{
	int i;
	unsigned int flags = 0;

	flags = acquire_print_lock();

	for (i = 0; i < nbytes; i++) {
		if (*(buf + i) == '\n') {
			outbyte ('\r');
		}
		outbyte (*(buf + i));
	}

	release_print_lock(flags);

	return (nbytes);
}

static int stdio_close(struct _reent *ptr, int fd)
{
	return 0;
}

static void stdio_init(void)
{
	/* do nothing */
}

struct device_ops dev_stdio_in = {
	.name 		= "stdin",
	.open_r 	= stdio_open,
	.read_r 	= stdio_read,
	.write_r 	= stdio_write,
	.close_r 	= stdio_close,
	.init 		= stdio_init
};

struct device_ops dev_stdio_out = {
	.name 		= "stdout",
	.open_r 	= stdio_open,
	.read_r 	= stdio_read,
	.write_r 	= stdio_write,
	.close_r 	= stdio_close,
	.init 		= stdio_init
};

struct device_ops dev_stdio_err = {
	.name 		= "stderr",
	.open_r 	= stdio_open,
	.read_r 	= stdio_read,
	.write_r 	= stdio_write,
	.close_r 	= stdio_close,
	.init 		= stdio_init
};
