/*************************************************************************
 Copyright 2003-2010 Netlogic Microsystems ("Netlogic"). All rights
 reserved.
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
*******************************#NETL_2#**********************************/

package require Tk

if {$tcl_platform(platform) == "windows"} {
  package require registry
}

wm title . "NLM GDB launcher"

frame .hostFrame
label .hostFrame.hostAddrLabel -text "Remote Host:"
entry .hostFrame.hostAddr 
pack .hostFrame.hostAddrLabel -padx 4 -pady 2 -side left
pack .hostFrame.hostAddr -padx 4 -pady 2 -side right

frame .portFrame
label .portFrame.ipPortLabel -text "Base Port :" 
entry .portFrame.ipPort
pack .portFrame.ipPortLabel -padx 4 -pady 2 -side left
pack .portFrame.ipPort -padx 4 -pady 2 -side right

#
# load defaults
#
if {$tcl_platform(platform) == "windows"} {
    if [catch {registry get  HKEY_CURRENT_USER\\Software\\NLM LastHost} res] {
       # first install no value yet
    } else {
       .hostFrame.hostAddr insert end $res
    }
    if [catch {registry get  HKEY_CURRENT_USER\\Software\\NLM LastPort} res] {
       # first install no value yet
       .portFrame.ipPort insert end 5000
    } else {
       .portFrame.ipPort insert end $res
    }
} else {
   if [catch {open $env(HOME)/.nlmrc r} fp] {
       # no file, which is ok - or some other error
   } else {
       gets $fp line
       if {[scan buf "Host:%s Port:%d" lasthost lastport] == 2} {
          entry .hostFrame.hostAddr insert lasthost
          entry .portFrame.ipPort insert $lastport
       }    
       close $fp
   }
}

frame .threadFrame
label .threadFrame.threadLabel -text "Thread :" 
tk_optionMenu .threadFrame.threadList threadval  \
    "Core 0 Thread 0"  "Core 0 Thread 1" "Core 0 Thread 2"  "Core 0 Thread 3" \
    "Core 1 Thread 0"  "Core 1 Thread 1" "Core 1 Thread 2"  "Core 1 Thread 3" \
    "Core 2 Thread 0"  "Core 2 Thread 1" "Core 2 Thread 2"  "Core 2 Thread 3" \
    "Core 3 Thread 0"  "Core 3 Thread 1" "Core 3 Thread 2"  "Core 3 Thread 3" \
    "Core 4 Thread 0"  "Core 4 Thread 1" "Core 4 Thread 2"  "Core 4 Thread 3" \
    "Core 5 Thread 0"  "Core 5 Thread 1" "Core 5 Thread 2"  "Core 5 Thread 3" \
    "Core 6 Thread 0"  "Core 6 Thread 1" "Core 6 Thread 2"  "Core 6 Thread 3" \
    "Core 7 Thread 0"  "Core 7 Thread 1" "Core 7 Thread 2"  "Core 7 Thread 3" 
pack .threadFrame.threadLabel -padx 4 -pady 2 -side left
pack .threadFrame.threadList -padx 4 -pady 2 -side right

frame .buttons
button .buttons.launch -text "Launch" -command "launchInsight"
button .buttons.close -text Close -command "destroy ."
pack .buttons.close .buttons.launch -side left -expand 1

pack .hostFrame .portFrame .threadFrame -side top -fill x
pack .buttons -side bottom -fill x -pady 2m

#
# put the cursor in the host window
#
focus .hostFrame.hostAddr

proc launchInsight {} {
    global threadval env tcl_platform
    set host [.hostFrame.hostAddr get]
    set port [.portFrame.ipPort get]

    scan $threadval "Core %d Thread %d" core thread
    if {[string compare $host ""] == 0} {
        tk_messageBox  -title "Bad Host value"  -type ok \
             -message "Please specify the host name or IP address."
	focus .hostFrame.hostAddr
    } elseif [catch {set conport [expr $port + $core*4 + $thread]} cause] {
        tk_messageBox -title "Bad port value" -message -type ok \
            -message "Invalid value for port number, should be numeric!" 
	focus .portFrame.ipPort
    } else {
        #
        # save value for later use
        #
        if {$tcl_platform(platform) == "windows"} {
           catch {registry set  HKEY_CURRENT_USER\\Software\\NLM LastHost $host} res
           catch {registry set  HKEY_CURRENT_USER\\Software\\NLM LastPort $port} res
       } else {
           if [catch {open $env(HOME)/.nlmrc w} fp] {
              # ignore errors - no big loss
           } else {
              puts  $fp line "Host:$host Port:$port"
              close $fp
           }
       }

       set env(GDBTK_PROFILE) "c${core}t${thread}"
       set env(GDBTK_TARGET_SPEC) "$host:$conport"
       exec mips64-insight &
    }
}
