#ifndef _TEST_DATA_H_
#define _TEST_DATA_H_
#define FRAG_TXT_SIZE 300
#define TXT_SIZE 1000
#if 0
#define NR_TESTS_PER_ALGO 6
#define NR_FRAG_TESTS_PER_ALGO 2
#endif


#define NLM_AES_BLOCK_LEN 16
#define NLM_AES_IV_LEN NLM_AES_BLOCK_LEN
#define NLM_AES128_KEY_LEN 16
#define NLM_AES192_KEY_LEN 24
#define NLM_AES256_KEY_LEN 32
#define NLM_SHA1_BLOCK_LEN 64 
#define NLM_SHA256_BLOCK_LEN 64
#define NLM_SHA384_BLOCK_LEN 128
#define NLM_SHA512_BLOCK_LEN 128
#define NLM_SHA1_KEY_LEN NLM_SHA1_BLOCK_LEN
#define NLM_SHA256_KEY_LEN NLM_SHA256_BLOCK_LEN
#define NLM_SHA384_KEY_LEN NLM_SHA384_BLOCK_LEN
#define NLM_SHA512_KEY_LEN NLM_SHA512_BLOCK_LEN

#define NLM_MD5_BLOCK_LEN 64
#define NLM_MD5_KEY_LEN NLM_MD5_BLOCK_LEN

#define NR_TESTS_AES128_CBC 4 /*rfc 3602 */
#define NR_FRAG_TESTS_AES128_CBC 2 /* rfc 3602 */

#define NR_TESTS_MD5 0 /* rfc 2104 */
#define NR_TESTS_MD5_HMAC 3 /* rfc 2104 */

/*http://csrc.nist.gov/publications/fips/fips180-2/fips180-2withchangenotice.pdf*/
/* Non hmac test cases are taken from above fips180-2 spec */
/*rfc 4868 */
/* hmac test cases are taken from above rfc */
#define NR_TESTS_SHA1 1 /* sha1 with hmac taken from http://grouper.ieee.org/groups/1619/email/msg01966.html */
#define NR_TESTS_SHA1_HMAC 1 /* sha1 with hmac taken from http://grouper.ieee.org/groups/1619/email/msg01966.html */
/* file saved as E:\Sandip\Technical External Docs\IPSec Docs\aes-cbc-hmac-test-vectors */
#define NR_TESTS_SHA256 1 
#define NR_TESTS_SHA256_HMAC 4 
#define NR_TESTS_SHA384 1 
#define NR_TESTS_SHA384_HMAC 4 
#define NR_TESTS_SHA512 1 
#define NR_TESTS_SHA512_HMAC 3 

#define NR_TESTS_AES128_CBC_SHA256 4
#define NR_TESTS_AES256_CBC_SHA1 1
#define NR_TESTS_AES128_CBC_SHA1 2

struct test_data{
	int enc;
	int len;
	int auth_len;
	int aad_len;
	int auth_key_len;
	int digest_len;
	unsigned char cipher_key[128];
	unsigned char auth_key[128];
	unsigned char iv[128];
	unsigned char in[TXT_SIZE * 2];
	unsigned char out[TXT_SIZE * 2];
	unsigned char digest[128];
	unsigned char aad[128];
	unsigned char auth_postdata[128];
};

struct test_params{
	int cipher_alg;
	int cipher_mode;
	int auth_alg;
	int auth_mode;
	int hmac;
	int cipher_iv_len;
	int cipher_key_len;
	int max_auth_key_len;
	int designer_desc_valid;
	uint64_t designer_desc[4];
	char comment[120];
};

struct frag_test_data{
	int enc;
	int digest_len;
	int aad_len;
	int auth_len;
	int auth_key_len;
	int hmac;
	int len[11];
	unsigned char cipher_key[64];
	unsigned char auth_key[64];
	unsigned char iv[256];

	unsigned char in[11][FRAG_TXT_SIZE];

	unsigned char out[11][FRAG_TXT_SIZE];
	unsigned char digest[64];
	unsigned char aad[64];
	unsigned char auth_postdata[64];
	int nr_frags;
};

struct cipher_auth_test_config{
	struct test_params *params;
	struct test_data *tests;
	int nr_tests;
};

#endif /* _TEST_DATA_H_ */
