#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <inttypes.h>
#include <string.h>
#include <sys/nlm_dev.h>
#include "dtr_test_data.h"

//#define DEBUG 1

uint32_t *segment_length;
const uint32_t segment_lengths [NUM_SEGMENTS_PER_DEVICE] = {2, 3, 5};
char*** data;
char *recovp[NUM_SEGMENTS_PER_DEVICE];
uint32_t row, column;

void print_str_in_hex (const char *out_str, const uint32_t len)
{
#ifdef DEBUG
  int i;
  for (i=0; i < len; i++)
    printf ("%x ", out_str[i]);
  printf ("\n");
#endif
}

void test_dtr (void)
{
  uint32_t i, ret = 0, *arr_str_size = NULL, size, in_size, out_size;
  char *in_str = NULL, *out_str = NULL;
  char **arr_in_str = NULL, **arr_out_str = NULL;
  struct message_type_0_inputs *msg_0_input =  NULL;
  
  /* First run to test MSG_TYPE_TRANFER: plain transfer */
  size = sizeof (msg_0_inputs)/sizeof (struct message_type_0_inputs);
  for (i = 0; i < size; i++)
  {
    msg_0_input = &msg_0_inputs[i];
    in_size = strlen (msg_0_input->in);
    out_size = strlen (msg_0_input->out);

    switch (i % 3)
    {
      case 0: /* no overlap */
        in_str = malloc (in_size);
        out_str = malloc (out_size);
        memcpy (in_str, msg_0_input->in, in_size);
        break;
      case 1: /* overlapping */
        in_str = malloc (in_size + in_size/2);
        out_str = in_str + in_size/2;
        memset (in_str, 0, (in_size + in_size/2));
        memcpy (in_str, msg_0_input->in, in_size);
        break;
      case 2: /* no overlap */
        out_str = malloc (2 * in_size);
        in_str = out_str + in_size;
        memset (out_str, 0, (2 * in_size));
        memcpy (in_str, msg_0_input->in, in_size);
        break;
    }

    if (msg_0_input->perform_csum == 0 && msg_0_input->perform_crc == 0)
      ret = nlm_dtr_transfer (in_str, out_size, out_str);

    print_str_in_hex (out_str, out_size);
    print_str_in_hex (msg_0_input->out, out_size);

    if (ret == 0)
    {
      if (memcmp (out_str, msg_0_input->out, out_size) == 0)
        printf ("####### DTR_APP: Outputs matched ############\n");
      else
      {
        printf ("####### DTR_APP: Outputs -not-matched #######\n");
        exit (1);
      }
    }
    switch (i % 3)
    {
      case 0: 
        free (in_str);
        free (out_str);
        break;
      case 1: 
        free (in_str);
        break;
      case 2: 
        free (out_str);
        break;
    }
  }

  /* Second run to test MSG_TYPE_REGULAR_LIST: transfer from multiple inputs to resp outputs of resp lengths */
  size = sizeof (msg_0_inputs)/sizeof (struct message_type_0_inputs);
  arr_in_str = malloc (sizeof (char *) * size);
  arr_str_size   = malloc (sizeof (uint32_t) * size);
  arr_out_str = malloc (sizeof (char *) * size);

  for (i = 0; i < size; i++)
  {
    msg_0_input = &msg_0_inputs[i];
    in_size = strlen (msg_0_input->in);
    out_size = strlen (msg_0_input->out);

    switch (i % 3)
    {
      case 0: /* no overlap */
        arr_in_str[i] = malloc (in_size);
        arr_out_str[i] = malloc (out_size);
        break;
      case 1: /* overlapping */
        arr_in_str[i] = malloc (in_size + in_size/2);
        arr_out_str[i] = arr_in_str[i] + in_size/2;
        memset (arr_in_str[i], 0, (in_size + in_size/2));
        break;
      case 2: /* no overlap */
        arr_out_str[i] = malloc (2 * in_size);
        arr_in_str[i] = arr_out_str[i] + in_size;
        memset (arr_out_str[i], 0, (2 * in_size));
        break;
    }
    memcpy (arr_in_str[i], msg_0_input->in, in_size);
    arr_str_size[i] = in_size;
  }

  ret = nlm_dtr_multi_transfer ((const char **)arr_in_str, arr_str_size, arr_out_str, size);

  if (ret == 0)
  {
    for (i = 0; i < size; i++)
    {
      out_str = arr_out_str[i];
      in_str = arr_in_str[i];
      out_size = arr_str_size[i];
      msg_0_input = &msg_0_inputs[i];
      
      print_str_in_hex (out_str, out_size);
      print_str_in_hex (msg_0_input->out, out_size);

      if (memcmp (out_str, msg_0_input->out, out_size) == 0)
        printf ("####### DTR_APP: Outputs matched ############\n");
      else
        printf ("####### DTR_APP: Outputs -not-matched #######\n");
      switch (i % 3)
      {
        case 0: 
          free (in_str);
          free (out_str);
          break;
        case 1: 
          free (in_str);
          break;
        case 2: 
          free (out_str);
          break;
      }
    }
  }
  else
  {
    printf ("ERROR: returned from driver \n");
  }
}

static void makedata (const uint32_t raid_type)
{
  uint32_t i, j, k;

  if (raid_type == RAID_TYPE_5)
  {
    row = NUM_DEVICES + NUM_RAID_DEVICES;
    column = NUM_SEGMENTS_PER_DEVICE;
    segment_length = (uint32_t *) segment_lengths;
  }
  else
  {
    row = sizeof (rai6_test_data [0])/sizeof (rai6_test_data [0][0]);
    column = sizeof (rai6_test_data)/sizeof (rai6_test_data [0]);
    column = column/SEGMENT_LENGTH_RAID_6;
    segment_length = malloc (sizeof (uint32_t) * column);
    for (i = 0; i < column; i++)
      segment_length [i] = SEGMENT_LENGTH_RAID_6;
  }

  // Fill the data for (NUM_DEVICES)
  /* Memory Allocation and filling of data */
  data = malloc (sizeof (char *) * row);
  for (i = 0; i < row; i++) 
  {
    data [i] = malloc (sizeof (char *) * column);
    for (j = 0; j < column; j++)
    {
      if (raid_type == RAID_TYPE_5)
      {
        data[i][j] = malloc (segment_length[j]);
        if ((i+1 == row-1) || (i == row-1))
          memset(data [i][j], 0xba, segment_length[j]);
        else
          for (k = 0; k < segment_length[j]; k++)
            data[i][j][k] = rand();
      }
      else
      {
        data[i][j] = malloc (SEGMENT_LENGTH_RAID_6);
        if ((i+1 == row-1) || (i == row-1))
          memset(data [i][j], 0xba, SEGMENT_LENGTH_RAID_6);
        else
          for (k = 0; k < SEGMENT_LENGTH_RAID_6; k++)
            data[i][j][k] = rai6_test_data [j * SEGMENT_LENGTH_RAID_6 + k] [i];
      }
    }
  }

  if (raid_type == RAID_TYPE_5)
  {
    for (j = 0; j < NUM_SEGMENTS_PER_DEVICE; j++)
    {
      recovp[j] = malloc (segment_length[j]);
      memset (recovp[j], 0, segment_length[j]);
    }

    for (i = 0; i < NUM_SEGMENTS_PER_DEVICE; i++)
      for (j = 0; j < segment_length[i]; j++)
        for (k = 0; k < NUM_DEVICES; k++)
        {
          recovp[i][j] ^= data[k][i][j];
        }
  }
}

void freedata (uint32_t raid_type)
{
  int i, j;

  /* Memory Allocation and filling of data */
  for (i = 0; i < row; i++) 
  {
    for (j = 0; j < column; j++)
      free (data[i][j]);
    free (data [i]);
  }
  free (data);

  if (raid_type == RAID_TYPE_5)
    for (j = 0; j < column; j++)
      free (recovp[j]);
  else
    free (segment_length);
}

void printdata (uint32_t raid_type)
{
  uint32_t i, j, k, seg_length;

  for (i = 0; i < column; i++)
  {
    for (k = 0; k < row; k++)
    {
      printf ("%d,%d: ", k, i);
      if (raid_type == RAID_TYPE_5)
        seg_length = segment_length[i];
      else
        seg_length = SEGMENT_LENGTH_RAID_6;

      for (j = 0; j < seg_length; j++)
        printf ("%02X", data [k][i][j] & 0xff);
      printf ("\t");
    }
    printf ("\n");
  }

  if (raid_type == RAID_TYPE_5)
  {
    for (i = 0; i < column; i++)
    {
      seg_length = segment_length[i];
      printf ("P,%d: ", i);
      for (j = 0; j < seg_length; j++)
        printf ("%02X", recovp [i][j] & 0xff);
      printf ("\n");
    }
  }
}

void call_driver (uint32_t write_operation, uint32_t raid_type, uint32_t p_device_id, uint32_t q_device_id)
{
  uint32_t i, j;
  uint32_t ret;

  ret = nlm_dtr_raid (NUM_DEVICES + raid_type,
                      column,
                      (char ***) data,
                      segment_length,
                      write_operation,
                      raid_type,
                      p_device_id,
                      q_device_id);
  if (ret)
  {
    printf ("####### DTR_APP_RAID: ERROR in Operation ############\n");
    exit (1);
  }
  if (raid_type == RAID_TYPE_5)
  {
    for (i = 0; i < column && write_operation ; i++)
      for (j = 0; j < segment_length[i]; j++)
      {
        if (recovp[i][j] != data[p_device_id - 1][i][j])
        {
          printf ("####### DTR_APP_RAID: Outputs not matched ############\n");
          return;
        }
      }
    if (write_operation)
      printf ("####### DTR_APP_RAID: Outputs matched ############\n");
    else
      printf ("####### DTR_APP_RAID: No Read & Check Errors ###########\n");
  }

  if (raid_type == RAID_TYPE_6)
  {
    for (i = 0; i < column && write_operation ; i++)
      for (j = 0; j < segment_length[i]; j++)
      {
        if ( rai6_test_data [i * SEGMENT_LENGTH_RAID_6 + j][q_device_id - 1]!= data[q_device_id - 1][i][j]
             && rai6_test_data [i * SEGMENT_LENGTH_RAID_6 + j][p_device_id - 1]!= data[p_device_id - 1][i][j])
        {
          printf ("####### DTR_APP_RAID: Outputs not matched ############\n");
          return;
        }
      }
    if (write_operation)
      printf ("####### DTR_APP_RAID: Outputs matched ############\n");
    else
      printf ("####### DTR_APP_RAID: No Read & Check Errors ###########\n");
  }
}

void test_raid ()
{
  uint32_t raid_type, p_device_id, q_device_id, write_operation;

  raid_type = RAID_TYPE_5;
  p_device_id = NUM_DEVICES + 1;
  q_device_id = 0xff;
  write_operation = WRITE_OPERATION;

  if (raid_type == RAID_TYPE_5 && write_operation == WRITE_OPERATION)
  {
    makedata (raid_type);
    printdata(raid_type);
  }

  //Write operation for RAID 5
  call_driver (write_operation, raid_type, p_device_id, q_device_id);
  //Read & check operation for RAID 5
  call_driver (!write_operation, raid_type, p_device_id, q_device_id);
  //Done with RAID 5, clear data
  freedata (raid_type);

  raid_type = RAID_TYPE_6;
  p_device_id = NUM_DEVICES + 1;
  q_device_id = NUM_DEVICES + 2;
  write_operation = WRITE_OPERATION;

  if (raid_type == RAID_TYPE_6 && write_operation == WRITE_OPERATION)
  {
    makedata (raid_type);
    printdata(raid_type);
  }

  //Write operation for RAID 6
  call_driver (write_operation, raid_type, p_device_id, q_device_id);
  //Read & check operation for RAID 6
  call_driver (!write_operation, raid_type, p_device_id, q_device_id);
  //Done with RAID 6, clear data
  freedata (raid_type);

  return;
}

int main(int argc, char *argv[])
{
  printf ("----------- DTR Test Started---------\n");
  test_dtr ();
  test_raid ();
  printf ("Done. exiting... %s\n",__FUNCTION__);
  return 0;
}

