/***********************************************************************
  Copyright 2003-2010 Netlogic Microsystems (“Netlogic”). All rights
  reserved.
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.
*****************************#NETL_2#********************************/
#include <stdio.h>
#include <sys/mman.h>
#include <sched.h>
#include <signal.h>
#include <string.h>
#include <stdint.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <mempool.h>
#include <sys/nlm_dev.h>

//#define HW_EMULATOR    1
#define SINGLE_THREADED 1

#ifdef HW_EMULATOR
/* TODO: remove HAL code */
#include "nlm_hal_fmn.h"
#endif

/* HACK! To be removed */
#define XLP_NET_TX_VC_BASE 476

#define MAX_PKTS       50
#define APP_PORT_BASE  0
#define NUM_APP_PORTS  3

#define ICMP_PKT_SIZE  74
#define PKT_ALIGN_LEN  64
#define MAX_CPUS       32
#define MAX_GMAC_PORT  18

#define CLONE_BUDDY_FLGS (CLONE_VM | CLONE_FS | CLONE_FILES | CLONE_SIGHAND | SIGCHLD )

//#define DBG        1
#ifdef DBG
#define log_dbg     printf
#define log_pkt     printf
#else
#define log_dbg(...)
#define log_pkt(...)
//    #define log_err(...)
#endif
#define log_err
#define log_info   printf

int if_fd[MAX_GMAC_PORT];

static __inline__ void poe_print(unsigned long long msg0) {
	log_dbg("POE nextfid  = %llu (0x%llx)\n"
		"    nextdist = %llu (0x%llx)\n"
		"    nextdest = %llu (0x%llx)\n"
		"    msgaddr  = 0x%llx\n"
		"    fid      = %llu (0x%llx)\n",
		(msg0 >> 48) & 0xffff, (msg0 >> 48) & 0xffff,
		(msg0 >> 44) & 0xf, (msg0 >> 44) & 0xf,
		(msg0 >> 32) & 0xfff, (msg0 >> 32) & 0xfff,
		(msg0 >> 16) & 0xffff,
		(msg0) & 0xffff, (msg0) & 0xffff);
}

static __inline__ void rx_print(unsigned long long msg0) {
	log_dbg("RX  context = %llu\n"
		"    length  = %llu (0x%llx)\n"
		"    address = 0x%010llx\n"
		"    unclass = %llu\n"
		"    err     = %llu\n"
		"    IPcksm  = %llu\n"
		"    TCPcksm = %llu\n"
		"    prepad  = %llu\n"
		"    p2p     = %llu\n",
		(msg0 >> 54) & 0x3ff,
		(msg0 >> 40) & 0x3fff, (msg0 >> 40) & 0x3fff,
		(msg0) & 0xffffffffc0ULL,
		(msg0 >> 5) & 0x1,
		(msg0 >> 4) & 0x1,
		(msg0 >> 3) & 0x1,
		(msg0 >> 2) & 0x1,
		(msg0 >> 1) & 0x1,
		(msg0) & 0x1);
}

static __inline__ void buf_print(unsigned char *buf, unsigned int len) {
	int i;
	for (i = 0; i < len; ++i) {
		log_dbg(" %02x", buf[i]);
		if (i % 8 == 7) log_dbg(" ");
		if (i % 32 == 31) log_dbg("\n");
	}
	log_dbg("\n");
}

int init_all_gmac(void)
{
	int inf = 0;
	char nae_str[50];
	int end_inf = APP_PORT_BASE + NUM_APP_PORTS;

	for( inf = APP_PORT_BASE; inf < end_inf; inf ++) {

		sprintf(nae_str, "/sysconfig/node0/nae/if%d",inf);
		if_fd[inf] = open(nae_str, O_RDWR, S_IRUSR | S_IWUSR);
		if (if_fd[inf] < 0) {
			printf("HyperExec_ERROR: Failed to open gmac port%d (%s) !! \n", inf, nae_str);
			return -1;
		}
	}

	return 0;
}

int process_xaui(void *args)
{
	uint64_t laddr;
	unsigned int cntl = 0, flags = 0, context = 0;;
	int size, rxc = 0, txc = 0;
	int ec = 0;

	while (1) {

		/* RX */
		size = nae_recv(&laddr, &cntl);

		/* no message received */
		if (size == 0) continue;

		context = cntl & 0x3ff;

		if (size < 0) {
			ec++;
			/* printf("-%d size %d\n",ec, size); */
			flags = -size;

			if (flags & (NLM_NAE_RX_ERR | NLM_NAE_RX_TXC)) {
				nae_send(context, laddr, 0, NLM_NAE_TX_FREE_IN);
				continue;
			}

			if (flags & NLM_NAE_RX_TXERR) {
				/* printf("Transmit underrun/collision!\n"); */
				continue;
			}

			printf("Unsupported rx flags (%d)!\n", flags);
			exit(-1);
		}

		rxc++;
//		printf("%d: Rx%d laddr 0x%llx size %d flags 0x%x\n", cpu, rxc, (unsigned long long) laddr, size, flags);

#if 1
		/* TX */
		if (nae_send(context, laddr, size, NLM_NAE_HW_BUF_MGMT) == 0) { 
			txc++;
		/* printf("%d\n",txc); */
		}
		
#else
	txc = 0; /* for compilation */
#endif
	}
	printf("XLP XAUI Loopback app completed\n");
	exit(0);
}

static void xaui_lb_app()
{
	long instance = 1;
	char *child_stack;
	int  childid = 0, size = 32 * 1024;

	for(;;) {
		/* parent will clone the child till no cpus are left
		 */
		if((child_stack = malloc(size)) == NULL) {
			printf("[%s] Mem alloc failed \n", __FUNCTION__);
			break;
		}

		/* stack grows from top to bottom
		 */
		child_stack = child_stack + size - 64;

		childid = clone(process_xaui, child_stack,
				CLONE_BUDDY_FLGS, NULL);
		if (childid < 0) {
			printf("[Parent] : Failed to clone!\n");
			break;
		}

		instance++;
		printf("[Parent] : Child process (%d) created\n", childid);
	}

	/* Also run the app on the master cpu */
	process_xaui(0);
}

int main(int argc, char *argv[])
{
	int cpu = getpid();

	if (cpu != 0) {
		log_dbg("This application is only configured to run on cpu0 %d.\n", cpu);
		return -1;
	}
	log_info("[cpu%d] Running nae template app...\n", cpu);

#ifdef HW_EMULATOR
	nlm_hal_fmn_init(0xf);
#endif

	if (init_all_gmac() < 0)
		return -1;

	xaui_lb_app();

	return 0;
}
