#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <inttypes.h>
#include <sys/time.h>
#include <string.h>
#include <time.h>
#include <sched.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <semaphore.h>
#include <signal.h>
#include <sys/time.h>
#include <mempool.h>

#define MEM_TYPE 				uint64_t
#define MEM_PATTERN 			(MEM_TYPE)0xdeadbeefdeadbeefULL
#define Mic_secs_Per_Second     1000000.0
#define CLONE_BUDDY_FLGS (CLONE_VM | CLONE_FS | CLONE_FILES | CLONE_SIGHAND | SIGCHLD )

extern int clock_gettime(clockid_t clock_id, struct timespec *ts);
static volatile int timer_test_done = 0;

static void mem_test_child(void)
{
	int fd;
	void *addr;

	fd = shm_open("/netosshm", O_CREAT | O_RDWR, 0600);
	if(fd < 0) {
		printf("shm_open failed \n");
		return;
	}
	addr = mmap(NULL, 100, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
	if(addr == MAP_FAILED) {
		printf("mmap failed on fd %d\n", fd);
		return;
	}

	printf("MMaped addr content : %s\n", (char *)addr);
}

static int child_function(void *arg)
{
	int i=0, msize = (1<<20); /* 1MB */
	int nblocks = msize / sizeof(MEM_TYPE);
	MEM_TYPE *mem = 0;

	printf("Child process running pid=%d arg=%lx\n", getpid(), (long)arg);
	
	/* malloc */
	printf("Child: allocating memory (size=%d bytes)...\n", msize);
	mem = malloc(msize);
	if (!mem) {
		printf("Child: unable to allocate memory (size=%d bytes)\n", msize);
		printf("pid = %d exiting (failure)\n", getpid());
		exit(EXIT_FAILURE);
	}

	printf("Child:Writing to memory...@ 0x%lx\n", (long)mem);
	for(i=0;i<nblocks;i++) {
		mem[i] = MEM_PATTERN;
	}

	printf("Child:Reading from memory @ 0x%lx\n", (long)mem);
	for(i=0;i<nblocks;i++) {
		if (mem[i] != MEM_PATTERN) {
			printf("Child:Memory read at byte[%x] failed!\n", i);
			printf("pid = %d exiting (failure)\n", getpid());
			exit(EXIT_FAILURE);
		}
	}
	free(mem);
	
	if((long)arg == 1)
		mem_test_child();

	printf("pid = %d exiting (success)\n", getpid());
	exit(EXIT_SUCCESS);
}

static void launch_children(void)
{
	long instance = 1;
	char *child_stack;
	int  childid = 0, size = 32 * 1024;

	for(;;) {
		/* parent will clone the child till no cpus are left
		 */
		if((child_stack = malloc(size)) == NULL) {
			printf("[%s] Mem alloc failed \n", __FUNCTION__);
			break;
		}

		/* stack grows from top to bottom 
		 */
		child_stack = child_stack + size - 64;
		
		childid = clone(child_function, child_stack, 
				CLONE_BUDDY_FLGS, (void *)instance);
		if (childid < 0) {
			printf("[Parent] : Failed to clone!\n");
			break;
		}
		
		instance++;
		printf("[Parent] : Child process (%d) created\n", childid);
	}
}

static void test_floating_point(void)
{
	double User_Time = 0.142057;
	double Microseconds, Dhrystones_Per_Second, Vax_Mips;
	int    Number_Of_Runs = 1000000;

	Microseconds = User_Time * Mic_secs_Per_Second / (double)Number_Of_Runs ;
	Dhrystones_Per_Second = (double)Number_Of_Runs / User_Time;
	Vax_Mips = Dhrystones_Per_Second / 1757.0;

	printf ("Microseconds for one run through Dhrystone: ");
	printf ("%7.1f \n", Microseconds);
	printf ("Dhrystones per Second:                      ");
	printf ("%10.1f \n", Dhrystones_Per_Second);
	printf ("VAX MIPS rating = %10.3f \n",Vax_Mips);
	printf ("\n");
}

static void test_time_functions(void)
{
	struct timespec sts, ets;
	int i = 0, num_loops = 100000000;

	printf("Testing clock_gettime for %d loops\n", num_loops);
	clock_gettime(CLOCK_REALTIME, &sts);

	for(i=0;i<num_loops;i++) {}

	clock_gettime(CLOCK_REALTIME, &ets);

	printf("\tstart: <sec=%ld nsec=%ld>, end:<sec=%ld nsec=%ld>\n",
		(unsigned long)sts.tv_sec, (unsigned long)sts.tv_nsec,
		(unsigned long)ets.tv_sec, (unsigned long)ets.tv_nsec);
}

static void test_gettimeofday(void)
{
	struct timeval stp, etp;
	int i = 0, num_loops = 100000000;
	
	printf("Testing gettimeofday for %d loops\n", num_loops);
	gettimeofday(&stp, NULL);

	for(i=0;i<num_loops;i++) {}

	gettimeofday(&etp, NULL);

	printf("\tstart: <sec=%ld usec=%ld>, end:<sec=%ld usec=%ld>\n",
		(unsigned long)stp.tv_sec, (unsigned long)stp.tv_usec,
		(unsigned long)etp.tv_sec, (unsigned long)etp.tv_usec);
}

void mem_pool_test()
{
	mpoolptr_t pool;
	int minelem = 10, elsize = 100, align =64, flag = 0;
	void *addr[100], *virt;
	unsigned long long phys;
	int i;

	pool = mpcreate("route", minelem, elsize, align, flag);
	if(pool == NULL) {
		printf("mpcreate failed \n");
		return;
	}
	for(i = 0; i < minelem; i++) {
		addr[i] = mpalloc(pool, flag);
		if(!addr[i])
			printf("mpalloc failed\n");
	}
	phys = mpgetphys(pool, addr[0]);
	virt = mpgetvirt(pool, phys);

	for(i = 0; i < minelem; i++) {
		mpfree(pool, addr[i]);
	}
	mpdestroy(pool);
	printf("Mem pool test passed\n");
}

static void mem_test_master(void)
{
	void *addr;
	int fd, rv;

	fd = shm_open("/netosshm", O_CREAT | O_RDWR, 0600);
	if(fd < 0) {
		printf("shm_open failed \n");
		return;
	}

	rv = ftruncate(fd, 100);
	if(rv < 0) {
		printf("ftrucate failed on fd %d\n", fd);
		return;
	}

	addr = mmap(NULL, 100, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
	if(addr == (void *)-1) {
		printf("ftrucate failed on fd %d\n", fd);
		return;
	}
	strcpy(addr, "Netos shared mem working.");

	rv = munmap(addr, 100);
	if(rv < 0) {
		printf("mnumap failed on fd %d\n", fd);
		return;
	}
	rv = close(fd);
	if(rv < 0) {
		printf("close failed on fd %d\n", fd);
		return;
	}
#if 0
	rv = shm_unlink("/netos_shm");
	if(rv < 0) {
		printf("close failed on fd %d\n", fd);
		return;
	}
#endif
}

static void test_semaphore()
{
    sem_t *sem;
    sem = sem_open("test_sem", O_CREAT, 0, 1);
    if(!sem){
        printf("Semaphore allocation failed!!\n");
	return;
    }else{
        sem_wait(sem);
        printf("Semaphore addr:%p\n",sem);
        sem_post(sem);
    }
    sem_wait(sem);
    printf("Semaphore test passed.\n\n");
    sem_post(sem);
    sem_close(sem);
}

static void timer_test(int sig)
{
    static int no_of_times;
    struct itimerval itimer;
    int ret = 0;

    printf("Timer called %d times..\n", no_of_times);
    no_of_times++;
    
	if (no_of_times == 5) {
        itimer.it_value.tv_sec = 0;
        itimer.it_value.tv_usec = 0;
        itimer.it_interval.tv_sec = 0;
        itimer.it_interval.tv_usec = 0;
        ret = setitimer(0, &itimer, NULL);
        timer_test_done = 1;

        if (ret) {
            printf("setitimer failed!!!\n");
        }
    }

    return;
}

static void test_timers(void)
{
    struct itimerval itimer;
    signal(SIGALRM, timer_test); 
    itimer.it_value.tv_sec = 0;
    itimer.it_value.tv_usec = 1000;
    itimer.it_interval.tv_sec = 0;
    itimer.it_interval.tv_usec = 1000;
    setitimer(0, &itimer, NULL);
    while(!timer_test_done);
}

int main(int argc, char *argv[])
{
	MEM_TYPE *mem = 0;
	unsigned char *mptr = 0;

	int i, size = (1<<20);				/* Attempt to malloc 1M */
	int nblocks = size / sizeof(MEM_TYPE);

	test_semaphore();
	test_timers();
	test_floating_point();
	test_gettimeofday();
	test_time_functions();

	mem_pool_test();
	mem_test_master();

	printf("In main pid = %d\n", getpid());
	launch_children();

	printf("Allocating memory (size=%d bytes)...\n", size);
	mem = malloc(size);
	if (!mem) {
		printf("Unable to allocate memory of size %d bytes\n", size);
		goto err_exit;
	}

	printf("Writing to memory...@ 0x%lx\n", (long)mem);
	for(i=0;i<nblocks;i++) {
		mem[i] = MEM_PATTERN;
	}

	printf("Reading from memory @ 0x%lx\n", (long)mem);
	for(i=0;i<nblocks;i++) {
		if (mem[i] != MEM_PATTERN) {
			printf("memory read at byte[%x] failed!\n", i);
			goto err_exit;
		}
	}

	/* Test memcpy */
	mptr = (unsigned char *)malloc(512);
	if (!mptr) {
		printf("Malloc failed!\n");
		goto err_exit;
	}
	printf("Invoking memcpy...\n");
	memcpy(mptr, (unsigned char *)mem, 512);

	/* Test strcmp */
	if (strncmp((char *)mptr, (char *)mem, 512) == 0) {
		printf("String Compare OK!\n");
	}
	else {
		printf("String Compare Failed!\n");
	}

	printf("pid = %d exiting (success)\n", getpid());
	exit(EXIT_SUCCESS);
err_exit:
	printf("pid = %d exiting (failure)\n", getpid());
	exit(EXIT_FAILURE);
}
